#include "Batch_Data_Proc.h"


Batch_Data_Proc::Batch_Data_Proc()
{
}


Batch_Data_Proc::~Batch_Data_Proc()
{
}

void Batch_Data_Proc::Batch_Mean_Var_2D_Value_Label(string fileName_Cat, int *simul_Label, int *num_Simul_Label, int p_RepNum, int num_Segments, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Cat: file name header (i.e. "gn" for genotype data)
	//int *simul_Label: 1-D array simulation label
	//int *num_Simul_Label: the number of elements in the simulation label
	//int p_RepNum: replication number (i.e. 0)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;
	string tmp_Str_0;

	//sets a label for file outputs
	fileRW.Get_Simul_Label(&tmp_Str_0, simul_Label, num_Simul_Label, p_RepNum);

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Cat + "_" + tmp_Str_0 + ".csv";//file content index

	//calculates means and variance in batch without label
	Batch_Mean_Var_2D_Value(tmp_Str_1, num_Segments, num_Row_Skip, num_Col_Skip, sort_Order);
}

void Batch_Data_Proc::Batch_Mean_Var_2D_Value_Frequency_Label(string fileName_Val, string fileName_Freq, int *simul_Label, int *num_Simul_Label, int p_RepNum, int num_Segments, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Val: the complete name of file containing the value for each category (i.e. genotype_Label.csv)
	//The number of columns for the value spread sheet should match with the number of act
	//string fileName_Fre: file name header (i.e. "gn" for genotype data)
	//int *simul_Label: 1-D array simulation label
	//int *num_Simul_Label: the number of elements in the simulation label
	//int p_RepNum: replication number (i.e. 0)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;
	string tmp_Str_0;

	//sets a label for file outputs
	fileRW.Get_Simul_Label(&tmp_Str_0, simul_Label, num_Simul_Label, p_RepNum);

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Freq + "_" + tmp_Str_0 + ".csv";//file content index

	//calculates means and variance in batch (value * frequency data) without label
	Batch_Mean_Var_2D_Value_Frequency(fileName_Val, tmp_Str_1, num_Segments, num_Row_Skip, num_Col_Skip, sort_Order);
}

void Batch_Data_Proc::Batch_Mean_Var_2D_Value(string fileName_Cat, int num_Segments, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Cat: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;

	//File name creation
	string tmp_Str;
	tmp_Str = "mv_" + fileName_Cat;//file content index

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Cat;//file content index

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D_String(tmp_Str_1, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	int i, j, num_Repeats;

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop

	//sets the data sort order
	if (sort_Order == 0)
	{//row sorted
		num_Repeats = num_Rows_Data;
	}
	else
	{//column sorted
		num_Repeats = num_Columns_Data;
	}//end else

	int num_Stats = 3;//the number of statistics types

	double **p_Mean = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Mean[i] = new double[num_Segments];
	}//end repeat loop

	double **p_Variance = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Variance[i] = new double[num_Segments];
	}//end repeat loop

	double **p_SD = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_SD[i] = new double[num_Segments];
	}//end repeat loop

	Data_Manipulation dataMani;//an object for data manipulation
	//calculates means
	dataMani.Get_Mean_SD_Pop_Value_2D(p_Mean, p_Variance, p_SD, p_Data, num_Rows_Data, num_Columns_Data, num_Segments, sort_Order);

	//sets the number of cells to skip
	int num_Cel_Skip;

	if (sort_Order == 0)
	{
		num_Cel_Skip = num_Col_Skip;
	}
	else
	{
		num_Cel_Skip = num_Row_Skip;
	}//end else

	int num_Columns_Out = (num_Stats * num_Segments) + num_Cel_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	int k, tmp_Seg, tmp_Stat_Seq;

	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				columnHeader[i][j] = "Sequence";
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					columnHeader[i][j] = "Mean";
					break;
				case 1:
					columnHeader[i][j] = "Variance";
					break;
				case 2:
					columnHeader[i][j] = "SD";
					break;
				default:
					columnHeader[i][j] = "Undefined";
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Repeats; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				if (sort_Order == 0)
				{//for row sorted
					tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];
				}
				else
				{//for column sorted
					tmp_String_Seq = tmp_Arr_String[j][(i + num_Col_Skip)];
				}//end else

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					output_Double[i][j] = p_Mean[i][tmp_Seg];
					break;
				case 1:
					output_Double[i][j] = p_Variance[i][tmp_Seg];
					break;
				case 2:
					output_Double[i][j] = p_SD[i][tmp_Seg];
					break;
				default:
					output_Double[i][j] = 0.0;
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Repeats, &num_Columns_Out);

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	//de-allocate memory
	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Mean[i];
	}//end repeat loop

	delete[] p_Mean;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Variance[i];
	}//end repeat loop

	delete[] p_Variance;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_SD[i];
	}//end repeat loop

	delete[] p_SD;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Output;
}

void Batch_Data_Proc::Batch_Multiple_Source_Mean_Var_2D_Value(vector<string> fileName_Cat, int *num_Simul_Label, int p_RepNum, int num_Segments, int num_Row_Skip, int num_Col_Skip)
{
	//vector<string> fileName_Cat: full file name (i.e. "gn.csv" for genotype data) vector
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;

	//the number of files to process
	int num_Files = (int)fileName_Cat.size();
	int i, j, k;
	string tmp_Str_1;//Data file name
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data
	int num_Rows_Data, num_Columns_Data;
	vector<vector<string> > tmp_Arr_String_0;//string vector for storing data and lables

	//gets data sizes
	if (num_Files > 0)
	{//the first file
		i = 0;//initialization

		//imports data from existing file	
		tmp_Str_1 = fileName_Cat[i];//file content index
		int size_String_Data = (int)tmp_Str_1.length();
		//The name of the file containing data
		char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
		//the number of cells in char* should be (the length of string + 1)
		fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
		//sets a label for file outputs

		//imports string data from a data file
		fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String_0, &num_Rows_Raw, &num_Columns_Raw);

		num_Rows_Data = num_Rows_Raw - num_Row_Skip;//the number of data rows
		num_Columns_Data = num_Columns_Raw - num_Col_Skip;//the number of data columns
	}//end if

	double ***p_Data = new double**[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double*[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = new double[num_Files];

			for (k = 0; k < num_Files; k++)
			{
				p_Data[i][j][k] = 0.0;
			}//end file loop
		}//end column loop
	}//end row loop	

	for (i = 1; i < num_Files; i++)
	{
		//imports data from existing file	
		tmp_Str_1 = fileName_Cat[i];//file content index
		int size_String_Data = (int)tmp_Str_1.length();
		//The name of the file containing data
		char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
		//the number of cells in char* should be (the length of string + 1)
		fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
		//sets a label for file outputs

		//Temporary data containers
		vector<vector<string> > tmp_Arr_String;//string vector for storing data
		vector<vector<double> > tmp_Arr_Double;//string vector for storing data
		int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

		//imports string data from a data file
		fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
		fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

		num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
		num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

		//adds new data from a file
		for (j = 0; j < num_Rows_Data; j++)
		{
			for (k = 0; k < num_Columns_Data; k++)
			{
				p_Data[j][k][i] = tmp_Arr_Double[j][k];
			}//end column loop
		}//end row loop
	}//source file loop

	//sets data containers
	int num_Stats = 3;//the number of statistics types

	double **p_Mean = new double *[num_Rows_Data];

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Mean[i] = new double[num_Columns_Data];
	}//end repeat loop

	double **p_Variance = new double *[num_Rows_Data];

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Variance[i] = new double[num_Columns_Data];
	}//end repeat loop

	double **p_SD = new double *[num_Rows_Data];

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_SD[i] = new double[num_Columns_Data];
	}//end repeat loop

	//calculates descriptive statistics
	double tmp_Sum;//local sum
	double tmp_Sum_Diff_Sq;//local sum of squared difference
	double tmp_Diff;
	double tmp_Mean;
	double tmp_Var;
	double d_Num_Files = (double)num_Files;
	double d_Num_Files_Minus_One = d_Num_Files - 1.0;

	for (i = 0; i < num_Rows_Data; i++)
	{
		for (j = 0; j < num_Columns_Data; j++)
		{
			tmp_Sum = 0.0;//local sum
			tmp_Sum_Diff_Sq = 0.0;//local sum of squared difference

			//calculates means
			for (k = 0; k < num_Files; k++)
			{
				tmp_Sum += p_Data[i][j][k];
			}//end file loop

			tmp_Mean = tmp_Sum / d_Num_Files;//local mean
			p_Mean[i][j] = tmp_Mean;

			for (k = 0; k < num_Files; k++)
			{
				tmp_Diff = p_Data[i][j][k] - tmp_Mean;//diviation
				tmp_Sum_Diff_Sq += (tmp_Diff * tmp_Diff);//squared dviation
			}//end file loop

			tmp_Var = (tmp_Sum_Diff_Sq / d_Num_Files_Minus_One);
			p_Variance[i][j] = tmp_Var;

			if (tmp_Var > 0.0)
			{
				p_SD[i][j] = (pow(tmp_Var, 0.5));
			}
			else
			{
				p_SD[i][j] = 0.0;
			}//end else
		}//end column
	}//end row loop

	//new file name creation
	string tmp_Str;
	tmp_Str = "sv_" + fileName_Cat[0];//file content index

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//the number of columns in output
	int num_Columns_Out = (num_Stats * num_Columns_Data) + num_Col_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	int tmp_Seg, tmp_Stat_Seq;

	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Col_Skip)
			{//sequence
				columnHeader[i][j] = "Sequence";
			}
			else
			{//statistics
				k = j - num_Col_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					columnHeader[i][j] = "Mean";
					break;
				case 1:
					columnHeader[i][j] = "Variance";
					break;
				case 2:
					columnHeader[i][j] = "SD";
					break;
				default:
					columnHeader[i][j] = "Undefined";
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Rows_Data];

	for (i = 0; i < num_Rows_Data; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Rows_Data; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Col_Skip)
			{//sequence
				tmp_String_Seq = tmp_Arr_String_0[(i + num_Row_Skip)][j];

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				k = j - num_Col_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					output_Double[i][j] = p_Mean[i][tmp_Seg];
					break;
				case 1:
					output_Double[i][j] = p_Variance[i][tmp_Seg];
					break;
				case 2:
					output_Double[i][j] = p_SD[i][tmp_Seg];
					break;
				default:
					output_Double[i][j] = 0.0;
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Rows_Data, &num_Columns_Out);

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	//de-allocate memory
	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Mean[i];
	}//end repeat loop

	delete[] p_Mean;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Variance[i];
	}//end repeat loop

	delete[] p_Variance;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_SD[i];
	}//end repeat loop

	delete[] p_SD;

	for (i = 0; i < num_Rows_Data; i++)
	{
		for (j = 0; j < num_Columns_Data; j++)
		{
			delete[] p_Data[i][j];
		}//end column loop

		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Output;
}

void Batch_Data_Proc::Batch_Mean_Var_2D_Value_Frequency(string fileName_Val, string fileName_Freq, int num_Segments, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Val: the complete name of file containing the value for each category (i.e. genotype_Label.csv)
	//The number of columns for the value spread sheet should match with the number of act
	//string fileName_Fre: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;
	int size_String;
	int i, j, num_Repeats;

	//imports values
	size_String = (int)fileName_Val.length();
	char *tmp_Str_Value = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(fileName_Val, tmp_Str_Value);//copy string and paste on char*
	vector<vector<string> > tmp_Arr_String_Value;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double_Value;//string vector for storing data
	int num_Rows_Value;//the number of data rows
	int num_Columns_Value;//the number of data columns

	fileRW.CSV_Read_String_2D(tmp_Str_Value, tmp_Arr_String_Value, &num_Rows_Value, &num_Columns_Value);

	int num_Rows_Null = 0;//the number of data rows
	int num_Columns_Null = 0;//the number of data columns
	//converts 2-D string to 2-D double
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String_Value, tmp_Arr_Double_Value, num_Rows_Null, num_Columns_Null);

	double* value_1D = new double[num_Columns_Value];

	//sets the values for categories
	for (j = 0; j < num_Columns_Value; j++)
	{
		value_1D[j] = tmp_Arr_Double_Value[0][j];
	}//end column loop

	//File name creation
	string tmp_Str;
	tmp_Str = "mv_" + fileName_Freq;//file content index

	size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Freq;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop

	//sets the data sort order
	if (sort_Order == 0)
	{//row sorted
		num_Repeats = num_Rows_Data;
	}
	else
	{//column sorted
		num_Repeats = num_Columns_Data;
	}//end else

	int num_Stats = 3;//the number of statistics types

	double **p_Mean = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Mean[i] = new double[num_Segments];
	}//end repeat loop

	double **p_Variance = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Variance[i] = new double[num_Segments];
	}//end repeat loop

	double **p_SD = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_SD[i] = new double[num_Segments];
	}//end repeat loop

	Data_Manipulation dataMani;//an object for data manipulation
	//calculates means
	dataMani.Get_Mean_SD_Pop_Value_Fixed_Freq_2D(p_Mean, p_Variance, p_SD, value_1D, p_Data, num_Rows_Data, num_Columns_Data, num_Segments, sort_Order);

	//sets the number of cells to skip
	int num_Cel_Skip;

	if (sort_Order == 0)
	{
		num_Cel_Skip = num_Col_Skip;
	}
	else
	{
		num_Cel_Skip = num_Row_Skip;
	}//end else

	int num_Columns_Out = (num_Stats * num_Segments) + num_Cel_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	int k, tmp_Seg, tmp_Stat_Seq;

	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				columnHeader[i][j] = "Sequence";
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					columnHeader[i][j] = "Mean";
					break;
				case 1:
					columnHeader[i][j] = "Variance";
					break;
				case 2:
					columnHeader[i][j] = "SD";
					break;
				default:
					columnHeader[i][j] = "Undefined";
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Repeats; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				if (sort_Order == 0)
				{//for row sorted
					tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];
				}
				else
				{//for column sorted
					tmp_String_Seq = tmp_Arr_String[j][(i + num_Col_Skip)];
				}//end else

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					output_Double[i][j] = p_Mean[i][tmp_Seg];
					break;
				case 1:
					output_Double[i][j] = p_Variance[i][tmp_Seg];
					break;
				case 2:
					output_Double[i][j] = p_SD[i][tmp_Seg];
					break;
				default:
					output_Double[i][j] = 0.0;
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Repeats, &num_Columns_Out);

	delete[] value_1D;

	delete[] tmp_Str_Value;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	//de-allocate memory
	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Mean[i];
	}//end repeat loop

	delete[] p_Mean;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Variance[i];
	}//end repeat loop

	delete[] p_Variance;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_SD[i];
	}//end repeat loop

	delete[] p_SD;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Data;
	delete[] fileName_Output;
}

string Batch_Data_Proc::Batch_Product_Value_Frequency(vector<double> &p_Val, string fileName_Freq, int num_Row_Skip, int num_Col_Skip)
{
	//string fileName_Val: the complete name of file containing the value for each category (i.e. genotype_Label.csv)
	//The number of columns for the value spread sheet should match with the number of act
	//string fileName_Fre: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted)	
	File_RW fileRW;
	int size_String;
	int i, j, num_Repeats;

	int num_Columns_Value = (int)p_Val.size();//the number of data columns
	double* value_1D = new double[num_Columns_Value];

	//sets the values for categories
	for (j = 0; j < num_Columns_Value; j++)
	{
		value_1D[j] = p_Val[j];
	}//end column loop

	//File name creation
	string tmp_Str;
	tmp_Str = "p_" + fileName_Freq;//file content index

	size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Freq;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];
	}//end row loop

	//calculates the product of a value and a frequency
	//sets the data sort order
	//row sorted
	for (i = 0; i < num_Rows_Data; i++)
	{
		for (j = 0; j < num_Columns_Data; j++)
		{
			//calculates the product of a value and a frequency
			p_Data[i][j] = value_1D[j] * tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop	

	int num_Stats = 3;//the number of statistics types	
	int num_Cel_Skip = num_Col_Skip;//sets the number of cells to skip
	int num_Columns_Out = (int)tmp_Arr_String[0].size();

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];

		for (j = 0; j < num_Columns_Out; j++)
		{
			columnHeader[i][j] = tmp_Arr_String[i][j];
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	num_Repeats = num_Rows_Data;

	//sets the file output data
	double** output_Double = new double*[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Repeats; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];
				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				output_Double[i][j] = p_Data[i][(j - num_Cel_Skip)];
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Repeats, &num_Columns_Out);

	delete[] value_1D;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}

string Batch_Data_Proc::Batch_Mean_Var_2D_Value_Frequency_Raw(vector<double> &p_Val, string fileName_Freq, int num_Segments, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Val: the complete name of file containing the value for each category (i.e. genotype_Label.csv)
	//The number of columns for the value spread sheet should match with the number of act
	//string fileName_Fre: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;
	int size_String;
	int i, j, num_Repeats;

	int num_Columns_Value = (int)p_Val.size();//the number of data columns
	double* value_1D = new double[num_Columns_Value];

	//sets the values for categories
	for (j = 0; j < num_Columns_Value; j++)
	{
		value_1D[j] = p_Val[j];
	}//end column loop

	//File name creation
	string tmp_Str;
	tmp_Str = "mv_" + fileName_Freq;//file content index

	size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Freq;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop

	//sets the data sort order
	if (sort_Order == 0)
	{//row sorted
		num_Repeats = num_Rows_Data;
	}
	else
	{//column sorted
		num_Repeats = num_Columns_Data;
	}//end else

	int num_Stats = 3;//the number of statistics types

	double **p_Mean = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Mean[i] = new double[num_Segments];
	}//end repeat loop

	double **p_Variance = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Variance[i] = new double[num_Segments];
	}//end repeat loop

	double **p_SD = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_SD[i] = new double[num_Segments];
	}//end repeat loop

	Data_Manipulation dataMani;//an object for data manipulation
	//calculates means
	dataMani.Get_Mean_SD_Pop_Value_Fixed_Freq_2D(p_Mean, p_Variance, p_SD, value_1D, p_Data, num_Rows_Data, num_Columns_Data, num_Segments, sort_Order);

	//sets the number of cells to skip
	int num_Cel_Skip;

	if (sort_Order == 0)
	{
		num_Cel_Skip = num_Col_Skip;
	}
	else
	{
		num_Cel_Skip = num_Row_Skip;
	}//end else

	int num_Columns_Out = (num_Stats * num_Segments) + num_Cel_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	int k, tmp_Seg, tmp_Stat_Seq;

	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				columnHeader[i][j] = "Sequence";
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					columnHeader[i][j] = "Mean";
					break;
				case 1:
					columnHeader[i][j] = "Variance";
					break;
				case 2:
					columnHeader[i][j] = "SD";
					break;
				default:
					columnHeader[i][j] = "Undefined";
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Repeats; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				if (sort_Order == 0)
				{//for row sorted
					tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];
				}
				else
				{//for column sorted
					tmp_String_Seq = tmp_Arr_String[j][(i + num_Col_Skip)];
				}//end else

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					output_Double[i][j] = p_Mean[i][tmp_Seg];
					break;
				case 1:
					output_Double[i][j] = p_Variance[i][tmp_Seg];
					break;
				case 2:
					output_Double[i][j] = p_SD[i][tmp_Seg];
					break;
				default:
					output_Double[i][j] = 0.0;
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Repeats, &num_Columns_Out);

	delete[] value_1D;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	//de-allocate memory
	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Mean[i];
	}//end repeat loop

	delete[] p_Mean;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Variance[i];
	}//end repeat loop

	delete[] p_Variance;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_SD[i];
	}//end repeat loop

	delete[] p_SD;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}

string Batch_Data_Proc::Batch_Mean_Var_2D_Value_Frequency_Raw_Add_Char(vector<double> &p_Val, string fileName_Freq, string p_Character_To_Add, int num_Segments, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Val: the complete name of file containing the value for each category (i.e. genotype_Label.csv)
	//The number of columns for the value spread sheet should match with the number of act
	//string fileName_Fre: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;
	int size_String;
	int i, j, num_Repeats;

	int num_Columns_Value = (int)p_Val.size();//the number of data columns
	double* value_1D = new double[num_Columns_Value];

	//sets the values for categories
	for (j = 0; j < num_Columns_Value; j++)
	{
		value_1D[j] = p_Val[j];
	}//end column loop

	//File name creation
	string tmp_Str;
	tmp_Str = "mv_" + p_Character_To_Add + "_" + fileName_Freq;//file content index

	size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Freq;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop

	//sets the data sort order
	if (sort_Order == 0)
	{//row sorted
		num_Repeats = num_Rows_Data;
	}
	else
	{//column sorted
		num_Repeats = num_Columns_Data;
	}//end else

	int num_Stats = 3;//the number of statistics types

	double **p_Mean = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Mean[i] = new double[num_Segments];
	}//end repeat loop

	double **p_Variance = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_Variance[i] = new double[num_Segments];
	}//end repeat loop

	double **p_SD = new double *[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		p_SD[i] = new double[num_Segments];
	}//end repeat loop

	Data_Manipulation dataMani;//an object for data manipulation
	//calculates means
	dataMani.Get_Mean_SD_Pop_Value_Fixed_Freq_2D(p_Mean, p_Variance, p_SD, value_1D, p_Data, num_Rows_Data, num_Columns_Data, num_Segments, sort_Order);

	//sets the number of cells to skip
	int num_Cel_Skip;

	if (sort_Order == 0)
	{
		num_Cel_Skip = num_Col_Skip;
	}
	else
	{
		num_Cel_Skip = num_Row_Skip;
	}//end else

	int num_Columns_Out = (num_Stats * num_Segments) + num_Cel_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	int k, tmp_Seg, tmp_Stat_Seq;

	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				columnHeader[i][j] = "Sequence";
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					columnHeader[i][j] = "Mean";
					break;
				case 1:
					columnHeader[i][j] = "Variance";
					break;
				case 2:
					columnHeader[i][j] = "SD";
					break;
				default:
					columnHeader[i][j] = "Undefined";
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Repeats];

	for (i = 0; i < num_Repeats; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Repeats; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Cel_Skip)
			{//sequence
				if (sort_Order == 0)
				{//for row sorted
					tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];
				}
				else
				{//for column sorted
					tmp_String_Seq = tmp_Arr_String[j][(i + num_Col_Skip)];
				}//end else

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				k = j - num_Cel_Skip;
				tmp_Seg = (k / num_Stats);//data segment
				tmp_Stat_Seq = k - (tmp_Seg * num_Stats);//statistics type

				switch (tmp_Stat_Seq)
				{
				case 0:
					output_Double[i][j] = p_Mean[i][tmp_Seg];
					break;
				case 1:
					output_Double[i][j] = p_Variance[i][tmp_Seg];
					break;
				case 2:
					output_Double[i][j] = p_SD[i][tmp_Seg];
					break;
				default:
					output_Double[i][j] = 0.0;
					break;
				}//end switch
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Repeats, &num_Columns_Out);

	delete[] value_1D;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	//de-allocate memory
	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Mean[i];
	}//end repeat loop

	delete[] p_Mean;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_Variance[i];
	}//end repeat loop

	delete[] p_Variance;

	for (i = 0; i < num_Repeats; i++)
	{
		delete[] p_SD[i];
	}//end repeat loop

	delete[] p_SD;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}

string Batch_Data_Proc::Batch_Delta_2D_Value(string fileName_Cat, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Cat: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;

	//File name creation
	string tmp_Str;
	tmp_Str = "d_" + fileName_Cat;//file content index

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Cat;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	int i, j;

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop

	double **p_Delta = new double*[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Delta[i] = new double[num_Columns_Data];
	}//end row loop

	Data_Manipulation dataMani;//an object for data manipulation
	//calculates means
	dataMani.Get_2D_Delta(p_Delta, p_Data, &num_Rows_Data, &num_Columns_Data, sort_Order);

	//sets the number of cells to skip
	int num_Columns_Out = num_Columns_Data + num_Col_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			columnHeader[i][j] = tmp_Arr_String[i][j];
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Rows_Data];

	for (i = 0; i < num_Rows_Data; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Rows_Data; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Col_Skip)
			{//sequence
				tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				output_Double[i][j] = p_Delta[i][(j - num_Col_Skip)];
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Rows_Data, &num_Columns_Out);

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Delta[i];
	}//end row loop

	delete[] p_Delta;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}

string Batch_Data_Proc::Batch_Delta_Delta_2D_Value(string fileName_Cat, int num_Row_Skip, int num_Col_Skip, int sort_Order)
{
	//string fileName_Cat: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int sort_Order: data sort direction, //sort_Order: (0: row sorted, 1: column sorted)	
	File_RW fileRW;

	//File name creation
	string tmp_Str;
	tmp_Str = "dd_" + fileName_Cat;//file content index

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Cat;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<double> > tmp_Arr_Double;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);
	fileRW.Convert_String_2D_To_Double_2D(tmp_Arr_String, tmp_Arr_Double, num_Row_Skip, num_Col_Skip);

	int num_Rows_Data = (int)tmp_Arr_Double.size();//the number of data rows
	int num_Columns_Data = (int)tmp_Arr_Double[0].size();//the number of data columns

	double **p_Data = new double*[num_Rows_Data];//raw data

	int i, j;

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Data[i] = new double[num_Columns_Data];

		for (j = 0; j < num_Columns_Data; j++)
		{
			p_Data[i][j] = tmp_Arr_Double[i][j];
		}//end column loop
	}//end row loop

	double **p_Delta = new double*[num_Rows_Data];//raw data

	for (i = 0; i < num_Rows_Data; i++)
	{
		p_Delta[i] = new double[num_Columns_Data];
	}//end row loop

	Data_Manipulation dataMani;//an object for data manipulation
	//calculates means
	dataMani.Get_2D_Delta_Delta(p_Delta, p_Data, &num_Rows_Data, &num_Columns_Data, sort_Order);

	int num_Columns_Out = num_Columns_Data + num_Col_Skip;

	//sets the column headers for the output file
	int num_String_Rows = 1;
	string** columnHeader = new string*[num_String_Rows];

	for (i = 0; i < num_String_Rows; i++)
	{
		columnHeader[i] = new string[num_Columns_Out];
	}//end row loop

	//sets the column headers
	for (i = 0; i < num_String_Rows; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			columnHeader[i][j] = tmp_Arr_String[i][j];
		}//end column loop
	}//end row loop

	//writes the column headers on the output file
	fileRW.CSV_Write_Add_String_2D(fileName_Output, columnHeader, &num_String_Rows, &num_Columns_Out);

	//sets the file output data
	double** output_Double = new double*[num_Rows_Data];

	for (i = 0; i < num_Rows_Data; i++)
	{
		output_Double[i] = new double[num_Columns_Out];
	}//end row loop

	string tmp_String_Seq;

	for (i = 0; i < num_Rows_Data; i++)
	{
		for (j = 0; j < num_Columns_Out; j++)
		{
			if (j < num_Col_Skip)
			{//sequence
				tmp_String_Seq = tmp_Arr_String[(i + num_Row_Skip)][j];

				//converts string to double
				fileRW.String_To_Double(&output_Double[i][j], &tmp_String_Seq);
			}
			else
			{//statistics
				output_Double[i][j] = p_Delta[i][(j - num_Col_Skip)];
			}//end else
		}//end column loop
	}//end row loop

	//writes the out data on the output file
	fileRW.CSV_Write_Add_Double_2D(fileName_Output, output_Double, &num_Rows_Data, &num_Columns_Out);

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] output_Double[i];
	}//end row loop

	delete[] output_Double;

	for (i = 0; i < num_String_Rows; i++)
	{
		delete[] columnHeader[i];
	}//end row loop

	delete[] columnHeader;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Delta[i];
	}//end row loop

	delete[] p_Delta;

	for (i = 0; i < num_Rows_Data; i++)
	{
		delete[] p_Data[i];
	}//end row loop

	delete[] p_Data;

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}

string Batch_Data_Proc::Batch_Trim_2D_String_Data_Interval_Row_Label(string fileName_Cat, int num_Row_Skip, int num_Col_Skip, int p_Row_Interval)
{
	//trims data in a csv file
	//to keep the row and the column labels set 1) num_Row_Skip = the number of column label rows, and 2) num_Col_Skip = 0
	//keeps column lables
	//string fileName_Cat: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int p_Row_Interval: sampling interval (i. e. p_Row_Interval = 100, rows are sampled in every 100 rows)
	File_RW fileRW;

	//File name creation
	string tmp_Str;
	tmp_Str = "iv_" + fileName_Cat;//file content index

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Cat;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<string> > tmp_Arr_Trimmed;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);

	Data_Manipulation datMani;//object for data processing
	datMani.Trim_2D_String_Data_Interval_Keep_Column_Label(tmp_Arr_Trimmed, tmp_Arr_String, num_Row_Skip, num_Col_Skip, p_Row_Interval);

	//writes trimmed 2-D string vector on a file
	fileRW.CSV_Write_Add_String_2D_Vector(fileName_Output, tmp_Arr_Trimmed);

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}

string Batch_Data_Proc::Batch_Trim_2D_String_Data_Interval(string fileName_Cat, int num_Row_Skip, int num_Col_Skip, int p_Row_Interval)
{
	//trims data in a csv file
	//string fileName_Cat: full file name (i.e. "gn.csv" for genotype data)
	//int num_Segments: the number of segments in the data (i.e. 2 if the data is separated into two parts)
	//int num_Row_Skip: the number of rows to skip reading (not to data headers)
	//int num_Col_Skip: the number of columns to skip reading (not to read data headers)
	//int p_Row_Interval: sampling interval (i. e. p_Row_Interval = 100, rows are sampled in every 100 rows)
	File_RW fileRW;

	//File name creation
	string tmp_Str;
	tmp_Str = "iv_" + fileName_Cat;//file content index

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	//imports data from existing file	
	string tmp_Str_1;//Data file name
	tmp_Str_1 = fileName_Cat;//file content index
	int size_String_Data = (int)tmp_Str_1.length();
	//The name of the file containing data
	char *fileName_Data = new char[(size_String_Data + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str_1, fileName_Data);//copy string and paste on char*
	//sets a label for file outputs

	//Temporary data containers
	vector<vector<string> > tmp_Arr_String;//string vector for storing data
	vector<vector<string> > tmp_Arr_Trimmed;//string vector for storing data
	int num_Rows_Raw, num_Columns_Raw;//the numbers of rows and columns for imported string data

	//imports string data from a data file
	fileRW.CSV_Read_String_2D(fileName_Data, tmp_Arr_String, &num_Rows_Raw, &num_Columns_Raw);

	Data_Manipulation datMani;//object for data processing
	datMani.Trim_2D_String_Data_Interval(tmp_Arr_Trimmed, tmp_Arr_String, num_Row_Skip, num_Col_Skip, p_Row_Interval);

	//writes trimmed 2-D string vector on a file
	fileRW.CSV_Write_Add_String_2D_Vector(fileName_Output, tmp_Arr_Trimmed);

	delete[] fileName_Data;
	delete[] fileName_Output;

	return tmp_Str;
}