#include "Data_Manipulation.h"


Data_Manipulation::Data_Manipulation()
{
}


Data_Manipulation::~Data_Manipulation()
{
}

void Data_Manipulation::Sort_Double_One_D(double *p_Reference, int *p_Size_dataSet, int *p_Index_New, int *p_Size_SortedSet)
{
	//p_Reference: 1-d array with double values
	//p_Size_dataSet: the size of p_Reference
	//p_Index_New: new indeces for elements in p_Reference after sorting
	//p_Size_SortedSet: the size of new sorted set not having redundancy

	//sorts the value and returns the new data index (*_Index_New) correspond to the old index for (*p_Reference)
	int size_D = (*p_Size_dataSet);

	if (size_D > 0)
	{
		std::set<double> sortedSet;
		std::set<double>::iterator it;

		//adds the first element
		sortedSet.insert(p_Reference[0]);

		int i;

		if (size_D > 1)
		{//adds new elements
			for (i = 1; i < size_D; i++)
			{
				it = sortedSet.find(p_Reference[i]);

				if (it == sortedSet.end())
				{// a new element
					sortedSet.insert(p_Reference[i]);
				}//end if
			}//end element loop
		}
		else
		{
			(*p_Size_SortedSet) = 1;//one elements
		}//end else

		//the size of sorted values
		int sortedSize = (int)sortedSet.size();
		(*p_Size_SortedSet) = sortedSize;

		//assigns to the new index number sorted by the reference value
		for (i = 0; i < size_D; i++)
		{
			it = sortedSet.find(p_Reference[i]);
			p_Index_New[i] = (int)std::distance(sortedSet.begin(), it);
		}//end i loop
	}
	else
	{
		(*p_Size_SortedSet) = 0;//no elements
	}//end else
}

void Data_Manipulation::Sort_Double_Two_D(double **p_Reference, int *p_Size_A_dataSet, int *p_Size_B_dataSet, int **p_Index_New, int *p_Size_SortedSet)
{
	//p_Reference: 2-d array with double values
	//p_Size_A_dataSet: the size the first dimension for p_Reference
	//p_Size_A_dataSet: the size the second dimension for p_Reference
	//p_Index_New: new indeces for elements in p_Reference after sorting
	//p_Size_SortedSet: the size of new sorted set not having redundancy

	int size_TwoD = (*p_Size_A_dataSet) * (*p_Size_B_dataSet);

	double *tmp_DataSet = new double[size_TwoD];


	int i, j, cnt;
	cnt = 0;//counter

	int end0 = (*p_Size_A_dataSet);
	int end1 = ((*p_Size_B_dataSet));

	//converts 2D to 1D
	for (i = 0; i < end0; i++)
	{
		for (j = 0; j < end1; j++)
		{
			tmp_DataSet[cnt] = p_Reference[i][j];
			cnt++;
		}//end second dim
	}//end first dim

	int *tmp_Index = new int[size_TwoD];

	//sorts data in 1D
	Sort_Double_One_D(tmp_DataSet, &size_TwoD, tmp_Index, p_Size_SortedSet);

	//converts 1D to 2D
	cnt = 0;

	for (i = 0; i < end0; i++)
	{
		for (j = 0; j < end1; j++)
		{
			p_Index_New[i][j] = tmp_Index[cnt];
			cnt++;
		}//end second dim
	}//end first dim

	//memory deallocation
	delete[] tmp_Index;
	delete[] tmp_DataSet;
}

void Data_Manipulation::Sort_Double_Three_D(double ***p_Reference, int *p_Size_A_dataSet, int *p_Size_B_dataSet, int *p_Size_C_dataSet, int ***p_Index_New, int *p_Size_SortedSet)
{
	//p_Reference: 2-d array with double values
	//p_Size_A_dataSet: the size the first dimension for p_Reference
	//p_Size_A_dataSet: the size the second dimension for p_Reference
	//p_Index_New: new indeces for elements in p_Reference after sorting
	//p_Size_SortedSet: the size of new sorted set not having redundancy

	int size_ThreeD = (*p_Size_A_dataSet) * (*p_Size_B_dataSet) * (*p_Size_C_dataSet);

	double *tmp_DataSet = new double[size_ThreeD];


	int i, j, k, cnt;
	cnt = 0;//counter

	int end0 = (*p_Size_A_dataSet);
	int end1 = (*p_Size_B_dataSet);
	int end2 = (*p_Size_B_dataSet);

	//converts 3D to 1D
	for (i = 0; i < end0; i++)
	{
		for (j = 0; j < end1; j++)
		{
			for (k = 0; k < end2; k++)
			{
				tmp_DataSet[cnt] = p_Reference[i][j][k];
				cnt++;
			}//end third dim
		}//end second dim
	}//end first dim

	int *tmp_Index = new int[size_ThreeD];

	//sorts data in 1D
	Sort_Double_One_D(tmp_DataSet, &size_ThreeD, tmp_Index, p_Size_SortedSet);

	//converts 1D to 2D
	cnt = 0;

	//converts 3D to 1D
	for (i = 0; i < end0; i++)
	{
		for (j = 0; j < end1; j++)
		{
			for (k = 0; k < end2; k++)
			{
				p_Index_New[i][j][k] = tmp_Index[cnt];
				cnt++;
			}//end third dim
		}//end second dim
	}//end first dim

	//memory deallocation
	delete[] tmp_Index;
	delete[] tmp_DataSet;
}

void Data_Manipulation::Example_Sort_Double_One_D(void)
{
	int num_Elements = 10;
	double * ref = new double[num_Elements];
	int *index = new int[num_Elements];

	int i, new_Size;
	for (i = 0; i < num_Elements; i++)
	{
		ref[i] = (double)i;
	}//end element loop

	ref[0] = 2.0;

	Sort_Double_One_D(ref, &num_Elements, index, &new_Size);

	std::cout << "New sorted set size is " << new_Size << std::endl << std::endl;

	for (i = 0; i < num_Elements; i++)
	{
		std::cout << "Index[" << i << "]: " << index[i] << std::endl;
	}//end element loop

	std::cout << "Type a character: " << std::endl;

	char tt;
	std::cin >> tt;

	delete[] ref;
	delete[] index;
}

void Data_Manipulation::Example_Sort_Double_Two_D(void)
{
	int i, j;
	int num_ElementsA = 10;
	int num_ElementsB = 10;

	double **ref = new double*[num_ElementsA];

	for (i = 0; i < num_ElementsA; i++)
	{
		ref[i] = new double[num_ElementsB];

		for (j = 0; j < num_ElementsB; j++)
		{
			ref[i][j] = (double)(i * j);
		}//end element loop
	}//end element loop

	int **index = new int*[num_ElementsA];

	for (i = 0; i < num_ElementsA; i++)
	{
		index[i] = new int[num_ElementsB];
	}//end element loop

	int new_Size;

	Sort_Double_Two_D(ref, &num_ElementsA, &num_ElementsB, index, &new_Size);

	std::cout << "New sorted set size is " << new_Size << std::endl << std::endl;


	for (i = 0; i < num_ElementsA; i++)
	{
		for (j = 0; j < num_ElementsB; j++)
		{
			std::cout << "Index[" << i << "][" << j << "]: " << index[i][j] << std::endl;
		}//end element loop
	}//end element loop

	std::cout << "Type a character: " << std::endl;

	char tt;
	std::cin >> tt;

	for (i = 0; i < num_ElementsA; i++)
	{
		delete[] ref[i];
	}//end element loop

	delete[] ref;

	for (i = 0; i < num_ElementsA; i++)
	{
		delete[] index[i];
	}//end element loop

	delete[] index;
}

void Data_Manipulation::ReArrange_4D_Mat(double ****mat_Rearranged, double ****mat, int nCls, int *s)
{
	//nCls: the number of classes for each dimension
	//s: vector indicating the sequence order 0: increasing order , 1: decreasing order
	//s[0]: 1st dimension order, s[1]: second dimension order, s[2]: third dimension order, s[3]:fourth dimension order
	const int dim = 4;
	int *counter = new int[dim];//counter
	int *start = new int[dim];//start point
	int *cursor = new int[dim];//current position for a rearrange matrix cursor
	int *increment = new int[dim];//increment for iteration for a rearranged matrix

	for (int i = 0; i < dim; i++)
	{
		if (s[i] == 0)
		{//increasing order
			start[i] = 0;
			increment[i] = 1;//increment
		}
		else
		{
			start[i] = (nCls - 1);
			increment[i] = (-1);//increment
		}//else
	}//end dimension loop

	cursor[0] = start[0];
	counter[0] = 0;
	while (counter[0] < nCls)
	{
		cursor[1] = start[1];
		counter[1] = 0;
		while (counter[1] < nCls)
		{
			cursor[2] = start[2];
			counter[2] = 0;
			while (counter[2] < nCls)
			{
				cursor[3] = start[3];
				counter[3] = 0;
				while (counter[3] < nCls)
				{//sets a rearranged matrix
					mat_Rearranged[cursor[0]][cursor[1]][cursor[2]][cursor[3]] = mat[counter[0]][counter[1]][counter[2]][counter[3]];
					cursor[3] += increment[3];
					counter[3]++;
				}//

				cursor[2] += increment[2];
				counter[2]++;
			}//

			cursor[1] += increment[1];
			counter[1]++;
		}//

		cursor[0] += increment[0];
		counter[0]++;
	}//

	delete[] start;
	delete[] cursor;
	delete[] increment;
	delete[] counter;
}

void Data_Manipulation::ReArrange_2D_Mat(double **mat_Rearranged, double **mat, int nCls, int *s)
{
	//nCls: the number of classes for each dimension
	//s: vector indicating the sequence order 0: increasing order , 1: decreasing order
	//s[0]: 1st dimension order, s[1]: second dimension order, s[2]: third dimension order, s[3]:fourth dimension order
	const int dim = 2;
	int *counter = new int[dim];//counter
	int *start = new int[dim];//start point
	int *cursor = new int[dim];//current position for a rearrange matrix cursor
	int *increment = new int[dim];//increment for iteration for a rearranged matrix

	for (int i = 0; i < dim; i++)
	{
		if (s[i] == 0)
		{//increasing order
			start[i] = 0;
			increment[i] = 1;//increment
		}
		else
		{
			start[i] = (nCls - 1);
			increment[i] = (-1);//increment
		}//else
	}//end dimension loop

	cursor[0] = start[0];
	counter[0] = 0;

	while (counter[0] < nCls)
	{
		cursor[1] = start[1];
		counter[1] = 0;
		while (counter[1] < nCls)
		{
			mat_Rearranged[cursor[0]][cursor[1]] = mat[counter[0]][counter[1]];
			cursor[1] += increment[1];
			counter[1]++;
		}//

		cursor[0] += increment[0];
		counter[0]++;
	}//

	delete[] start;
	delete[] cursor;
	delete[] increment;
	delete[] counter;
}

void Data_Manipulation::Get_MaxDiff_4D_Mat(double *p_Max, double ****mat0, double ****mat1, int nCls)
{
	int i, j, k, l;
	double tmp_Max = 0.0;

	//calculates the sums
	for (i = 0; i < nCls; i++)
	{
		for (j = 0; j < nCls; j++)
		{
			for (k = 0; k < nCls; k++)
			{
				for (l = 0; l < nCls; l++)
				{
					if ((abs((mat0[i][j][k][l] - mat1[i][j][k][l]))) > tmp_Max)
					{
						tmp_Max = (abs((mat0[i][j][k][l] - mat1[i][j][k][l])));
					}//end if
				}//end 4 d loop
			}//end 3 d loop
		}//end 2 d loop
	}//end 1 d loop

	(*p_Max) = tmp_Max;//sets the maximum difference
}

void Data_Manipulation::Get_MaxDiff_2D_Mat(double *p_Max, double **mat0, double **mat1, int nCls)
{
	int i, j;
	double tmp_Max = 0.0;

	//calculates the sums
	for (i = 0; i < nCls; i++)
	{
		for (j = 0; j < nCls; j++)
		{
			if ((abs((mat0[i][j] - mat1[i][j]))) > tmp_Max)
			{
				tmp_Max = (abs((mat0[i][j] - mat1[i][j])));
			}//end if
		}//end 2 d loop
	}//end 1 d loop

	(*p_Max) = tmp_Max;//sets the maximum difference
}

void Data_Manipulation::Get_CDF_4D_Discrete_Self(double ****map, int numCela, int numCelb, int numCelc, int numCeld)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j, k, l;

	//1. Initial process
	//cumul_Map[0][0][0][0] = map[0][0][0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		map[i][0][0][0] = map[i][0][0][0] + map[(i - 1)][0][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		map[0][i][0][0] = map[0][i][0][0] + map[0][(i - 1)][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelc; i++)
	{
		map[0][0][i][0] = map[0][0][i][0] + map[0][0][(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCeld; i++)
	{
		map[0][0][0][i] = map[0][0][0][i] + map[0][0][0][(i - 1)];
	}//end dimention 1 loop


	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			map[i][j][0][0] = map[i][j][0][0] + map[(i - 1)][j][0][0] + map[i][(j - 1)][0][0] - map[(i - 1)][(j - 1)][0][0];
		}
	}//end dimention 1 loop

	//Dimension 1,3
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			map[i][0][k][0] = map[i][0][k][0] + map[(i - 1)][0][k][0] + map[i][0][(k - 1)][0] - map[(i - 1)][0][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,4
	for (i = 1; i < numCela; i++)
	{
		for (l = 1; l < numCeld; l++)
		{
			map[i][0][0][l] = map[i][0][0][l] + map[(i - 1)][0][0][l] + map[i][0][0][(l - 1)] - map[(i - 1)][0][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			map[0][j][k][0] = map[0][j][k][0] + map[0][(j - 1)][k][0] + map[0][j][(k - 1)][0] - map[0][(j - 1)][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,4
	for (j = 1; j < numCelb; j++)
	{
		for (l = 1; l < numCeld; l++)
		{
			map[0][j][0][l] = map[0][j][0][l] + map[0][(j - 1)][0][l] + map[0][j][0][(l - 1)] - map[0][(j - 1)][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 3,4
	for (k = 1; k < numCelc; k++)
	{
		for (l = 1; l < numCeld; l++)
		{
			map[0][0][k][l] = map[0][0][k][l] + map[0][0][(k - 1)][l] + map[0][0][k][(l - 1)] - map[0][0][(k - 1)][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 3 process
	//Dimension 1,2,3
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				map[i][j][k][0] = map[i][j][k][0] + map[(i - 1)][j][k][0] + map[i][(j - 1)][k][0] + map[i][j][(k - 1)][0] - map[(i - 1)][(j - 1)][k][0] - map[(i - 1)][j][(k - 1)][0] - map[i][(j - 1)][(k - 1)][0] + map[(i - 1)][(j - 1)][(k - 1)][0];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,2,4
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (l = 1; l < numCeld; l++)
			{
				map[i][j][0][l] = map[i][j][0][l] + map[(i - 1)][j][0][l] + map[i][(j - 1)][0][l] + map[i][j][0][(l - 1)] - map[(i - 1)][(j - 1)][0][l] - map[(i - 1)][j][0][(l - 1)] - map[i][(j - 1)][0][(l - 1)] + map[(i - 1)][(j - 1)][0][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,3,4
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				map[i][0][k][l] = map[i][0][k][l] + map[(i - 1)][0][k][l] + map[i][0][(k - 1)][l] + map[i][0][k][(l - 1)] - map[(i - 1)][0][(k - 1)][l] - map[(i - 1)][0][k][(l - 1)] - map[i][0][(k - 1)][(l - 1)] + map[(i - 1)][0][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3,4
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				map[0][j][k][l] = map[0][j][k][l] + map[0][(j - 1)][k][l] + map[0][j][(k - 1)][l] + map[0][j][k][(l - 1)] - map[0][(j - 1)][(k - 1)][l] - map[0][(j - 1)][k][(l - 1)] - map[0][j][(k - 1)][(l - 1)] + map[0][(j - 1)][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 4 process
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				for (l = 1; l < numCeld; l++)
				{
					map[i][j][k][l] = map[i][j][k][l] + map[(i - 1)][j][k][l] + map[i][(j - 1)][k][l] + map[i][j][(k - 1)][l] + map[i][j][k][(l - 1)] + (map[i][(j - 1)][(k - 1)][(l - 1)] + map[(i - 1)][j][(k - 1)][(l - 1)] + map[(i - 1)][(j - 1)][k][(l - 1)] + map[(i - 1)][(j - 1)][(k - 1)][l])
						- (map[(i - 1)][(j - 1)][k][l] + map[(i - 1)][j][(k - 1)][l] + map[(i - 1)][j][k][(l - 1)] + map[i][(j - 1)][(k - 1)][l] + map[i][(j - 1)][k][(l - 1)] + map[i][j][(k - 1)][(l - 1)]) - map[(i - 1)][(j - 1)][(k - 1)][(l - 1)];
				}//end dimension 4 loop
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop
}

void Data_Manipulation::Get_CDF_2D_Discrete_Self(double **map, int numCela, int numCelb)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j;

	//1. Initial process
	//cumul_Map[0][0][0][0] = map[0][0][0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		map[i][0] = map[i][0] + map[(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		map[0][i] = map[0][i] + map[0][(i - 1)];
	}//end dimention 1 loop

	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			map[i][j] = map[i][j] + map[(i - 1)][j] + map[i][(j - 1)] - map[(i - 1)][(j - 1)];
		}//end dimention 2 loop
	}//end dimention 1 loop
}

void Data_Manipulation::Get_CDF_4D_Discrete(double ****cumul_Map, double ****map, int numCela, int numCelb, int numCelc, int numCeld)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j, k, l;

	//1. Initial process
	cumul_Map[0][0][0][0] = map[0][0][0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		cumul_Map[i][0][0][0] = map[i][0][0][0] + cumul_Map[(i - 1)][0][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		cumul_Map[0][i][0][0] = map[0][i][0][0] + cumul_Map[0][(i - 1)][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelc; i++)
	{
		cumul_Map[0][0][i][0] = map[0][0][i][0] + cumul_Map[0][0][(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCeld; i++)
	{
		cumul_Map[0][0][0][i] = map[0][0][0][i] + cumul_Map[0][0][0][(i - 1)];
	}//end dimention 1 loop


	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			cumul_Map[i][j][0][0] = map[i][j][0][0] + cumul_Map[(i - 1)][j][0][0] + cumul_Map[i][(j - 1)][0][0] - cumul_Map[(i - 1)][(j - 1)][0][0];
		}
	}//end dimention 1 loop

	//Dimension 1,3
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			cumul_Map[i][0][k][0] = map[i][0][k][0] + cumul_Map[(i - 1)][0][k][0] + cumul_Map[i][0][(k - 1)][0] - cumul_Map[(i - 1)][0][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,4
	for (i = 1; i < numCela; i++)
	{
		for (l = 1; l < numCeld; l++)
		{
			cumul_Map[i][0][0][l] = map[i][0][0][l] + cumul_Map[(i - 1)][0][0][l] + cumul_Map[i][0][0][(l - 1)] - cumul_Map[(i - 1)][0][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			cumul_Map[0][j][k][0] = map[0][j][k][0] + cumul_Map[0][(j - 1)][k][0] + cumul_Map[0][j][(k - 1)][0] - cumul_Map[0][(j - 1)][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,4
	for (j = 1; j < numCelb; j++)
	{
		for (l = 1; l < numCeld; l++)
		{
			cumul_Map[0][j][0][l] = map[0][j][0][l] + cumul_Map[0][(j - 1)][0][l] + cumul_Map[0][j][0][(l - 1)] - cumul_Map[0][(j - 1)][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 3,4
	for (k = 1; k < numCelc; k++)
	{
		for (l = 1; l < numCeld; l++)
		{
			cumul_Map[0][0][k][l] = map[0][0][k][l] + cumul_Map[0][0][(k - 1)][l] + cumul_Map[0][0][k][(l - 1)] - cumul_Map[0][0][(k - 1)][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 3 process
	//Dimension 1,2,3
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				cumul_Map[i][j][k][0] = map[i][j][k][0] + cumul_Map[(i - 1)][j][k][0] + cumul_Map[i][(j - 1)][k][0] + cumul_Map[i][j][(k - 1)][0] - cumul_Map[(i - 1)][(j - 1)][k][0] - cumul_Map[(i - 1)][j][(k - 1)][0] - cumul_Map[i][(j - 1)][(k - 1)][0] + cumul_Map[(i - 1)][(j - 1)][(k - 1)][0];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,2,4
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (l = 1; l < numCeld; l++)
			{
				cumul_Map[i][j][0][l] = map[i][j][0][l] + cumul_Map[(i - 1)][j][0][l] + cumul_Map[i][(j - 1)][0][l] + cumul_Map[i][j][0][(l - 1)] - cumul_Map[(i - 1)][(j - 1)][0][l] - cumul_Map[(i - 1)][j][0][(l - 1)] - cumul_Map[i][(j - 1)][0][(l - 1)] + cumul_Map[(i - 1)][(j - 1)][0][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,3,4
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				cumul_Map[i][0][k][l] = map[i][0][k][l] + cumul_Map[(i - 1)][0][k][l] + cumul_Map[i][0][(k - 1)][l] + cumul_Map[i][0][k][(l - 1)] - cumul_Map[(i - 1)][0][(k - 1)][l] - cumul_Map[(i - 1)][0][k][(l - 1)] - cumul_Map[i][0][(k - 1)][(l - 1)] + cumul_Map[(i - 1)][0][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3,4
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				cumul_Map[0][j][k][l] = map[0][j][k][l] + cumul_Map[0][(j - 1)][k][l] + cumul_Map[0][j][(k - 1)][l] + cumul_Map[0][j][k][(l - 1)] - cumul_Map[0][(j - 1)][(k - 1)][l] - cumul_Map[0][(j - 1)][k][(l - 1)] - cumul_Map[0][j][(k - 1)][(l - 1)] + cumul_Map[0][(j - 1)][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 4 process
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				for (l = 1; l < numCeld; l++)
				{
					cumul_Map[i][j][k][l] = map[i][j][k][l] + cumul_Map[(i - 1)][j][k][l] + cumul_Map[i][(j - 1)][k][l] + cumul_Map[i][j][(k - 1)][l] + cumul_Map[i][j][k][(l - 1)] + (cumul_Map[i][(j - 1)][(k - 1)][(l - 1)] + cumul_Map[(i - 1)][j][(k - 1)][(l - 1)] + cumul_Map[(i - 1)][(j - 1)][k][(l - 1)] + cumul_Map[(i - 1)][(j - 1)][(k - 1)][l])
						- (cumul_Map[(i - 1)][(j - 1)][k][l] + cumul_Map[(i - 1)][j][(k - 1)][l] + cumul_Map[(i - 1)][j][k][(l - 1)] + cumul_Map[i][(j - 1)][(k - 1)][l] + cumul_Map[i][(j - 1)][k][(l - 1)] + cumul_Map[i][j][(k - 1)][(l - 1)]) - cumul_Map[(i - 1)][(j - 1)][(k - 1)][(l - 1)];
				}//end dimension 4 loop
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop
}

void Data_Manipulation::Get_CDF_2D_Discrete(double **cumul_Map, double **map, int numCela, int numCelb)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j;

	//1. Initial process
	cumul_Map[0][0] = map[0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		cumul_Map[i][0] = map[i][0] + cumul_Map[(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		cumul_Map[0][i] = map[0][i] + cumul_Map[0][(i - 1)];
	}//end dimention 1 loop


	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			cumul_Map[i][j] = map[i][j] + cumul_Map[(i - 1)][j] + cumul_Map[i][(j - 1)] - cumul_Map[(i - 1)][(j - 1)];
		}//end dimention 2 loop
	}//end dimention 1 loop
}

void Data_Manipulation::Cdf_4D_Observed_Expected(double ****mat, double ****mat_Exp, int numCls, double *num_Observations)
{
	//numCls: the number of classes for each dimension
	//mat: the map of 4-dimensional descrete probability distribution
	//mat_Exp: expected map
	double totSum;

	//calculates expected frequencies
	Get_Exp_Freq_4D_Discrete(mat_Exp, &totSum, mat, numCls);//calculates the expected frequency
	//calculates cdf of expected pdf
	Get_CDF_4D_Discrete_Self(mat_Exp, numCls, numCls, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	//convert real values to fractions
	Get_Fraction_Dim_4_Discrete(mat, totSum, numCls);//convert elements into fractions by using the total sum
	//calculates cdf of observed pdf
	Get_CDF_4D_Discrete_Self(mat, numCls, numCls, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	(*num_Observations) = totSum;//sets the total number of observations
	//Verification
	//cout << "1-D Expected value: " << (4.0) << " Observed value: " << map[0][0][0][(numCeld - 1)] << endl;
	//cout << "2-D Expected value: " << (4.0*4.0) << " Observed value: " << map[0][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[0][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][0][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][0] << endl;
	//cout << "4-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "4-D Expected valuefor 3 3 3 2: " << (4.0*4.0*4.0*3.0) << " Observed value: " << map[3][3][3][2] << endl;
	//cout << "Success!"<<endl;
}

void Data_Manipulation::Cdf_2D_Observed_Expected(double **mat, double **mat_Exp, int numCls, double *num_Observations)
{
	//numCls: the number of classes for each dimension
	//mat: the map of 4-dimensional descrete probability distribution
	//mat_Exp: expected map
	double totSum;

	//calculates expected frequencies
	Get_Exp_Freq_2D_Discrete(mat_Exp, &totSum, mat, numCls);//calculates the expected frequency
	//calculates cdf of expected pdf
	Get_CDF_2D_Discrete_Self(mat_Exp, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	//convert real values to fractions
	Get_Fraction_Dim_2_Discrete(mat, totSum, numCls);//convert elements into fractions by using the total sum
	//calculates cdf of observed pdf
	Get_CDF_2D_Discrete_Self(mat, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	(*num_Observations) = totSum;//sets the total number of observations
	//Verification
	//cout << "1-D Expected value: " << (4.0) << " Observed value: " << map[0][0][0][(numCeld - 1)] << endl;
	//cout << "2-D Expected value: " << (4.0*4.0) << " Observed value: " << map[0][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[0][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][0][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][0] << endl;
	//cout << "4-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "4-D Expected valuefor 3 3 3 2: " << (4.0*4.0*4.0*3.0) << " Observed value: " << map[3][3][3][2] << endl;
	//cout << "Success!"<<endl;
}

void Data_Manipulation::Get_Exp_Freq_4D_Discrete(double ****mat_Exp, double *p_TotSum, double ****mat, int nc)
{
	//double ****mat_Exp: expected frequencies
	//double *p_TotSum: the total sum
	//double ****mat: raw matrix
	//int nc: the number of classes for each dimension

	//calculates the expected frequency
	int i, j, k, l;
	const int dim = 4;
	double **frequency = new double *[dim];//expected frequency for each dimension
	double totSum = 0.0;//The total sum of elements

	//memory allocation
	for (i = 0; i < dim; i++)
	{
		frequency[i] = new double[nc];
	}//end i loop

	Get_Freq_Dim_4_Discrete(mat, &totSum, nc, frequency);//gets the expected frequency for each dimension

	(*p_TotSum) = totSum;//sets the total sum

	//sets the expected frequency
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			for (k = 0; k < nc; k++)
			{
				for (l = 0; l < nc; l++)
				{
					mat_Exp[i][j][k][l] = frequency[0][i] * frequency[1][j] * frequency[2][k] * frequency[3][l];
				}//end 4d loop
			}//end 3d loop
		}//end 2d loop
	}//end 1d loop

	//memory deallocation
	for (i = 0; i < dim; i++)
	{
		delete[] frequency[i];
	}//end i loop

	delete[] frequency;
}

void Data_Manipulation::Get_Exp_Freq_2D_Discrete(double **mat_Exp, double *p_TotSum, double **mat, int nc)
{
	//double ****mat_Exp: expected frequencies
	//double *p_TotSum: the total sum
	//double ****mat: raw matrix
	//int nc: the number of classes for each dimension

	//calculates the expected frequency
	int i, j;
	const int dim = 2;
	double **frequency = new double *[dim];//expected frequency for each dimension
	double totSum = 0.0;//The total sum of elements

	//memory allocation
	for (i = 0; i < dim; i++)
	{
		frequency[i] = new double[nc];
	}//end i loop

	Get_Freq_Dim_2_Discrete(mat, &totSum, nc, frequency);//gets the expected frequency for each dimension

	(*p_TotSum) = totSum;//sets the total sum

	//sets the expected frequency
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			mat_Exp[i][j] = frequency[0][i] * frequency[1][j];
		}//end 2d loop
	}//end 1d loop

	//memory deallocation
	for (i = 0; i < dim; i++)
	{
		delete[] frequency[i];
	}//end i loop

	delete[] frequency;
}

void Data_Manipulation::Get_Fraction_Dim_4_Discrete(double ****mat, double p_TotSum, int nc)
{
	//double ****mat: 4d matrix
	//double * p_TotSum: Pre-calculated Total sum
	//int nc: the number of classes in each dimension
	int i, j, k, l;

	if (p_TotSum != 0.0)
	{
		//calculates the sums
		for (i = 0; i < nc; i++)
		{
			for (j = 0; j < nc; j++)
			{
				for (k = 0; k < nc; k++)
				{
					for (l = 0; l < nc; l++)
					{
						mat[i][j][k][l] /= p_TotSum;
					}//end 4 d loop
				}//end 3 d loop
			}//end 2 d loop
		}//end 1 d loop
	}//end if
}

void Data_Manipulation::Get_Fraction_Dim_2_Discrete(double **mat, double p_TotSum, int nc)
{
	//double ****mat: 4d matrix
	//double * p_TotSum: Pre-calculated Total sum
	//int nc: the number of classes in each dimension
	int i, j;

	if (p_TotSum != 0.0)
	{
		//calculates the sums
		for (i = 0; i < nc; i++)
		{
			for (j = 0; j < nc; j++)
			{
				mat[i][j] /= p_TotSum;
			}//end 2 d loop
		}//end 1 d loop
	}//end if
}

void Data_Manipulation::Get_Freq_Dim_4_Discrete(double ****mat, double *p_TotSum, int nc, double **frq)
{
	//double ****mat: 4d matrix
	//double * p_TotSum: Total sum
	//int nc: the number of classes in each dimension
	//double *frq: dim[dimentions (=4), class ] frequencies in each dimension
	int i, j, k, l;
	const int dim = 4;
	double tmp;

	//initialization
	for (j = 0; j < dim; j++)
	{
		for (i = 0; i < nc; i++)
		{
			frq[j][i] = 0.0;
		}//end class loop
	}//end dimension loop

	//calculates the sums
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			for (k = 0; k < nc; k++)
			{
				for (l = 0; l < nc; l++)
				{
					tmp = mat[i][j][k][l];
					frq[0][i] += tmp;
					frq[1][j] += tmp;
					frq[2][k] += tmp;
					frq[3][l] += tmp;
				}//end 4 d loop
			}//end 3 d loop
		}//end 2 d loop
	}//end 1 d loop

	double totSum = 0.0;

	//calculates the total sum
	for (i = 0; i < nc; i++)
	{
		totSum += frq[0][i];
	}//end 1 d loop

	(*p_TotSum) = totSum;//sets the total sum

	//calculates the class frequencies for each dimension

	if (totSum > 0.0)
	{
		for (j = 0; j < dim; j++)
		{
			for (i = 0; i < nc; i++)
			{
				frq[j][i] /= totSum;
			}//end class loop
		}//end dimension loop
	}//end if
}

void Data_Manipulation::Get_Freq_Dim_2_Discrete(double **mat, double *p_TotSum, int nc, double **frq)
{
	//double **mat: 2d matrix
	//double * p_TotSum: Total sum
	//int nc: the number of classes in each dimension
	//double *frq: dim[dimentions (=2), class ] frequencies in each dimension
	int i, j;
	const int dim = 2;
	double tmp;

	//initialization
	for (j = 0; j < dim; j++)
	{
		for (i = 0; i < nc; i++)
		{
			frq[j][i] = 0.0;
		}//end class loop
	}//end dimension loop

	//calculates the sums
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			tmp = mat[i][j];
			frq[0][i] += tmp;
			frq[1][j] += tmp;
		}//end 2 d loop
	}//end 1 d loop

	double totSum = 0.0;

	//calculates the total sum
	for (i = 0; i < nc; i++)
	{
		totSum += frq[0][i];
	}//end 1 d loop

	(*p_TotSum) = totSum;//sets the total sum

	//calculates the class frequencies for each dimension

	if (totSum > 0.0)
	{
		for (j = 0; j < dim; j++)
		{
			for (i = 0; i < nc; i++)
			{
				frq[j][i] /= totSum;
			}//end class loop
		}//end dimension loop
	}//end if
}

void Data_Manipulation::Get_Delta(double *p_Delta, double *p_Data, int *p_Num_Ele_Data)
{
	//returns the first-degree difference of an array
	//p_Delta (x_{t + h} - x_{t}): an array for the first difference of p_Data with ((p_Num_Ele_Data)) elements
	//The last element is 0.0
	//p_Data (x_{t}): an array contains data x with (p_Num_Ele_Data) elements
	//p_Num_Ele_Data: the number of elements in p_Data

	if ((*p_Num_Ele_Data)  > 1)
	{//if there are more than 1 elements
		int i, end_0;
		end_0 = (*p_Num_Ele_Data) - 1;

		//calculates the first-degree difference
		for (i = 0; i < end_0; i++)
		{
			//calculates the first-degree difference
			p_Delta[i] = p_Data[(i + 1)] - p_Data[i];
		}//end element loop

		p_Delta[end_0] = 0.0;//the last lelement
	}//end if
}

void Data_Manipulation::Get_2D_Delta(double **p_Delta, double **p_Data, int *p_Num_Rows, int *p_Num_Columns, int sort_Order)
{
	//sort_Order: 0: row -sorted (default, 1: column sorted)
	int num_Data_Unit;//the number of data in a unit
	int num_Sets;//the number of sets
	int i, j;

	if (sort_Order == 0)
	{//row sorted
		num_Data_Unit = (*p_Num_Rows);
		num_Sets = (*p_Num_Columns);//the number of sets

		double* tmp = new double[num_Data_Unit];
		double* tmp_Delta = new double[num_Data_Unit];

		for (i = 0; i < num_Sets; i++)
		{
			//sets unit data
			for (j = 0; j < num_Data_Unit; j++)
			{
				tmp[j] = p_Data[j][i];
			}//end data unit loop

			//calculates delta
			Get_Delta(tmp_Delta, tmp, &num_Data_Unit);

			//sets unit delta data
			for (j = 0; j < num_Data_Unit; j++)
			{
				p_Delta[j][i] = tmp_Delta[j];
			}//end data unit loop
		}//end data set loop

		delete[] tmp_Delta;
		delete[] tmp;
	}
	else
	{//column sorted
		num_Data_Unit = (*p_Num_Columns);
		num_Sets = (*p_Num_Rows);//the number of sets
		double* tmp = new double[num_Data_Unit];
		double* tmp_Delta = new double[num_Data_Unit];

		for (i = 0; i < num_Sets; i++)
		{
			//sets unit data
			for (j = 0; j < num_Data_Unit; j++)
			{
				tmp[j] = p_Data[i][j];
			}//end data unit loop

			//calculates delta
			Get_Delta(tmp_Delta, tmp, &num_Data_Unit);

			//sets unit delta data
			for (j = 0; j < num_Data_Unit; j++)
			{
				p_Delta[i][j] = tmp_Delta[j];
			}//end data unit loop
		}//end data set loop

		delete[] tmp_Delta;
		delete[] tmp;
	}//end else
}

void Data_Manipulation::Get_2D_Delta_Delta(double **p_Delta, double **p_Data, int *p_Num_Rows, int *p_Num_Columns, int sort_Order)
{
	//sort_Order: 0: row -sorted (default, 1: column sorted)
	int num_Data_Unit;//the number of data in a unit
	int num_Sets;//the number of sets
	int i, j;

	if (sort_Order == 0)
	{//row sorted
		num_Data_Unit = (*p_Num_Rows);
		num_Sets = (*p_Num_Columns);//the number of sets

		double* tmp = new double[num_Data_Unit];
		double* tmp_Delta = new double[num_Data_Unit];

		for (i = 0; i < num_Sets; i++)
		{
			//sets unit data
			for (j = 0; j < num_Data_Unit; j++)
			{
				tmp[j] = p_Data[j][i];
			}//end data unit loop

			//calculates delta
			Get_Delta_Delta(tmp_Delta, tmp, &num_Data_Unit);

			//sets unit delta data
			for (j = 0; j < num_Data_Unit; j++)
			{
				p_Delta[j][i] = tmp_Delta[j];
			}//end data unit loop
		}//end data set loop

		delete[] tmp_Delta;
		delete[] tmp;
	}
	else
	{//column sorted
		num_Data_Unit = (*p_Num_Columns);
		num_Sets = (*p_Num_Rows);//the number of sets
		double* tmp = new double[num_Data_Unit];
		double* tmp_Delta = new double[num_Data_Unit];

		for (i = 0; i < num_Sets; i++)
		{
			//sets unit data
			for (j = 0; j < num_Data_Unit; j++)
			{
				tmp[j] = p_Data[i][j];
			}//end data unit loop

			//calculates delta
			Get_Delta_Delta(tmp_Delta, tmp, &num_Data_Unit);

			//sets unit delta data
			for (j = 0; j < num_Data_Unit; j++)
			{
				p_Delta[i][j] = tmp_Delta[j];
			}//end data unit loop
		}//end data set loop

		delete[] tmp_Delta;
		delete[] tmp;
	}//end else
}

void Data_Manipulation::Get_Delta_Delta(double *p_Delta_Delta, double *p_Data, int *p_Num_Ele_Data)
{
	//returns the second-degree difference of an array
	//p_Delta_Delta (x_{t + h} - x_{t}): an array for the second-degree difference of p_Data with ((p_Num_Ele_Data) ) elements
	//The last two elements are 0.0
	//p_Data (x_{t}): an array contains data x with (p_Num_Ele_Data) elements
	//p_Num_Ele_Data: the number of elements in p_Data

	if ((*p_Num_Ele_Data)  > 2)
	{//if there are more than 2 elements
		int i, end_0;
		end_0 = (*p_Num_Ele_Data) - 2;//for the second difference

		double *tmp_Delta = new double[(*p_Num_Ele_Data)];

		//gets the first-degree difference
		Get_Delta(tmp_Delta, p_Data, p_Num_Ele_Data);

		//calculates the second-degree difference
		for (i = 0; i < end_0; i++)
		{
			//calculates the second-degree difference
			p_Delta_Delta[i] = tmp_Delta[(i + 1)] - tmp_Delta[i];
		}//end element loop

		p_Delta_Delta[end_0] = 0.0;//the second-last element
		p_Delta_Delta[(end_0 + 1)] = 0.0;//the last element

		delete[] tmp_Delta;
	}//end if
}

void Data_Manipulation::Get_Mean_SD_Pop_Value_Freq(double *p_Mean, double *p_Variance, double *p_SD, double *p_Value, double *p_Frequency, int num_Elements)
{
	//For data without frequency, use an array with values 1.0 as p_Frequency
	//double *p_Mean : mean
	//double *p_Variance: variance
	//double *p_SD: standard deviation without correction
	//double *p_Value: data values
	//double *p_Frequency: frequency (or abundance)
	//int num_Elements: the number of elements
	double total_Sum = 0.0;
	double total_Frequency = 0.0;
	int i;

	for (i = 0; i < num_Elements; i++)
	{
		total_Sum += (p_Value[i] * p_Frequency[i]);
		total_Frequency += p_Frequency[i];
	}//end element loop

	double tmp_Mean = 0.0;
	double sum_Square_Diff = 0.0;
	double tmp_Diff;
	double variance = 0.0;

	if (total_Frequency > 0.0)
	{
		//calculates mean
		tmp_Mean = total_Sum / total_Frequency;

		for (i = 0; i < num_Elements; i++)
		{
			tmp_Diff = (p_Value[i] - tmp_Mean);
			sum_Square_Diff += ((tmp_Diff * tmp_Diff) * p_Frequency[i]);
		}//end element loop

		//calculates variance
		variance = sum_Square_Diff / total_Frequency;
	}//end if

	//sets mean
	(*p_Mean) = tmp_Mean;

	//sets variance
	(*p_Variance) = variance;

	//calculates standard deviation
	if (variance > 0.0)
	{
		(*p_SD) = pow(variance, 0.5);
	}
	else
	{
		(*p_SD) = 0.0;
	}//end else
}

void Data_Manipulation::Get_Mean_SD_Pop_Value(double *p_Mean, double *p_Variance, double *p_SD, double *p_Value, int num_Elements)
{
	//double *p_Mean : mean
	//double *p_Variance: variance
	//double *p_SD: standard deviation without correction
	//double *p_Value: data values
	//int num_Elements: the number of elements

	double total_Sum = 0.0;
	double total_Freq = (double)num_Elements;
	double tmp_Mean = 0.0;
	double variance = 0.0;
	int i;

	if (total_Freq > 0.0)
	{
		for (i = 0; i < num_Elements; i++)
		{
			total_Sum += p_Value[i];
		}//end element loop

		//calculates mean
		tmp_Mean = total_Sum / total_Freq;

		//calculates variance
		double sum_Square_Diff = 0.0;
		double tmp_Diff;

		for (i = 0; i < num_Elements; i++)
		{
			tmp_Diff = p_Value[i] - tmp_Mean;
			sum_Square_Diff += (tmp_Diff * tmp_Diff);
		}//end element loop

		variance = sum_Square_Diff / total_Freq;
	}//end if

	//sets mean
	(*p_Mean) = tmp_Mean;

	//sets variance
	(*p_Variance) = variance;

	//calculates standard deviation
	if (variance > 0.0)
	{
		(*p_SD) = pow(variance, 0.5);
	}
	else
	{
		(*p_SD) = 0.0;
	}//end else
}

void Data_Manipulation::Get_Mean_SD_Pop_Value_2D(double **p_Mean, double **p_Variance, double **p_SD, double **p_Value, int num_Rows, int num_Columns, int num_Sets, int sort_Order)
{
	//if there are 2 traits, the number of sets (num_Sets) is 2.
	//p_Mean: 2D array: dim[sequence][set]
	//p_Variance: 2D array: dim[sequence][set]
	//p_SD: 2D array: dim[sequence][set]
	//p_Value: 2D array
	//sort_Order: (0: row sorted, 1: column sorted)

	double tmp_Mean;
	double tmp_SD;
	double tmp_Variance;
	int i, j, k, num_Batch_Unit, num_Batch_Sequence, tmp_Tab_Start;

	double *tmp_Arr;

	if (sort_Order == 0)
	{//if row sorted
		num_Batch_Unit = num_Columns / num_Sets;
		num_Batch_Sequence = num_Rows;
	}
	else
	{//if column sorted
		num_Batch_Unit = num_Rows / num_Sets;
		num_Batch_Sequence = num_Columns;
	}//end else

	tmp_Arr = new double[num_Batch_Unit];

	for (i = 0; i < num_Batch_Sequence; i++)
	{
		for (j = 0; j < num_Sets; j++)
		{
			tmp_Tab_Start = j * num_Batch_Unit;//start point

			//sets the unit data set
			if (sort_Order == 0)
			{//if row sorted
				for (k = 0; k < num_Batch_Unit; k++)
				{
					tmp_Arr[k] = p_Value[i][(k + tmp_Tab_Start)];
				}//end k loop
			}
			else
			{//if column sorted
				for (k = 0; k < num_Batch_Unit; k++)
				{
					tmp_Arr[k] = p_Value[(k + tmp_Tab_Start)][i];
				}//end k loop
			}//end else

			//calculates mean, variance and standard deviation for the unit data
			Get_Mean_SD_Pop_Value(&tmp_Mean, &tmp_Variance, &tmp_SD, tmp_Arr, num_Batch_Unit);

			//sets mean, variance and standard deviation for the unit data
			p_Mean[i][j] = tmp_Mean;
			p_Variance[i][j] = tmp_Variance;
			p_SD[i][j] = tmp_SD;
		}//end set loop
	}//end i loop

	//de-allocate memory
	delete[] tmp_Arr;
}

void Data_Manipulation::Get_Mean_SD_Pop_Value_Freq_2D(double **p_Mean, double **p_Variance, double **p_SD, double **p_Value, double **p_Frequency, int num_Rows, int num_Columns, int num_Sets, int sort_Order)
{
	//Category values (p_Value) change
	//if there are 2 traits, the number of sets (num_Sets) is 2.
	//p_Mean: 2D array: dim[sequence][set]
	//p_Variance: 2D array: dim[sequence][set]
	//p_SD: 2D array: dim[sequence][set]
	//p_Value: 2D array
	//sort_Order: (0: row sorted, 1: column sorted)

	double tmp_Mean;
	double tmp_SD;
	double tmp_Variance;
	int i, j, k, num_Batch_Unit, num_Batch_Sequence, tmp_Tab_Start;

	double *tmp_Arr;
	double *tmp_Freq;

	if (sort_Order == 0)
	{//if row sorted
		num_Batch_Unit = num_Columns / num_Sets;
		num_Batch_Sequence = num_Rows;
	}
	else
	{//if column sorted
		num_Batch_Unit = num_Rows / num_Sets;
		num_Batch_Sequence = num_Columns;
	}//end else

	tmp_Arr = new double[num_Batch_Unit];
	tmp_Freq = new double[num_Batch_Unit];

	for (i = 0; i < num_Batch_Sequence; i++)
	{
		for (j = 0; j < num_Sets; j++)
		{
			tmp_Tab_Start = j * num_Batch_Unit;//start point

			//sets the unit data set
			if (sort_Order == 0)
			{//if row sorted
				for (k = 0; k < num_Batch_Unit; k++)
				{
					tmp_Arr[k] = p_Value[i][(k + tmp_Tab_Start)];
					tmp_Freq[k] = p_Frequency[i][(k + tmp_Tab_Start)];
				}//end k loop
			}
			else
			{//if column sorted
				for (k = 0; k < num_Batch_Unit; k++)
				{
					tmp_Arr[k] = p_Value[(k + tmp_Tab_Start)][i];
					tmp_Freq[k] = p_Frequency[(k + tmp_Tab_Start)][i];
				}//end k loop
			}//end else

			//calculates mean, variance and standard deviation for the unit data
			Get_Mean_SD_Pop_Value_Freq(&tmp_Mean, &tmp_Variance, &tmp_SD, tmp_Arr, tmp_Freq, num_Batch_Unit);

			//sets mean, variance and standard deviation for the unit data
			p_Mean[i][j] = tmp_Mean;
			p_Variance[i][j] = tmp_Variance;
			p_SD[i][j] = tmp_SD;
		}//end set loop
	}//end i loop

	//de-allocate memory
	delete[] tmp_Arr;
	delete[] tmp_Freq;
}

void Data_Manipulation::Get_Mean_SD_Pop_Value_Fixed_Freq_2D(double **p_Mean, double **p_Variance, double **p_SD, double *p_Value, double **p_Frequency, int num_Rows, int num_Columns, int num_Sets, int sort_Order)
{
	//Category values (p_Value) do not change
	//if there are 2 traits, the number of sets (num_Sets) is 2.
	//p_Mean: 2D array: dim[sequence][set]
	//p_Variance: 2D array: dim[sequence][set]
	//p_SD: 2D array: dim[sequence][set]
	//p_Value: 1D array (Fixed) category values for each row (or column)
	//p_Frequency: 2D array for the abundance of the (fixed) category
	//sort_Order: (0: row sorted, 1: column sorted)

	double tmp_Mean;
	double tmp_SD;
	double tmp_Variance;
	int i, j, k, num_Batch_Unit, num_Batch_Sequence, tmp_Tab_Start;

	double *tmp_Freq;

	if (sort_Order == 0)
	{//if row sorted
		num_Batch_Unit = num_Columns / num_Sets;
		num_Batch_Sequence = num_Rows;
	}
	else
	{//if column sorted
		num_Batch_Unit = num_Rows / num_Sets;
		num_Batch_Sequence = num_Columns;
	}//end else

	tmp_Freq = new double[num_Batch_Unit];

	for (i = 0; i < num_Batch_Sequence; i++)
	{
		for (j = 0; j < num_Sets; j++)
		{
			tmp_Tab_Start = j * num_Batch_Unit;//start point

			//sets the unit data set
			if (sort_Order == 0)
			{//if row sorted
				for (k = 0; k < num_Batch_Unit; k++)
				{
					tmp_Freq[k] = p_Frequency[i][(k + tmp_Tab_Start)];
				}//end k loop
			}
			else
			{//if column sorted
				for (k = 0; k < num_Batch_Unit; k++)
				{
					tmp_Freq[k] = p_Frequency[(k + tmp_Tab_Start)][i];
				}//end k loop
			}//end else

			//calculates mean, variance and standard deviation for the unit data
			Get_Mean_SD_Pop_Value_Freq(&tmp_Mean, &tmp_Variance, &tmp_SD, p_Value, tmp_Freq, num_Batch_Unit);

			//sets mean, variance and standard deviation for the unit data
			p_Mean[i][j] = tmp_Mean;
			p_Variance[i][j] = tmp_Variance;
			p_SD[i][j] = tmp_SD;
		}//end set loop
	}//end i loop

	//de-allocate memory
	delete[] tmp_Freq;
}

void Data_Manipulation::Trim_2D_String_Data_Interval(vector<vector<string>> &tmp_New_Data, vector<vector<string>> &tmp_Data, int num_Row_Skip, int num_Column_Skip, int p_Row_Interval)
{
	//vector<vector<string>> tmp_New_Data: new data (vector without size)
	//vector<vector<string>> tmp_Data: template raw data
	//int num_Row_Skip: the number of rows to skip
	//int num_Column_Skip: the number of columns to skip
	//int p_Interval: sampling interval

	int size_Row_Raw = (int)tmp_Data.size();
	int size_Col_Raw = ((int)tmp_Data[0].size());

	int size_Row_New = 1 + ((size_Row_Raw - num_Row_Skip) / p_Row_Interval);//the number of rows for a trimmed data
	int size_Col_New = ((int)tmp_Data[0].size()) - num_Column_Skip;
	tmp_New_Data.resize(size_Row_New, vector<string>(size_Col_New));//sets the size of a trimmed data

	int i, j;
	int counter_Row = 0;
	int counter_Col;

	//sets elements in a trimmed data
	for (i = num_Row_Skip; i < size_Row_Raw; i += p_Row_Interval)
	{
		counter_Col = 0;

		for (j = num_Column_Skip; j < size_Col_Raw; j++)
		{
			tmp_New_Data[counter_Row][counter_Col] = tmp_Data[i][j];
			counter_Col++;
		}//end column loop

		counter_Row++;
	}//end row loop
}

void Data_Manipulation::Trim_2D_String_Data_Interval_Keep_Column_Label(vector<vector<string>> &tmp_New_Data, vector<vector<string>> &tmp_Data, int num_Row_Skip, int num_Column_Skip, int p_Row_Interval)
{
	//keeps column label
	//vector<vector<string>> tmp_New_Data: new data (vector without size)
	//vector<vector<string>> tmp_Data: template raw data
	//int num_Row_Skip: the number of rows to skip
	//int num_Column_Skip: the number of columns to skip
	//int p_Interval: sampling interval

	int size_Row_Raw = (int)tmp_Data.size();
	int size_Col_Raw = ((int)tmp_Data[0].size());

	int size_Row_New = num_Row_Skip + 1 + ((size_Row_Raw - num_Row_Skip) / p_Row_Interval);//the number of rows for a trimmed data
	int size_Col_New = ((int)tmp_Data[0].size()) - num_Column_Skip;
	tmp_New_Data.resize(size_Row_New, vector<string>(size_Col_New));//sets the size of a trimmed data

	int i, j;
	int counter_Row = 0;
	int counter_Col;

	//sets column headers
	//sets elements in a trimmed data
	for (i = 0; i < num_Row_Skip; i++)
	{
		counter_Col = 0;

		for (j = num_Column_Skip; j < size_Col_Raw; j++)
		{
			tmp_New_Data[counter_Row][counter_Col] = tmp_Data[i][j];
			counter_Col++;
		}//end column loop

		counter_Row++;
	}//end row loop


	//sets elements in a trimmed data
	for (i = num_Row_Skip; i < size_Row_Raw; i += p_Row_Interval)
	{
		counter_Col = 0;

		for (j = num_Column_Skip; j < size_Col_Raw; j++)
		{
			tmp_New_Data[counter_Row][counter_Col] = tmp_Data[i][j];
			counter_Col++;
		}//end column loop

		counter_Row++;
	}//end row loop
}