#ifndef MATPOP_H_
#define MATPOP_H_

#include <iostream>
#include <string>
#include <Eigen/Dense>
#include <Eigen/Eigenvalues>
#include "File_RW.h"

using namespace std;
using namespace Eigen;
//using namespace boost::numeric::ublas;

class Matrix_Population_Model
{
private:
	//transition matrics
	double****** transitionMat;//dim[deme, niche, species, sex, row, column]; survival rate to the same life stage
	double****** fundMat;//dim[deme, niche, species, sex, row, column]; fundimental matrix
	double****** var_FundMat;//dim[deme, niche, species, sex, row, column]; variance of fundimental matrix
	double**** stableStageDist;//dim[deme, niche, species, life stage]; stable stage distribution
	double***** lifeExpectancy_Mean;//dim[deme, niche, species, life stage]; stable stage distribution
	double*** lamda_Pop;//dim[deme, niche, species]; population growth rate
	double*** dds_Pop;//dim[deme, niche, species]; density-dependent survival rate making lambda = 1

	int num_Deme;
	int num_Niche;
	int num_Spc;
	int num_Sex;
	int** num_Stg;
	int maxStg;
	char* fileName_Output;//Output file name
public:
	Matrix_Population_Model(int p_Num_Deme, int p_Num_Niche, int p_Num_Spc, int p_Num_Sex, int** p_Num_Stg, string p_Label);
	~Matrix_Population_Model();
	//dim[(((num_Deme - 1) * (num_Niche)) + (num_Niche - 1) + 1), species, sex, life stage, row, column]; survival rate to the same life stage
	void Set_Vital_Rat(double***** P, double***** G, double**** F);//sets vital rates

	//Analysis run
	void Run_Analysis(void);

	void Set_Lamda_Pop(void);
	void Set_Lamda(double *p_Lamda, double *p_StbDist, int p_Deme, int p_Niche, int p_Spc, int p_Sex);//returns the population growth rate (for female)
	void Set_Stable_DDS(double *p_DDS, int p_Deme, int p_Niche, int p_Spc, int p_Sex);//returns the population growth rate (for female)
	void Get_Stable_DDS(double*** p_DDS_Stable);//returns stable density-depenent-survival rate
	void Get_Stable_Stage_Dist(double**** p_StableStageDist);//returns the stable-stage distribution

	//calculates age-related characteristics
	void Set_AgeChar(void);
	//sets fundamental matrix
	void Set_FundMat(double **p_FundMat, double **p_Var_FundMat, double *p_LifeExp, int p_Deme, int p_Niche, int p_Spc, int p_Sex);

	//writes files
	void Set_Output(void);
};

#endif /* MATPOP_H_ */