#include "RanGen.h"

RanGen::RanGen(const double** p_Alle_Frq, const int *p_n_Locus, const int *p_n_Allele)
{
	//Alle_Frq is the frequencies of alleles for each locus
	n_Locus = (*p_n_Locus);
	n_Allele = (*p_n_Allele);

	cumul_Alle_Frq = new double*[n_Locus];
	int locus;

	for (locus = 0; locus < n_Locus; locus++){
		cumul_Alle_Frq[locus] = new double[n_Allele];
	}//end locus loop

	int allele;

	//sets the cumulative allele frequencies
	for (locus = 0; locus < n_Locus; locus++){
		cumul_Alle_Frq[locus][0] = p_Alle_Frq[locus][0];//the first allele type

		for (allele = 1; allele < n_Allele; allele++){
			cumul_Alle_Frq[locus][allele] = cumul_Alle_Frq[locus][(allele - 1)] + p_Alle_Frq[locus][allele];
		}//end allele loop
	}//end locus loop
}//end constructor

RanGen::~RanGen(void)
{
	int locus;

	for (locus = 0; locus < n_Locus; locus++){
		delete[] cumul_Alle_Frq[locus];
	}//end locus loop

	delete[] cumul_Alle_Frq;
}

void RanGen::Get_Random_Gamete(unsigned short int *gamete, const int *p_num_Locus, variate_generator<mt19937&, uniform_real_distribution<> > &uni_Ran_Dev)
{
	//generates a random gamete
	int locus;
	int allele;
	double ranDev;//uniform random deviate

	for (locus = 0; locus < n_Locus; locus++){
		ranDev = uni_Ran_Dev();//temporarily stores a uniform random deviate in gamete

		for (allele = 0; allele < n_Allele; allele++){
			if (ranDev <= cumul_Alle_Frq[locus][allele]){
				break;
			}//end if
		}//end allele loop

		gamete[locus] = allele;//chosen allele
	}//end locus loop
}
