/*
* SimulProc.h
*
*  Created on: Dec 4, 2014
*      Author: jungk_000
*/

#ifndef SIMULPROC_H_
#define SIMULPROC_H_

#include <iostream>
#include <vector>
#include <cmath>
#include <intrin.h>
#include <string>//for csv file
#include <sstream>//for csv file

#include <boost/math/special_functions/round.hpp>
#include <boost/random/uniform_real_distribution.hpp>
#include <boost/random/uniform_int_distribution.hpp>
#include <boost/random/binomial_distribution.hpp>
//#include <boost/random/poisson_distribution.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/lognormal.hpp>

#include "Agent.h"
#include "File_RW.h"
#include "Data_Manipulation.h"
#include "Batch_Data_Proc.h"

//quantile for 4 sigma in normal distribution
#define QUANTILE_FOUR_SIGMA_LEFT 0.0000315
#define QUANTILE_FOUR_SIGMA_RIGHT 0.9999685

//plasticity trait number
#define TRAIT_NUMBER_PLASTICITY_BODYLENGTH 2
#define TRAIT_NUMBER_MATING_ASSORTATIVE 3

//assortative mating
#define BIN_NUMBER_MALE_DEFAULT 0
#define BIN_NUMBER_FEMALE_DEFAULT 0

#define NUMBER_MATE_ENCOUNTER 5

//sex
#define SEX_MALE 0
#define SEX_FEMALE 1

using namespace std;
using namespace boost::random;

class Simul_Proc {
private:
	///////////////////
	//plasticity mode//
	///////////////////
	//true: plasticity on, false: plasticity off
	bool mode_Plasticity;

	///////////////////////////
	//assortative mating mode//
	///////////////////////////
	//true: assortative mating mode on, false: assortative mating mode off
	bool mode_Assortative_Mating;

	int **mode_Gene_Trait;//trait mode
	//mode_Gene_Trait[spc][0]: Overall gene-trait relation ship
	//0:All loci are QTL

	//mode_Gene_Trait[spc][1]: Mating
	//0: assortative mating power function
	//1: Randome mating with a niche
	//2: Assortative mating normal function

	//mode_Gene_Trait[spc][2]: Behavior

	//mode_Gene_Trait[spc][3]: Trait-Resource use determined by Morphology
	//0: low-pass filter (mouth size (body-size)) + high-pass filter (gill)
	//These traits affect the efficiency of using resouce in the defined range of size.

	//mode_Gene_Trait[spc][4]: Gene-Trait value determination
	//0: Synergetic (Multiplicative)
	//1: Additive

	//mode_Gene_Trait[spc][5]: initial genotypic ondition
	//0: 0 (Null state)
	//1:  1(Full state)
	//2:  1(Intermediate state)
	//3: customized initial state

	int *mode_Forage_Physic_Spc;//Mode for physical foraging trait
	//0: Compound Sigmoidal Attack Rate Function

	int num_Tstep;//the number of time steps
	double unit_T_Step;//the unit time step size
	int interval_sampling;//sampling interval
	int counter_Sampling;//counter for sampling (Sampling occurs when counter_Sampling == 0)
	int tstep_Key_Buffer_Output;//a key time step for output-data buffering
	bool tick_t;//tick for t (0 or 1)

	int timeStep_Reintroduction;//the time step for re-introducing the prototype morph.
	bool status_Reintroduction;//True: re-introducing species, False: not re-introducing a core species

	//Vitality
	double ******P_G;//dim[deme, niche, species, sex, life stage, cumulative P(0) or G(1)]; survival rate to the same life stage
	double *prop_Female_At_Birth;//dim[species]; the proportion of females at birth
	//0: male, 1:female

	//Movement
	double mov_Patch;//rate of movement between adjacent patches
	double mov_Patch_half;//0.5 * mov_Patch
	double **mov_Niche_Cumul;//dim[niche, niche];cumulative rates of movement between niches within a patch
	double **mov_Niche;//dim[niche, niche];chancd of movement or stay between niches within a patch

	vector<vector<Agent*> > agt_M;//dim[species, agent index]; List of dead agents (agent index number) in the agent list vector
	vector<vector<Agent*> > agt;//dim[species, agent index]; List contains information of agents

	int *celNum_M;//dim[species], the number of cells used in agt_M
	int *celNum_Agt;//dim[species], the number of cells used in agt

	vector<vector<vector<vector<vector<Agent*> > > > > agtLst_0;//dim[deme * niche, species, sex, life stage], list of agent genetic-data index number
	vector<vector<vector<vector<vector<Agent*> > > > > agtLst_1;//dim[deme * niche, species, sex, life stage], list of agent genetic-data index number

	int *****size_AgtLst;//dim[deme, niche, species, sex, life stage], the size of agtLst_0
	int *****celNum_0;//dim[deme, niche, species, sex, life stage], the number of cells used in agtLst_0
	int *****celNum_1;//dim[deme, niche, species, sex, life stage], the number of cells used in agtLst_0

	//the number of cells
	//dim[max_Num_Bin_Bodylength, sex];
	int **size_MatingList_Discrete;
	//temporary counter for mating list
	//dim[max_Num_Bin_Bodylength, sex];
	int **celNum_MatingList_Discrete;
	//temporary counter for mating list
	//dim[max_Num_Bin_Bodylength, sex];
	int **maxCelNum_MatingList_Discrete;
	vector<vector<vector<Agent*> > > matingList_Discrete;//dim[mating class, sex, agent index number]temporary container for mating

	//large step pre-search for mate
	int *num_MateSearchLargeStep_Discrete;
	//the maximum large step pre-search for mate
	int max_Num_MateSearchLargeStep_Discrete;
	//step size for a large step pre-mate-search
	//dim[species, max_Num_MateSearchLargeStep_Discrete]
	double **mateSearchLargeStep_Discrete;

	//Assortative mating parameters
	double *female_Choosiness_Intensity;//dim[species]; weighting factor for a mate frequency and female mate choosiness
	double *female_Choosiness_Scale;//dim[species]; weighting factor for a mate frequency and female mate choosiness

	//temporary data structura for the new borns
	vector<vector<Agent*> > new_AgtLst;//dim[species, agent index]; List of newly-fertilized agents (agent index number) in the agent list vector
	int *celNum_New_AgtLst;//dim[species]; the number of cells used in new_AgtLst
	int *size_New_AgtLst;//dim[species]; the total number of cells in new_AgtLst
	vector<vector<int> > new_AgtLst_Deme;//dim[species, agent index]; List of the deme of newly-fertilized agents (agent index number) in the agent list vector
	vector<vector<int> > new_AgtLst_Niche;//dim[species, agent index]; List of the niche of newly-fertilized agents (agent index number) in the agent list vector

	//Discrete trait computation
	//temporary data structures for survival, development, and movement
	int cls_UpperBound_Spc_Discrete;//the maximum number of classes for the upper bound trait
	int cls_LowerBound_Spc_Discrete;//the maximum number of classes for the lower bound trait
	int **size_VitaList_Discrete;//dim[upper bound trait, lower bound trait]; the number of cells
	int **celNum_VitaList_Discrete;//dim[upper bound trait, lower bound trait]; temporary counter for mating list
	int **maxCelNum_VitaList_Discrete;//dim[upper bound trait, lower bound trait]; the maximum number of temporary counter for mating list
	//temporary container for survival, development, and movement
	//dim[upper bound trait, lower bound trait, agent index number]
	std::vector< std::vector< std::vector<Agent*> > > vitaList_Discrete;

	//Gene-Trait
	int **loc_Trt;//dim[species, locus]the trait number controlled by a locus
	double **trt_Loc;//dim[species, trait]; the number of loci used to control a respective trait
	//the number of loci used to control a respective trait
	//dim[species, trait];
	int **trt_Loc_Int;
	//i.e. loc_Trt[2][2] contains the trait number determined by the thrid locus in the third species
	int **num_Clas_Trt_Spc;//dim[spc, trait]; the number of possible classes for each trait
	int *max_Num_Clas_Trt_Spc;//dim[trait]; the maximum number of of possible classes for each trait;
	int max_Num_Clas_Trt;//the maximum number of trait classes
	//the number of foraging traits = 2
	int m_Num_Traits_Foraging;

	//Initial allele frequencies for customized runs
	vector<vector<double> > ini_Bi_Allelic_Frequency;//dim[species, locus]

	//the efficiency for foraging resource for a species
	//dim[resource cluster, resource class]
	double **esp_Discrete;
	//functional response
	//dim[resource cluster, resource class]
	double **frsp;
	//foraging time for discrete traits
	double **ft_Discrete;

	//Bodylength-bodymass conversion factor
	//dim[species]
	double *scalingFactor_BodyLength_BodyMass_Conversion;
	double *exponent_BodyLength_BodyMass_Conversion;

	double *upper_Bound_Max;//dim[species]: the maximum bound of the upper-bound trait
	double *upper_Bound_Min;//dim[species]: the minimum bound of the upper-bound trait
	double *lower_Bound_Max;//dim[species]: the maximum bound of the lower-bound trait
	double *lower_Bound_Min;//dim[species]: the minimum bound of the lower-bound trait

	//the exponent for the upper-foraging bound trait
	double *exponent_Upper_Bound;
	//the exponent for the lower foraging bound trait
	double *exponent_Lower_Bound;

	double ****bodyMass;//dim[species, sex, life stage, upper-bound phenotype]: the body mass (g) of a trait at a life stage
	double ****bodyLength;//dim[species, sex, life stage, upper-bound phenotype]: the body length of a trait at a life stage	

	//Default degree of maturation in body length
	//dim[species, sex, life stage, upper-bound phenotype]
	double ****degree_Dev_Bodylength_Default;

	//body length
	//the number of bins for body length for each species
	//[species]
	int *num_Bin_Bodylength;
	//the maximum number of bins for bodylength trait
	int max_Num_Bin_Bodylength;
	//the maximum number of genotypes for gillraker trait
	int max_Num_Genotype_GillRaker;
	//the width of a bin for body length for each species
	//assumes that the minimum body length is 0.0.
	//[species]
	double *bin_Width_BodyLength;
	//scaling factor for the effect of available resource mass 
	//on the development of bodylength
	//[species]
	double *ddg;

	//the maximum bodyLength for a species
	//dim[species]
	double *bodyLength_Max_Spc;
	//the minimum bodyLength for a species
	//dim[species]
	double *bodyLength_Min_Spc;

	//the maximum bodyLength for a species
	//dim[species]
	double *bodyMass_Max;//dim[species]: the maximum body mass (g) of a sex
	//the minimum bodyLength for a species
	//dim[species]
	double *bodyMass_Min;
	double ****gillRaker;//dim[species, sex, life stage, lower-bound phenotype]: the body length of a trait at a life stage

	//metabolic rate = k * (biomass)^(metabolicRate_Biomass_Factor)
	//dim[species]
	double *metabolicRate_Biomass_Factor;
	//The degree of growth at the adult stage
	//dim[species, sex]
	double **growth_Target_Spc;

	//The exponent for von Bertallanffy function
	//dim[species, sex];
	double **exponent_VonBertallanffy;

	//Mating-behavior traits
	double **mul_Dim_Trt_Loc_QTL_Mtg;//multiplier for creating dimension for traits
	//i.e. for two traits for two loci, mul_Dim_Trt_Loc_QTL[spc][0] = 3, mul_Dim_Trt_Loc_QTL[spc][1] = 1 

	//Early sexual maturation
	double ****early_Sexual_Maturation;//dim[species, sex, life stage, upper-bound phenotype]: the chance of early sexaul maturation

	//Plasticity
	//Minimum fraction for producing random deviate with positive value
	double ltrm;
	//the proportion of the minimum aymptotic body length to the maximum asymptotic body length
	//dim[species]
	double *ltg_VB;
	//the factor controlling environmental effect on bodylength growth
	//dim[species]
	double *ce_VB;
	//the factor controlling the effect of interaction 
	//between genetic and environmental factors on
	//bodylength growth
	//dim[species]
	double *ci_VB;
	//the proportion of sexual maturation initiation change
	//dim[species]
	double *ltg_VB_Sx;
	//individual coefficient variance in bodylength growth
	//dim[species]
	double *cv_BodyLength_Growth;

	//the exponent for controlling the effect of 
	//the available resource mass on the initiation of sexual activities
	//dim[species]
	//default = 0.0
	double *smPlb;

	//the proportion of females at birth
	//dim[species]
	double *proportion_Female_At_Birth;
	//the maximum fertility
	//dim[species]
	double *maximum_Fertility;
	//covariance variation for the maximum fertility
	//dim[species]
	double *maximum_Fertility_CV;

	//(Core Species) Reduction in body length at a life stage caused  as the cost of early sexual maturation
	//dim[species]: the maximum body mass (g) of a species when the state of growth is 100% (1.0)
	//default = 1.0; (no cost)
	//the maximum cost = 0.0 (no bodylength for the earliest sexually matured genotype)
	double *cost_Reduction_EarlySexMat;

	//Genetic distance
	vector<vector<double> > gen_Dst_Map;//dim[species, locus gap (= (the number of loci - 1))]

	//(Core Species) Functional Response (Hunting)
	//the density of hunted prey = (a * Ttot * (N^(q))) / (1.0 + (a * Ttot * (th) * (N^(q))))
	//a. Agents hunting resource species
	double ****th_Spc_RscSpc;//dim[species, resource species], th: the fraction of time used for handling prey during hunting (0<= th <=1)
	double ****q_Spc_RscSpc;//dim[species, resource species], q: Type I: q = 1.0, Type II: q = 1.0, Type III: q = 2.0

	//Resource Acquisition (Hunting)
	double *delta_Hunt_Bound;//dim[species]; The parameter controlling generalist penality in the efficacy of hunting at around the boundaries
	double *k_Hunt_Bound;//dim[species]; The parameter controlling sigmoidal decrease in the efficacy of hunting at around the boundaries

	//Resource choice scaling factor regarding resource body mass
	//default = 1.0;
	double *exponent_Choice_Resource_BodyMass;

	int *****initial_Num_Spc;//dim[species], the initial number of species

	//////////////////////
	//Resource//
	//////////////////////
	//Resource species: Genetics
	int* num_Locus_Rsc;//dim[resource species]; the number of loci in a resource species
	int* num_Position_Rsc;//dim[resource species]; the number of loci in a resource species
	double* mut_Rat_Rsc;//dim[resource species]; the mutation rate of a resource species
	int max_Num_PhenTyp_Rsc;//the maximum number of phenotypes in resource species
	int* num_PhenoTyp_Rsc;//dim[resource species]; the number of phenotypes of a resource species
	int tot_Num_Rsc_Cls;//the total number of resource classes

	//Resource species: Demographic
	double ****phen_D_Rsc;//dim[deme, niche, resource species, phenotype class]; the densities of resources phenotypic classes
	double ****phen_D_Rsc_Fixed;//dim[deme, niche, resource species, phenotype class]; the densities of resources phenotypic classes after-burn-In (Fixed density)
	bool consistancy_Resource;//True: Resource abundance dos not change after burn in period (0), False: resource abundance changes after burn in period (1)
	double **mean_BioMass_Rsc;//dim[resource species, phenotype class]: the mean sizes (individual biomass) of phenotypic classes of resource species
	//resource boimass at the left and the right side of a bin
	//dim[resource species, phenotype class, bin left(0) or right (1) side]: the mean sizes(individual biomass) of phenotypic classes of resource species
	double ***bin_Rsc_BioMass;

	//The number of individuals = c*(1/biomass)^(alpha)
	//The parameter determining the exponent of the average biomass of individuals of a resource species (The number of individuals = C (1/biomass)^(alpha))
	//double **alpha_BioMass_Abundance;//dim[deme, niche]: //The number of individuals = c*(1/biomass)^(alpha)
	//double **c_BioMass_Abundance;//dim[deme, niche] : //The number of individuals = c*(1/biomass)^(alpha)
	double ***d_Rsc;//dim[deme, niche, resource species]; the densities of resources
	double ****prop_Density_Rsc;//dim[deme, niche, resource species, phenotype class]; the densities of resources
	double ***intrinsic_Rsc;//dim[deme, niche, resource species]; intrinsic growth rate for resource species
	double *growthRate_Rsc;//[phenotype class]; temporary container for resource species population growth rates
	double *tmp_Rsc_Cls;//[phenotype class]; temporary container for resource species density

	//Density
	double ***density_Spc;//dim[deme, niche, species]: the abundance of species
	double ***density_Rsc;//dim[deme, niche, resource species]: the abundance of resource species
	double ***density_Rsc_Fixed;//dim[deme, niche, resource species]: the abundance of resource species

	//(Resource Species) Functional Response (Hunting)
	//the density of hunted prey = (a * Ttot * (N^(q))) / (1.0 + (a * Ttot * (th) * (N^(q))))
	//a. Agents hunting resource species
	double ****th_Rsc_RscSpc;//dim[deme, niche, species, resource species, resource species], th: the fraction of time used for handling prey during hunting (0<= th <=1)
	double ****q_Rsc_RscSpc;//dim[deme, niche, species, resource species, resource species], q: Type I: q = 1.0, Type II: q = 1.0, Type III: q = 2.0

	double ****sum_Cmt_Rsc_Spc;//dim[deme, niche, resource type, phenotype class]: the number of individuals using a class of a resource species
	double *****sum_Spc_Rsc;//dim[deme, niche, species, resource species, resource phenotypic class]: the number of individuals of a specie using a class of a resource species

	//Temporary data container for discrete traits
	//dim[species, bodylength bin, gillraker genotype]
	int ***celNum_Spc_Trt_Local_Discrete;//the number of agents for a trait type for species

	//Dencity Dependence
	double *****Cd;//dim[deme, niche, species, sex, life stage]; parameter for density dependence
	double ***b_MassIntake_DDS;//dim[species, sex, life stage]; Exponent (b) controlling the relationship between mass intake and density-dependent survival

	//Lotka-Volterra interaction
	//the number of competing species = the number of species in agent + the number of resource species
	double ****LV_Loss;//dim[deme, niche, competing species(num_Sum_Spc), competing species(num_Sum_Spc)]; matrix of coefficients for loss by prey in a Lotka-Volterra equation
	//i.e. LV_Cmt_a[species #2][species # 5] is the coefficient for the effect of the abundance of species #5 on the abundance of species #2
	double ****LV_Gain;//dim[deme, niche, competing species(num_Sum_Spc), competing species(num_Sum_Spc)]; matrix of coefficients for gain by predator in a Lotka-Volterra equation
	//i.e. LV_Cmt_a[species #2][species # 5] is the coefficient for the effect of the abundance of species #5 on the abundance of species #2


	//Energetics for core species
	//Nutrietional value of resource per unit prey weight (The maximum value = 1)
	//Nutritional value = (Unit mass of prey / Unit Mass of Predator)^(k_NutritionValue)
	double **k_NutritionValue;//dim[core species, num_Sum_Spc]; nutritional value of prey

	mt19937 *randomSeed_Mut;//pointer for a random seed for mutation
	mt19937 *randomSeed_Trt;//pointer for a random seed for mutation
	mt19937 *randomSeed;//pointer for a random seed
	uniform_real_distribution<double> *uni_Real;//pointer for a uniform real distribution
	variate_generator<mt19937&, uniform_real_distribution<double> > *rand_Uni_Real;//uniform real random variate generator
	//To get a random deviate, const double random_Deviate =(*uni_Real_Rand)();

	mt19937 *randomSeed_Mtg;//pointer for a random seed
	uniform_real_distribution<double> *uni_Real_Mtg;//pointer for a uniform real distribution
	variate_generator<mt19937&, uniform_real_distribution<double> > *rand_Uni_Real_Mtg;//uniform real random variate generator

	mt19937 *randomSeed_int;//pointer for a random seed
	uniform_int_distribution<int> *uni_int;//pointer for a uniform integer distribution[0,1]
	variate_generator<mt19937&, uniform_int_distribution<int> > *rand_Uni_int;//uniform integer random variate generator

	//File management
	File_RW csvRW;//object for file management
	bool status_Simul_File_Creation;//file creation status; true: simulation files are created ,false: files are not created
	//File name for trait data
	char *fileName_Trait;
	string fileName_Trait_Str;
	//File name for trait data
	char *fileName_Trait_Comb;
	string fileName_Trait_Comb_Str;
	//File name for mate pair bodysize
	char *fileName_Bodysize_Mtg;
	string fileName_Bodysize_Mtg_Str;
	//File name for mate pair bodysize summary
	char *fileName_Bodysize_Mtg_Adt;
	string fileName_Bodysize_Mtg_Adt_Str;
	//File name for mate pair bodysize genotype
	char *fileName_Bodysize_Genotype_Mtg;
	string fileName_Bodysize_Genotype_Mtg_Str;
	//File name for mate pair bodysize genotype
	char *fileName_Bodysize_Genotype_Mtg_Adt;
	string fileName_Bodysize_Genotype_Mtg_Adt_Str;
	//File name for mate pair gillraker genotype
	char *fileName_GillRaker_Genotype_Mtg;
	string fileName_GillRaker_Genotype_Mtg_Str;
	//File name for mate pair gillraker genotype summary
	char *fileName_GillRaker_Genotype_Mtg_Adt;
	string fileName_GillRaker_Genotype_Mtg_Adt_Str;

	//File name for trait data
	char *fileName_Genome;
	string fileName_Genome_Str;
	//File name for resource density data
	char *fileName_Density;
	string fileName_Density_Str;
	//char *fileName_Resource_Class_Bodymass;//File name for resource density data
	//
	char *fileName_KolmogorovSmirnov_Mtg_Genotype;
	string fileName_KolmogorovSmirnov_Mtg_Genotype_Str;
	//
	char *fileName_KolmogorovSmirnov_Mtg_Bodymass;
	string fileName_KolmogorovSmirnov_Mtg_Bodymass_Str;
	//
	char *fileName_KolmogorovSmirnov_Mtg_GillRaker;
	string fileName_KolmogorovSmirnov_Mtg_GillRaker_Str;

	int tot_Num_Cel_Trait_Class;//the total number of cells in the trait-output-data buffer
	int tot_Num_Cel_Genome;//the total number of cells in the trait-output-data buffer
	int tot_Num_Cel_KolmogorovSmirnov;//the total number of cells in the Kolmogorov-Smirnov-test-output-data buffer
	int num_Param_KolmogorovSmirnov;//the number of parameters for Kolmogorov-Smirnov test

	//Trait Data Sampling
	//The range of all traits is between 0.0 and 1.0
	int **int_Num_Class_Trait;//dim[species, trait]; the number of classes (for histogram)
	double **double_Num_Class_Trait_Minus_One;//dim[species, trait]; the number of classes in double (for histogram)for a trait
	//double_Num_Class_Trait = (double)(int_Num_Class_Trait - 1)
	//trait class = (int)(trait value * double_Num_Class_Trait)
	//It assumes that the maximum trait value is 1.0

	int ****tmp_Data_Buffer_Trait;//dim[deme * niche, species, trait, trait class]
	int *****data_Buffer_Trait;//dim[sampling interval, deme * niche, species, trait, trait class]
	int **data_Buffer_Trait_2D;//to write files for data_Buffer_Trait

	//Combinatory trait type
	int ****tmp_Data_Buffer_Trait_Foraging_Combination;//dim[deme, niche, species, upper-boud trait class * lower-boud trait class]
	int ****data_Buffer_Trait_Combination;//dim[deme, niche, species, upper-boud trait class * lower-boud trait class]
	int **data_Buffer_Trait_Combination_2D;//to write files for data_Buffer_Trait_Combination

	int *num_Trait_Combination;//dim[species]; the number of trait combinations
	int **multiplier_Trait_Combination;//dim[species, trait]; The multiplier for the trait combination
	int tot_Num_Cel_Trait_Comb;//the total number of cells for trait combination per sampling time step
	int max_Num_Trait_Combination;

	//Genotypes of Mating Pairs
	double ****data_Buffer_Mtg_Trait;//dim[male upper-boud trait, male lower-boud trait class, female upper-boud trait, female lower-boud trait class]:Expected 4-D data	
	double*******tmp_Data_Buffer_Mtg_Trait;//dim[deme, niche, species, male upper-boud trait, male lower-boud trait class, female upper-boud trait, female lower-boud trait class]
	double ****exp_Data_Buffer_Mtg_Trait;//dim[male upper-boud trait, male lower-boud trait class, female upper-boud trait, female lower-boud trait class]:Expected 4-D data

	//Mating pair phenotype Bodysize
	//the number of bodysize class is 'max_Num_Clas_Trt'
	int *****data_Buffer_Bodysize_Mtg;//dim[deme, niche, species, male bodysize phenotype class, female bodysize phenotype class]
	//Mating pair genotype Bodysize
	//the number of bodysize class is 'max_Num_Clas_Trt'
	int *****data_Buffer_Bodysize_Genotype_Mtg;//dim[deme, niche, species, male bodysize genotype class, female bodysize genotype class]
	//Mating pair genotype gillraker
	//the number of gillraker class is 'max_Num_Clas_Trt'
	int *****data_Buffer_Gillraker_Genotype_Mtg;//dim[deme, niche, species, male bodysize genotype class, female bodysize genotype class]
	//numRow = 1;//the total number of cells in the trait-output-data buffer
	//numCol = 1 + (num_Deme * num_Niche * num_Spc * max_Num_Clas_Trt * max_Num_Clas_Trt);//add a column for time step
	int **data_Buffer_Bodysize_Mtg_2D;//to write files for data_Buffer_Trait

	//mating pairs with respect to bodymass
	int max_Num_Bodymass_Cls;
	int **num_Bodymass_Cls;//dim[species, sex]: the number of bodymass classes
	//contains bodymass index number for each category
	int ****index_Trait_Bodymass;//dim[species, sex, life stage, trait #0]
	double **data_Buffer_Mtg_Bodymass;//dim[male body mass, female body mass]
	double **exp_Data_Buffer_Mtg_Bodymass;//dim[male body mass, female body mass]
	double *****tmp_Data_Buffer_Mtg_Bodymass;//dim[deme, niche, species, male body mass class, female body mass class]

	//mating pairs with respect to the number of gill rakers
	int max_Num_GillRaker_Cls;
	int **num_GillRaker_Cls;//dim[species, sex]: the number of bodymass classes
	//contains gill-raker index number for each category
	int ****index_Trait_GillRaker;//dim[species, sex, life stage, trait #1 class]
	double **data_Buffer_Mtg_GillRaker;//dim[male body mass class, female body mass class]
	double **exp_Data_Buffer_Mtg_GillRaker;//dim[male body mass class, female body mass class]
	double *****tmp_Data_Buffer_Mtg_GillRaker;//dim[deme, niche, species, male body mass class, female body mass class]

	//Kolmogorov-Smirnov test results
	double ****data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype;//dim[deme, niche,, species, statisticss(Z, D, p-value, N, adjusted N)]
	double ****data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass;//dim[deme, niche,, species, statisticss(Z, D, p-value, N, adjusted N)]
	double ****data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker;//dim[deme, niche,, species, statisticss(Z, D, p-value, N, adjusted N)]

	//Density Data Sampling
	double *****data_Buffer_RscCls;//dim[sampling interval, deme, niche, Resource species,Size class]
	double **data_Buffer_RscCls_2D;//for writing files for data_Buffer_RscCls

	//Genetic data sampling
	double scaling_Multiplier_Allele;//scaling multiplier for allele(10^(-the number of digit of the maximum number of allele types per locus))
	//(i. e. if the maximum number of allele types per locus is 259, scaling_Multiplier_Allele is 10^(-3))
	int ****tmp_Data_Buffer_Allele;//dim[deme * niche * species, locus, allele, (tottal alleles - allele)]
	int *****data_Buffer_Allele;//dim[sampling interval, deme * niche * species, locus, allele, (tottal alleles - allele)]
	int **data_Buffer_Allele_2D;//to write files for data_Buffer_Allele

	//shared class
	boost::math::normal *std_Norm_Dist;//standard normal distribution

	//Standard normal distribution
	boost::math::normal std_Norm_KS_Test;
public:
	int num_Deme;//the maximum number of demes
	int num_Niche;//the maximum number of niches
	int tot_Deme_Niche;//((num_Deme - 1) * (num_Niche)) + ((num_Niche - 1)) + 1;
	int num_Spc;//the number of species
	int tot_Deme_Niche_Spc;//((num_Deme - 1) * (num_Niche * num_Spc)) + ((num_Niche - 1) * num_Spc) + (num_Spc - 1) + 1;
	int num_Sex;//the number of sexes
	int max_Num_Stg;//the maximum number of life stage
	int **num_Stg;//the number of life stages
	//the number of life stages for calculating
	//bodylength growth for von Bertallanffy function
	//dim[species]
	double **num_Stg_BodyLength_Growth;
	//the number of immature life stages having chance of early reproduction
	int **num_Immat_Stg_SexMaturation;
	int num_Niche_Num_Spc;//num_Niche * Num_Spc

	int far_Right_Deme;//the deme located on the far right side

	int num_RscSpc;////the number of resource species
	int num_Sum_Spc;//the sum of species types (num_Spc + num_RscSpc)

	int max_Num_Locus;//the maximum number of locus
	int *num_Locus;//dim[species]; the number of loci
	int *num_Locus_Mor;//dim[species]; the number of loci used for determining morphology
	int *num_Position;//dim[species]; the number of positions per locus
	int max_Num_Allele;//the maximum number of allele types per locus
	int max_Num_Position;//the maximum number of positions
	double **mut_Rat;//dim[species, locus]; mutation rate
	int *num_Trait;//dim[species]; the number of traits
	int max_Num_Trait;//the maximum number of trait

	uint64_t rdtsc(void);//returns cpu clock cycle (for generating a seed for random number generator)

	//constructor
	Simul_Proc(char *p_FileName_Setting, int *setArr, int **p_Mode, int *p_SimulStr, int *p_Num_Locus, int *p_Num_Position, double **p_MutRat, int *p_Num_Trait, int **p_Num_Stg, int **p_Num_Immat_Stg_SexMaturation, double *p_PropFeBrth, double***** p_P, double***** p_G, int ***** p_Initial_Num_Spc, double *p_MetabolicRate_Biomass_Factor);
	//destructor
	virtual ~Simul_Proc();

	//sets genetic structures
	void Set_Gen_Struct(int *p_Num_Locus, int *p_Num_Position, double **p_MutRat, int *p_Num_Trait);
	void Set_Gen_Trait(int **p_Loc_Trt, bool p_Switch_Assortative_Mating, double *p_Female_Choosiness_Scale, double *p_Female_Choosiness_Intensity, double *p_MetabolicRate_Biomass_Factor);//gets the relationship between genes and traits
	void Set_Ini_Allele_Frequency(vector<vector<double> > &p_Ini_Bi_Allelic_Frequency);

	//sets simulation structures
	void Set_Simul_Struct(int *****p_Initial_Num_Spc);
	//sets vitality structurs
	void Set_Vitality_Struct(int **p_Num_Stg, int **p_Num_Immat_Stg_SexMaturation, double *p_PropFeBrth, double***** p_P, double***** p_G, double *p_MetabolicRate_Biomass_Factor);
	//sets statistical functions
	void Set_Stat_Dst(char * fileName_Output);
	//sets resource
	void Set_Resource(void);
	void Set_Resource_Burn_In(void);//sets the quantity of resource after burn-in period
	void Reset_Resource_Fixed(void);//resets the quantity of resource to the fixed resource abundance after burn-in period
	void Set_Resource_Consistancy(int p_Consistancy_Resource);//sets the consistancy in resource

	//Discrete Vital List
	//sets data container for discrete vital list
	void Set_Container_Vital_List_Discrete(void);

	void Set_Data_Rsc_Density_Tstep(void);//sets the densities of resource species for each time step
	void Set_Resource(double ***intra_Specific_Interaction_LV, int* p_num_Locus_Rsc, int* p_num_Position_Rsc, double* p_mut_Rat_Rsc, int* p_num_PhenoTyp_Rsc, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc);
	void Set_Biomass_Resource(double **p_Trt_RscSpc_Mean_Sd);//sets the mean sizes (individual biomass) of phenotypic classes of resource species
	//sets the mean and standard deviations of the sizes (individual biomass) of phenotypic classes of resource species
	void Set_Mean_SD_BioMass_Resource(double** p_Trt_RscSpc_Mean_Sd, double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc);

	//sets parameter values regarding species interaction
	void Set_Spc_Interaction(void);
	void Set_Spc_Interaction(double *****p_Cd, double ***p_b_MassIntake_DDS, double ****h_Spc_RscSpc, double ****qq_Spc_RscSpc, double *p_K_Hunt_Bound, double *p_Delta_Hunt_Bound, double *p_Exponent_Choice_Resource_BodyMass, int *p_Mode_Forage_Physic_Spc);
	void Set_Lotka_Volterra_Parameters(char * fileName_Output, double *p_b_NutritionalValue, int among_Resource_Interaction, double **neg_Eff_Spc_Rsc_ScalingFactor, double ***p_intrinsic_Rsc, double ***p_Intra_Specific_Interaction_LV, double **a_BiomassRatio_Interaction, double **b_BiomassRatio_Interaction, double **a_BiomassRatio_Interaction_Gain, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double ****p_Stable_Stage_Dist, double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc, double ****p_H_Rsc_RscSpc, double ****p_Qq_Rsc_RscSpc);//sets parameters for the Lotka-Volterra equation
	//sets movement
	void Set_Movement(void);
	void Set_Movement(double* p_Mov_Patch, double** p_Mov_Niche);
	//sets foraging traits
	void Set_ForagingTrait(double **p_Growth_Target_Spc);
	//sets fertility
	void Set_Fertility(double**** p_F, double**** p_F_CV, double *p_Prop_Female_At_Birth);
	//Sets data storage
	void Set_Data_Output_Buffer(int** p_Class_Trait);
	//sets functional response for resource
	void Set_Functional_Response_Spc(double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double **p_Trt_RscSpc_Mean_Sd);
	//sets functional response for resource
	void Set_Functional_Response_Rsc(double ****p_H_Rsc_RscSpc, double ****p_Qq_Rsc_RscSpc, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double **p_Trt_RscSpc_Mean_Sd);

	//Sets the bodymass and sexual maturation of core species
	void Set_Body_Length_Mass_SexMat(double *max_Max_Body_Length, double *min_Min_Body_Length, double *bodyLength_Max, double *bodyLength_Min, double *p_Exp_Length_To_Growth, double *p_Cst_Length_To_Growth);

	//sets foraging trait bounds
	void Set_Foraging_Trait_Bound(double *p_Upper_Bound_Min, double *p_Lower_Bound_Max, double *p_Lower_Bound_Min);

	//sets parameters regarding plasticity
	void Set_Plasticity(double *p_DDG, double *p_Cost_Reduction_EarlySexMat, double *p_SmPlb);

	//sets parameters regarding plasticity
	void Set_Plasticity_Update(bool *p_Mode_Plasticity, double *p_ltg_VB, double *p_ce_VB, double *p_ci_VB, double *p_ltg_VB_Sx, double *p_sigma_BodyLength_Growth);

	const int Proc_Simul(int *p_Cnt, int *p_Total_Num_Simul, int *p_Replicate);//simulation
	void Proc_Stat(int *p_totNumSimul, int *p_Num_Replicate, int *p_Cnt_Simul);//Processes for statistical analyses
	void Burn_In_Rsc(int p_N_Burn_In);//only simulate resource species before core-species simulation starts
	void Set_Initial_Agent(void);//sets agents for the initial populations
	void Set_Reintroduction_Agent(int *p_Current_Tstep);//re-introduction of a core species
	void Set_TimeStep_Reintroduction_Core_Spc(int *p_Status_Re_Introsuction, int *p_TimeStep);//sets the ine step for reintroducing a core species
	void Set_Status_Reintroduction_Core_Spc(bool status_Reintroduction);//sets the status of the reintroduction of a core species

	//(Mode_Trait_Rsc[spc] == 0):The number of resource-use traits = the number of resource types
	//Discrete trait process
	//Survival, development, sexual maturation
	//Survival, growth (maturation) (No movement)
	void Surv_Grw_Trait_Rsc_Discrete(void);

	//discrete data processes
	//sorts individuals into phenotypes by using vitaList
	void Set_VitaList_Discrete(int p_Deme, int p_Niche, int p_Spc, int p_Sex, int p_Stg);
	//counts individuals sorted into phenotypes by using vitaList
	void Set_VitaList_CelNum_Discrete(int p_Deme, int p_Niche, int p_Spc);

	//returns the fertility predicted by using the body weight
	void Get_Mean_Fertility_Predicted_By_Bodymass(double *p_Fertility, double *p_Bodymass);

	//returns the group status of the possibility of sexual maturation
	void Get_Sexual_Maturation_Group_State(double *p_sexMat_Current, double *p_sexMat_Next, bool *null_Sexual_Maturation_Activation_Current, bool *full_Sexual_Maturation_Activation_Current, bool *null_Sexual_Maturation_Activation_Next, bool *full_Sexual_Maturation_Activation_Next);

	//get a factor to convert the number of female offspring to the number of male and female off spring
	void Get_Offsprinf_Convergence_Factor_From_FemaleOffspring(double *p_Factor, double *p_Prop_Females_At_Birth);

	//returns the chance of getting sexually active at a given life stage
	void Get_Chance_Sexual_Maturation(double *p_Chance_Sexual_Maturation_stage, int * p_Spc, int *p_Current_Lifestage, int *p_Sex, double *p_Allele_Dosage, int *p_Num_Stg_Immat_Sex_Mat);

	void Remove_Agent(int *p_Deme, int *p_Niche, int *p_Spc, Agent* p_ID, int *p_CelNumM);//removes an agent
	//Adds (to t) an agent with genetic information for the initial population
	void Add_Agent_Unit_IniPop(int p_Deme, int p_Niche, int p_Spc, int p_Sex, int p_Stg, vector<unsigned short int> &genome_0_New, vector<unsigned short int> &genome_1_New, vector<int> &trait);
	//Adds (to (t + 1)) an agent with genetic information with pre-zygotic processes to (t + 1)
	//If the tick is true, Adds an agent with genetic information with pre-zygotic processes to (t + 1)
	void Add_Mut_Agent_Unit(int *p_Deme, int *p_Niche, int *p_Spc, vector<unsigned short int> &genome_0_New, vector<unsigned short int> &genome_1_New, vector<int> &trait, int *p_CellNum, int *p_Size_Agt, vector<Agent*> &p_Agt_Sink, vector<Agent*> &p_Agt_Source, int *p_CelNum_M, int *p_celNum_New_AgtLst, bool *p_Status_SexualMaturation);

	//Transfer an agent from (t) to (t + 1)
	//Universal agent transfer function
	void Transfer_Agent_Unit(vector<Agent*> &p_Agt_Sink, vector<Agent*> &p_Agt_Source, Agent *p_ID, int *p_CelNum, int *p_Size_AgtLst);

	//If the tick_t is true with movement
	void Transfer_Agent_Unit_Mov_Tick_True(int *p_Deme, int *p_Niche, int *p_Deme_New, int *p_Niche_New, int *p_Spc, int *p_Sex, int *p_Stg, Agent *p_ID);
	//if tick_t is false with movement
	void Transfer_Agent_Unit_Mov_Tick_False(int *p_Deme, int *p_Niche, int *p_Deme_New, int *p_Niche_New, int *p_Spc, int *p_Sex, int *p_Stg, Agent *p_ID);

	//gets gamete from an agent without linkage disequillibrium
	void Get_Gamete_No_Linkage(vector<unsigned short int> &gamete, int p_Spc, Agent *p_ID);
	//gets gamete from an agent with linkage disequillibrium
	void Get_Gamete_Linkage(vector<unsigned short int> &gamete, int *p_Spc, int *p_AgtLibIndex);

	//gets gamete with allele 0
	void Get_Null_Gamete(vector<unsigned short int> &gamete, int *p_num_Locus, int *p_Spc);
	//gets gamete with allele 1
	void Get_Full_Gamete(vector<unsigned short int> &gamete, int *p_num_Locus, int *p_Spc);

	//gets gamete derived from bi-allelic frequencies
	void Get_Random_Drawn_Bi_Allelic_Gamete(vector<unsigned short int> &gamete, int *p_num_Locus, int *p_Spc, vector<vector<double> > &p_Bi_Allelic_Frequency);

	//sets trait to given gametes
	void Set_Trait(vector<int> &trait, vector<unsigned short int> &gamete_0, vector<unsigned short int> &gamete_1, int p_Spc);
	//re-sets the traits of an agent after mutation
	void Set_Trait(vector<int> &trait, int p_Spc, Agent * p_ID);

	//sets foraging time by using Sergie algorithm
	void Get_Resource_Preference_Sergie_Rsc(vector<double> &trait, int p_Num_Physical_Trait);

	//Genetic Mutation
	void Mut_Bi_Alle_Eq(void);//bi-alleleic mutation with equal, constant mutation rate
	void Get_Num_Binom_Dist(int *num_Mut_Per_Tstep, int *max_Num_Events, double *p_MutationRate);

	//Resource Processes
	void Rsc_Proc(void);//main resource process	
	void Set_Ini_Rsc(double ***p_intrinsic_Rsc, double **p_Trt_RscSpc_Mean_Sd, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance);//sets the initial densities of resource species

	//Local-level Aggregate Process for discrete traits
	void Local_Aggregate_Prc_Discrete(int *p_Deme, int *p_Niche);

	//Density
	void Set_Spc_Den(void);//sets the density of species
	void Set_Prop_Density_Rsc(void);//calculates the proportion of resource species
	//(mode_Gene_Trait[3] == 0):The number of resource-use traits = the number of resource types

	//initializes the number of individuals foraging resource classes
	void Set_Ini_Cumul_Indiv_Rsc_Cls(double **p_Sum_Cmt_Rsc_Spc, double ***p_Sum_Spc_Rsc);

	//sets the number of indivuduals competing to acquire a class of a resource species
	void Set_Cmt_Spc_Rsc_Pheotype_Cls_Local_Discrete(int deme, int niche);

	//sets sum data for discrete trait container
	//sets local agent numbers
	void Set_CelNum_Spc_Trt_Local_Discrete(int deme, int niche);

	//Lotka-Volterra Interaction
	void LotkaVolterra_Competition_Discrete(double *h, double *den_Tgt, double *itr_Grw, double *d_CmtSpc, double *coef_Cmt, int *num_Competing_Species);
	void LV_Compet_Discrete_Rsc(int p_RscSpc, int p_Deme, int p_Niche, double h);

	//Beverton-Holt function
	void Beverton_Holt_DenDep(double *p_previous_Density, double *p_DenDepSurRat, double *p_C, double *p_b_MassIntake_DDS);
	void Ricker_DenDep(double *p_previous_Density, double *p_DenDepSurRat, double *p_C);

	//Functional response
	void FunctionalResponse_Rate(double *huntingRate, double *a, double *th, double *denPrey, double *q);//returns the rate of prey attacked

	//returns functional response
	//functional response: p_Frsp [resource cluster][size class]
	void Get_FunctionalResponse_Esp(double **p_Frsp_Rate, double **p_Esp, double **p_Density_Rsc, int p_Deme, int p_Niche, int p_Spc);

	//Foraging Efficiency: Function for describing the decrease in the efficiency of hunting at around the upper and lower size boundaries
	void ESP_Sigmoid_Func(double *p_Hunt_Efficiency, double *p_x, double *p_UpperBound, double *p_LowerBound, double *p_K, double *p_Delta);
	//sets the efficacy of the use of prey affected by prey size (ESP)
	void Set_ESP_BodyMass(double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc, double *bodyMass_Max, double *bodyMass_Min, double *p_Exp_Length_To_Growth, double *p_Cst_Length_To_Growth, double *p_Upper_Bound_Min, double *p_Lower_Bound_Max, double *p_Lower_Bound_Min, double *p_Exponent_Upper_Bound, double *p_Exponent_Lower_Bound);

	//returns foraging efficiency of a species for resource size classes
	//p_Esp_Discrete[resource cluster][size class]
	void Get_ESP_Discrete(double **p_Esp_Discrete, int *p_Spc, int *p_Bin_BodyLength, int *p_GillRaker_Genotype);
	//tests foraging efficacy for discrete traits
	void Test_ESP_Discrete(void);

	//returns the bin number for bodylength for foraging
	void Get_Bin_Number_Degree_Of_Development_Body_Length(int *bin_Number, double *p_DegreeOfDevelopment_Current, double *p_Unit_Development_Foraging, double *p_Bin_Width, double *constant_VonBertallanffy, double *exponent_VonBertallanffy);
	//returns bodylength from a bin number
	void Get_BodyLength_From_Bin_Number(double *p_BodyLength, int *p_Bin_Number, int *p_Species);
	//returns bodymass from a bin number
	void Get_BodyMass_From_Bin_Number(double *p_BodyMass, int *p_Bin_Number, int *p_Species);
	//returns bodymass from a bin number
	void Get_BodyMass_From_BodyLength(double *p_BodyMass, double *p_BodyLength, int *p_Species);

	//returns upper-bound-foraging trait value from bodylength
	void Get_Upper_Bound_Value_From_BodyLength(double *p_Upper_Bound_Value, double *p_BodyLength, int *p_Species, double *p_Exponent_Upper_Bound);
	//returns gillraker value from genetic information and bodylength
	void Get_GillRaker_Value_From_BodyLength(double *p_GillRaker_Value, double *p_BodyLength, int *p_Num_Allele_GillRaker, int *p_Species, double *p_Exponent_Lower_Bound);

	//returns the bin width for calculating discrete bodylength at mating
	void Get_Bin_Width_Discrete_BodyLength_Mating(double *p_Bodylength_Bin_Width, int *p_Spc, int *p_Num_Bins);
	//returns the bodylength bin number for a mating individual
	void Get_Bin_Number_Bodylength_Mating_Indiv(int *p_BinNumber, double *p_Bodylength, double *p_Minimum_Bodylength, double *p_Bin_width_BodyLength, int *p_Tot_Num_Bins);

	//Body-length Development functions
	//returns the bin number for the current body length
	void Get_Bin_Number_BodyLength(int *p_bodyLength_Bin, double *p_Current_BodyLength, double *p_Max_BodyLength, double *p_Min_BodyLength, int *p_Num_Bin);
	//sets the bin width for bodylength
	void Set_Bin_Width_BodyLength(double *p_BodyLength_Max);

	//Normal function was used
	void ESP_Norm_Func(double *p_Hunt_Efficiency, double *p_Mu, double *p_Delta, double *binWidth_Rsc, double *bin_Left, double *bin_Right);

	//Ontogeny function
	void Get_Von_Bertalaffy_Growth_Weight(double *GrowthState, int *current_LifeStage, int *total_LifeStage, double *final_GrowthState, double *exp_Length_To_Weight);//returns the degree of growth by using von Bertalaffy function
	void Get_Von_Bertalaffy_Growth_Length(double *GrowthState, int *current_LifeStage, int *total_LifeStage, double *final_GrowthState);//returns the degree of growth by using von Bertalaffy function

	//Foraging time

	//foraging time fore discrete traits
	//returns the normalization contant for foraging time
	void Get_ForagingTime_Normalization_Constant_Discrete(double *normalizationConstant, double **p_Frsp, int p_Deme, int p_Niche, int p_Spc);
	//returns the proportion of foraging time for a class of a resource species
	void Get_Prop_ForagingTime_Discrete(double *p_Prop_ForagingTime, double * p_Scaled_Resource_Mass, double **p_FunctionalResponse, double *normalizationConstant, double *p_Nutritional_Value, int *p_Deme, int *p_Niche, int *p_Spc, int *p_Rsc, int *p_rCls, int *p_Num_Spc);
	//returns foraging time for a resource size class by using functional response
	void Get_Foraging_Time_Functional_Response(double **p_foraging_Time, double **p_Frsp, int p_Deme, int p_Niche, int p_Spc);

	//Common function
	//returns the total available mass for each agent
	void Get_Available_Resource_Mass_Individual_Discrete(double *p_Available_Mass_For_Indiv, double **p_Frsp, double **p_Ft_Discrete, double **p_Density_Resource, double **p_Sum_Cmt_Rsc_Spc, int spc);

	//returns the basic metabolic rate by using bodylength bin number
	void Get_Basic_Metabolic_Rate_From_Bin_BodyLength(double *p_Maximum_Metabolic_Rate, double *p_Bin_Width_BodyLength, int *p_Bin_Number_BodyLength, double *p_Gd_a, double *p_Gd_b, double *p_Exponent_MetabilicRate_BodyMass);

	//returns the metabolic rate by using bodylength
	void Get_Metabolic_Rate_From_BodyLength(double *p_Metabolic_Rate, double *p_BodyLength, double *p_Gd_a, double *p_Gd_b, double *p_Exponent_MetabilicRate_BodyMass);

	//returns edd for reflecting resource availability to development and survival
	void Get_EDD(double *p_Edd, double *p_Rci, double *p_Metabolic_Rate, double *p_Ddg, double *p_Ddc);

	//returns the degree of development in body length
	void Get_LKE(double *p_LKE, double *p_EDD, double *p_LKRN, double *p_LKN);

	//returns density-dependent-survival rate
	//by using the metabolic rate and available resource mass
	void Get_DDS_MetabolicRate_Rci(double *p_DDS, double *p_Ddc, double *p_Rci, double *p_MetabolicRate);

	//sexual activity related functions
	//reproductive process
	void Rpd_Mtg_Cls_Discrete(void);
	//Unit reproduction process (default)
	void Mtg_Unit_Discrete(int p_Deme, int p_Niche, int p_Spc);
	//Unit reproduction process for female choice-driven assortative mating
	void Mtg_Unit_Discrete_Assortative_Female_Choice(int p_Deme, int p_Niche, int p_Spc);

	//sorts mates by trait_0 within a niche
	void Set_Mates_By_BodyLength(int p_Deme, int p_Niche, int p_Spc);
	//sorts females by bodymass and put males in one bin (0)
	void Set_Female_Mates_By_BodyLength_Male_By_One(int p_Deme, int p_Niche, int p_Spc);

	//calculates the cumulative probability of choosing males
	//by using bodylength
	void Set_Cumul_Prob_Choice_Trait_Freq_BodyLength(double *p_Cumul_Prob_choice, double *p_Sum_Prod_Freq_Preference, int p_Spc, int *p_Bin_BodyLength_Female, int *p_LargeStepSearch);
	//calculates mate compatibility
	void Get_Competibility_Mates_BodyMass_Abs(double *p_Mate_Competibility, double *p_BodyMass_A, double *p_BodyMass_B, double *p_Choosiness_Phenotype, double *p_Choosiness_Scale, double *_Max_Bodylength, double *_Min_Bodylength);
	//returns upper & lower bound of fertility
	void Get_Bound_Fertility(int *p_Upper_Bound, int *p_Lower_Bound, double *p_Mean_Fertility, double *p_SD_Fertility);
	//calculates mean and standard deviation for fertility
	void Get_Mean_SD_Fertility_From_BodyMass(double *p_Mean_Fertility, double *p_SD_Fertility, double *p_BodyMass, int *p_Species);
	//checks fertility
	void Check_Fertility(int *p_Fertility, int *p_Upper_Bound, int *p_Lower_Bound);
	//sex determination at birth
	void Get_Sex_New_Born(int *p_Sex, double *p_Prop_Female_At_Birth);

	//resets the data buffer for mating
	void Set_Rpd_Memory_Cnt_Discrete(void);
	//converts bodylength to bin number
	void Convert_BodyLength_To_Bin_Number(int *p_Bin_Number, double *p_BodyLength, double *p_Unit_Bin_Width, int *p_Maximun_Bin_Number);



	//converts mu and sd for normal distribution to m and v for log-normal distribution
	void Convert_Mu_SD_NormDist_To_M_V_LogNormDist(double *p_M, double *p_V, double *p_Mu, double *p_SD);

	//returns a random deviate derived from log-normal distribution
	void Get_Random_Deviate_Log_Normal_Dist(double *p_Random_Deviate_LogNormalDist, double *p_Random_Deviate_UniformDist, double *p_M, double *p_V);

	//returns random bodylength growth
	void Get_Random_Growth_BodyLength(double *p_Random_Deviate_LogNormalDist, double *p_Random_Deviate_UniformDist, double *p_M, double *p_V, double *p_Mu, double *p_SD, double *p_Upper_Limit);

	//returns a bodylength growth factor regarding to age
	void Get_VBK(double *p_VBK, double *p_K, double *p_Age, double *p_Denominator, double *p_TimeStep);

	//returns a sexual maturation factor regarding to genetics and environment
	void Get_SxIniB(Agent *p_ID, double *p_SxIniB, double *p_LGX, double *p_Edd, double *p_Ce, double *p_Ci, double *p_Allele_Dose_BodyLength, int *p_Stg, int *p_Age_Rprd_Ini, int *p_Age_Rprd_Last, double *p_Inverse_Num_Repd_Imm_Stg_Double);

	//return asymptotic bodylength based on von Bertallanffy function
	void Get_VBC_Foraging(double *p_VBc, double *p_Ltg, double *p_Edd, double *p_Ltrn, double *p_Lgn, double *p_Ce, double *p_Ci);

	//return asymptotic bodylength based on von Bertallanffy function
	void Get_VBC(double *p_VBc, double *p_Ltg, double *p_Edd, double *p_Ltrn, double *p_Lgn, double *p_Ce, double *p_Ci);

	//returns baseline growth
	void Get_Baseline_Growth_BodyLength(double *p_Baseline_Growth_BodyLength, double *p_Maximum_Asymp_BodyLength, double *p_Lgn, double *p_VBK);

	//return the cumulative degree of growth over age based on von Bertallanffy function
	void Get_Cumulative_VBk(double *p_VBk_Cumul, double *p_K, double *p_Age, double *p_TotNumStg);

	//return bodylength based on von Bertallanffy function
	void Get_BodyLength_Von_Bertallanffy(double *p_BodyLength, double *p_BodyLength_Asymptotic_Max, double *p_VBC, double *p_VBK_Cumulative);

	//return bodylength based on von Bertallanffy function for foraging
	void Get_BodyLength_Von_Bertallanffy_Foraging(double *p_BodyLength, double *p_BodyLength_Previous, double *p_BodyLength_Asymptotic_Max, double *p_VBC, double *p_VBK);

	//set the effect of environment on bodylength growth
	void Get_CE_Trait(double *p_CE, vector<int> &p_Trait, double *p_Max_Num_Allele_Trait_Plasticity, double *p_CE_Fixed);

	//updates a tick for changing time steps
	void Switch_Tick_t(void);


	///////////////////////////////////////////////////////////////
	//Output Data Management//
	///////////////////////////////////////////////////////////////
	//Agent-Unit Output Data Buffer In/Out
	void Data_Buffer_Agent_In(int *p_Deme, int *p_Niche, int p_Spc, Agent *p_ID);//Data-In: adds a data point
	void Data_Buffer_Agent_Out(int *p_Deme, int *p_Niche, int p_Spc, Agent *p_ID);//Data-Out: removes a data point
	void Data_Buffer_Agent_Out_In(int *p_Deme, int *p_Niche, int *p_Deme_New, int *p_Niche_New, int p_Spc, Agent *p_ID);//Data-Out-IN: change the data location from the old location to a new location

	//adds mate data to buffer
	void Data_Buffer_Mating_Pair_Add_Discrete(int *p_Deme, int *p_Niche, int *p_Spc, Agent *p_ID_Male, Agent *p_ID_Female, int *p_Bin_Num_Bodylength_Male, int *p_Bin_Num_Bodylength_Female, int *p_Bin_Num_GillRaker_Male, int *p_Bin_Num_GillRaker_Female, double *p_tmp_Bodylength, double *p_Minimum_Bodylength, double *p_Bin_width_BodyLength, int *p_Tot_Num_Bins);

	//Output Data Buffer Management
	void Data_Output_Buffer_Initialization(int *p_Counter_Sampling, int *p_CrntTstep);//initializes data-out buffers

	void Data_Output_Buffer_Set_Cell_Trait(void);//sets the number of cells for data output
	void Data_Output_Buffer_Set_Cell_Genome(void);//sets the number of cells for data output
	void Data_Output_Buffer_Set_Cell_KolmogorovSmirnov(void);//sets the number of cells for data output

	//Calculating output for mating patterns
	void Data_Output_KolmogorovSmirnov_4D_Mtg_Genotype(int *p_Counter_Sampling);
	void Data_Output_KolmogorovSmirnov_2D_Mtg_Bodymass(int *p_Counter_Sampling);
	void Data_Output_KolmogorovSmirnov_2D_Mtg_GillRaker(int *p_Counter_Sampling);

	void Data_Output_Buffer_Write(int *p_Counter_Sampling, int *p_Current_Tstep);//writes output data
	void Data_Output_Buffer_Write_Trait(void);//writes trait output data
	void Data_Output_Buffer_Write_Trait_Comb(int *p_Current_Tstep);//writes trait output data
	void Data_Output_Buffer_Write_Bodylength_Mate_Pair(int *p_Current_Tstep);//writes mate pair bodylength data
	void Data_Output_Buffer_Write_Genome(void);//writes genome output data
	void Data_Output_Buffer_Write_Density(void);//writes resource densities output data
	void Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_Genotype(int *p_Current_Tstep);//writes trait output data
	void Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_Bodymass(int *p_Current_Tstep);//writes trait output data
	void Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_GillRaker(int *p_Current_Tstep);//writes trait output data

	//File Management
	void Set_File_Name(string string_Simul_Counter);//sets file names
	void Set_Simul_File_Output(void);//creates output files
	void Set_File_Column_Header(void);//sets column-headers for output files
	void Set_File_Column_Header_Trait(void);//sets column-headers for trait output files
	void Set_File_Column_Header_Trait_Comb(void);//sets column-headers for trait combinationoutput files
	void Set_File_Column_Header_Bodysize_Mtg(void);//sets colmn headers for mate pair bodysize
	void Set_File_Column_Header_Bodysize_Genotype_Mtg(void);//sets colmn headers for mate pair bodysize genotype
	void Set_File_Column_Header_GillRaker_Genotype_Mtg(void);//sets colmn headers for mate pair gillraker genotype

	void Set_File_Column_Header_Genome(void);//sets column-headers for genome output files
	void Set_File_Column_Header_Density(void);//sets column-headers for resource density output files
	void Set_File_Column_Header_KolmogorovSmirnov(char *p_FileName);//sets column-headers for Kolmogorov-Smirnov test output

	void Set_File_Resource_Bodymass(char *p_FileName_Setting);

	//set category values for output files
	void Set_File_Column_Header_Trait_Value(vector<double> &tmp_TraitValue);//sets trait values for output files
	void Set_File_Column_Header_Trait_Single_Value(vector<double> &tmp_TraitValue, int *p_NumTraits, int *p_Num_Class_Trait);//sets trait values for output files
	void Set_File_Column_Header_Density_Value(vector<double> &tmp_Bodymass);//sets bodymass for resource classes

	//sets Data Buffers
	void Set_Data_Buffer_Trait_Tstep(int p_Counter_Sampling);//sets data buffer for trait at each time step
	void Set_Data_Buffer_Trait_Comb(int *p_Counter_Sampling);//sets data buffer for trait combination
	void Set_Data_Buffer_Trait_Comb_Initial_Population(void);//sets data buffer for trait combination for the initial population
	void Set_Data_Buffer_Genome_Tstep(int p_Counter_Sampling);//sets data buffer for trait at each time step
	void Set_Data_Buffer_Density_Tstep(int p_Counter_Sampling);//sets data buffer for resource densities at each time step

	//data container memory allocation
	void Set_Data_Buffer_Mtg_Bodymass(void);//sets data buffer for mating with respect to bodymass
	void Set_Data_Buffer_Mtg_GillRaker(void);//sets data buffer for mating with respect to gill-raker

	//de-allocate memory
	void Deallocate_Memory_Data_Buffer_Mtg_Bodymass(void);//de-allocates memory for data buffer for mating with respect to bodymass
	void Deallocate_Memory_Data_Buffer_Mtg_GillRaker(void);//de-allocates memory for data buffer for mating with respect to gill-raker
	void Deallocate_Memory_Simul_FileName(void);//de-allocates memory for file names
	void Deallocate_Memory_Data_Buffer(void);//de-allocates memory for data buffer for files 

	//Program mode
	void Get_Deme_Niche_Mode(int *p_Deme_Niche_Mode);
	//unsigned long long rdtsc();//returns cpu clock cycle number

	//Memory Management
	void Opt_Memory_Cnt(int *p_CrntTime);//optimizes the counters for memory management

	//Memory management for discrete temporary data container
	//optimizes the counters for memory management
	void Opt_Memory_Cnt_Discrete(void);
	//sets the counters for memory management regarding survival
	void Set_Vita_Memory_Cnt_Discrete(void);

	//Computational routine
	void Get_MaxDiff_4D_Mat(double *p_Max, double ****mat0, double ****mat1, int nCls);//finds the maximum difference in 4-D matrix
	void Get_MaxDiff_2D_Mat(double *p_Max, double **mat0, double **mat1, int nCls);//finds the maximum difference in 4-D matrix

	//Re-sorting data
	void ReArrange_4D_Mat(double ****mat_Rearranged, double ****mat, int nCls, int *s);//Re-arrange 4D matrix in a specific order
	void ReArrange_2D_Mat(double **mat_Rearranged, double **mat, int nCls, int *s);//Re-arrange 4D matrix in a specific order

	void Get_Freq_Dim_4_Discrete(double ****mat, double *p_TotSum, int nc, double **frq);//calculates frequencies of classes in each dimension in 4-dimensional matrix
	void Get_Freq_Dim_2_Discrete(double **mat, double *p_TotSum, int nc, double **frq);//calculates frequencies of classes in each dimension in 4-dimensional matrix

	void Get_Exp_Freq_4D_Discrete(double ****mat_Exp, double *p_TotSum, double ****mat, int nc);//calculates the expected frequency
	void Get_Exp_Freq_2D_Discrete(double **mat_Exp, double *p_TotSum, double **mat, int nc);//calculates the expected frequency

	void Get_Fraction_Dim_4_Discrete(double ****mat, double p_TotSum, int nc);//convert elements into fractions by using the total sum
	void Get_Fraction_Dim_2_Discrete(double **mat, double p_TotSum, int nc);//convert elements into fractions by using the total sum

	void Get_CDF_4D_Discrete(double ****cumul_Map, double ****map, int numCela, int numCelb, int numCelc, int numCeld);//Gets the cumulative 4 dimensional discrete cumulative distribution
	void Get_CDF_2D_Discrete(double **cumul_Map, double **map, int numCela, int numCelb);//Gets the cumulative 2 dimensional discrete cumulative distribution

	void Get_CDF_4D_Discrete_Self(double ****map, int numCela, int numCelb, int numCelc, int numCeld);//Gets the cumulative 4 dimensional discrete cumulative distribution
	void Get_CDF_2D_Discrete_Self(double **map, int numCela, int numCelb);//Gets the cumulative 4 dimensional discrete cumulative distribution

	void Cdf_4D_Observed_Expected(double ****mat, double ****mat_Exp, int numCls, double *num_Observations);//calculates the cumulative probability density for 4-D observed values and their expected values
	void Cdf_2D_Observed_Expected(double **mat, double **mat_Exp, int numCls, double *num_Observations);//calculates the cumulative probability density for 4-D observed values and their expected values

	void Get_Kolmogorov_Smirnov_Test_SampleSize(double *p_AdjustedN, double N0, double N1);//calculates the sample size for Kolmogorov_Smirnov_Test
	void Get_Kolmogorov_Smirnov_Test_Z_Inf(double *Z_Inf, double *Z, double D, double N0);//calculates the infinite Z for Kolmogorov_Smirnov_Test
	void Get_Kolmogorov_Smirnov_Test_P_Value(double *p_PValue, double *p_PValue_Peacock, double *Z_Inf);//calculates the infinite Z for Kolmogorov_Smirnov_Test

	//Displays status
	void Display_Simul_Setup(int *p_totNumSimul, int *p_Num_Replicate, int *p_Cnt_Simul);//display the simulation settings
	void Display_Simul_Status(int *p_tStep);//displays the simulation status
	//displays variable values
	void DisplayVariable(int *tStep);
	//displays a statistical process mode
	void Display_StatisticalProcessMode(void);

	//Check-in
	void Check_In(string tmp_Str);

	//Descriptive statistics process
	void Set_Stats_Trait(void);//sets the means and standard deviations for traits
	void Set_Stats_Rsc(void);//sets the means and standard deviations for traits
};

#endif /* SIMULPROC_H_ */

