//============================================================================
// Name        : ABM_EP_OpenMP.cpp : Defines the entry point for the console application.
// Author      : Jungkoo Kang
// Version     :
// Copyright   : Your copyright notice
// Description : Hello World in C++, Ansi-style
//============================================================================

#include "stdafx.h"

//To find memory leak
//#define CRTDBG_MAP_ALLOC
//#include <stdlib.h>
//#include <crtdbg.h>

#include <iostream>
#include <fstream>//for csv file
#include <sstream>//for csv file
#include <string>//for csv file
#include <vector>
#include <ctime>
#include <cmath>
#include <boost/math/special_functions/round.hpp>
#include <boost/random/uniform_real_distribution.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/poisson_distribution.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include "Simul_Proc.h"//simulation processes
#include "Agent.h"
#include "File_RW.h"
#include "Matrix_Population_Model.h"
#include "Data_Manipulation.h"
#include "Batch_Analysis.h"

using namespace std;
//using namespace boost::math;
using namespace boost::random;

//void Function_Test(void)
//{
//string test_String;
//cout << "Type any character to terminate this program" << endl;
//cin >> test_String;//to wait
//}

//int GPU_If_i(float *x, float *c)
//{//returns 1 if x == c, otherwise returns 0
//float z = 1.0f + (*x) * (*x);
//float k = -(*x)*(*c);

//return (int)(1.0f - ceilf(((z + k) / (z + fabs(k)))));
//}

//float GPU_If_f(float *x, float *c)
//{//returns 1 if x == c, otherwise returns 0
//float z = 1.0f + (*x) * (*x);
//float k = -(*x)*(*c);

//return (1.0f - ceilf(((z + k) / (z + fabs(k)))));
//}

void Set_Plasticity_Param(double *p_Plasticity_Vec, int lgn, int ce, int ci, int ddg, int lgx, int cv_Bl_Growth, int ini_Freq_Allele_Plasticity)
{
	//p_Plasticity_Vec[0]: lgn
	//p_Plasticity_Vec[1]: ce
	//p_Plasticity_Vec[2]: ci
	//p_Plasticity_Vec[3]: ddg
	//p_Plasticity_Vec[4]: lgx
	//p_Plasticity_Vec[5]: cv_Bl_Growth
	//p_Plasticity_Vec[6]: ini_Freq_Allele_Plasticity
	//double default_BodyLength_Max = 273.68;

	//lgn
	double default_Lgn = 0.25;

	switch (lgn){
	case 0:
		p_Plasticity_Vec[0] = 0.125;
		break;
	case 1:
		p_Plasticity_Vec[0] = 0.25;
		break;
	case 2:
		p_Plasticity_Vec[0] = (0.25 + 0.5) / 2.0;
		break;
	case 3:
		p_Plasticity_Vec[0] = 0.5;
		break;
	case 4:
		p_Plasticity_Vec[0] = 0.75;
		break;
	default:
		p_Plasticity_Vec[0] = default_Lgn;
		break;
	}

	//ce
	switch (ce){
	case 0:
		p_Plasticity_Vec[1] = 0.0;
		break;
	case 1:
		p_Plasticity_Vec[1] = 0.25;
		break;
	case 2:
		p_Plasticity_Vec[1] = 0.5;
		break;
	case 3:
		p_Plasticity_Vec[1] = 0.75;
		break;
	case 4:
		p_Plasticity_Vec[1] = 1.0;
		break;
	default:
		p_Plasticity_Vec[1] = 0.0;
		break;
	}//end switch

	//ci
	switch (ci){
	case 0:
		p_Plasticity_Vec[2] = 0.0;
		break;
	case 1:
		p_Plasticity_Vec[2] = 0.25;
		break;
	case 2:
		p_Plasticity_Vec[2] = 0.5;
		break;
	case 3:
		p_Plasticity_Vec[2] = 0.75;
		break;
	case 4:
		p_Plasticity_Vec[2] = 1.0;
		break;
	default:
		p_Plasticity_Vec[2] = 0.0;
		break;
	}//end switch

	//ddg
	switch (ddg){
	case 0:
		p_Plasticity_Vec[3] = 0.25;
		break;
	case 1:
		p_Plasticity_Vec[3] = 0.5;
		break;
	case 2:
		p_Plasticity_Vec[3] = 0.75;
		break;
	case 3:
		p_Plasticity_Vec[3] = 1.0;
		break;
	default:
		p_Plasticity_Vec[3] = 0.75;
		break;
	}//end switch

	p_Plasticity_Vec[3] = log(p_Plasticity_Vec[3]);

	//lgx
	switch (lgx){
	case 0:
		p_Plasticity_Vec[4] = 0.0;
		break;
	case 1:
		p_Plasticity_Vec[4] = 0.25;
		break;
	case 2:
		p_Plasticity_Vec[4] = 0.5;
		break;
	case 3:
		p_Plasticity_Vec[4] = 0.75;
		break;
	case 4:
		p_Plasticity_Vec[4] = 1.0;
		break;
	default:
		p_Plasticity_Vec[4] = 1.0;
		break;
	}//end switch

	//cv_Bl_Growth
	switch (cv_Bl_Growth){
	case 0:
		p_Plasticity_Vec[5] = 0.001;
		break;
	case 1:
		p_Plasticity_Vec[5] = 0.01;
		break;
	case 2:
		p_Plasticity_Vec[5] = 0.1;
		break;
	case 3:
		p_Plasticity_Vec[5] = 0.2;
		break;
	case 4:
		p_Plasticity_Vec[5] = 0.4;
		break;
	default:
		p_Plasticity_Vec[5] = 0.01;
		break;
	}//end switch

	//ini_Freq_Allele_Plasticity
	switch (ini_Freq_Allele_Plasticity){
	case 0:
		p_Plasticity_Vec[6] = 0.0;
		break;
	case 1:
		p_Plasticity_Vec[6] = 0.01;
		break;
	case 2:
		p_Plasticity_Vec[6] = 0.5;
		break;
	case 3:
		p_Plasticity_Vec[6] = 0.99;
		break;
	case 4:
		p_Plasticity_Vec[6] = 1.0;
		break;
	default:
		p_Plasticity_Vec[6] = 0.0;
		break;
	}//end switch
}

void Simulation_Default(int p_Program_Mode, int *simul_Label, int *num_Simul_Label, int *p_Simul, bool *gene_Mode, int *simul_Basic_Setting, double *foragingTrait, double *p_CD_DDS, int *p_Mode, double *p_Mating, double *p_Setting_Resource, double *p_Plasticity_Vec)
{
	bool switch_Plasticity = gene_Mode[0];
	bool switch_Assortative_Mating = gene_Mode[1];

	int deme, niche, spc, sex, stg, trt, loc, num_Cat, i;

	//File name creation
	File_RW fileRW;
	string tmp_Str_0;
	string tmp_Str;

	tmp_Str = "st";//file content index

	//sets a label for file outputs
	fileRW.Get_Simul_Label(&tmp_Str_0, simul_Label, num_Simul_Label, p_Simul[2]);

	tmp_Str += tmp_Str_0 + ".csv";

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	string **s_Tag;
	int s_Tag_Row = 1;
	int s_Tag_Col = 7;
	s_Tag = new string*[s_Tag_Row];

	for (spc = 0; spc < s_Tag_Row; spc++){
		s_Tag[spc] = new string[s_Tag_Col];

		for (i = 0; i < s_Tag_Col; i++){
			s_Tag[spc][i] = " ";
		}//end i loop
	}//end row loop

	//Simulation Settings
	int simulStr[1];
	int num_Spc = 1, num_Deme = 1, num_Niche = 1, num_Sex = 2, num_RscSpc = simul_Basic_Setting[1], max_Num_Stg = 2, max_Num_Allele = 2;
	int max_Num_PhenTyp_Rsc = 21;//the maximum number of phenotypes in resource species
	int num_Mode_Category = 10;

	//Species mode
	int **mode;
	mode = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		mode[spc] = new int[num_Mode_Category];

		for (i = 0; i < num_Mode_Category; i++)
		{
			mode[spc][i] = p_Mode[i];
		}//end i loop

		//Overall gene-trait relation ship
		//0:All loci are QTL
		//mode[spc][0] = 0;

		//Mating
		//0: (with an error in mating group searching)trait_Mtg is used to determine the effect of traits on mating.  All loci are QTL. 
		//1: Randome mating with a niche
		//mode[spc][1] = p_Mode[1];

		//Behavior
		//0: Preference by physical capacity
		//An agent spend more time on a resource species class it can consume better than other resource species classes.
		//mode[spc][2] = 0;

		//Trait-Resource use determined by Morphology
		//0: low-pass filter (mouth size (body-size)) + high-pass filter (gill)
		//These traits affect the efficiency of using resouce in the defined range of size.
		//mode[spc][3] = 0;

		//Gene-Trait value determination
		//0: Synergetic
		//1: Additive
		//mode[spc][4] = 0;

		//(not used) mode_Gene_Trait[spc][5]: initial genotypic ondition 
		//0: 0 (Null state)
		//1:  1(Full state)
	}//end species loop

	s_Tag[0][0] = "(Core Species) Species Mode";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Trait";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, mode, &num_Spc, &num_Mode_Category);

	//Time steps
	simulStr[0] = simul_Basic_Setting[4];//the total number of time steps

	s_Tag[0][0] = "The Number of Time Steps";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Category";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, simulStr, &s_Tag_Col);

	//Life stages
	int **num_Stg;
	num_Stg = new int*[num_Spc];
	max_Num_Stg = 0;

	for (i = 0; i < num_Spc; i++){
		num_Stg[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			if (sex == 0)//male
			{
				num_Stg[i][sex] = 4;
			}
			else//female
			{
				num_Stg[i][sex] = 5;
			}//end else

			if (num_Stg[i][sex] > max_Num_Stg){
				max_Num_Stg = num_Stg[i][sex];
			}//end if
		}//end sex loop
	}//end i loop

	s_Tag[0][0] = "(Core Species) The Number of Life Stages";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Sex";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, num_Stg, &num_Spc, &num_Sex);

	//The number of immature life stages for sexual maturation
	int **num_Stg_Immat_Sex;
	num_Stg_Immat_Sex = new int*[num_Spc];

	for (i = 0; i < num_Spc; i++){
		num_Stg_Immat_Sex[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			if (sex == 0)//male
			{
				num_Stg_Immat_Sex[i][sex] = simul_Basic_Setting[2];
			}
			else//female
			{
				num_Stg_Immat_Sex[i][sex] = simul_Basic_Setting[3];
			}//end else
		}//end sex loop
	}//end i loop

	s_Tag[0][0] = "(Core Species) The Number of Immature Life Stages having chance of early reproduction";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Sex";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, num_Stg_Immat_Sex, &num_Spc, &num_Sex);

	//The number of loci
	int *num_Locus = new int[num_Spc];//the number of loci
	int max_Num_Locus = 0;

	for (spc = 0; spc < num_Spc; spc++){
		if (switch_Assortative_Mating == true)
		{//assortative mating mode
			num_Locus[spc] = 2 * simul_Basic_Setting[0];
		}
		else if (switch_Plasticity == true)
		{//plasticity mode
			num_Locus[spc] = simul_Basic_Setting[0] + (simul_Basic_Setting[0] / 2);
		}
		else
		{//default
			num_Locus[spc] = simul_Basic_Setting[0];// 1 * multiplier;
		}//end else

		if (num_Locus[spc] > max_Num_Locus)
		{
			max_Num_Locus = num_Locus[spc];
		}//end if
	}//end species loop

	s_Tag[0][0] = "(Core Species) The Number of Loci";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, num_Locus, &num_Spc);

	//The number of position
	int *num_Position;//the number of position
	num_Position = new int[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		num_Position[spc] = 2;
	}//end i loop

	s_Tag[0][0] = "(Core Species) The Number of Positions";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, num_Position, &num_Spc);

	double prop_Upper_Bound_Loci = 0.5;//the proportion of the upper-bound foraging trait loci 
	vector<vector<double>> ini_Bi_Allelic_Frequency;//initial bi-allelic frequencies

	for (spc = 0; spc < num_Spc; spc++){
		ini_Bi_Allelic_Frequency.push_back(vector<double>());

		if (switch_Assortative_Mating == true)
		{//assortative mating mode
			for (loc = 0; loc < num_Locus[spc]; loc++){
				if (loc < (num_Locus[spc] / 4))
				{//for the upper bound loci
					ini_Bi_Allelic_Frequency[spc].push_back(foragingTrait[4]);//initialization
				}
				else if (loc < ((num_Locus[spc] / 2)))
				{//for the lower bound loci
					ini_Bi_Allelic_Frequency[spc].push_back((1.0 - foragingTrait[4]));//initialization
				}
				else if (loc < ((3 * num_Locus[spc] / 4)))
				{//for plasticity loci
					ini_Bi_Allelic_Frequency[spc].push_back((p_Plasticity_Vec[6]));//initialization
				}
				else
				{//for assortative mating loci
					ini_Bi_Allelic_Frequency[spc].push_back((p_Mating[2]));//initialization
				}//end else
			}//end locus loop
		}
		else if (switch_Plasticity == true)
		{//plasticity mode
			for (loc = 0; loc < num_Locus[spc]; loc++){
				if (loc < (num_Locus[spc] / 3))
				{//for the upper bound loci
					ini_Bi_Allelic_Frequency[spc].push_back(foragingTrait[4]);//initialization
				}
				else if (loc < (2 * (num_Locus[spc] / 3)))
				{//for the lower bound loci
					ini_Bi_Allelic_Frequency[spc].push_back((1.0 - foragingTrait[4]));//initialization
				}
				else
				{//plasticity loci
					ini_Bi_Allelic_Frequency[spc].push_back((p_Plasticity_Vec[6]));//initialization
				}//end else
			}//end locus loop
		}
		else
		{//default
			for (loc = 0; loc < num_Locus[spc]; loc++){
				if (((double)loc) < (((double)num_Locus[spc]) * prop_Upper_Bound_Loci))
				{//for the upper bound loci
					ini_Bi_Allelic_Frequency[spc].push_back(foragingTrait[4]);//initialization
				}
				else
				{//for the lower bound loci
					ini_Bi_Allelic_Frequency[spc].push_back((1.0 - foragingTrait[4]));//initialization
				}//end else			
			}//end locus loop
		}
	}//end i loop

	double **tmp_Ini_Bi_Allele_Freq = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		tmp_Ini_Bi_Allele_Freq[spc] = new double[max_Num_Locus];

		for (loc = 0; loc < max_Num_Locus; loc++){
			tmp_Ini_Bi_Allele_Freq[spc][loc] = -1.0;//initialization
		}//end locus loop

		//sets a frequency for generating the output
		for (loc = 0; loc < num_Locus[spc]; loc++){
			tmp_Ini_Bi_Allele_Freq[spc][loc] = ini_Bi_Allelic_Frequency[spc][loc];//initialization
		}//end locus loop
	}//end i loop

	s_Tag[0][0] = "(Core Species) Initial Bi-Allelic Frequencies";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][0] = "Locus";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, tmp_Ini_Bi_Allele_Freq, &num_Spc, &max_Num_Locus);

	//de-allocate memory
	for (spc = 0; spc < num_Spc; spc++){
		delete[] tmp_Ini_Bi_Allele_Freq[spc];
	}//end i loop

	delete[] tmp_Ini_Bi_Allele_Freq;

	//The number of traits
	int *num_Trait;//the number of traits
	num_Trait = new int[num_Spc];
	int max_Num_Trait = 0;

	for (spc = 0; spc < num_Spc; spc++){
		if (switch_Assortative_Mating == true)
		{//assortative mating mode
			num_Trait[spc] = 4;
		}
		else if (switch_Plasticity == true)
		{//plasticity mode
			num_Trait[spc] = 3;
		}
		else
		{//default mode
			num_Trait[spc] = 2;
		}//end else

		if (num_Trait[spc] > max_Num_Trait){
			max_Num_Trait = num_Trait[spc];
		}//end if
	}//end species loop

	s_Tag[0][0] = "(Core Species) The Number of Traits";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, num_Trait, &num_Spc);

	//assigns loci to traits
	int **loc_Trt;//dim[species, locus]the trait number controlled by a locus
	loc_Trt = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		loc_Trt[spc] = new int[max_Num_Locus];
		int end0 = num_Locus[spc];

		for (loc = 0; loc <max_Num_Locus; loc++){
			loc_Trt[spc][loc] = (-1);//initialization
		}//end locus loop

		if (switch_Assortative_Mating == true)
		{
			for (loc = 0; loc < end0; loc++)
			{
				if (loc < (num_Locus[spc] / 4))
				{
					loc_Trt[spc][loc] = 0;//Upper-bound trait
				}
				else if (loc < ((num_Locus[spc] / 2)))
				{
					loc_Trt[spc][loc] = 1;//Lower-bound trait
				}
				else if (loc < (3 * num_Locus[spc] / 4))
				{
					loc_Trt[spc][loc] = 2;//Plasticity trait
				}
				else
				{
					loc_Trt[spc][loc] = 3;//Mating trait
				}//end else
			}//end locus loop
		}
		else if (switch_Plasticity == true)
		{//plastisity mode
			for (loc = 0; loc < end0; loc++){
				if (loc < (num_Locus[spc] / 3))
				{
					loc_Trt[spc][loc] = 0;//Upper-bound trait
				}
				else if (loc < (2 * (num_Locus[spc] / 3)))
				{
					loc_Trt[spc][loc] = 1;//Lower-bound trait
				}
				else
				{
					loc_Trt[spc][loc] = 2;//Plasticity trait
				}//end else
			}//end locus loop
		}
		else
		{//default mode
			for (loc = 0; loc < end0; loc++){
				if (((double)loc) < (((double)num_Locus[spc]) * prop_Upper_Bound_Loci))
				{
					loc_Trt[spc][loc] = 0;//Upper-bound trait
				}
				else
				{
					loc_Trt[spc][loc] = 1;//Lower-bound trait
				}//end else
			}//end locus loop
		}//else
	}//end species loop

	s_Tag[0][0] = "(Core Species) Locus-Trait Match";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Locus";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, loc_Trt, &num_Spc, &max_Num_Locus);

	//The number of classes for each traits for constructing output histograms
	int **num_Class_Trait;
	num_Class_Trait = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		num_Class_Trait[spc] = new int[max_Num_Trait];

		if (switch_Assortative_Mating == true)
		{
			for (trt = 0; trt < max_Num_Trait; trt++){
				if (trt < num_Trait[spc])
				{//For upper, lower bound trait scenario
					num_Class_Trait[spc][trt] = ((num_Locus[spc] / 4) * num_Position[spc]) + 1;
				}
				else
				{
					num_Class_Trait[spc][trt] = 1;
				}//end else			
			}//end trait loop
		}
		else if (switch_Plasticity == true)
		{
			for (trt = 0; trt < max_Num_Trait; trt++){
				if (trt < num_Trait[spc]){//For upper, lower bound trait scenario
					num_Class_Trait[spc][trt] = ((num_Locus[spc] / 3) * num_Position[spc]) + 1;
				}
				else
				{
					num_Class_Trait[spc][trt] = 1;
				}//end else			
			}//end trait loop
		}
		else
		{//default mode
			for (trt = 0; trt < max_Num_Trait; trt++){
				if (trt < num_Trait[spc]){//For upper, lower bound trait scenario
					num_Class_Trait[spc][trt] = ((int)(num_Locus[spc] / 2) * num_Position[spc]) + 1;
				}
				else
				{
					num_Class_Trait[spc][trt] = 1;
				}//end else			
			}//end trait loop
		}//end else
	}//end species loop

	//The number of classes for each traits for constructing output histograms
	s_Tag[0][0] = "(Core Species) The Number of Classes for each Traits for Constructing Output";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Trait";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, num_Class_Trait, &num_Spc, &max_Num_Trait);

	//Mutation rates
	double **mut_Rat = new double *[num_Spc];//mutation rates	

	for (spc = 0; spc < num_Spc; spc++){
		mut_Rat[spc] = new double[max_Num_Locus];

		for (loc = 0; loc < max_Num_Locus; loc++){
			mut_Rat[spc][loc] = 0.0;
		}//end locus loop

		int end0 = num_Locus[spc];

		for (loc = 0; loc < end0; loc++){
			mut_Rat[spc][loc] = 0.00001;//0.00001
		}//end locus loop
	}//end species loop

	s_Tag[0][0] = "(Core Species) Mutation Rate";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Locus";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, mut_Rat, &num_Spc, &max_Num_Locus);

	//The proportion of females at birth
	double *propFemBirth;
	propFemBirth = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		propFemBirth[spc] = 0.5;//the proportion of females at birth
	}//end species loop	

	s_Tag[0][0] = "(Core Species) The Proportion of Females at Birth";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, propFemBirth, &num_Spc);

	//Probability that an agent survives to the same life stage
	double***** P;
	P = new double****[num_Deme];
	double adult_Male_SurvivalRate = 0.6281;//0.5
	double adult_Female_SurvivalRate = 0.6281;//0.5
	double juvenile_Survival_Rate = 0.67032;
	double incremental_Improvement_Survival = 0.0;
	double dist_Juvenile_Male_P_G = 0.0;//Distribution to survival in survival and growth distribution
	double dist_Juvenile_Female_P_G = 0.0;//Distribution to survival in survival and growth distribution

	for (deme = 0; deme < num_Deme; deme++){
		P[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			P[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				P[deme][niche][spc] = new double*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					P[deme][niche][spc][sex] = new double[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						P[deme][niche][spc][sex][stg] = 0.0;
					}//end life stage loop

					for (stg = 0; stg < num_Stg[spc][sex]; stg++){
						//P (survival rate) + G (survival + growth rate)
						if (stg == (num_Stg[spc][sex] - 1)){//for the last life stage
							if (sex == 0)//male
							{
								P[deme][niche][spc][sex][stg] = adult_Male_SurvivalRate;
							}
							else//female
							{
								P[deme][niche][spc][sex][stg] = adult_Female_SurvivalRate;
							}
						}
						else if (stg < (num_Stg[spc][sex] - 1)){//immature life stage
							if (sex == 0)//male
							{
								P[deme][niche][spc][sex][stg] = (juvenile_Survival_Rate + incremental_Improvement_Survival *((double)stg)) * dist_Juvenile_Male_P_G;
							}
							else//female
							{
								P[deme][niche][spc][sex][stg] = (juvenile_Survival_Rate + incremental_Improvement_Survival *((double)stg))* dist_Juvenile_Female_P_G;
							}//end else
						}
						else{//does not exist
							P[deme][niche][spc][sex][stg] = 0.0;
						}//end else						
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) P";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	s_Tag[0][4] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_5DD(fileName_Output, P, &num_Deme, &num_Niche, &num_Spc, &num_Sex, &max_Num_Stg);

	//Probability that an agent survives and develops to the next life stage
	double***** G;
	G = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		G[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			G[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				G[deme][niche][spc] = new double*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					G[deme][niche][spc][sex] = new double[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						G[deme][niche][spc][sex][stg] = 0.0;
					}//end life stage loop

					for (stg = 0; stg < max_Num_Stg; stg++){
						//P (survival rate) + G (survival + growth rate)
						if (stg == (num_Stg[spc][sex] - 1)){//for the last life stage
							G[deme][niche][spc][sex][stg] = 0.0;
						}
						else if (stg < (num_Stg[spc][sex] - 1)){//immature life stage
							if (sex == 0)//male
							{
								G[deme][niche][spc][sex][stg] = (juvenile_Survival_Rate + incremental_Improvement_Survival *((double)stg)) * (1.0 - dist_Juvenile_Male_P_G);
							}
							else//female
							{
								G[deme][niche][spc][sex][stg] = (juvenile_Survival_Rate + incremental_Improvement_Survival *((double)stg)) * (1.0 - dist_Juvenile_Female_P_G);
							}//end else
						}
						else{//not exist
							G[deme][niche][spc][sex][stg] = 0.0;
						}//end else
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) G";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	s_Tag[0][4] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_5DD(fileName_Output, G, &num_Deme, &num_Niche, &num_Spc, &num_Sex, &max_Num_Stg);

	//Fertility (only counts female offspring)
	double**** F;//Mean in fertility following normal distribution
	F = new double***[num_Deme];
	sex = 1;//for females

	for (deme = 0; deme < num_Deme; deme++){
		F[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			F[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				F[deme][niche][spc] = new double[max_Num_Stg];

				for (stg = 0; stg < max_Num_Stg; stg++){
					F[deme][niche][spc][stg] = 0.0;
				}//end life stage loop

				for (stg = 0; stg < num_Stg[spc][sex]; stg++){
					//P (survival rate) + G (survival + growth rate)
					if (stg == (num_Stg[spc][sex] - 1)){
						F[deme][niche][spc][stg] = 121.27;//Alewife: 121.27
					}
					else{
						F[deme][niche][spc][stg] = 0.0;
					}//end else
				}//end life stage loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) F";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, F, &num_Deme, &num_Niche, &num_Spc, &max_Num_Stg);

	//stable stage distribution
	double**** stableStageDist;//dim[deme, niche, species, life stage]; stable stage distribution
	stableStageDist = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		stableStageDist[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			stableStageDist[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				stableStageDist[deme][niche][spc] = new double[max_Num_Stg];

				for (stg = 0; stg < max_Num_Stg; stg++){
					stableStageDist[deme][niche][spc][stg] = 0.0;
				}//end row loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//Stable Density-Dependent Survival Rate
	double*** stableDDS;//dim[deme, niche, species]; stable density-dependent-survival rate
	stableDDS = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		stableDDS[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			stableDDS[deme][niche] = new double[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				stableDDS[deme][niche][spc] = 0.0;
			}//end species loop
		}//end niche loop
	}//end deme loop

	///////////////////////////////////////////////
	//Matrix Population Model//
	///////////////////////////////////////////////
	if (num_Spc > 0)
	{
		Matrix_Population_Model MatPopMol(num_Deme, num_Niche, num_Spc, num_Sex, num_Stg, tmp_Str_0);
		MatPopMol.Set_Vital_Rat(P, G, F);//sets transition matrix
		MatPopMol.Run_Analysis();//runs analyses
		MatPopMol.Get_Stable_Stage_Dist(stableStageDist);
		MatPopMol.Get_Stable_DDS(stableDDS);
	}//end if
	////////////////////////////

	//Stable-Stage Distribution
	s_Tag[0][0] = "(Core Species) Stable-Stage Distribution";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, stableStageDist, &num_Deme, &num_Niche, &num_Spc, &max_Num_Stg);

	//Stable Density-Dependent Survival Rate
	s_Tag[0][0] = "(Core Species) Stable Density-Dependent Survival Rate";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 3;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_3DD(fileName_Output, stableDDS, &num_Deme, &num_Niche, &num_Spc);

	//The initial number of agents
	int***** initial_Num_Spc;
	initial_Num_Spc = new int****[num_Deme];

	bool introduction_Adult_Only = true;
	double tmp_Prop_Pop = 0.0;

	for (deme = 0; deme < num_Deme; deme++)
	{
		const double tot_Num_Indiv_Per_Sex_Per_Niche = 500.0;
		int end0;
		initial_Num_Spc[deme] = new int***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++)
		{
			initial_Num_Spc[deme][niche] = new int**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++)
			{
				initial_Num_Spc[deme][niche][spc] = new int*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++)
				{
					end0 = num_Stg[spc][sex];
					initial_Num_Spc[deme][niche][spc][sex] = new int[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++)
					{
						initial_Num_Spc[deme][niche][spc][sex][stg] = 0;//initialization
					}//end life stage loop

					for (stg = 0; stg < end0; stg++)
					{
						//adult only introduction mode
						if (introduction_Adult_Only == true)
						{//adult-only introduction
							if (stg == (num_Stg[spc][sex] - 1))
							{
								tmp_Prop_Pop = 1.0;
							}
							else
							{
								tmp_Prop_Pop = 0.0;
							}//end else
						}
						else
						{//following stable stage distribution
							tmp_Prop_Pop = stableStageDist[deme][niche][spc][stg];
						}//end else

						//P (survival rate) + G (survival + growth rate)
						if (sex == 1)// for females
						{
							initial_Num_Spc[deme][niche][spc][sex][stg] = boost::math::iround((tmp_Prop_Pop * tot_Num_Indiv_Per_Sex_Per_Niche * propFemBirth[spc]));
						}
						else//for males
						{
							initial_Num_Spc[deme][niche][spc][sex][stg] = boost::math::iround((tmp_Prop_Pop * tot_Num_Indiv_Per_Sex_Per_Niche * (1.0 - propFemBirth[spc])));
						}//end else
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) The Initial Number of Species";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	s_Tag[0][4] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_5DD(fileName_Output, initial_Num_Spc, &num_Deme, &num_Niche, &num_Spc, &num_Sex, &max_Num_Stg);

	double**** F_CV;//Standard Deviation in Fertility Following Normal Distribution
	F_CV = new double***[num_Deme];
	sex = 1;//for females

	for (deme = 0; deme < num_Deme; deme++){
		F_CV[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			F_CV[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				F_CV[deme][niche][spc] = new double[max_Num_Stg];

				for (stg = 0; stg < max_Num_Stg; stg++){
					F_CV[deme][niche][spc][stg] = 0.0;
				}//end life stage loop

				for (stg = 0; stg < num_Stg[spc][sex]; stg++){
					if (stg == (num_Stg[spc][sex] - 1)){
						F_CV[deme][niche][spc][stg] = 0.2;
					}
					else{
						F_CV[deme][niche][spc][stg] = 0.2;
					}//end else
				}//end life stage loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) Coefficient Variation in Fertility Following Normal Distribution";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, F_CV, &num_Deme, &num_Niche, &num_Spc, &max_Num_Stg);

	//(Core Species) Female Choosiness Scale for Male Phenotype
	//Error rate in choosing the right mating class
	double *female_Choosiness_Scale = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		female_Choosiness_Scale[spc] = p_Mating[1];
	}//end species loop

	s_Tag[0][0] = "(Core Species) Female Choosiness Scale for Male Phenotype";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, female_Choosiness_Scale, &num_Spc);

	//(Core Species) Female Choosiness Intensity for Male Phenotype
	//Error rate in choosing the right mating class
	double *female_Choosiness_Intensity = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		female_Choosiness_Intensity[spc] = p_Mating[0];//0.5;
	}//end species loop

	s_Tag[0][0] = "(Core Species) Female Choosiness Intensity for Male Phenotype";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, female_Choosiness_Intensity, &num_Spc);

	//movement rate between demes
	double *mov_Deme = new double[1];
	mov_Deme[0] = 0.0;

	s_Tag[0][0] = "(Core Species) Movement Rate between Demes";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "per individual rate";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, mov_Deme, &num_Cat);

	double** mov_Niche;//movement rate between niches
	mov_Niche = new double*[num_Niche];

	for (niche = 0; niche < num_Niche; niche++){
		mov_Niche[niche] = new double[num_Niche];

		for (int niche_0 = 0; niche_0 < num_Niche; niche_0++){
			if (niche == niche_0)
			{//chance to stay
				mov_Niche[niche][niche_0] = 1.0;
			}
			else
			{//chance to move
				mov_Niche[niche][niche_0] = 0.0;
			}//end else
		}//end species loop
	}//end species loop

	//Movement Rate Between Niches
	s_Tag[0][0] = "(Core Species) Movement Rate Between Niches";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Niche";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, mov_Niche, &num_Niche, &num_Niche);

	//(Core Species) The Maximum Body Mass (g) of a Species with the Maximum Trait Value at 100% Maturation
	double *bodyLength_Max = new double[num_Spc];//dim[species]: the maximum body mass (g) of a species when the state of growth is 100% (1.0)

	for (spc = 0; spc < num_Spc; spc++){
		bodyLength_Max[spc] = 273.6842;//the unit: mm
	}//end species loop

	s_Tag[0][0] = "(Core Species) The Maximum Body Length of a Species";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, bodyLength_Max, &num_Spc);


	//(Core Species) Reduction in body length at a life stage caused  as the cost of early sexual maturation
	double *cost_Reduction_EarlySexMat = new double[num_Spc];//dim[species]: the maximum body mass (g) of a species when the state of growth is 100% (1.0)

	for (spc = 0; spc < num_Spc; spc++){
		cost_Reduction_EarlySexMat[spc] = foragingTrait[2];//the unit: g
	}//end species loop

	s_Tag[0][0] = "(Core Species) Cost of Early Sexual Maturation in Body Length at a Given Time";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, cost_Reduction_EarlySexMat, &num_Spc);


	//(Core Species) The Scaling Factor for Biomass to Prey Size Range Ratio
	//(k_Bm_Psr = log10( (the maximum adult core species biomass / the minimum adult core species biomass) / (the maximum prey size / the maximum prey size for the lowest-trait values adult) )
	//minimum_Size_At_Max_Maturation = (bodyMass_Max[spc] * (pow(10.0, k_Bm_Psr[spc]))) / upper_Limt_Trait_Magnitude
	double *bodyLength_Min = new double[num_Spc];//dim[species]: the maximum body mass (g) of a species when the state of growth is 100% (1.0)

	for (spc = 0; spc < num_Spc; spc++){
		bodyLength_Min[spc] = bodyLength_Max[spc] * cost_Reduction_EarlySexMat[spc];//the unit: g
	}//end species loop

	s_Tag[0][0] = "(Core Species) The Minimum Body Length of a Species";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, bodyLength_Min, &num_Spc);

	//the exponent for controlling the effect of 
	//the available resource mass on the initiation of sexual activities
	//dim[species]
	//default = 0.0
	double *smPlb = new double[num_Spc];//dim[species]: the maximum body mass (g) of a species when the state of growth is 100% (1.0)

	for (spc = 0; spc < num_Spc; spc++){
		smPlb[spc] = 0.0;
	}//end species loop

	s_Tag[0][0] = "(Core Species) smPlb: the effect of resource on sexual maturation";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, smPlb, &num_Spc);

	//metabolic Rate-Biomass Factor
	double *metabolicRate_Biomass_Factor = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		metabolicRate_Biomass_Factor[spc] = 3.0 / 4.0;
	}//end species loop

	s_Tag[0][0] = "(Core Species) alpha: metabolic rate = Cd*(biomass)^(alpha)";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, metabolicRate_Biomass_Factor, &num_Spc);

	////the mass of resource taken by an individual when core species population growth rate is 1
	//double mass_Resource_Per_Indiv_At_Stable;//the mass of resource taken by an individual when core species population growth rate is 1

	//Parameter for density-dependent
	double exponent_CD = 1.0;
	double***** Cd;
	Cd = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		Cd[deme] = new double***[num_Niche];
		int end0;

		for (niche = 0; niche < num_Niche; niche++){
			Cd[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				Cd[deme][niche][spc] = new double*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					end0 = num_Stg[spc][sex];
					Cd[deme][niche][spc][sex] = new double[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						Cd[deme][niche][spc][sex][stg] = 0.0;//initialization
					}//end life stage loop

					for (stg = 0; stg < end0; stg++){
						//the mass of resource taken by an individual when core species population growth rate is 0
						//mass_Resource_Per_Indiv_At_Stable = rsc_K[deme][niche][0] / (spc_K[deme][niche][spc] * frac_Comepeting_Indiv[spc]);

						//Parameter for density-dependent (stops population growth when the mass of resource taken by an individual is mass_Resource_Per_Indiv_At_Stable)
						Cd[deme][niche][spc][sex][stg] = (*p_CD_DDS);
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) k (Denisty-Dependent-Survival Factor): metabolic rate = k*(biomass)^(alpha)";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	s_Tag[0][4] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_5DD(fileName_Output, Cd, &num_Deme, &num_Niche, &num_Spc, &num_Sex, &max_Num_Stg);

	//intrinsic rate of increase for resource species
	double***b_MassIntake_DDS;//resource density
	b_MassIntake_DDS = new double**[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		int tmp_Stg = 0;
		b_MassIntake_DDS[spc] = new double*[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			b_MassIntake_DDS[spc][sex] = new double[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				b_MassIntake_DDS[spc][sex][stg] = 0.0;
			}//end species loop

			tmp_Stg = num_Stg[spc][sex];

			for (stg = 0; stg < tmp_Stg; stg++){
				b_MassIntake_DDS[spc][sex][stg] = 1.0;// 1.0;
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "Exponent (b) controlling the relationship between mass intake and density-dependent survival";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 3;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Sex";
	s_Tag[0][2] = "Life stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_3DD(fileName_Output, b_MassIntake_DDS, &num_Spc, &num_Sex, &max_Num_Stg);

	double *b_NutritionalValue = new double[num_Spc];//dim[species]:(1.0)

	for (spc = 0; spc < num_Spc; spc++){
		b_NutritionalValue[spc] = 0.0;//the unit: g
	}//end species loop

	s_Tag[0][0] = "(Core Species) Exponent (b) determining the nutritional value of prey";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, b_NutritionalValue, &num_Spc);

	//von Bertalaffy function
	double **growth_Target_Spc = new double*[num_Spc];//dim[species];The degree of growth at the adult stage

	for (spc = 0; spc < num_Spc; spc++){
		growth_Target_Spc[spc] = new double[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			growth_Target_Spc[spc][sex] = 0.95;
		}//end sex loop
	}//end species loop

	s_Tag[0][0] = "(Core Species) von Bertalaffy Function Growth Target at the Last Life Stage";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Sex";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, growth_Target_Spc, &num_Spc, &num_Sex);

	//constant (a) for converting growth in length to growth in weight
	double *cst_Length_To_Growth = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++)
	{
		cst_Length_To_Growth[spc] = (-5.28911);
	}//end species loop

	s_Tag[0][0] = "(Core Species) constant (a) for converting growth degree in length to weight";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, cst_Length_To_Growth, &num_Spc);

	//ltn: controls the fraction of observed bodylength range
	double *ltg_VB = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++)
	{
		ltg_VB[spc] = p_Plasticity_Vec[0];
	}//end species loop

	s_Tag[0][0] = "(Core Species) LGN Von Bertallanffy";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, ltg_VB, &num_Spc);

	//constant (a) for converting growth in length to growth in weight
	double *ce_VB = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++)
	{
		ce_VB[spc] = p_Plasticity_Vec[1];
	}//end species loop

	s_Tag[0][0] = "(Core Species) CE Von Bertallanffy";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, ce_VB, &num_Spc);

	//constant (a) for converting growth in length to growth in weight
	double *ci_VB = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++)
	{
		ci_VB[spc] = p_Plasticity_Vec[2];
	}//end species loop

	s_Tag[0][0] = "(Core Species) CI Von Bertallanffy";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, ci_VB, &num_Spc);

	//the proportion of sexual maturation initiation change
	double *ltg_VB_Sx = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++)
	{
		ltg_VB_Sx[spc] = p_Plasticity_Vec[4];
	}//end species loop

	s_Tag[0][0] = "(Core Species) ltg_VB_Sx: the proportion of sexual maturation initiation change";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, ltg_VB_Sx, &num_Spc);

	//coefficent variantion in bodylength growth
	double *cv_BodyLength_Growth = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++)
	{
		cv_BodyLength_Growth[spc] = p_Plasticity_Vec[5];
	}//end species loop

	s_Tag[0][0] = "(Core Species) cv_BodyLength_Growth: coefficient variation for bodylength growth";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, cv_BodyLength_Growth, &num_Spc);

	//exponent (b) for converting growth in length to growth in weight
	double *exp_Length_To_Growth = new double[num_Spc];//dim[species];

	for (spc = 0; spc < num_Spc; spc++){
		exp_Length_To_Growth[spc] = 3.0637;
	}//end species loop

	s_Tag[0][0] = "(Core Species) exponent (b) for converting growth degree in length to weight";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, exp_Length_To_Growth, &num_Spc);

	double**** metabolicRate_CoreSpc;
	metabolicRate_CoreSpc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		metabolicRate_CoreSpc[deme] = new double**[num_Niche];
		int end0;

		for (niche = 0; niche < num_Niche; niche++){
			metabolicRate_CoreSpc[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				metabolicRate_CoreSpc[deme][niche][spc] = new double[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					end0 = num_Stg[spc][sex];
					//metabolicRate_CoreSpc[deme][niche][spc][sex] = Cd[deme][niche][spc][sex][(end0 - 1)] * pow((bodyMass_Max[spc] * growth_Target_Spc[spc][sex]), metabolicRate_Biomass_Factor[spc]);//((1.0 / pow(stableDDS[deme][niche][spc], ((double)tmp_Stg))) - 1.0) * mass_Resource_Per_Indiv_At_Stable;
					metabolicRate_CoreSpc[deme][niche][spc][sex] = pow((bodyLength_Max[spc] * growth_Target_Spc[spc][sex]), metabolicRate_Biomass_Factor[spc]);//((1.0 / pow(stableDDS[deme][niche][spc], ((double)tmp_Stg))) - 1.0) * mass_Resource_Per_Indiv_At_Stable;
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) Maximum Metabolic Rate";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, metabolicRate_CoreSpc, &num_Deme, &num_Niche, &num_Spc, &num_Sex);

	//Mode for physical foraging trait
	int *mode_Forage_Physic_Spc = new int[num_Spc];//Mode for physical foraging trait
	//Do not use 0: Normal hunting function 
	for (spc = 0; spc < num_Spc; spc++){
		mode_Forage_Physic_Spc[spc] = 0;//1: the default setting (Compound Sigmoidal Resource Use Function)
	}//end species loop

	s_Tag[0][0] = "(Core Species) Physical Foraging Trait Mode";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, mode_Forage_Physic_Spc, &num_Spc);

	//(Core Species) The Steepness in the Efficacy of Hunting at around the Boundaries
	double *k_Hunt_Bound;
	k_Hunt_Bound = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		k_Hunt_Bound[spc] = foragingTrait[1];// 10.0;//0.0: No generality penalty, +infinity: the maximum generality penalty
	}//end species loop

	s_Tag[0][0] = "(Core Species) The Steepness in the Efficacy of Hunting at around the Boundaries";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, k_Hunt_Bound, &num_Spc);

	//(Core Species) Generality Penalty(delta) in Compound Sigmoidal Hunting Efficacy Function
	double *delta_Hunt_Bound;
	delta_Hunt_Bound = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		delta_Hunt_Bound[spc] = foragingTrait[0];//0.0: No generality penalty, +infinity: the maximum generality penalty
	}//end species loop

	s_Tag[0][0] = "(Core Species) Generalist Penalty (delta) in Compound Sigmoidal Hunting Efficacy Function";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, delta_Hunt_Bound, &num_Spc);

	//exponent_Choice_Resource_BodyMass
	double *exponent_Choice_Resource_BodyMass;
	exponent_Choice_Resource_BodyMass = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		exponent_Choice_Resource_BodyMass[spc] = foragingTrait[6];// foragingTrait[0];//1.0
	}//end species loop

	s_Tag[0][0] = "(Core Species) Exponent for resource bodymass in foraging time assignment";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, exponent_Choice_Resource_BodyMass, &num_Spc);


	//(Core Species) Exponent for Upper-bound Trait
	double *exponent_UpperBound_Trait;
	exponent_UpperBound_Trait = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		exponent_UpperBound_Trait[spc] = 1.853;//0.0: No generality penalty, +infinity: the maximum generality penalty
	}//end species loop

	//s_Tag[0][0] = "(Core Species) Exponent for Upper-bound Trait";
	//s_Tag_Col = 1;
	//fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	//num_Cat = 1;
	//s_Tag[0][0] = "Species";
	//fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	//fileRW.CSV_Write_Add_Double_1DD(fileName_Output, exponent_UpperBound_Trait, &num_Spc);

	//(Core Species) Exponent for Lower-bound Trait
	double *exponent_LowerBound_Trait;
	exponent_LowerBound_Trait = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		exponent_LowerBound_Trait[spc] = foragingTrait[5];//0.0: No generality penalty, +infinity: the maximum generality penalty
	}//end species loop

	s_Tag[0][0] = "(Core Species) scaling factor for Lower-foraging bound";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, exponent_LowerBound_Trait, &num_Spc);

	//(Core Species) The minimum bound of the upper-bound trait
	double *upper_Bound_Min;
	upper_Bound_Min = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		upper_Bound_Min[spc] = 0.01;//
	}//end species loop

	//s_Tag[0][0] = "(Core Species) The minimum bound of the upper-bound trait";
	//s_Tag_Col = 1;
	//fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	//num_Cat = 1;
	//s_Tag[0][0] = "Species";
	//fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	//fileRW.CSV_Write_Add_Double_1DD(fileName_Output, upper_Bound_Min, &num_Spc);

	//(Core Species) The maximum bound of the lower-bound trait
	double *lower_Bound_Max;
	lower_Bound_Max = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		lower_Bound_Max[spc] = 0.02;//
	}//end species loop

	s_Tag[0][0] = "(Core Species) The maximum bound of the lower-bound trait";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, lower_Bound_Max, &num_Spc);

	//(Core Species) The minimum bound of the upper-bound trait
	double *lower_Bound_Min;
	lower_Bound_Min = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		lower_Bound_Min[spc] = 0.001;//2015: 0.001 0.0025
	}//end species loop

	s_Tag[0][0] = "(Core Species) The minimum bound of the lower-bound trait";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, lower_Bound_Min, &num_Spc);

	//Functional Response: Handling Time
	double ****th_Spc_SpcRsc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		th_Spc_SpcRsc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			th_Spc_SpcRsc[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				th_Spc_SpcRsc[deme][niche][spc] = new double[num_RscSpc];

				for (i = 0; i < num_RscSpc; i++){
					th_Spc_SpcRsc[deme][niche][spc][i] = foragingTrait[3];
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) Functional Response: Handling Time";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, th_Spc_SpcRsc, &num_Deme, &num_Niche, &num_Spc, &num_RscSpc);

	//Functional Response: Response Mode
	double ****q_Spc_SpcRsc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		q_Spc_SpcRsc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			q_Spc_SpcRsc[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				q_Spc_SpcRsc[deme][niche][spc] = new double[num_RscSpc];

				for (i = 0; i < num_RscSpc; i++){
					q_Spc_SpcRsc[deme][niche][spc][i] = 1.0;
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) Functional Response: Response Mode";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, q_Spc_SpcRsc, &num_Deme, &num_Niche, &num_Spc, &num_RscSpc);

	//Resource Consistancy
	int * tmp_RscCst = new int[1];

	tmp_RscCst[0] = ((int)p_Setting_Resource[9]);

	s_Tag[0][0] = "Consistancy in resource";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Category (0: Not consistant; 1: Consistant)";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, tmp_RscCst, &s_Tag_Col);

	//deallocate memory
	delete[] tmp_RscCst;

	//Resource species: Genetics
	//Th number of loci for resource species
	int* num_Locus_Rsc = new int[num_RscSpc];//dim[resource species]; the number of loci in a resource species

	for (spc = 0; spc < num_RscSpc; spc++){
		num_Locus_Rsc[spc] = 10;
	}//end resource species loop

	s_Tag[0][0] = "(Resource Species) The Number of Loci";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, num_Locus_Rsc, &num_RscSpc);

	//The Number of Positions for a Resource Species
	int* num_Position_Rsc = new int[num_RscSpc];//dim[resource species]; the number of loci in a resource species

	for (spc = 0; spc < num_RscSpc; spc++){
		num_Position_Rsc[spc] = 2;
	}//end resource species loop

	s_Tag[0][0] = "(Resource Species) The Number of Positions";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, num_Position_Rsc, &num_RscSpc);

	//The Mutation Rate for a Resource Species
	double* mut_Rat_Rsc = new double[num_RscSpc];//dim[resource species]; the mutation rate of a resource species

	for (spc = 0; spc < num_RscSpc; spc++){
		mut_Rat_Rsc[spc] = 0.00001;
	}//end resource species loop

	s_Tag[0][0] = "(Resource Species) Mutation Rate";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, mut_Rat_Rsc, &num_RscSpc);

	//The Number of Phenotypes for a Resource Species
	int* num_PhenoTyp_Rsc = new int[num_RscSpc];
	max_Num_PhenTyp_Rsc = 0;

	for (spc = 0; spc < num_RscSpc; spc++){
		num_PhenoTyp_Rsc[spc] = num_Locus_Rsc[spc] * num_Position_Rsc[spc] + 1;

		if (num_PhenoTyp_Rsc[spc] > max_Num_PhenTyp_Rsc){
			max_Num_PhenTyp_Rsc = num_PhenoTyp_Rsc[spc];//the maximum number of phenotypes in resource species
		}//end if
	}//end resource species loop

	s_Tag[0][0] = "(Resource Species)  The Number of Phenotypes";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, num_PhenoTyp_Rsc, &num_RscSpc);

	//The maximum mean size of a resource species in a niche
	double *max_Mean_BioMass_Rsc = new double[1];

	if (num_RscSpc <= 2){//For 2-resource-cluster setting
		max_Mean_BioMass_Rsc[0] = 0.223607;
	}
	else{//for three resource setting: max_Mean_BioMass_Rsc[0] = 5.0;
		max_Mean_BioMass_Rsc[0] = 5.0;//10% of the maximum weight of a core species 
	}//end else

	s_Tag[0][0] = "(Resource Species)  The Maximum Mean Size";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Biomass (g)";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, max_Mean_BioMass_Rsc, &num_Cat);

	//The minimum mean size of a resource species in a niche
	double *min_Mean_BioMass_Rsc = new double[1];
	min_Mean_BioMass_Rsc[0] = 0.01;

	s_Tag[0][0] = "(Resource Species) The Minimum Mean Size";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Biomass (g)";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, min_Mean_BioMass_Rsc, &num_Cat);

	//Size-Structured Food Web
	//The coefficient variation (=standard error / mean) of the size of a resource species in a niche
	double *CV_BioMass_Rsc = new double[1];
	CV_BioMass_Rsc[0] = 0.3;

	s_Tag[0][0] = "(Resource Species) The Coefficient Variation (=Standard Deviation / Mean) of the Size";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Coefficient Variation (CV)";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, CV_BioMass_Rsc, &num_Cat);

	//Scaling Factor: Negative Effect of Core Species on Resource Species
	double **coreSpc_Neg_Effect_ScalingFactor = new double*[num_Deme];

	for (deme = 0; deme < num_Deme; deme++)
	{
		coreSpc_Neg_Effect_ScalingFactor[deme] = new double[num_Niche];

		for (niche = 0; niche < num_Niche; niche++)
		{
			coreSpc_Neg_Effect_ScalingFactor[deme][niche] = p_Setting_Resource[8];// 1.0;
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Resource Species) Scaling Factor: Negative Effect of Core Species on Resource Species";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, coreSpc_Neg_Effect_ScalingFactor, &num_Deme, &num_Niche);

	//Size-Structured Food Web
	//The parameter determining the exponent of the average biomass of individuals of a resource species (The number of individuals = C (1/biomass)^(alpha))
	double **alpha_BioMass_Abundance = new double*[num_Deme];

	for (deme = 0; deme < num_Deme; deme++)
	{
		alpha_BioMass_Abundance[deme] = new double[num_Niche];

		for (niche = 0; niche < num_Niche; niche++)
		{
			alpha_BioMass_Abundance[deme][niche] = p_Setting_Resource[7];// 3.0 / 4.0;
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Resource Species) alpha in (The number of individuals = c*(1/biomass)^(alpha))";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, alpha_BioMass_Abundance, &num_Deme, &num_Niche);

	//Size-Structured Food Web
	//(Resource Species) c in (The number of individuals = c*(1/biomass)^(alpha))
	double **c_BioMass_Abundance = new double*[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		c_BioMass_Abundance[deme] = new double[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			c_BioMass_Abundance[deme][niche] = p_Setting_Resource[6];
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Resource Species) c in (The number of individuals = c*(1/biomass)^(alpha))";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, c_BioMass_Abundance, &num_Deme, &num_Niche);

	//Functional Response: Handling Time
	double ****th_Rsc_RscRsc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		th_Rsc_RscRsc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			th_Rsc_RscRsc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				th_Rsc_RscRsc[deme][niche][spc] = new double[num_RscSpc];

				for (i = 0; i < num_RscSpc; i++){
					th_Rsc_RscRsc[deme][niche][spc][i] = p_Setting_Resource[5];
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Resource Species) Functional Response: Handling Time";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Resource Species";
	s_Tag[0][3] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, th_Rsc_RscRsc, &num_Deme, &num_Niche, &num_RscSpc, &num_RscSpc);

	//Functional Response: Response Mode
	double ****q_Rsc_RscRsc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		q_Rsc_RscRsc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			q_Rsc_RscRsc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				q_Rsc_RscRsc[deme][niche][spc] = new double[num_RscSpc];

				for (i = 0; i < num_RscSpc; i++){
					q_Rsc_RscRsc[deme][niche][spc][i] = 1.0;
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Resource Species) Functional Response: Response Mode";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Resource Species";
	s_Tag[0][3] = "Resource Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, q_Rsc_RscRsc, &num_Deme, &num_Niche, &num_RscSpc, &num_RscSpc);

	//intrinsic rate of increase for resource species
	double***rsc_intr;//resource density
	rsc_intr = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		rsc_intr[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			rsc_intr[deme][niche] = new double[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){

				if (spc == 0)
				{
					rsc_intr[deme][niche][spc] = p_Setting_Resource[0];
				}
				else
				{
					rsc_intr[deme][niche][spc] = 0.0;
				}//end else
			}//end resource species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "Scaling factor for the Intrinsic Rate of Increase (r) for Resource Species";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 3;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][1] = "Resource species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_3DD(fileName_Output, rsc_intr, &num_Deme, &num_Niche, &num_RscSpc);

	//Intra-specific-density-dependent effect in the Lotka-Volterra equation
	int num_Sum_Spc = num_Spc + num_RscSpc;
	double ***intra_Specific_Interaction_LV = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		intra_Specific_Interaction_LV[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			intra_Specific_Interaction_LV[deme][niche] = new double[num_Sum_Spc];

			for (spc = 0; spc < num_Sum_Spc; spc++){
				//reflect the biomass-Numerical abundance relationship to set this
				intra_Specific_Interaction_LV[deme][niche][spc] = p_Setting_Resource[1];
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "M (Intra-Specific Effect (Lotka-Volterra)): effect = M  / [c_abundance_biomass * (1.0 / (biomass)^(alpha))]";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 3;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][1] = "(Core + Resource)Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_3DD(fileName_Output, intra_Specific_Interaction_LV, &num_Deme, &num_Niche, &num_Sum_Spc);

	//Interaction among resource species
	int *interaction_Among_RscSpc = new int[1];
	interaction_Among_RscSpc[0] = 1;//0: No interaction among resource species, 1: interaction among resource species

	s_Tag[0][0] = "Resource Species Interaction";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "(0: No interaction, 1: Interaction)";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, interaction_Among_RscSpc, &num_Cat);

	//Parameter (a) for the equation determining alpha (alpha = a * (bioMass of a predator / biomass of a prey)^b) in the Lotka-Volterra equation
	double **a_BiomassRatio_Interaction = new double*[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		a_BiomassRatio_Interaction[deme] = new double[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			a_BiomassRatio_Interaction[deme][niche] = p_Setting_Resource[2];//(= 1 / the the number of core species at carrying capacity)
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "a: Inter-Specific Interaction (Loss) (Lotka-Volterra) (alpha = a * (bioMass of a prey / biomass of a predator)^b)";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, a_BiomassRatio_Interaction, &num_Deme, &num_Niche);

	//Parameter (a) for the equation determining alpha (alpha = a * (bioMass of a predator / biomass of a prey)^b) in the Lotka-Volterra equation
	double **b_BiomassRatio_Interaction = new double*[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		b_BiomassRatio_Interaction[deme] = new double[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			b_BiomassRatio_Interaction[deme][niche] = p_Setting_Resource[4];
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "b: Inter-Specific Interaction (Loss) (Lotka-Volterra) (alpha = a * (bioMass of a prey / biomass of a predator)^b)";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, b_BiomassRatio_Interaction, &num_Deme, &num_Niche);

	//Parameter (a) for the equation determining alpha (alpha = a * (bioMass of a predator / biomass of a prey)^b) in the Lotka-Volterra equation
	double **a_BiomassRatio_Interaction_Gain = new double*[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		a_BiomassRatio_Interaction_Gain[deme] = new double[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			a_BiomassRatio_Interaction_Gain[deme][niche] = p_Setting_Resource[3];//1.0 * 0.001;//
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "a: Inter-Specific Interaction (Lotka-Volterra) (Gain) (alpha = a * (bioMass of a predator / biomass of a prey)^b)";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_2DD(fileName_Output, a_BiomassRatio_Interaction_Gain, &num_Deme, &num_Niche);


	///////////////////////////////////
	//parameters regarding plasticity//
	///////////////////////////////////

	//scaling factor for the effect of available resource mass 
	//on the development of bodylength
	//[species]
	//default = 10.0
	double *p_DDG = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		p_DDG[spc] = p_Plasticity_Vec[3];
	}//end species loop

	s_Tag[0][0] = "DDG: scaling factor(resource mass-bodylength Development)";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, p_DDG, &num_Spc);

	//////////////////
	//Basic Settings//
	//////////////////
	int tmp_Num_Param = 8;
	int *setArr = new int[tmp_Num_Param];
	setArr[0] = num_Deme;//the number of demes
	setArr[1] = num_Niche;//the number of niches
	setArr[2] = num_Spc;//the number of species
	setArr[3] = num_Sex;//the number of sexes
	setArr[4] = max_Num_Stg;//the maximum number of life stages
	setArr[5] = num_RscSpc;//the number of resource species
	setArr[6] = max_Num_Allele;//the maximum number of allele types per locus
	setArr[7] = max_Num_PhenTyp_Rsc;//the maximum number of phenotypes in resource species

	//Basic Settings
	s_Tag[0][0] = "Basic Settings";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Category";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_1DD(fileName_Output, setArr, &tmp_Num_Param);

	////////////////////////////////
	//Simulation Unit//
	////////////////////////////////
	Simul_Proc sim(fileName_Output, setArr, mode, simulStr, num_Locus, num_Position, mut_Rat, num_Trait, num_Stg, num_Stg_Immat_Sex, propFemBirth, P, G, initial_Num_Spc, metabolicRate_Biomass_Factor);
	sim.Set_File_Name(tmp_Str_0);//sets out file names

	//Initial population re-introduction
	int status_Re_Introduction = 0;
	int timeStep_Re_Introduction = 15000;
	sim.Set_TimeStep_Reintroduction_Core_Spc(&status_Re_Introduction, &timeStep_Re_Introduction);//sets the time step for re-introducing a core species

	if (num_RscSpc > 0){
		sim.Set_Resource(intra_Specific_Interaction_LV, num_Locus_Rsc, num_Position_Rsc, mut_Rat_Rsc, num_PhenoTyp_Rsc, alpha_BioMass_Abundance, c_BioMass_Abundance, max_Mean_BioMass_Rsc[0], min_Mean_BioMass_Rsc[0], CV_BioMass_Rsc[0]);
	}//end if

	sim.Set_Spc_Interaction(Cd, b_MassIntake_DDS, th_Spc_SpcRsc, q_Spc_SpcRsc, k_Hunt_Bound, delta_Hunt_Bound, exponent_Choice_Resource_BodyMass, mode_Forage_Physic_Spc);
	sim.Set_Movement(&mov_Deme[0], mov_Niche);
	sim.Set_Gen_Trait(loc_Trt, switch_Assortative_Mating, female_Choosiness_Scale, female_Choosiness_Intensity, metabolicRate_Biomass_Factor);
	sim.Set_ForagingTrait(growth_Target_Spc);

	if (num_RscSpc > 0)
	{
		//sets the efficacy of the use of prey affected by prey size (ESP)
		sim.Set_ESP_BodyMass(max_Mean_BioMass_Rsc[0], min_Mean_BioMass_Rsc[0], CV_BioMass_Rsc[0], bodyLength_Max, bodyLength_Min, exp_Length_To_Growth, cst_Length_To_Growth, upper_Bound_Min, lower_Bound_Max, lower_Bound_Min, exponent_UpperBound_Trait, exponent_LowerBound_Trait);
		sim.Set_Lotka_Volterra_Parameters(fileName_Output, b_NutritionalValue, interaction_Among_RscSpc[0], coreSpc_Neg_Effect_ScalingFactor, rsc_intr, intra_Specific_Interaction_LV, a_BiomassRatio_Interaction, b_BiomassRatio_Interaction, a_BiomassRatio_Interaction_Gain, alpha_BioMass_Abundance, c_BioMass_Abundance, stableStageDist, max_Mean_BioMass_Rsc[0], min_Mean_BioMass_Rsc[0], CV_BioMass_Rsc[0], th_Rsc_RscRsc, q_Rsc_RscRsc);
		sim.Set_File_Resource_Bodymass(fileName_Output);
		sim.Set_Resource_Consistancy(((int)p_Setting_Resource[9]));//sets the consistancy in resource
	}//end if

	sim.Set_Fertility(F, F_CV, propFemBirth);
	//sets parameters regarding plasticity
	sim.Set_Plasticity(p_DDG, cost_Reduction_EarlySexMat, smPlb);
	sim.Set_Plasticity_Update(&switch_Plasticity, ltg_VB, ce_VB, ci_VB, ltg_VB_Sx, cv_BodyLength_Growth);
	sim.Set_Data_Output_Buffer(num_Class_Trait);

	//sets customized Initial allele frequency
	sim.Set_Ini_Allele_Frequency(ini_Bi_Allelic_Frequency);
	///////////////////min_Mean_BioMass_Rsc

	delete[] ltg_VB;
	delete[] ce_VB;
	delete[] ci_VB;
	delete[] ltg_VB_Sx;
	delete[] cv_BodyLength_Growth;

	//deallocate memory for plasticity
	delete[] p_DDG;
	delete[] smPlb;

	//deallocates memory for writing data on file
	for (spc = 0; spc < s_Tag_Row; spc++){
		delete[] s_Tag[spc];
	}//end row loop

	delete[] s_Tag;

	delete[] fileName_Output;

	delete[] interaction_Among_RscSpc;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			delete[] intra_Specific_Interaction_LV[deme][niche];
		}//end niche loop

		delete[] intra_Specific_Interaction_LV[deme];
	}//end deme loop

	delete[] intra_Specific_Interaction_LV;

	for (deme = 0; deme < num_Deme; deme++){
		delete[] a_BiomassRatio_Interaction_Gain[deme];
	}//end deme loop

	delete[] a_BiomassRatio_Interaction_Gain;

	for (deme = 0; deme < num_Deme; deme++){
		delete[] b_BiomassRatio_Interaction[deme];
	}//end deme loop

	delete[] b_BiomassRatio_Interaction;

	for (deme = 0; deme < num_Deme; deme++){
		delete[] a_BiomassRatio_Interaction[deme];
	}//end deme loop

	delete[] a_BiomassRatio_Interaction;
	delete[] CV_BioMass_Rsc;
	delete[] max_Mean_BioMass_Rsc;
	delete[] min_Mean_BioMass_Rsc;

	for (deme = 0; deme < num_Deme; deme++){
		delete[] c_BioMass_Abundance[deme];
	}//end deme loop

	delete[] c_BioMass_Abundance;

	for (deme = 0; deme < num_Deme; deme++){
		delete[] alpha_BioMass_Abundance[deme];
	}//end deme loop

	delete[] alpha_BioMass_Abundance;

	for (deme = 0; deme < num_Deme; deme++){
		delete[] coreSpc_Neg_Effect_ScalingFactor[deme];
	}//end deme loop

	delete[] coreSpc_Neg_Effect_ScalingFactor;
	delete[] metabolicRate_Biomass_Factor;

	delete[] female_Choosiness_Scale;
	delete[] female_Choosiness_Intensity;

	delete[] setArr;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			delete[] stableDDS[deme][niche];
		}//end niche loop

		delete[] stableDDS[deme];
	}//end deme loop

	delete[] stableDDS;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] stableStageDist[deme][niche][spc];
			}//end species loop

			delete[] stableStageDist[deme][niche];
		}//end niche loop

		delete[] stableStageDist[deme];
	}//end deme loop

	delete[] stableStageDist;

	delete[] cst_Length_To_Growth;
	delete[] exp_Length_To_Growth;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] growth_Target_Spc[spc];
	}//end species loop

	delete[] growth_Target_Spc;

	delete[] exponent_LowerBound_Trait;
	delete[] exponent_UpperBound_Trait;

	delete[] lower_Bound_Max;
	delete[] lower_Bound_Min;
	delete[] upper_Bound_Min;

	delete[] mode_Forage_Physic_Spc;
	delete[] k_Hunt_Bound;
	delete[] delta_Hunt_Bound;
	delete[] exponent_Choice_Resource_BodyMass;

	delete[] b_NutritionalValue;
	delete[] cost_Reduction_EarlySexMat;
	delete[] bodyLength_Min;
	delete[] bodyLength_Max;

	delete[] num_Locus_Rsc;
	delete[] num_Position_Rsc;
	delete[] mut_Rat_Rsc;
	delete[] num_PhenoTyp_Rsc;

	//Functional Response: Handling Time
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				delete[] th_Rsc_RscRsc[deme][niche][spc];
			}//end species loop

			delete[] th_Rsc_RscRsc[deme][niche];
		}//end niche loop

		delete[] th_Rsc_RscRsc[deme];
	}//end deme loop

	delete[] th_Rsc_RscRsc;

	//Functional Response: Response Mode
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				delete[] q_Rsc_RscRsc[deme][niche][spc];
			}//end species loop

			delete[] q_Rsc_RscRsc[deme][niche];
		}//end niche loop

		delete[] q_Rsc_RscRsc[deme];
	}//end deme loop

	delete[] q_Rsc_RscRsc;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] q_Spc_SpcRsc[deme][niche][spc];
			}//end species loop

			delete[] q_Spc_SpcRsc[deme][niche];
		}//end niche loop

		delete[] q_Spc_SpcRsc[deme];
	}//end deme loop

	delete[] q_Spc_SpcRsc;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] th_Spc_SpcRsc[deme][niche][spc];
			}//end species loop

			delete[] th_Spc_SpcRsc[deme][niche];
		}//end niche loop

		delete[] th_Spc_SpcRsc[deme];
	}//end deme loop

	delete[] th_Spc_SpcRsc;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] loc_Trt[spc];
	}//end species loop

	delete[] loc_Trt;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Class_Trait[spc];
	}//end species loop

	delete[] num_Class_Trait;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] initial_Num_Spc[deme][niche][spc][sex];
				}//end sex loop

				delete[] initial_Num_Spc[deme][niche][spc];
			}//end species loop

			delete[] initial_Num_Spc[deme][niche];
		}//end niche loop

		delete[] initial_Num_Spc[deme];
	}//end deme loop

	delete[] initial_Num_Spc;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] metabolicRate_CoreSpc[deme][niche][spc];
			}//end species loop

			delete[] metabolicRate_CoreSpc[deme][niche];
		}//end niche loop

		delete[] metabolicRate_CoreSpc[deme];
	}//end deme loop

	delete[] metabolicRate_CoreSpc;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			delete[] b_MassIntake_DDS[spc][sex];
		}//end niche loop

		delete[] b_MassIntake_DDS[spc];
	}//end deme loop

	delete[] b_MassIntake_DDS;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] Cd[deme][niche][spc][sex];
				}//end sex loop

				delete[] Cd[deme][niche][spc];
			}//end species loop

			delete[] Cd[deme][niche];
		}//end niche loop

		delete[] Cd[deme];
	}//end deme loop

	delete[] Cd;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			delete[] rsc_intr[deme][niche];
		}//end niche loop

		delete[] rsc_intr[deme];
	}//end deme loop

	delete[] rsc_intr;

	delete[] mov_Deme;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] P[deme][niche][spc][sex];
				}//end sex loop
				delete[] P[deme][niche][spc];
			}//end species loop
			delete[] P[deme][niche];
		}//end niche loop
		delete[] P[deme];
	}//end deme loop

	delete[] P;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] G[deme][niche][spc][sex];
				}//end sex loop
				delete[] G[deme][niche][spc];
			}//end species loop
			delete[] G[deme][niche];
		}//end niche loop
		delete[] G[deme];
	}//end deme loop

	delete[] G;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] F[deme][niche][spc];
			}//end species loop

			delete[] F[deme][niche];
		}//end niche loop
		delete[] F[deme];
	}//end deme loop

	delete[] F;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] F_CV[deme][niche][spc];
			}//end species loop

			delete[] F_CV[deme][niche];
		}//end niche loop
		delete[] F_CV[deme];
	}//end deme loop

	delete[] F_CV;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] mut_Rat[spc];
	}//end species loop

	delete[] mut_Rat;

	for (niche = 0; niche < num_Niche; niche++){
		delete[] mov_Niche[niche];
	}//end species loop

	delete[] mov_Niche;
	delete[] num_Locus;
	delete[] num_Position;
	delete[] num_Trait;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Stg_Immat_Sex[spc];
	}//end species loop

	delete[] num_Stg_Immat_Sex;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Stg[spc];
	}//end species loop

	delete[] num_Stg;

	delete[] propFemBirth;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] mode[spc];
	}//end species loop

	delete[] mode;

	if (p_Program_Mode == 0)
	{//runs a simulation
		sim.Proc_Simul(&p_Simul[0], &p_Simul[1], &p_Simul[2]);
	}
	else if (p_Program_Mode == 1)
	{//runs statistical analyses
		sim.Proc_Stat(&p_Simul[0], &p_Simul[1], &p_Simul[2]);
	}
	else
	{

	}//end else
}

void Simul_Setting_Mating(double *p_Mating, int p_Female_Choosiness_Intensity, int p_Female_Choosiness_Scale, int ini_Freq_Allele_Assortative_Mating)
{
	//choosiness
	switch (p_Female_Choosiness_Intensity)
	{
	case 0:
		p_Mating[0] = 0.0;
		break;
	case 1:
		p_Mating[0] = 0.25;
		break;
	case 2:
		p_Mating[0] = 0.5;
		break;
	case 3:
		p_Mating[0] = 0.75;
		break;
	case 4:
		p_Mating[0] = 1.0;
		break;
	default:
		p_Mating[0] = 0.0;
		break;
	}//end switch

	//choosiness
	switch (p_Female_Choosiness_Scale)
	{
	case 0:
		p_Mating[1] = 0.0;
		break;
	case 1:
		p_Mating[1] = 0.25;
		break;
	case 2:
		p_Mating[1] = 0.5;
		break;
	case 3:
		p_Mating[1] = 1.0;
		break;
	case 4:
		p_Mating[1] = 2.0;
		break;
	case 5:
		p_Mating[1] = 4.0;
		break;
	case 6:
		p_Mating[1] = 8.0;
		break;
	default:
		p_Mating[1] = 1.0;
		break;
	}//end switch

	//ini_Freq_Allele_Assortative_Mating
	switch (ini_Freq_Allele_Assortative_Mating)
	{
	case 0:
		p_Mating[2] = 0.0;
		break;
	case 1:
		p_Mating[2] = 0.01;
		break;
	case 2:
		p_Mating[2] = 0.5;
		break;
	case 3:
		p_Mating[2] = 0.99;
		break;
	case 4:
		p_Mating[2] = 1.0;
		break;
	default:
		p_Mating[2] = 0.0;
		break;
	}//end switch
}

void Simul_Setting_Foraging_Trait(double *p_foraging_Trait, int *p_Lower_bound_Scaling_Factor, int *p_Exponent_Choice_Resource_BodyMass)
{
	switch ((*p_Lower_bound_Scaling_Factor))
	{
	case 0:
		p_foraging_Trait[5] = 0.0;
		break;
	case 1:
		p_foraging_Trait[5] = 0.125;
		break;
	case 2:
		p_foraging_Trait[5] = 0.25;
		break;
	case 3:
		p_foraging_Trait[5] = 0.5;
		break;
	case 4:
		p_foraging_Trait[5] = 0.75;
		break;
	case 5:
		p_foraging_Trait[5] = 1.0;
		break;
	default:
		p_foraging_Trait[5] = 0.125;
		break;
	}//end switch

	switch ((*p_Exponent_Choice_Resource_BodyMass))
	{
	case 0:
		p_foraging_Trait[6] = 1.0;
		break;
	case 1:
		p_foraging_Trait[6] = 1.1;
		break;
	case 2:
		p_foraging_Trait[6] = 1.2;
		break;
	case 3:
		p_foraging_Trait[6] = 1.3;
		break;
	case 4:
		p_foraging_Trait[6] = 1.4;
		break;
	case 5:
		p_foraging_Trait[6] = 1.5;
		break;
	default:
		p_foraging_Trait[6] = 1.0;
		break;
	}//end switch
}

void Simul_Setting_Basic(int *p_Num_Loci_Core_Spc, int *p_Num_RscSpc, double *p_GeneralityPenalty, double *p_Cost_EarSexMat_BodyLength, double *p_SteepnessAttackRate, double *p_CD_DDS, double *p_Handling_Time_Spc, double *p_Ini_BiAllelic_Freq, int *c_Num_Loci_Core_Spc, int *c_Num_RscSpc, int*c_GeneralityPenalty, int *cost_EarSexMat_BodyLength, int *c_SteepnessAttackRate, int *c_CD_DDS, int *c_Handling_Time_Spc, int *c_Ini_BiAllelic_Freq)
{
	switch ((*c_Num_Loci_Core_Spc))
	{
	case 0:
		(*p_Num_Loci_Core_Spc) = 2;
		break;
	case 1:
		(*p_Num_Loci_Core_Spc) = 4;
		break;
	case 2:
		(*p_Num_Loci_Core_Spc) = 6;
		break;
	case 3:
		(*p_Num_Loci_Core_Spc) = 8;
		break;
	case 4:
		(*p_Num_Loci_Core_Spc) = 10;
		break;
	case 5:
		(*p_Num_Loci_Core_Spc) = 12;
		break;
	case 6:
		(*p_Num_Loci_Core_Spc) = 14;
		break;
	case 7:
		(*p_Num_Loci_Core_Spc) = 16;
		break;
	default:
		break;
	}//end switch

	(*p_Num_RscSpc) = (*c_Num_RscSpc) + 2;

	const double GeneralistPenalty = 1.0;//1.0

	switch ((*c_GeneralityPenalty))
	{
	case 0:
		(*p_GeneralityPenalty) = 0.0 * GeneralistPenalty;
		break;
	case 1:
		(*p_GeneralityPenalty) = 0.125 * GeneralistPenalty;
		break;
	case 2:
		(*p_GeneralityPenalty) = 0.25 * GeneralistPenalty;
		break;
	case 3:
		(*p_GeneralityPenalty) = 0.375 * GeneralistPenalty;
		break;
	case 4:
		(*p_GeneralityPenalty) = 0.5 * GeneralistPenalty;
		break;
	case 5:
		(*p_GeneralityPenalty) = 0.625 * GeneralistPenalty;
		break;
	case 6:
		(*p_GeneralityPenalty) = 0.75 * GeneralistPenalty;
		break;
	case 7:
		(*p_GeneralityPenalty) = 0.875 * GeneralistPenalty;
		break;
	case 8:
		(*p_GeneralityPenalty) = 1.0 * GeneralistPenalty;
		break;
	case 9:
		(*p_GeneralityPenalty) = 2.0 * GeneralistPenalty;
		break;
	default:
		(*p_GeneralityPenalty) = 0.0 * GeneralistPenalty;
		break;
	}//end switch

	double default_SteepnessAttackRate = 10.0;

	switch ((*c_SteepnessAttackRate))
	{
	case 0:
		(*p_SteepnessAttackRate) = default_SteepnessAttackRate * 0.25;
		break;
	case 1:
		(*p_SteepnessAttackRate) = default_SteepnessAttackRate * 0.5;
		break;
	case 2:
		(*p_SteepnessAttackRate) = default_SteepnessAttackRate;
		break;
	case 3:
		(*p_SteepnessAttackRate) = default_SteepnessAttackRate * 2.0;
		break;
	case 4:
		(*p_SteepnessAttackRate) = default_SteepnessAttackRate * 4.0;
		break;
	default:
		break;
	}//end switch

	const double CD_DDS = 10.0;//Default 10.0

	switch ((*c_CD_DDS))
	{
	case 0:
		(*p_CD_DDS) = CD_DDS * 0.5;
		break;
	case 1:
		(*p_CD_DDS) = CD_DDS * 0.75;
		break;
	case 2:
		(*p_CD_DDS) = CD_DDS;
		break;
	case 3:
		(*p_CD_DDS) = CD_DDS * 1.25;
		break;
	case 4:
		(*p_CD_DDS) = CD_DDS * 1.5;
		break;
	default:
		//cout << "Line 2222: CD setting: " << (*c_CD_DDS) << endl;
		(*p_CD_DDS) = CD_DDS;
		break;
	}//end switch

	switch ((*cost_EarSexMat_BodyLength))
	{
	case 0:
		(*p_Cost_EarSexMat_BodyLength) = 1.0 - 0.0;
		break;
	case 1:
		(*p_Cost_EarSexMat_BodyLength) = 1.0 - 0.125;
		break;
	case 2:
		(*p_Cost_EarSexMat_BodyLength) = 1.0 - 0.25;
		break;
	case 3:
		(*p_Cost_EarSexMat_BodyLength) = 1.0 - 0.5;
		break;
	case 4:
		(*p_Cost_EarSexMat_BodyLength) = 1.0 - 0.75;
		break;
	case 5:
		(*p_Cost_EarSexMat_BodyLength) = 1.0 - 0.875;
		break;
	case 101://for landlocked Alewife
		(*p_Cost_EarSexMat_BodyLength) = 0.508;
		break;
	default:
		(*p_Cost_EarSexMat_BodyLength) = 0.508;
		break;
	}//end switch

	const double handling_Time_Spc = 1.0;//0.000001

	switch ((*c_Handling_Time_Spc))
	{
	case 0:
		(*p_Handling_Time_Spc) = handling_Time_Spc * 0.0;
		break;
	case 1:
		(*p_Handling_Time_Spc) = handling_Time_Spc * 0.05;
		break;
	case 2:
		(*p_Handling_Time_Spc) = handling_Time_Spc * 0.1;
		break;
	case 3:
		(*p_Handling_Time_Spc) = handling_Time_Spc * 0.25;
		break;
	case 4:
		(*p_Handling_Time_Spc) = handling_Time_Spc * 0.5;
		break;
	case 5:
		(*p_Handling_Time_Spc) = handling_Time_Spc;
		break;
	default:
		(*p_Handling_Time_Spc) = handling_Time_Spc * 0.1;
		break;
	}//end switch

	//_Ini_BiAllelic_Freq
	switch ((*c_Ini_BiAllelic_Freq))
	{
	case 0:
		(*p_Ini_BiAllelic_Freq) = 1.0;
		break;
	case 1:
		(*p_Ini_BiAllelic_Freq) = 0.9999;
		break;
	case 2:
		(*p_Ini_BiAllelic_Freq) = 0.999;
		break;
	case 3:
		(*p_Ini_BiAllelic_Freq) = 0.99;
		break;
	case 4:
		(*p_Ini_BiAllelic_Freq) = 0.5;
		break;
	case 5:
		(*p_Ini_BiAllelic_Freq) = 0.01;
		break;
	case 6:
		(*p_Ini_BiAllelic_Freq) = 0.001;
		break;
	case 7:
		(*p_Ini_BiAllelic_Freq) = 0.0001;
		break;
	case 8:
		(*p_Ini_BiAllelic_Freq) = 0.0;
		break;
	default:
		(*p_Ini_BiAllelic_Freq) = 0.99;
		break;
	}//end switch
}

void SimulSetting_Resource(double *p_Setting, int *c_Intrinsic_r_Resource, int *c_IntraSpecific_LV, int *c_Scale_Neg_LV, int *c_Exp_Neg_LV, int *c_Scale_Pos_LV, int *c_Rsc_Abundance_ScalingFactor, int *c_Alpha_Abundance_Biomass, int *c_Neg_Eff_Spc_Rsc_ScalingFactor, int *c_Handling_Time_Rsc, int *c_Ecosystem_Scaling_Factor, int *p_Consistancy_Resource)
{
	//sets the effect for intra-specific competition
	double scalingFactor_Overall = 200.0;//overall scaling factor Default: 200.0

	switch ((*c_Ecosystem_Scaling_Factor))
	{
	case 0:
		scalingFactor_Overall = scalingFactor_Overall * 0.125;
		break;
	case 1:
		scalingFactor_Overall = scalingFactor_Overall * 0.25;
		break;
	case 2:
		scalingFactor_Overall = scalingFactor_Overall  * 0.5;
		break;
	case 3:
		scalingFactor_Overall = scalingFactor_Overall * 0.75;
		break;
	case 4:
		scalingFactor_Overall = scalingFactor_Overall;
		break;
	case 5:
		scalingFactor_Overall = scalingFactor_Overall * 1.25;
		break;
	case 6:
		scalingFactor_Overall = scalingFactor_Overall * 1.5;
		break;
	case 7:
		scalingFactor_Overall = scalingFactor_Overall * 1.75;
		break;
	case 8:
		scalingFactor_Overall = scalingFactor_Overall * 2.0;
		break;
	default:
		break;
	}//end switch

	const double Intrinsic_Increase_Resource_Default = 1.0;//Default: 0.5

	switch ((*c_Intrinsic_r_Resource))
	{
	case 0:
		p_Setting[0] = Intrinsic_Increase_Resource_Default * 0.25;
		break;
	case 1:
		p_Setting[0] = Intrinsic_Increase_Resource_Default * 0.5;
		break;
	case 2:
		p_Setting[0] = Intrinsic_Increase_Resource_Default;
		break;
	case 3:
		p_Setting[0] = Intrinsic_Increase_Resource_Default * 2.0;
		break;
	case 4:
		p_Setting[0] = Intrinsic_Increase_Resource_Default * 4.0;
		break;
	default:
		p_Setting[0] = Intrinsic_Increase_Resource_Default;
		break;
	}//end switch

	//sets the effect for intra-specific competition
	//const double IntraSpecific_LV_Default = 0.5;
	const double IntraSpecific_LV_Default = 1.0;

	switch ((*c_IntraSpecific_LV))
	{
	case 0:
		p_Setting[1] = IntraSpecific_LV_Default * 0.25;
		break;
	case 1:
		p_Setting[1] = IntraSpecific_LV_Default * 0.5;
		break;
	case 2:
		p_Setting[1] = IntraSpecific_LV_Default;
		break;
	case 3:
		p_Setting[1] = IntraSpecific_LV_Default * 2.0;
		break;
	case 4:
		p_Setting[1] = IntraSpecific_LV_Default * 4.0;
		break;
	default:
		p_Setting[1] = IntraSpecific_LV_Default;
		break;
	}//end switch

	const double scale_Neg_LV = 0.0005 * (1.0 / scalingFactor_Overall);//Default: 0.0005

	switch ((*c_Scale_Neg_LV))
	{
	case 0:
		p_Setting[2] = scale_Neg_LV * 0.25;
		break;
	case 1:
		p_Setting[2] = scale_Neg_LV * 0.5;
		break;
	case 2:
		p_Setting[2] = scale_Neg_LV;
		break;
	case 3:
		p_Setting[2] = scale_Neg_LV * 2.0;
		break;
	case 4:
		p_Setting[2] = scale_Neg_LV * 4.0;
		break;
	default:
		p_Setting[2] = scale_Neg_LV;
		break;
	}//end switch

	switch ((*c_Exp_Neg_LV))//Default: 0.25
	{
	case 0:
		p_Setting[4] = 0.15;
		break;
	case 1:
		p_Setting[4] = 0.2;
		break;
	case 2:
		p_Setting[4] = 0.25;
		break;
	case 3:
		p_Setting[4] = 0.3;
		break;
	case 4:
		p_Setting[4] = 0.35;
		break;
	case 5:
		p_Setting[4] = 0.4;
		break;
	case 6:
		p_Setting[4] = 0.45;
		break;
	case 7:
		p_Setting[4] = 0.5;
		break;
	case 8:
		p_Setting[4] = 0.55;
		break;
	case 9:
		p_Setting[4] = 0.6;
		break;
	case 10:
		p_Setting[4] = 0.65;
		break;
	case 11:
		p_Setting[4] = 0.7;
		break;
	case 12:
		p_Setting[4] = 0.75;
		break;
	default:
		p_Setting[4] = 0.25;
		break;
	}//end switch

	const double scale_Pos_LV = 0.01;//Default: 0.01

	switch ((*c_Scale_Pos_LV))
	{
	case 0:
		p_Setting[3] = scale_Pos_LV * 0.25;
		break;
	case 1:
		p_Setting[3] = scale_Pos_LV * 0.5;
		break;
	case 2:
		p_Setting[3] = scale_Pos_LV;
		break;
	case 3:
		p_Setting[3] = scale_Pos_LV * 2.0;
		break;
	case 4:
		p_Setting[3] = scale_Pos_LV * 4.0;
		break;
	default:
		p_Setting[3] = scale_Pos_LV;
		break;
	}//end switch

	const double handling_Time_Rsc = 1.0;//Default0.000001

	switch ((*c_Handling_Time_Rsc))
	{
	case 0:
		p_Setting[5] = handling_Time_Rsc * 0.0;
		break;
	case 1:
		p_Setting[5] = handling_Time_Rsc * 0.05;
		break;
	case 2:
		p_Setting[5] = handling_Time_Rsc * 0.1;
		break;
	case 3:
		p_Setting[5] = handling_Time_Rsc * 0.25;
		break;
	case 4:
		p_Setting[5] = handling_Time_Rsc * 0.5;
		break;
	case 5:
		p_Setting[5] = handling_Time_Rsc;
		break;
	default:
		p_Setting[5] = handling_Time_Rsc * 0.1;
		break;
	}//end switch

	double slope_Abundance_Biomass = 25000.0 * scalingFactor_Overall;//default: 50000.0 for 0.26kg of the maximum bodymass

	switch ((*c_Rsc_Abundance_ScalingFactor))
	{//5.0
	case 0:
		p_Setting[6] = slope_Abundance_Biomass * 0.5;
		break;
	case 1:
		p_Setting[6] = slope_Abundance_Biomass * 0.75;
		break;
	case 2:
		p_Setting[6] = slope_Abundance_Biomass;
		break;
	case 3:
		p_Setting[6] = slope_Abundance_Biomass * 1.25;
		break;
	case 4:
		p_Setting[6] = slope_Abundance_Biomass * 1.5;
		break;
	case 5:
		p_Setting[6] = slope_Abundance_Biomass * 2.0;
		break;
	default:
		p_Setting[6] = slope_Abundance_Biomass;
		break;
	}//end switch

	switch ((*c_Alpha_Abundance_Biomass))
	{//1.18
	case 0:
		p_Setting[7] = 0.1;
		break;
	case 1:
		p_Setting[7] = 0.2;
		break;
	case 2:
		p_Setting[7] = 0.25;
		break;
	case 3:
		p_Setting[7] = 0.5;
		break;
	case 4:
		p_Setting[7] = 0.75;
		break;
	case 5:
		p_Setting[7] = 1.0;
		break;
	case 6:
		p_Setting[7] = 1.25;
		break;
	default:
		p_Setting[7] = 0.75;
		break;
	}//end switch

	double Neg_Eff_Spc_Rsc_ScalingFactor = 1.0;// 205.8145;//default: 1.0

	switch ((*c_Neg_Eff_Spc_Rsc_ScalingFactor))
	{//100.0
	case 0:
		p_Setting[8] = Neg_Eff_Spc_Rsc_ScalingFactor * 0.5;
		break;
	case 1:
		p_Setting[8] = Neg_Eff_Spc_Rsc_ScalingFactor * 0.75;
		break;
	case 2:
		p_Setting[8] = Neg_Eff_Spc_Rsc_ScalingFactor;
		break;
	case 3:
		p_Setting[8] = Neg_Eff_Spc_Rsc_ScalingFactor * 1.25;
		break;
	case 4:
		p_Setting[8] = Neg_Eff_Spc_Rsc_ScalingFactor * 1.5;
		break;
	default:
		p_Setting[8] = Neg_Eff_Spc_Rsc_ScalingFactor;
		break;
	}//end switch

	//resource consistancy setting
	p_Setting[9] = ((double)(*p_Consistancy_Resource));
}

void Matrix_Population_Model_Routine(void)
{//Matrix population model routine
	int deme, niche, spc, sex, stg, num_Cat, i;

	//File name creation
	File_RW fileRW;
	string tmp_Str_0 = "_MatPop";
	string tmp_Str;

	tmp_Str = "MatPop.csv";

	int size_String = (int)tmp_Str.length();
	char *fileName_Output = new char[(size_String + 1)];//one more cell for ending character array	
	//the number of cells in char* should be (the length of string + 1)
	fileRW.Copy_String_To_Char_Arr(tmp_Str, fileName_Output);//copy string and paste on char*
	fileRW.CSV_Create(fileName_Output);//creates a csv file

	string **s_Tag;
	int s_Tag_Row = 1;
	int s_Tag_Col = 7;
	s_Tag = new string*[s_Tag_Row];

	for (spc = 0; spc < s_Tag_Row; spc++){
		s_Tag[spc] = new string[s_Tag_Col];

		for (i = 0; i < s_Tag_Col; i++){
			s_Tag[spc][i] = " ";
		}//end i loop
	}//end row loop

	//Simulation Settings
	int program_Mode = 1;//0: Matrix Population Model, 1: Simulation + Matrix Population Model
	int num_Spc = 1, num_Deme = 1, num_Niche = 1, num_Sex = 2, num_RscSpc = 0, max_Num_Stg = 2, max_Num_Allele = 2;
	int max_Num_PhenTyp_Rsc = 21;//the maximum number of phenotypes in resource species
	int num_Mode_Category = 10;

	//Life stages
	int **num_Stg;
	num_Stg = new int*[num_Spc];
	max_Num_Stg = 0;

	for (i = 0; i < num_Spc; i++){
		num_Stg[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			if (sex == 0)//male
			{
				num_Stg[i][sex] = 10;
			}
			else//female
			{
				num_Stg[i][sex] = 10;
			}//end else

			if (num_Stg[i][sex] > max_Num_Stg){
				max_Num_Stg = num_Stg[i][sex];
			}//end if
		}//end sex loop
	}//end i loop

	s_Tag[0][0] = "(Core Species) The Number of Life Stages";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Sex";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, num_Stg, &num_Spc, &num_Sex);

	//The number of immature life stages for sexual maturation
	int **num_Stg_Immat_Sex;
	num_Stg_Immat_Sex = new int*[num_Spc];

	for (i = 0; i < num_Spc; i++){
		num_Stg_Immat_Sex[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			if (sex == 0)//male
			{
				num_Stg_Immat_Sex[i][sex] = 0;
			}
			else//female
			{
				num_Stg_Immat_Sex[i][sex] = 0;
			}//end else
		}//end sex loop
	}//end i loop

	s_Tag[0][0] = "(Core Species) The Number of Immature Life Stages having chance of early reproduction";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 2;
	s_Tag[0][0] = "Species";
	s_Tag[0][1] = "Sex";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Integer_2DD(fileName_Output, num_Stg_Immat_Sex, &num_Spc, &num_Sex);

	//The proportion of females at birth
	double *propFemBirth;
	propFemBirth = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		propFemBirth[spc] = 0.5;//the proportion of females at birth
	}//end species loop	

	s_Tag[0][0] = "(Core Species) The Proportion of Females at Birth";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 1;
	s_Tag[0][0] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_1DD(fileName_Output, propFemBirth, &num_Spc);

	//Probability that an agent survives to the same life stage
	double***** P;
	P = new double****[num_Deme];
	double adult_Male_SurvivalRate = 0.928;//0.5
	double adult_Female_SurvivalRate = 0.928;//0.5
	double juvenile_Survival_Rate = 0.928;
	double incremental_Improvement_Survival = 0.0;
	double dist_Juvenile_Male_P_G = 0.690733;//Distribution to survival in survival and growth distribution
	double dist_Juvenile_Female_P_G = 0.690733;//Distribution to survival in survival and growth distribution
	double infant_Surv = 0.0;
	double infant_Growth = 0.902;

	for (deme = 0; deme < num_Deme; deme++){
		P[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			P[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				P[deme][niche][spc] = new double*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					P[deme][niche][spc][sex] = new double[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						P[deme][niche][spc][sex][stg] = 0.0;
					}//end life stage loop

					for (stg = 0; stg < num_Stg[spc][sex]; stg++){
						//P (survival rate) + G (survival + growth rate)
						if (stg == 9)
						{
							P[deme][niche][spc][sex][stg] = 0.991;
						}
						else
						{
							P[deme][niche][spc][sex][stg] = 0.0;
						}//end else						
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) P";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	s_Tag[0][4] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_5DD(fileName_Output, P, &num_Deme, &num_Niche, &num_Spc, &num_Sex, &max_Num_Stg);

	//Probability that an agent survives and develops to the next life stage
	double***** G;
	G = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		G[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			G[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				G[deme][niche][spc] = new double*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					G[deme][niche][spc][sex] = new double[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						G[deme][niche][spc][sex][stg] = 0.0;
					}//end life stage loop

					for (stg = 0; stg < max_Num_Stg; stg++){
						//P (survival rate) + G (survival + growth rate)
						if (stg == 0)
						{
							G[deme][niche][spc][sex][stg] = 0.634;
						}
						else if (stg < 5)
						{
							G[deme][niche][spc][sex][stg] = 0.802;
						}
						else if (stg < 9)
						{
							G[deme][niche][spc][sex][stg] = 0.961;
						}
						else
						{
							G[deme][niche][spc][sex][stg] = 0.0;
						}//end else
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) G";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 5;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Sex";
	s_Tag[0][4] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_5DD(fileName_Output, G, &num_Deme, &num_Niche, &num_Spc, &num_Sex, &max_Num_Stg);

	//Fertility (only counts female offspring)
	double**** F;//Mean in fertility following normal distribution
	F = new double***[num_Deme];
	sex = 1;//for females

	for (deme = 0; deme < num_Deme; deme++){
		F[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			F[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				F[deme][niche][spc] = new double[max_Num_Stg];

				for (stg = 0; stg < max_Num_Stg; stg++){
					F[deme][niche][spc][stg] = 0.0;
				}//end life stage loop

				for (stg = 0; stg < num_Stg[spc][sex]; stg++){
					//P (survival rate) + G (survival + growth rate)
					if (stg == 9)
					{
						F[deme][niche][spc][stg] = 0.233;
					}
					else
					{
						F[deme][niche][spc][stg] = 0.0;
					}//end else
				}//end life stage loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	s_Tag[0][0] = "(Core Species) F";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, F, &num_Deme, &num_Niche, &num_Spc, &max_Num_Stg);

	//stable stage distribution
	double**** stableStageDist;//dim[deme, niche, species, life stage]; stable stage distribution
	stableStageDist = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		stableStageDist[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			stableStageDist[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				stableStageDist[deme][niche][spc] = new double[max_Num_Stg];

				for (stg = 0; stg < max_Num_Stg; stg++){
					stableStageDist[deme][niche][spc][stg] = 0.0;
				}//end row loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//Stable Density-Dependent Survival Rate
	double*** stableDDS;//dim[deme, niche, species]; stable density-dependent-survival rate
	stableDDS = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		stableDDS[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			stableDDS[deme][niche] = new double[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				stableDDS[deme][niche][spc] = 0.0;
			}//end species loop
		}//end niche loop
	}//end deme loop

	///////////////////////////////////////////////
	//Matrix Population Model//
	///////////////////////////////////////////////
	if (num_Spc > 0)
	{
		Matrix_Population_Model MatPopMol(num_Deme, num_Niche, num_Spc, num_Sex, num_Stg, tmp_Str_0);
		MatPopMol.Set_Vital_Rat(P, G, F);//sets transition matrix
		MatPopMol.Run_Analysis();//runs analyses
		MatPopMol.Get_Stable_Stage_Dist(stableStageDist);
		MatPopMol.Get_Stable_DDS(stableDDS);
	}//end if
	////////////////////////////

	//Stable-Stage Distribution
	s_Tag[0][0] = "(Core Species) Stable-Stage Distribution";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 4;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	s_Tag[0][3] = "Life Stage";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_4DD(fileName_Output, stableStageDist, &num_Deme, &num_Niche, &num_Spc, &max_Num_Stg);

	//Stable Density-Dependent Survival Rate
	s_Tag[0][0] = "(Core Species) Stable Density-Dependent Survival Rate";
	s_Tag_Col = 1;
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	num_Cat = 3;
	s_Tag[0][0] = "Deme";
	s_Tag[0][1] = "Niche";
	s_Tag[0][2] = "Species";
	fileRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	fileRW.CSV_Write_Add_Double_3DD(fileName_Output, stableDDS, &num_Deme, &num_Niche, &num_Spc);

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			delete[] stableDDS[deme][niche];
		}//end niche loop

		delete[] stableDDS[deme];
	}//end deme loop

	delete[] stableDDS;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] stableStageDist[deme][niche][spc];
			}//end species loop

			delete[] stableStageDist[deme][niche];
		}//end niche loop

		delete[] stableStageDist[deme];
	}//end deme loop

	delete[] stableStageDist;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] P[deme][niche][spc][sex];
				}//end sex loop
				delete[] P[deme][niche][spc];
			}//end species loop
			delete[] P[deme][niche];
		}//end niche loop
		delete[] P[deme];
	}//end deme loop

	delete[] P;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] G[deme][niche][spc][sex];
				}//end sex loop
				delete[] G[deme][niche][spc];
			}//end species loop
			delete[] G[deme][niche];
		}//end niche loop
		delete[] G[deme];
	}//end deme loop

	delete[] G;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] F[deme][niche][spc];
			}//end species loop

			delete[] F[deme][niche];
		}//end niche loop
		delete[] F[deme];
	}//end deme loop

	delete[] F;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Stg_Immat_Sex[spc];
	}//end species loop

	delete[] num_Stg_Immat_Sex;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Stg[spc];
	}//end species loop

	delete[] num_Stg;

	delete[] propFemBirth;

	for (spc = 0; spc < s_Tag_Row; spc++){
		delete[] s_Tag[spc];
	}//end row loop

	delete[] s_Tag;

	delete[] fileName_Output;
}

void Simul_Test_Debug(void)
{

}

void Test_Sort(void)
{
	//data sorting example
	Data_Manipulation dm;
	//dm.Example_Sort_Double_One_D();
	dm.Example_Sort_Double_Two_D();
}

void Test_File_Read(void)
{
	File_RW fileRW;
	vector<vector<string>> tmpStr;
	vector<vector<double>> tmpDbl;
	int numRows, numColumns;
	string fileName_Str = "resourceMass.csv";

	fileRW.CSV_Read_String_2D_String(fileName_Str, tmpStr, &numRows, &numColumns);

	int numHeaders_Row = 0;//the number of header rows
	int numHeaders_Col = 0;//the number of header columns

	//converts 2-D string to 2-D double
	fileRW.Convert_String_2D_To_Double_2D(tmpStr, tmpDbl, numHeaders_Row, numHeaders_Col);

	int i, j, end_Row, end_Col;
	end_Row = (int)tmpStr.size();
	end_Col = (int)tmpStr[0].size();

	for (i = 0; i < end_Row; i++)
	{
		for (j = 0; j < end_Col; j++)
		{
			cout << "row: " << i << " col: " << j << " : " << tmpDbl[i][j] << endl;
		}//end column loop
	}//end row loop

	cout << "The number of rows is " << numRows << " The number of columns is " << numColumns << endl;

	char tt;
	cout << "Type a character: " << endl;
	cin >> tt;
}

void Batch_Analysis_Time_Label(string p_Header_FileName_input, string name_Trait, string name_Col, int p_Col_Num, int col_Num_Time_Label, int p_Year_Begin, int p_Year_Interval, int p_num_Year_Segments, int num_Replicates)
{
	//Batch analysis for data files with time labels
	//p_Header_FileName: header of a file name
	//i.e. p_Header_FileName = "mv_tr"
	//p_Col_Num: column number to process in file
	//i.e. //the selected column in data 1: mean, 2: variance, 3: sd, 4: mean, 5: variance, 6: sd
	//name_Trait: trait name
	//i.e. name_Trait = "U_"
	//col_Num_Time_Label: time lable column number
	//i.e. col_Num_Time_Label = 0;//(the first column)
	//name_Col = "yr_Mu_";

	//input file information
	string file_Tag = p_Header_FileName_input;//input file tag
	int num_Rows_To_Skip = 1;//the number of rows to skip in a file
	int num_Columns_To_Skip = 1;//the number of columns to skip in a file
	//int num_Replicates;//The number of replicates

	//Output file setting
	string id_Tag_Main = "mr_";

	//for data type conversion
	ostringstream convert;
	//initialize data transition buffer
	convert.str("");

	convert << p_Year_Interval;//integer -> string
	//output file name tag
	id_Tag_Main = id_Tag_Main + convert.str();
	//initialize data transition buffer
	convert.str("");

	//output file name tag
	id_Tag_Main = id_Tag_Main + name_Col;

	int begin_Year = p_Year_Begin;//default: 1
	int interval_Year = p_Year_Interval;//Long-term: 5000, short-term: 100
	int num_Segments_Year = p_num_Year_Segments;
	int end_Year = begin_Year + (interval_Year * num_Segments_Year) + 1;
	int start_Row = begin_Year;//the row for starting to count
	int end_Row = end_Year;//the row for finishing to count

	//simulation block
	int num_Files = 0;//the number of files to process
	int mode_Batch_Analysis = 0;//batch analysis mode
	int num_Simul_Label = 10;
	int *simul_Label = new int[num_Simul_Label];//sets setting for identiing simulations

	for (int i = 0; i < num_Simul_Label; i++)
	{
		simul_Label[i] = 0;//initialization
	}//end mode loop

	//2015 03 26_Random
	int counter = 0;
	int replicate = 0;

	string tmp_Str_0;
	File_RW fileRW;

	int num_Categories = 7;//the number of categories

	vector<int> num_Class(num_Categories);

	int i;

	//the number of elements for each category
	num_Class[0] = 7;//
	num_Class[1] = 9;//
	num_Class[2] = 9;//
	num_Class[3] = 5;//
	num_Class[4] = 5;//
	num_Class[5] = 9;//
	num_Class[6] = 9;//

	string tmp_Str;
	vector<string> fileNameVector;//file names
	vector<int> cat0;//category
	vector<int> cat1;//category
	vector<int> cat2;//category
	vector<int> cat3;//category
	vector<int> cat4;//category
	vector<int> cat5;//category
	vector<int> cat6;//category

	////////////////////////////////////
	//File name setting//
	////////////////////////////////////
	int c_0, c_1, c_2, c_3, c_4, c_5, c_6, c_7, c_8, c_9;

	for (replicate = 0; replicate < num_Replicates; replicate++)
	{
		counter = 0;

		for (c_0 = 4; c_0 < 7; c_0++)
		{//Default 3; 3,8
			for (c_1 = 2; c_1 < 9; c_1 += 3)
			{//Default: 0; 0,11 //p_Mode[5] = 1;//initial genotypic condition (0: Null, 1: Full, 2: Mid)
				for (c_2 = 4; c_2 < 5; c_2 += 4)
				{//Default: 4, 4,7
					for (c_3 = 2; c_3 < 3; c_3++)
					{//Default = 2//2,9
						for (c_4 = 2; c_4 < 3; c_4++)
						{//Default: 6; 0,9;
							for (c_5 = 1; c_5 < 2; c_5++)
							{//Default:0//0,7
								for (c_6 = 1; c_6 < 2; c_6++)
								{//Default 2; 1,2
									for (c_7 = 1; c_7 < 2; c_7++)
									{//Default 2; 0,5
										for (c_8 = 0; c_8 < 9; c_8++)
										{//Default 2; 0,5 (initial bi-alleleic frequency default: 3, 0-8)
											for (c_9 = 0; c_9 < 1; c_9++)
											{//Default 2; 0,5
												//Simulation label
												simul_Label[0] = c_0;
												simul_Label[1] = c_1;
												simul_Label[2] = c_2;
												simul_Label[3] = c_3;
												simul_Label[4] = c_4;
												simul_Label[5] = c_5;
												simul_Label[6] = c_6;
												simul_Label[7] = c_7;
												simul_Label[8] = c_8;
												simul_Label[9] = c_9;

												//sets a label for file outputs
												fileRW.Get_Simul_Label(&tmp_Str_0, simul_Label, &num_Simul_Label, replicate);

												//sets a file name
												tmp_Str = file_Tag + tmp_Str_0 + ".csv";
												fileNameVector.push_back(tmp_Str);

												cat0.push_back(c_0);
												cat1.push_back(c_1);
												cat2.push_back(c_2);
												cat3.push_back(c_3);
												cat4.push_back(c_4);
												cat5.push_back(c_8);//c_5
												cat6.push_back(c_7);

												num_Files++;//increases the number of files to process
											}//end 9
										}//end 8
									}//end 7
								}//end 6					
							}//end 5
						}//end 4
					}//end 3
				}//end 2
			}//end 1
		}//end 0
	}//end replicate loop

	delete[] simul_Label;

	int num_Columns_Data;//the number of columns (excluding headers) in the data in a file

	//time lable string vector
	vector<string> timeLabel_Str;

	//gets the number of columns in a data set
	if (num_Files > 0)
	{
		int tmp_Num_Row, tmp_Num_Col;
		vector<vector<string> > tmpArr;
		fileRW.CSV_Read_String_2D_String(fileNameVector[0], tmpArr, &tmp_Num_Row, &tmp_Num_Col);

		//sets the number of columns in the data
		num_Columns_Data = tmp_Num_Col - num_Columns_To_Skip;

		//the number of rows containing data
		int tmp_Num_Row_Data = tmp_Num_Row - num_Rows_To_Skip;
		//set time lable size
		timeLabel_Str.resize(tmp_Num_Row_Data);

		//set time lable
		if (tmp_Num_Row > 0)
		{
			for (int i = num_Rows_To_Skip; i < tmp_Num_Row; i++)
			{
				timeLabel_Str[(i - num_Rows_To_Skip)] = tmpArr[i][col_Num_Time_Label];
			}//end row loop
		}//end if
	}//end if

	//counter object
	Batch_Analysis b_Anal(num_Class);

	int num_Row_Selected;//the selected row in data
	int num_Column_Selected = 1;//the selected column in data
	string id_Tag;//output file name tag
	string str_Yr;
	string str_Option;
	int year;//year	
	//////////////////////////////////////////
	//Initial response loop//
	//////////////////////////////////////////

	//////////////////////////////////
	//Means and SDs//
	//////////////////////////////////

	//create total sum file
	//for the upper-bound trait
	str_Option = name_Trait;//Upper-bound trait
	string tmp_File_Name_Tot_U = id_Tag_Main + str_Option +"Aggregate" + ".csv";
	//create a file
	fileRW.CSV_Create_String(tmp_File_Name_Tot_U);

	for (year = begin_Year; year < end_Year; year += interval_Year)
	{
		num_Row_Selected = year;//the selected row in data

		//get year in string here
		//converts integer year to string
		str_Yr = timeLabel_Str[(year - 1)];

		//for the upper-bound trait
		//condition
		str_Option = name_Trait;//Upper-bound trait
		//sets a file ID tag
		id_Tag = id_Tag_Main + str_Option + str_Yr;
		std::cout << "ID_Tag: " << id_Tag << std::endl;

		//calculates means and SDs
		num_Column_Selected = p_Col_Num;//the selected column in data 1: mean, 2: variance, 3: sd, 4: mean, 5: variance, 6: sd
		string tmp_File_Output = b_Anal.Set_Mean_SD(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, num_Row_Selected, num_Column_Selected, cat0, cat1, cat2, cat3, cat4, cat5, cat6);
		int tmp_Year;
		fileRW.String_To_Integer(&tmp_Year, &timeLabel_Str[(year - 1)]);
		//add a time column
		fileRW.CSV_Add_Column_Integer(tmp_File_Output, tmp_Year);
		//add a csv file to a csv file
		fileRW.CSV_Add_CSV_To_CSV(tmp_File_Name_Tot_U, tmp_File_Output);
	}//end year loop

	////////////////////////////////////////////
	//Maximum & Minimum//
	////////////////////////////////////////////
	start_Row = begin_Year;//start row
	end_Row = end_Year;//end row

	//Upper-bound trait
	num_Column_Selected = 1;//the selected column in data 1: mean, 2: variance, 3: sd, 4: mean, 5: variance, 6: sd
	str_Option = name_Trait;//Upper-bound trait
	//sets a file ID tag
	id_Tag = id_Tag_Main + str_Option;
	//b_Anal.Set_Max_Stat(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, num_Column_Selected, start_Row, end_Row, cat0, cat1, cat2, cat3, cat4, cat5, cat6);
	//b_Anal.Set_Min_Stat(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, num_Column_Selected, start_Row, end_Row, cat0, cat1, cat2, cat3, cat4, cat5, cat6);


	//////////////////////////////////////////////////////////////////////////////////////////////////////////
	//counts the number of data sets exceeding the threshold//
	//////////////////////////////////////////////////////////////////////////////////////////////////////////
	vector<int> state_Column(num_Columns_Data);

	//sets condition state
	for (i = 0; i < num_Columns_Data; i++)
	{
		//vector<bool> &state_Column: the state of column
		//to return true all conditions for all columns should be true
		//0: no count
		//1: (x < p_constant)
		//2: (x > p_constant)
		//3: (x <= p_constant)
		//4: (x >= p_constant)
		//5: (x == p_constant)
		if (i == 0)
		{//the first column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 1;//sets a condition
		}
		else if (i == 3)
		{//the fourth column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 0;//sets a condition
		}
		else
		{//0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 0;//sets a condition
		}//end else
	}//end data column loop

	vector<double> p_Constant(num_Columns_Data);

	//sets constants
	for (i = 0; i < num_Columns_Data; i++)
	{
		//vector<double> &p_Constant: constants for condition for the columns (excluding the columns not containing data)
		//i.e. p_Constant[0] = 1.0
		if (i == 0)
		{//the first column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			p_Constant[i] = 0.5;//sets a condition
		}
		else if (i == 3)
		{//the fourth column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			p_Constant[i] = 0.0;//sets a condition
		}
		else
		{
			p_Constant[i] = 0.0;//sets a condition
		}//end else
	}//end data column loop

	//sets a file ID tag
	id_Tag = id_Tag_Main + "less05U";
	//b_Anal.Count_Threshold(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, start_Row, end_Row, state_Column, p_Constant, cat0, cat1, cat2, cat3, cat4, cat5, cat6);

	//sets condition state
	for (i = 0; i < num_Columns_Data; i++)
	{
		//vector<bool> &state_Column: the state of column
		//to return true all conditions for all columns should be true
		//0: no count
		//1: (x < p_constant)
		//2: (x > p_constant)
		//3: (x <= p_constant)
		//4: (x >= p_constant)
		//5: (x == p_constant)
		if (i == 0)
		{//the first column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 0;//sets a condition
		}
		else if (i == 3)
		{//the fourth column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 2;//sets a condition
		}
		else
		{
			state_Column[i] = 0;//sets a condition
		}//end else
	}//end data column loop

	//sets constants
	for (i = 0; i < num_Columns_Data; i++)
	{
		//vector<double> &p_Constant: constants for condition for the columns (excluding the columns not containing data)
		//i.e. p_Constant[0] = 1.0
		if (i == 0)
		{//the first column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			p_Constant[i] = 0.0;//sets a condition
		}
		else if (i == 3)
		{//the fourth column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			p_Constant[i] = 0.5;//sets a condition
		}
		else
		{
			p_Constant[i] = 0.0;//sets a condition
		}//end else
	}//end data column loop

	//sets a file ID tag
	id_Tag = id_Tag_Main + "Greater05L";
	//b_Anal.Count_Threshold(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, start_Row, end_Row, state_Column, p_Constant, cat0, cat1, cat2, cat3, cat4, cat5, cat6);

	//sets condition state
	for (i = 0; i < num_Columns_Data; i++)
	{
		//vector<bool> &state_Column: the state of column
		//to return true all conditions for all columns should be true
		//0: no count
		//1: (x < p_constant)
		//2: (x > p_constant)
		//3: (x <= p_constant)
		//4: (x >= p_constant)
		//5: (x == p_constant)
		if (i == 0)
		{//the first column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 1;//sets a condition
		}
		else if (i == 3)
		{//the fourth column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			state_Column[i] = 2;//sets a condition
		}
		else
		{
			state_Column[i] = 0;//sets a condition
		}//end else
	}//end data column loop

	//sets constants
	for (i = 0; i < num_Columns_Data; i++)
	{
		//vector<double> &p_Constant: constants for condition for the columns (excluding the columns not containing data)
		//i.e. p_Constant[0] = 1.0
		if (i == 0)
		{//the first column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			p_Constant[i] = 0.5;//sets a condition
		}
		else if (i == 3)
		{//the fourth column 0: mean, 1: variance, 2: sd, 3: mean, 4: variance, 5: sd
			p_Constant[i] = 0.5;//sets a condition
		}
		else
		{
			p_Constant[i] = 0.0;//sets a condition
		}//end else
	}//end data column loop

	//sets a file ID tag
	id_Tag = id_Tag_Main + "less05U_Greater05L";
	//b_Anal.Count_Threshold(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, start_Row, end_Row, state_Column, p_Constant, cat0, cat1, cat2, cat3, cat4, cat5, cat6);
}

void Batch_Analysis_Row_Process_Time_Label(string p_Header_FileName_input, string name_Trait, string name_Col, int col_Num_Time_Label, int p_Year_Begin, int p_Year_Interval, int p_num_Year_Segments, int num_Replicates)
{
	//Batch analysis for data files in thw whole row with time labels
	//p_Header_FileName: header of a file name
	//i.e. p_Header_FileName = "mv_tr"
	//p_Col_Num: column number to process in file
	//i.e. //the selected column in data 1: mean, 2: variance, 3: sd, 4: mean, 5: variance, 6: sd
	//name_Trait: trait name
	//i.e. name_Trait = "U_"
	//col_Num_Time_Label: time lable column number
	//i.e. col_Num_Time_Label = 0;//(the first column)
	//name_Col = "yr_Mu_";

	//input file information
	string file_Tag = p_Header_FileName_input;//input file tag
	int num_Rows_To_Skip = 1;//the number of rows to skip in a file
	int num_Columns_To_Skip = 1;//the number of columns to skip in a file
	//int num_Replicates;//The number of replicates

	//Output file setting
	string id_Tag_Main = "mrow_";

	//for data type conversion
	ostringstream convert;
	//initialize data transition buffer
	convert.str("");

	convert << p_Year_Interval;//integer -> string
	//output file name tag
	id_Tag_Main = id_Tag_Main + convert.str();
	//initialize data transition buffer
	convert.str("");

	//output file name tag
	id_Tag_Main = id_Tag_Main + name_Col;

	int begin_Year = p_Year_Begin;//default: 1
	int interval_Year = p_Year_Interval;//Long-term: 5000, short-term: 100
	int num_Segments_Year = p_num_Year_Segments;
	int end_Year = begin_Year + (interval_Year * num_Segments_Year) + 1;
	int start_Row = begin_Year;//the row for starting to count
	int end_Row = end_Year;//the row for finishing to count

	//simulation block
	int num_Files = 0;//the number of files to process
	int mode_Batch_Analysis = 0;//batch analysis mode
	int num_Simul_Label = 10;
	int *simul_Label = new int[num_Simul_Label];//sets setting for identiing simulations

	for (int i = 0; i < num_Simul_Label; i++)
	{
		simul_Label[i] = 0;//initialization
	}//end mode loop

	//2015 03 26_Random
	int counter = 0;
	int replicate = 0;

	string tmp_Str_0;
	File_RW fileRW;

	int num_Categories = 7;//the number of categories

	vector<int> num_Class(num_Categories);

	//the number of elements for each category
	num_Class[0] = 7;//
	num_Class[1] = 9;//
	num_Class[2] = 9;//
	num_Class[3] = 5;//
	num_Class[4] = 5;//
	num_Class[5] = 9;//
	num_Class[6] = 9;//

	string tmp_Str;
	vector<string> fileNameVector;//file names
	vector<int> cat0;//category
	vector<int> cat1;//category
	vector<int> cat2;//category
	vector<int> cat3;//category
	vector<int> cat4;//category
	vector<int> cat5;//category
	vector<int> cat6;//category

	////////////////////////////////////
	//File name setting//
	////////////////////////////////////
	int c_0, c_1, c_2, c_3, c_4, c_5, c_6, c_7, c_8, c_9;

	for (replicate = 0; replicate < num_Replicates; replicate++)
	{
		counter = 0;

		for (c_0 = 4; c_0 < 7; c_0++)
		{//Default 3; 3,8
			for (c_1 = 2; c_1 < 9; c_1 += 3)
			{//Default: 0; 0,11 //p_Mode[5] = 1;//initial genotypic condition (0: Null, 1: Full, 2: Mid)
				for (c_2 = 4; c_2 < 5; c_2 += 4)
				{//Default: 4, 4,7
					for (c_3 = 2; c_3 < 3; c_3++)
					{//Default = 2//2,9
						for (c_4 = 2; c_4 < 3; c_4++)
						{//Default: 6; 0,9;
							for (c_5 = 1; c_5 < 2; c_5++)
							{//Default:0//0,7
								for (c_6 = 1; c_6 < 2; c_6++)
								{//Default 2; 1,2
									for (c_7 = 1; c_7 < 2; c_7++)
									{//Default 2; 0,5
										for (c_8 = 0; c_8 < 9; c_8++)
										{//Default 2; 0,5 (initial bi-alleleic frequency default: 3, 0-8)
											for (c_9 = 0; c_9 < 1; c_9++)
											{//Default 2; 0,5
												//Simulation label
												simul_Label[0] = c_0;
												simul_Label[1] = c_1;
												simul_Label[2] = c_2;
												simul_Label[3] = c_3;
												simul_Label[4] = c_4;
												simul_Label[5] = c_5;
												simul_Label[6] = c_6;
												simul_Label[7] = c_7;
												simul_Label[8] = c_8;
												simul_Label[9] = c_9;

												//sets a label for file outputs
												fileRW.Get_Simul_Label(&tmp_Str_0, simul_Label, &num_Simul_Label, replicate);

												//sets a file name
												tmp_Str = file_Tag + tmp_Str_0 + ".csv";
												fileNameVector.push_back(tmp_Str);

												cat0.push_back(c_0);
												cat1.push_back(c_1);
												cat2.push_back(c_2);
												cat3.push_back(c_3);
												cat4.push_back(c_4);
												cat5.push_back(c_8);//c_5
												cat6.push_back(c_7);

												num_Files++;//increases the number of files to process
											}//end 9
										}//end 8
									}//end 7
								}//end 6					
							}//end 5
						}//end 4
					}//end 3
				}//end 2
			}//end 1
		}//end 0
	}//end replicate loop

	delete[] simul_Label;

	int num_Columns_Data;//the number of columns (excluding headers) in the data in a file

	//time lable string vector
	vector<string> timeLabel_Str;

	//gets the number of columns in a data set
	if (num_Files > 0)
	{
		int tmp_Num_Row, tmp_Num_Col;
		vector<vector<string> > tmpArr;
		fileRW.CSV_Read_String_2D_String(fileNameVector[0], tmpArr, &tmp_Num_Row, &tmp_Num_Col);

		//sets the number of columns in the data
		num_Columns_Data = tmp_Num_Col - num_Columns_To_Skip;

		//the number of rows containing data
		int tmp_Num_Row_Data = tmp_Num_Row - num_Rows_To_Skip;
		//set time lable size
		timeLabel_Str.resize(tmp_Num_Row_Data);

		//set time lable
		if (tmp_Num_Row > 0)
		{
			for (int i = num_Rows_To_Skip; i < tmp_Num_Row; i++)
			{
				timeLabel_Str[(i - num_Rows_To_Skip)] = tmpArr[i][col_Num_Time_Label];
			}//end row loop
		}//end if
	}//end if

	//counter object
	Batch_Analysis b_Anal(num_Class);

	int num_Row_Selected;//the selected row in data
	string id_Tag;//output file name tag
	string str_Yr;
	string str_Option;
	int year;//year	
	//////////////////////////////////////////
	//Initial response loop//
	//////////////////////////////////////////

	//////////////////////////////////
	//Means and SDs//
	//////////////////////////////////

	//create total sum file
	//for the upper-bound trait
	str_Option = name_Trait;//Upper-bound trait
	string tmp_File_Name_Tot_U = id_Tag_Main + str_Option + "Aggregate" + ".csv";
	//create a file
	fileRW.CSV_Create_String(tmp_File_Name_Tot_U);

	for (year = begin_Year; year < end_Year; year += interval_Year)
	{
		num_Row_Selected = year;//the selected row in data

		//get year in string here
		//converts integer year to string
		str_Yr = timeLabel_Str[(year - 1)];

		//for the upper-bound trait
		//condition
		str_Option = name_Trait;//Upper-bound trait
		//sets a file ID tag
		id_Tag = id_Tag_Main + str_Option + str_Yr;
		std::cout << "ID_Tag: " << id_Tag << std::endl;

		//calculates means and SDs
		string tmp_File_Output = b_Anal.Set_Mean_SD_Batch_Row(id_Tag, fileNameVector, num_Rows_To_Skip, num_Columns_To_Skip, num_Row_Selected, cat0, cat1, cat2, cat3, cat4, cat5, cat6);
		int tmp_Year;
		fileRW.String_To_Integer(&tmp_Year, &timeLabel_Str[(year - 1)]);
		//add a time column
		fileRW.CSV_Add_Column_Integer(tmp_File_Output, tmp_Year);
		//add a csv file to a csv file
		fileRW.CSV_Add_CSV_To_CSV(tmp_File_Name_Tot_U, tmp_File_Output);
	}//end year loop	
}

void Simul_Test_Default(void)
{
	int num_Simul_Label = 10;
	int *simul_Label = new int[num_Simul_Label];//sets setting for identiing simulations

	for (int i = 0; i < num_Simul_Label; i++)
	{
		simul_Label[i] = 0;//initialization
	}//end mode loop

	int num_Cel_Simul_Counter = 3;
	int *simul_Counter = new int[num_Cel_Simul_Counter];//sets setting for counting simulations

	for (int i = 0; i < num_Cel_Simul_Counter; i++)
	{
		simul_Counter[i] = 0;//initialization
	}//end mode loop

	int num_Cel_Simul_Basic_Settings = 10;
	int *simul_Basic_Setting = new int[num_Cel_Simul_Basic_Settings];//sets setting for counting simulations

	for (int i = 0; i < num_Cel_Simul_Basic_Settings; i++)
	{
		simul_Basic_Setting[i] = 0;//initialization
	}//end mode loop

	int numMode = 10;
	int *p_Mode = new int[numMode];//Gene-Trait mode

	for (int i = 0; i < numMode; i++)
	{
		p_Mode[i] = 0;
	}//end mode loop

	double *foragingTrait = new double[numMode];//sets setting for foraging traits
	double *p_Setting_Resource = new double[numMode];//data container for storing resource-interaction parameter values
	double *p_Plasticity_Vec = new double[numMode];//contain parameter values regarding environmental effects
	//parameter values regarding mating
	double *p_Mating = new double[numMode];

	int i_Num_Loci_Core_Spc, i_Num_RscSpc;
	double d_GeneralityPenalty, d_SteepnessAttackRate, d_CD_DDS;

	int num_Replicates = 10;//The number of replicates

	//Default simulation
	int num_Loci_Core_Spc = 4;//the number of loci
	int i_Num_Stg_Early_SexMat_Male = 2;////the number of life stage showing early sexual maturation
	int i_Num_Stg_Early_SexMat_Female = 3;////the number of life stage showing early sexual maturation

	int generalityPenalty = 3;
	int cost_EarlySexMaturation = 101;//The cost of early sexual maturation in body length Default: 101//for landlocked Alewife
	int steepnessAttackRate = 2;//Default value = 10.0
	int CD_DDS = 2;//Default value = 10.0
	int int_Exponent_Choice_Resource_BodyMass = 0;//(Core Species) Exponent for resource bodymass in foraging time assignment

	//Resource interaction settings
	int ecosystem_Scaling_Factor = 4;
	int rsc_Abundance_ScalingFactor = 2;//scales the resource abundance by controlling the intraspecific effect
	int alpha_Abundance_Biomass = 4;//controls overall balance of biomass distribution default: 4
	int neg_Eff_Spc_Rsc_ScalingFactor = 2;//the scaling factor for the negative effect of a core species on resource species
	int c_Intrinsic_r_Resource = 2;
	int c_IntraSpecific_LV = 2;
	int c_Scale_Neg_LV = 2;
	int c_Exp_Neg_LV = 2;
	int c_Scale_Pos_LV = 2;
	int handling_Time_Rsc = 2;
	int handling_Time_Spc = 2;
	int exp_Upper_Bound = 0;//exponent for mapping the upper-bound trait
	int exp_Lower_Bound = 0;//exponent for mapping the lower-bound trait
	int consistancy_Resource = 0;////True: Resource abundance dos not change after burn in period (0), False: resource abundance changes after burn in period (1)
	int initial_Bi_Allelic_Freq = 0;//The initial allele frequency

	//Mating
	int f_Ch = 0;
	int f_Intensity = (-1);

	//2015 03 26_Random
	int counter = 0;
	int replicate = 0;

	//current setup
	int program_Mode = 0;//0: simulation, 1: statistics only
	//ecosystem_Scaling_Factor = 3;//The scale of overall system, Default = 4
	//num_Loci_Core_Spc = 1;//the number of loci

	//p_Mode[4] = 0;//Trait value determinationGene (Multiplicative (0) vs additive (1))
	//p_Mode[5] = 1;//initial genotypic condition (0: Null, 1: Full, 2: Mid)

	cost_EarlySexMaturation = 101;//The cost of early sexual maturation in body length; Default: 101//for landlocked Alewife, No cost: 0
	i_Num_Stg_Early_SexMat_Male = 2;////the number of life stage showing early sexual maturation Default: 1, No early sexual maturation: 0
	i_Num_Stg_Early_SexMat_Female = 3;////the number of life stage showing early sexual maturation Default: 1, No early sexual maturation: 0

	int ini_Freq_Allele_Assortative_Mating = (-1);

	//handling_Time_Spc = 2;// handling time for the core species
	//handling_Time_Rsc = 2;// handling time for the resource

	consistancy_Resource = 0;////True: Resource abundance dos not change after burn in period (0), False: resource abundance changes after burn in period (1)

	//plasticity
	int lgn, ce, ci, ddg, lgx, cv_Bl_Growth, ini_Freq_Allele_Plasticity;
	lgn = 2;//fixed value
	lgx = 4;//fixed value
	f_Intensity = 3;//fixed value
	int lower_bound_Scaling_Factor = 3;//fixed_Value
	counter = 0;
	
	int num_Gene_Mode = 2;
	//gene mode
	bool *gene_Mode = new bool[num_Gene_Mode];
	//plasticity gene
	gene_Mode[0] = false;
	//assortative mating gene
	gene_Mode[1] = false;

	//the number of resource clusters
	int num_RscSpc = 1;//x + 2; Default = 1

	//the number of time steps
	int num_TimeSteps = 5100;//5200;//25200;

	//the total number of simulations
	int tot_Simul = 3 * 3 * 2 * 9;//;

	for (replicate = 0; replicate < num_Replicates; replicate += num_Replicates)
	{//replicate = 0; replicate < num_Replicates; replicate += num_Replicates
		for (num_Loci_Core_Spc = 7; num_Loci_Core_Spc < 8; num_Loci_Core_Spc += 2)
		{//Default 3; 7,2
			for (initial_Bi_Allelic_Freq = 3; initial_Bi_Allelic_Freq < 4; initial_Bi_Allelic_Freq++)
			{//Default: 3; 0-8 	//0: 0 (Null state),	//1:  1(Full state),	//2:  1(Intermediate state),	//3: customized initial state
				for (alpha_Abundance_Biomass = 4; alpha_Abundance_Biomass < 7; alpha_Abundance_Biomass ++)
				{//Default: 4, 4,7
					for (c_Exp_Neg_LV = 2; c_Exp_Neg_LV < 9; c_Exp_Neg_LV += 3)
					{//Default = 2//2,9
						for (generalityPenalty = 4; generalityPenalty < 5; generalityPenalty += 4)
						{//Default: 4 //0,9
							for (steepnessAttackRate = 2; steepnessAttackRate < 3; steepnessAttackRate++)
							{//Default 2; 0-4
								for (handling_Time_Spc = 2; handling_Time_Spc < 3; handling_Time_Spc++)
								{//Default 2; 0,5
									for (int_Exponent_Choice_Resource_BodyMass = 0; int_Exponent_Choice_Resource_BodyMass < 1; int_Exponent_Choice_Resource_BodyMass++)
									{//default = 0; 0-5
										for (f_Ch = 0; f_Ch < 1; f_Ch++)
										{//Default:0//0-4
											for (ini_Freq_Allele_Assortative_Mating = 0; ini_Freq_Allele_Assortative_Mating < 1; ini_Freq_Allele_Assortative_Mating++)
											{//default: 0//0-4

												///////////////////
												//Plasticity loop//
												///////////////////
												for (lgn = 1; lgn < 2; lgn++)
												{//0-4, 1-3; default = 1
													for (ce = 0; ce < 1; ce++)
													{//p_Plasticity_Vec[1]: ce: 0-3, default: 0
														for (ci = 0; ci < 1; ci++)
														{//p_Plasticity_Vec[2]: ci: 0-4, default: 0
															for (ddg = 2; ddg < 3; ddg++)
															{//p_Plasticity_Vec[3]: ddg: 0-3, default: 2
																for (cv_Bl_Growth = 1; cv_Bl_Growth < 2; cv_Bl_Growth++)
																{//p_Plasticity_Vec[5]: cv_BodyLength_Growth: 0-2, default: 1
																	for (ini_Freq_Allele_Plasticity = 0; ini_Freq_Allele_Plasticity < 1; ini_Freq_Allele_Plasticity++)
																	{//0-4, default: 0

																		//////////////
																		//Extra loop//
																		//////////////
																		for (lower_bound_Scaling_Factor = 1; lower_bound_Scaling_Factor < 2; lower_bound_Scaling_Factor++)
																		{//0-5, default: 1
																			//set parameter values regarding plasticity
																			Set_Plasticity_Param(p_Plasticity_Vec, lgn, ce, ci, ddg, lgx, cv_Bl_Growth, ini_Freq_Allele_Plasticity);

																			//1. sets basic simulation settings
																			Simul_Setting_Basic(&i_Num_Loci_Core_Spc, &i_Num_RscSpc, &d_GeneralityPenalty, &foragingTrait[2], &d_SteepnessAttackRate, &d_CD_DDS, &foragingTrait[3], &foragingTrait[4], &num_Loci_Core_Spc, &num_RscSpc, &generalityPenalty, &cost_EarlySexMaturation, &steepnessAttackRate, &CD_DDS, &handling_Time_Spc, &initial_Bi_Allelic_Freq);

																			//Foraging trait setting
																			Simul_Setting_Foraging_Trait(foragingTrait, &lower_bound_Scaling_Factor, &int_Exponent_Choice_Resource_BodyMass);

																			//2. sets settings regarding mating
																			Simul_Setting_Mating(p_Mating, f_Ch, f_Intensity, ini_Freq_Allele_Assortative_Mating);

																			//3. sets settings regarding resource dynamics
																			SimulSetting_Resource(p_Setting_Resource, &c_Intrinsic_r_Resource, &c_IntraSpecific_LV, &c_Scale_Neg_LV, &c_Exp_Neg_LV, &c_Scale_Pos_LV, &rsc_Abundance_ScalingFactor, &alpha_Abundance_Biomass, &neg_Eff_Spc_Rsc_ScalingFactor, &handling_Time_Rsc, &ecosystem_Scaling_Factor, &consistancy_Resource);

																			//counters
																			simul_Counter[0] = counter;
																			simul_Counter[1] = tot_Simul;
																			simul_Counter[2] = replicate;

																			//basic settings
																			simul_Basic_Setting[0] = i_Num_Loci_Core_Spc;
																			simul_Basic_Setting[1] = i_Num_RscSpc;
																			simul_Basic_Setting[2] = i_Num_Stg_Early_SexMat_Male;//the number of life stage showing early sexual maturation
																			simul_Basic_Setting[3] = i_Num_Stg_Early_SexMat_Female;//the number of life stage showing early sexual maturation
																			simul_Basic_Setting[4] = num_TimeSteps;//the number of time steps
																			//foraging traits
																			foragingTrait[0] = d_GeneralityPenalty;
																			foragingTrait[1] = d_SteepnessAttackRate;
																			//foragingTrait[2] = : //the cost of early maturation in body length
																			//foragingTrait[3] = : //handling time
																			//foragingTrait[4] = 0.0;//initial bi-allelic frequency
																			//foragingTrait[5] = 4.0;//lower-bound scaling factor (lower_bound_Scaling_Factor)
																			//foragingTrait[6] = 1.0;//exponent_Choice_Resource_BodyMass



																			//Simulation label num_RscSpc
																			simul_Label[0] = alpha_Abundance_Biomass;
																			simul_Label[1] = c_Exp_Neg_LV;
																			simul_Label[2] = generalityPenalty;
																			simul_Label[3] = steepnessAttackRate;
																			simul_Label[4] = handling_Time_Spc;
																			simul_Label[5] = lgn;
																			simul_Label[6] = cv_Bl_Growth;
																			simul_Label[7] = lower_bound_Scaling_Factor;
																			simul_Label[8] = initial_Bi_Allelic_Freq;
																			simul_Label[9] = ini_Freq_Allele_Assortative_Mating;

																			Simulation_Default(program_Mode, simul_Label, &num_Simul_Label, simul_Counter, gene_Mode, simul_Basic_Setting, foragingTrait, &d_CD_DDS, p_Mode, p_Mating, p_Setting_Resource, p_Plasticity_Vec);

																			counter++;//increases the counter
																		}//end lower-bound scaling factor loop
																		////////
																		////////

																	}//end ini_Freq_Allele_Plasticity
																}//end cv_Bl_Growth		
															}//end ddg
														}//end ci
													}//end ce
												}//end lgn
												/////////////////
												/////////////////

											}//end ini_Freq_Allele_Assortative_Mating
										}//end degree female choosiness loop

									}//end int_Exponent_Choice_Resource_BodyMass									
								}//end handling_Time_Spc
							}//end steepnessAttackRate
						}//generalist penalty loop
					}//end expoenet for species interaction loop
				}//end alpha-abundance-biomass loop
			}//end initial genetic condition loop
		}//end loci loop
	}//end replicate loop

	delete[] simul_Label;
	delete[] foragingTrait;
	delete[] p_Plasticity_Vec;
	delete[] p_Mating;
	delete[] p_Setting_Resource;
	delete[] p_Mode;
	delete[] simul_Basic_Setting;
	delete[] simul_Counter;
	delete[] gene_Mode;
}

void Batch_Proc_Stats_Rsc(void)
{
	string label_Title = "rc";
	string label_Type = "abun";
	string label_time_Type = "yr_Mu_";
	int col_Num_Time_Label = 0;
	int p_Year_Begin = 1;//the minimum begin year is 1
	int p_Year_Interval = 10;
	int p_num_Year_Segments = 30;
	int num_Replicates = 10;
	//300 years
	Batch_Analysis_Row_Process_Time_Label(label_Title, label_Type, label_time_Type, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//900 years
	p_Year_Interval = 30;
	p_num_Year_Segments = 30;
	Batch_Analysis_Row_Process_Time_Label(label_Title, label_Type, label_time_Type, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//5000 years
	p_Year_Interval = 100;
	p_num_Year_Segments = 50;
	Batch_Analysis_Row_Process_Time_Label(label_Title, label_Type, label_time_Type, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
}

void Batch_Proc_Stats(void)
{
	/////////////////////////////
	//Batch analysis for traits//
	/////////////////////////////
	string p_Header_FileName_input;
	string name_Trait;
	string name_Interval_Type;
	int p_Col_Num;
	int col_Num_Time_Label;
	int p_Year_Begin;//the minimum begin year is 1
	int p_Year_Interval;
	int p_num_Year_Segments;
	int num_Replicates;

	///////////////////////////////////
	//Mating pair gillraker genotypes
	//short
	//mean
	p_Header_FileName_input = "mv_mtg_L_Gn_Sum";
	name_Trait = "LG_";
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	col_Num_Time_Label = 0;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 2;
	p_num_Year_Segments = 5;
	num_Replicates = 10;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 5;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 15;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 100;
	p_num_Year_Segments = 5;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	////////////////////////////////////
	//Mating pair body size genotypes
	//short
	//mean
	p_Header_FileName_input = "mv_mtg_U_Gn_Sum";
	name_Trait = "UG_";
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	col_Num_Time_Label = 0;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 2;
	p_num_Year_Segments = 5;
	num_Replicates = 10;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 5;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 15;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 100;
	p_num_Year_Segments = 5;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	///////////////////////////////////
	//Mating pair body size phenotypes
	//short
	//mean
	p_Header_FileName_input = "mv_mtg_U_Ph_Sum";
	name_Trait = "UP_";
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	col_Num_Time_Label = 0;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 2;
	p_num_Year_Segments = 5;
	num_Replicates = 10;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 5;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 15;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 100;
	p_num_Year_Segments = 5;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	///////////////////////////////////
	//population body size genotypes
	//short
	//mean
	p_Header_FileName_input = "mv_tr";
	name_Trait = "UGT_";
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	col_Num_Time_Label = 0;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 20;
	p_num_Year_Segments = 5;
	num_Replicates = 10;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 50;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 150;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 1;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 1000;
	p_num_Year_Segments = 5;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 3;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	///////////////////////////////////
	//population gillraker genotypes
	//short
	//mean
	p_Header_FileName_input = "mv_tr";
	name_Trait = "LGT_";
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 4;
	col_Num_Time_Label = 0;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 20;
	p_num_Year_Segments = 5;
	num_Replicates = 10;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 4;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 50;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//medium long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 4;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 150;
	p_num_Year_Segments = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);

	//Long
	//mean
	name_Interval_Type = "yr_Mu_";
	p_Col_Num = 4;
	p_Year_Begin = 1;//the minimum begin year is 1
	p_Year_Interval = 1000;
	p_num_Year_Segments = 5;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
	//sd
	name_Interval_Type = "yr_SD_";
	p_Col_Num = 6;
	Batch_Analysis_Time_Label(p_Header_FileName_input, name_Trait, name_Interval_Type, p_Col_Num, col_Num_Time_Label, p_Year_Begin, p_Year_Interval, p_num_Year_Segments, num_Replicates);
}

int _tmain(int argc, _TCHAR* argv[])
{
	//Main simulation function
	Simul_Test_Default();

	/////////////////////////////
	//Batch analysis for traits//
	/////////////////////////////
	//Batch_Proc_Stats();

	//////////////////////////////
	//Batch process for resource//
	//////////////////////////////
	//Batch_Proc_Stats_Rsc();

	//(short term)  Batch analysis for resource 
	//Batch_Analysis_Rsc_Term_Mu(1, 30, 10);
	//(long term) Batch analysis for resource
	//Batch_Analysis_Rsc_Term_Mu(1, 5000, 5);


	//Matrix Population Model
	//Matrix_Population_Model_Routine();

	//For debugging
	//Simul_Test_Debug();

	//Pause before terminating the run
	//Function_Test();

	//tests sort
	//Test_Sort();

	//tests file read
	//Test_File_Read();

	//To fin memory leak
	//_CrtDumpMemoryLeaks();

	return 0;
}