/*
* SimulProc.cpp
*
*  Created on: Dec 4, 2014
*      Author: jungk_000
*/

#include "Simul_Proc.h"

Simul_Proc::Simul_Proc(char *p_FileName_Setting, int *setArr, int **p_Mode, int *p_SimulStr, int *p_Num_Locus, int *p_Num_Position, double **p_MutRat, int *p_Num_Trait, int **p_Num_Stg, int **p_Num_Immat_Stg_SexMaturation, double *p_PropFeBrth, double***** p_P, double***** p_G, int *****p_Initial_Num_Spc, double *p_MetabolicRate_Biomass_Factor)
{
	status_Simul_File_Creation = false;//file creation status; true: simulation files are created ,false: files are not created

	std_Norm_Dist = new boost::math::normal(0.0, 1.0);//standard normal distribution
	num_Deme = setArr[0];
	num_Niche = setArr[1];
	tot_Deme_Niche = ((num_Deme - 1) * (num_Niche)) + (num_Niche - 1) + 1;
	num_Spc = setArr[2];
	tot_Deme_Niche_Spc = ((num_Deme - 1) * (num_Niche * num_Spc)) + ((num_Niche - 1) * num_Spc) + (num_Spc - 1) + 1;
	num_Sex = setArr[3];
	max_Num_Stg = setArr[4];

	num_Niche_Num_Spc = num_Niche * num_Spc;

	far_Right_Deme = num_Deme - 1;
	num_RscSpc = setArr[5];
	num_Sum_Spc = num_Spc + num_RscSpc;

	max_Num_Allele = setArr[6];
	max_Num_PhenTyp_Rsc = setArr[7];

	//the number of foraging traits = 2
	m_Num_Traits_Foraging = 2;

	//sets the multiplier for counting alleles
	double tmp_Max_Num_Allele = (double)max_Num_Allele;
	double counter_Double = 0.0;
	scaling_Multiplier_Allele = 1.0;

	while (scaling_Multiplier_Allele < tmp_Max_Num_Allele){
		counter_Double += 1.0;
		scaling_Multiplier_Allele = pow(10.0, counter_Double);
	}//end while loop

	scaling_Multiplier_Allele = 1.0 / scaling_Multiplier_Allele;

	int spc, i;
	int num_Mode_Category = 10;
	mode_Gene_Trait = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		mode_Gene_Trait[spc] = new int[num_Mode_Category];

		for (i = 0; i < num_Mode_Category; i++){
			mode_Gene_Trait[spc][i] = p_Mode[spc][i];
		}//end i loop
	}//end species loop

	num_Tstep = p_SimulStr[0];
	unit_T_Step = 1.0;
	timeStep_Reintroduction = 0;//the time step for re-introducing the prototype morph.
	status_Reintroduction = false;//True: re-introducing species, False: not re-introducing a core species
	consistancy_Resource = false;//

	//Sets the sampling interval
	const int max_Interval = 10;

	if (num_Tstep >= max_Interval)
	{
		interval_sampling = max_Interval;
	}
	else
	{
		interval_sampling = num_Tstep;
	}//end else

	tick_t = true;

	Set_Gen_Struct(p_Num_Locus, p_Num_Position, p_MutRat, p_Num_Trait);
	Set_Vitality_Struct(p_Num_Stg, p_Num_Immat_Stg_SexMaturation, p_PropFeBrth, p_P, p_G, p_MetabolicRate_Biomass_Factor);
	Set_Simul_Struct(p_Initial_Num_Spc);

	if (num_RscSpc > 0){
		Set_Resource();
	}//end if

	Set_Spc_Interaction();
	Set_Movement();
	Set_Stat_Dst(p_FileName_Setting);

	//assortative mating mode
	mode_Assortative_Mating = false;
}//end constructor

Simul_Proc::~Simul_Proc(void)
{
	delete[] mode_Forage_Physic_Spc;//deallocate memory
	delete std_Norm_Dist;//deallocate memory

	int i, j, k, spc, deme, niche, sex, stg, rsc;

	for (spc = 0; spc < num_Spc; spc++)
	{
		delete[] growth_Target_Spc[spc];//deallocate memory
	}

	delete[] growth_Target_Spc;

	for (spc = 0; spc < num_Spc; spc++)
	{
		delete[] exponent_VonBertallanffy[spc];//deallocate memory
	}

	delete[] exponent_VonBertallanffy;

	delete[] ltg_VB;
	delete[] ce_VB;
	delete[] ci_VB;
	delete[] ltg_VB_Sx;
	delete[] cv_BodyLength_Growth;

	delete[] cost_Reduction_EarlySexMat;
	delete[] smPlb;
	delete[] bodyLength_Max_Spc;
	delete[] bodyLength_Min_Spc;
	delete[] bodyMass_Min;
	delete[] proportion_Female_At_Birth;
	delete[] maximum_Fertility;
	delete[] maximum_Fertility_CV;

	for (i = 0; i < num_Spc; i++)
	{
		for (j = 0; j < cls_UpperBound_Spc_Discrete; j++)
		{
			delete[] celNum_Spc_Trt_Local_Discrete[i][j];
		}//end upper foraging bound loop

		delete[] celNum_Spc_Trt_Local_Discrete[i];
	}//end species loop

	delete[] celNum_Spc_Trt_Local_Discrete;

	delete[] num_Locus_Mor;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] mode_Gene_Trait[spc];
	}//end species loop

	delete[] mode_Gene_Trait;


	for (spc = 0; spc < num_Spc; spc++){
		delete[] mul_Dim_Trt_Loc_QTL_Mtg[spc];
	}//end species loop

	delete[] mul_Dim_Trt_Loc_QTL_Mtg;

	delete[] metabolicRate_Biomass_Factor;

	delete[] bodyMass_Max;

	delete[] num_Bin_Bodylength;
	delete[] bin_Width_BodyLength;
	delete[] ddg;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			for (stg = 0; stg < max_Num_Stg; stg++){
				delete[] early_Sexual_Maturation[spc][sex][stg];
			}//end life stage loop

			delete[] early_Sexual_Maturation[spc][sex];
		}//end sex loop

		delete[] early_Sexual_Maturation[spc];
	}//end species loop

	delete[] early_Sexual_Maturation;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			for (stg = 0; stg < max_Num_Stg; stg++){
				delete[] gillRaker[spc][sex][stg];
			}//end life stage loop

			delete[] gillRaker[spc][sex];
		}//end sex loop

		delete[] gillRaker[spc];
	}//end species loop

	delete[] gillRaker;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			for (stg = 0; stg < max_Num_Stg; stg++){
				delete[] bodyLength[spc][sex][stg];
			}//end life stage loop

			delete[] bodyLength[spc][sex];
		}//end sex loop

		delete[] bodyLength[spc];
	}//end species loop

	delete[] bodyLength;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			for (stg = 0; stg < max_Num_Stg; stg++){
				delete[] degree_Dev_Bodylength_Default[spc][sex][stg];
			}//end life stage loop

			delete[] degree_Dev_Bodylength_Default[spc][sex];
		}//end sex loop

		delete[] degree_Dev_Bodylength_Default[spc];
	}//end species loop

	delete[] degree_Dev_Bodylength_Default;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			for (stg = 0; stg < max_Num_Stg; stg++){
				delete[] bodyMass[spc][sex][stg];
			}//end life stage loop

			delete[] bodyMass[spc][sex];
		}//end sex loop

		delete[] bodyMass[spc];
	}//end species loop

	delete[] bodyMass;

	delete[] scalingFactor_BodyLength_BodyMass_Conversion;
	delete[] exponent_BodyLength_BodyMass_Conversion;

	delete[] upper_Bound_Max;//dim[species]: the maximum bound of the upper-bound trait
	delete[] upper_Bound_Min;//dim[species]: the minimum bound of the upper-bound trait
	delete[] lower_Bound_Max;//dim[species]: the maximum bound of the lower-bound trait
	delete[] lower_Bound_Min;//dim[species]: the minimum bound of the lower-bound trait

	delete[] exponent_Upper_Bound;
	delete[] exponent_Lower_Bound;

	//deallocates memory for esp_Discrete (foraging efficiency)
	for (int rsc = 0; rsc < num_RscSpc; rsc++)
	{
		delete[] esp_Discrete[rsc];
	}//end resource cluster loop

	delete[] esp_Discrete;

	//deallocates memory for functional response
	for (int rsc = 0; rsc < num_RscSpc; rsc++)
	{
		delete[] frsp[rsc];
	}//end resource cluster loop

	delete[] frsp;

	//deallocates memory for foraging time
	for (int rsc = 0; rsc < num_RscSpc; rsc++)
	{
		delete[] ft_Discrete[rsc];
	}//end resource cluster loop

	delete[] ft_Discrete;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Clas_Trt_Spc[spc];
	}//end species loop

	delete[] num_Clas_Trt_Spc;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] trt_Loc[spc];
	}//end species loop

	delete[] trt_Loc;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] trt_Loc_Int[spc];
	}//end species loop

	delete[] trt_Loc_Int;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] loc_Trt[spc];
	}//end species loop

	delete[] loc_Trt;

	//de-allocated memory for simulation file names
	Deallocate_Memory_Simul_FileName();

	int l, m, n;
	int tmp_Max_Trt = max_Num_Clas_Trt;

	for (l = 0; l < tmp_Max_Trt; l++)
	{
		for (m = 0; m < tmp_Max_Trt; m++)
		{
			for (n = 0; n < tmp_Max_Trt; n++)
			{
				delete[] exp_Data_Buffer_Mtg_Trait[l][m][n];
			}//end female lower bound loop

			delete[] exp_Data_Buffer_Mtg_Trait[l][m];
		}//end female upper bound loop

		delete[] exp_Data_Buffer_Mtg_Trait[l];
	}//end male lower bound loop

	delete[] exp_Data_Buffer_Mtg_Trait;

	for (l = 0; l < tmp_Max_Trt; l++)
	{
		for (m = 0; m < tmp_Max_Trt; m++)
		{
			for (n = 0; n < tmp_Max_Trt; n++)
			{
				delete[] data_Buffer_Mtg_Trait[l][m][n];
			}//end female lower bound loop

			delete[] data_Buffer_Mtg_Trait[l][m];
		}//end female upper bound loop

		delete[] data_Buffer_Mtg_Trait[l];
	}//end male lower bound loop

	delete[] data_Buffer_Mtg_Trait;

	for (i = 0; i < num_Deme; i++)
	{
		for (j = 0; j < num_Niche; j++)
		{
			for (k = 0; k < num_Spc; k++)
			{
				for (l = 0; l < tmp_Max_Trt; l++)
				{
					for (m = 0; m < tmp_Max_Trt; m++)
					{
						for (n = 0; n < tmp_Max_Trt; n++)
						{
							delete[] tmp_Data_Buffer_Mtg_Trait[i][j][k][l][m][n];
						}//end female lower bound loop

						delete[] tmp_Data_Buffer_Mtg_Trait[i][j][k][l][m];
					}//end female upper bound loop

					delete[] tmp_Data_Buffer_Mtg_Trait[i][j][k][l];
				}//end male lower bound loop

				delete[] tmp_Data_Buffer_Mtg_Trait[i][j][k];
			}//end male upper bound loop

			delete[] tmp_Data_Buffer_Mtg_Trait[i][j];
		}//end species loop

		delete[] tmp_Data_Buffer_Mtg_Trait[i];
	}//end niche loop

	delete[] tmp_Data_Buffer_Mtg_Trait;

	Deallocate_Memory_Data_Buffer_Mtg_Bodymass();//de-allocates memory for data buffer for mating with respect to bodymass
	Deallocate_Memory_Data_Buffer_Mtg_GillRaker();//de-allocates memory for data buffer for mating with respect to gill-raker

	for (spc = 0; spc < num_Spc; spc++){
		delete[] int_Num_Class_Trait[spc];
	}//end species loop

	delete[] int_Num_Class_Trait;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] double_Num_Class_Trait_Minus_One[spc];
	}//end species loop

	delete[] double_Num_Class_Trait_Minus_One;

	delete[] num_Trait_Combination;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] initial_Num_Spc[deme][niche][spc][sex];
				}//end sex loop

				delete[] initial_Num_Spc[deme][niche][spc];
			}//end species loop

			delete[] initial_Num_Spc[deme][niche];
		}//end niche loop

		delete[] initial_Num_Spc[deme];
	}//end deme loop

	delete[] initial_Num_Spc;

	for (niche = 0; niche < num_Niche; niche++){
		delete[] mov_Niche_Cumul[niche];
	}//end niche loop

	delete[] mov_Niche_Cumul;

	for (niche = 0; niche < num_Niche; niche++){
		delete[] mov_Niche[niche];
	}//end niche loop

	delete[] mov_Niche;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			delete[] density_Spc[deme][niche];
		}//end niche loop

		delete[] density_Spc[deme];
	}//end deme loop

	delete[] density_Spc;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			delete[] b_MassIntake_DDS[spc][sex];
		}//end niche loop

		delete[] b_MassIntake_DDS[spc];
	}//end deme loop

	delete[] b_MassIntake_DDS;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] Cd[deme][niche][spc][sex];
				}//end sex loop
				delete[] Cd[deme][niche][spc];
			}//end species loop
			delete[] Cd[deme][niche];
		}//end niche loop
		delete[] Cd[deme];
	}//end deme loop

	delete[] Cd;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] k_NutritionValue[spc];
	}//end species loop

	delete[] k_NutritionValue;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Sum_Spc; spc++){
				delete[] LV_Gain[deme][niche][spc];
			}//end species loop

			delete[] LV_Gain[deme][niche];
		}//end niche loop

		delete[] LV_Gain[deme];
	}//end deme loop

	delete[] LV_Gain;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Sum_Spc; spc++){
				delete[] LV_Loss[deme][niche][spc];
			}//end species loop

			delete[] LV_Loss[deme][niche];
		}//end niche loop

		delete[] LV_Loss[deme];
	}//end deme loop

	delete[] LV_Loss;

	delete[] k_Hunt_Bound;
	delete[] delta_Hunt_Bound;
	delete[] exponent_Choice_Resource_BodyMass;

	if (num_RscSpc > 0){//deallocate memory for resources
		delete[] num_Locus_Rsc;
		delete[] num_Position_Rsc;
		delete[] mut_Rat_Rsc;
		delete[] num_PhenoTyp_Rsc;

		for (spc = 0; spc < num_RscSpc; spc++){
			delete[] mean_BioMass_Rsc[spc];
		}//end species loop

		delete[] mean_BioMass_Rsc;

		for (spc = 0; spc < num_RscSpc; spc++)
		{
			for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++)
			{
				delete[] bin_Rsc_BioMass[spc][rsc];
			}//end phenotype loop

			delete[] bin_Rsc_BioMass[spc];
		}//end species loop

		delete[] bin_Rsc_BioMass;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					delete[] phen_D_Rsc_Fixed[deme][niche][spc];
				}//end species loop

				delete[] phen_D_Rsc_Fixed[deme][niche];
			}//end niche loop

			delete[] phen_D_Rsc_Fixed[deme];
		}//end deme loop

		delete[] phen_D_Rsc_Fixed;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					delete[] phen_D_Rsc[deme][niche][spc];
				}//end species loop

				delete[] phen_D_Rsc[deme][niche];
			}//end niche loop

			delete[] phen_D_Rsc[deme];
		}//end deme loop

		delete[] phen_D_Rsc;

		delete[] growthRate_Rsc;

		delete[] tmp_Rsc_Cls;

		int end0 = max_Num_Clas_Trt_Spc[0];
		int end1 = max_Num_Clas_Trt_Spc[1];

		delete[] max_Num_Clas_Trt_Spc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					delete[] sum_Cmt_Rsc_Spc[deme][niche][spc];
				}//end species loop

				delete[] sum_Cmt_Rsc_Spc[deme][niche];
			}//end niche loop

			delete[] sum_Cmt_Rsc_Spc[deme];
		}//end deme loop

		delete[] sum_Cmt_Rsc_Spc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_Spc; spc++){
					for (rsc = 0; rsc < num_RscSpc; rsc++){
						delete[] sum_Spc_Rsc[deme][niche][spc][rsc];
					}//end resource species loop

					delete[] sum_Spc_Rsc[deme][niche][spc];
				}//end species loop

				delete[] sum_Spc_Rsc[deme][niche];
			}//end niche loop

			delete[] sum_Spc_Rsc[deme];
		}//end deme loop

		delete[] sum_Spc_Rsc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				delete[] density_Rsc[deme][niche];
			}//end niche loop

			delete[] density_Rsc[deme];
		}//end deme loop

		delete[] density_Rsc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				delete[] density_Rsc_Fixed[deme][niche];
			}//end niche loop

			delete[] density_Rsc_Fixed[deme];
		}//end deme loop

		delete[] density_Rsc_Fixed;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					delete[] q_Rsc_RscSpc[deme][niche][spc];
				}//end species loop

				delete[] q_Rsc_RscSpc[deme][niche];
			}//end niche loop

			delete[] q_Rsc_RscSpc[deme];
		}//end deme loop

		delete[] q_Rsc_RscSpc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					delete[] th_Rsc_RscSpc[deme][niche][spc];
				}//end species loop

				delete[] th_Rsc_RscSpc[deme][niche];
			}//end niche loop

			delete[] th_Rsc_RscSpc[deme];
		}//end deme loop

		delete[] th_Rsc_RscSpc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_Spc; spc++){
					delete[] q_Spc_RscSpc[deme][niche][spc];
				}//end species loop

				delete[] q_Spc_RscSpc[deme][niche];
			}//end niche loop

			delete[] q_Spc_RscSpc[deme];
		}//end deme loop

		delete[] q_Spc_RscSpc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_Spc; spc++){
					delete[] th_Spc_RscSpc[deme][niche][spc];
				}//end species loop

				delete[] th_Spc_RscSpc[deme][niche];
			}//end niche loop

			delete[] th_Spc_RscSpc[deme];
		}//end deme loop

		delete[] th_Spc_RscSpc;

		//delete[] fileName_Resource_Class_Bodymass;
		delete[] fileName_Density;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				delete[] d_Rsc[deme][niche];
			}//end niche loop

			delete[] d_Rsc[deme];
		}//end deme loop

		delete[] d_Rsc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (rsc = 0; rsc < num_RscSpc; rsc++){
					delete[] prop_Density_Rsc[deme][niche][rsc];
				}//end resource species loop

				delete[] prop_Density_Rsc[deme][niche];
			}//end niche loop

			delete[] prop_Density_Rsc[deme];
		}//end deme loop

		delete[] prop_Density_Rsc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				delete[] intrinsic_Rsc[deme][niche];
			}//end niche loop

			delete[] intrinsic_Rsc[deme];
		}//end deme loop

		delete[] intrinsic_Rsc;
	}//end if(num_RscSpc > 0)

	for (spc = 0; spc < num_Spc; spc++){
		int tmp = (int)agt[spc].size();

		if (tmp > 0){
			for (i = 0; i < tmp; i++){
				agt[spc][i]->Deallocation_Memory(num_Locus[spc]);
				delete agt[spc][i];
			}//end individual loop
		}//end if
	}//end species loop

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] celNum_0[deme][niche][spc][sex];
				}//end sex loop

				delete[] celNum_0[deme][niche][spc];
			}//end species loop

			delete[] celNum_0[deme][niche];
		}//end niche loop

		delete[] celNum_0[deme];
	}//end deme loop

	delete[] celNum_0;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] celNum_1[deme][niche][spc][sex];
				}//end sex loop

				delete[] celNum_1[deme][niche][spc];
			}//end species loop

			delete[] celNum_1[deme][niche];
		}//end niche loop

		delete[] celNum_1[deme];
	}//end deme loop

	delete[] celNum_1;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					delete[] size_AgtLst[deme][niche][spc][sex];
				}//end sex loop
				delete[] size_AgtLst[deme][niche][spc];
			}//end species loop
			delete[] size_AgtLst[deme][niche];
		}//end niche loop
		delete[] size_AgtLst[deme];
	}//end deme loop

	delete[] size_AgtLst;

	delete[] celNum_New_AgtLst;
	delete[] size_New_AgtLst;

	delete[] celNum_M;
	delete[] celNum_Agt;

	delete[] num_Locus;
	delete[] num_Position;
	delete[] num_Trait;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] multiplier_Trait_Combination[spc];
	}//end species loop

	delete[] multiplier_Trait_Combination;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] tmp_Data_Buffer_Trait_Foraging_Combination[i][j][spc];
			}//end species loop

			delete[] tmp_Data_Buffer_Trait_Foraging_Combination[i][j];
		}//end niche loop

		delete[] tmp_Data_Buffer_Trait_Foraging_Combination[i];
	}//end deme loop

	delete[] tmp_Data_Buffer_Trait_Foraging_Combination;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				delete[] data_Buffer_Trait_Combination[i][j][spc];
			}//end species loop

			delete[] data_Buffer_Trait_Combination[i][j];
		}//end niche loop

		delete[] data_Buffer_Trait_Combination[i];
	}//end deme loop

	delete[] data_Buffer_Trait_Combination;

	tmp_Max_Trt = max_Num_Clas_Trt;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (l = 0; l < max_Num_Clas_Trt; l++){
					delete[] data_Buffer_Bodysize_Mtg[i][j][spc][l];//initialization
				}//end lower-bound trait loop

				delete[] data_Buffer_Bodysize_Mtg[i][j][spc];//initialization
			}//end species loop

			delete[] data_Buffer_Bodysize_Mtg[i][j];//initialization
		}//end niche loop

		delete[] data_Buffer_Bodysize_Mtg[i];//initialization
	}//end deme loop

	delete[] data_Buffer_Bodysize_Mtg;//initialization

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (l = 0; l < max_Num_Clas_Trt; l++){
					delete[] data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l];//initialization
				}//end lower-bound trait loop

				delete[] data_Buffer_Bodysize_Genotype_Mtg[i][j][spc];//initialization
			}//end species loop

			delete[] data_Buffer_Bodysize_Genotype_Mtg[i][j];//initialization
		}//end niche loop

		delete[] data_Buffer_Bodysize_Genotype_Mtg[i];//initialization
	}//end deme loop

	delete[] data_Buffer_Bodysize_Genotype_Mtg;//initialization

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (l = 0; l < max_Num_Clas_Trt; l++){
					delete[] data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l];//initialization
				}//end lower-bound trait loop

				delete[] data_Buffer_Gillraker_Genotype_Mtg[i][j][spc];//initialization
			}//end species loop

			delete[] data_Buffer_Gillraker_Genotype_Mtg[i][j];//initialization
		}//end niche loop

		delete[] data_Buffer_Gillraker_Genotype_Mtg[i];//initialization
	}//end deme loop

	delete[] data_Buffer_Gillraker_Genotype_Mtg;//initialization

	for (i = 0; i < tot_Deme_Niche; i++)
	{
		for (j = 0; j < num_Spc; j++)
		{//max_Num_Clas_Trt
			for (k = 0; k < max_Num_Trait; k++)
			{
				delete[] tmp_Data_Buffer_Trait[i][j][k];
			}

			delete[] tmp_Data_Buffer_Trait[i][j];
		}

		delete[] tmp_Data_Buffer_Trait[i];
	}

	delete[] tmp_Data_Buffer_Trait;

	int trt;

	for (i = 0; i < interval_sampling; i++){
		for (j = 0; j < tot_Deme_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (trt = 0; trt < max_Num_Trait; trt++){
					delete[] data_Buffer_Trait[i][j][spc][trt];
				}//end trait loop

				delete[] data_Buffer_Trait[i][j][spc];
			}//end species loop

			delete[] data_Buffer_Trait[i][j];
		}//end niche loop

		delete[] data_Buffer_Trait[i];
	}//end deme loop

	delete[] data_Buffer_Trait;

	for (i = 0; i < num_Deme; i++)
	{
		for (j = 0; j < num_Niche; j++)
		{
			for (k = 0; k < num_Spc; k++)
			{
				delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i][j][k];
			}//end species loop

			delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i][j];
		}//end niche loop

		delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i];
	}//end deme loop

	delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker;

	for (i = 0; i < num_Deme; i++)
	{
		for (j = 0; j < num_Niche; j++)
		{
			for (k = 0; k < num_Spc; k++)
			{
				delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i][j][k];
			}//end species loop

			delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i][j];
		}//end niche loop

		delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i];
	}//end deme loop

	delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass;

	for (i = 0; i < num_Deme; i++)
	{
		for (j = 0; j < num_Niche; j++)
		{
			for (k = 0; k < num_Spc; k++)
			{
				delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i][j][k];
			}//end species loop

			delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i][j];
		}//end niche loop

		delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i];
	}//end deme loop

	delete[] data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype;

	//Data buffer for resource species density
	for (i = 0; i < interval_sampling; i++)
	{
		for (j = 0; j < num_Deme; j++)
		{
			for (k = 0; k < num_Niche; k++)
			{
				for (l = 0; l < num_RscSpc; l++)
				{
					delete[] data_Buffer_RscCls[i][j][k][l];
				}//end resource species loop

				delete[] data_Buffer_RscCls[i][j][k];
			}

			delete[] data_Buffer_RscCls[i][j];
		}//end j loop

		delete[] data_Buffer_RscCls[i];
	}//end i loop

	delete[] data_Buffer_RscCls;

	//Genetic data buffer for each time step
	for (i = 0; i < tot_Deme_Niche_Spc; i++){
		for (j = 0; j < max_Num_Locus; j++){
			for (k = 0; k < max_Num_Allele; k++){
				delete[] tmp_Data_Buffer_Allele[i][j][k];
			}//end k loop

			delete[] tmp_Data_Buffer_Allele[i][j];
		}//end j loop

		delete[] tmp_Data_Buffer_Allele[i];
	}//end i loop

	delete[] tmp_Data_Buffer_Allele;

	//Genetic data buffer for each sampling interval
	for (i = 0; i < interval_sampling; i++){
		for (j = 0; j < tot_Deme_Niche_Spc; j++){
			for (k = 0; k < max_Num_Locus; k++){
				for (l = 0; l < max_Num_Allele; l++){
					delete[] data_Buffer_Allele[i][j][k][l];
				}//end k loop

				delete[] data_Buffer_Allele[i][j][k];
			}//end j loop

			delete[] data_Buffer_Allele[i][j];
		}//end deme loop

		delete[] data_Buffer_Allele[i];
	}//end sampling interval loop

	delete[] data_Buffer_Allele;

	for (i = 0; i < num_Spc; i++){
		delete[] mut_Rat[i];
	}//end species loop

	delete[] mut_Rat;

	delete[] num_MateSearchLargeStep_Discrete;

	for (i = 0; i < max_Num_Bin_Bodylength; i++){
		delete[] size_MatingList_Discrete[i];
	}//end mating class loop

	delete[] size_MatingList_Discrete;

	delete[] female_Choosiness_Scale;
	delete[] female_Choosiness_Intensity;

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		delete[] size_VitaList_Discrete[i];
	}//end mating class loop

	delete[] size_VitaList_Discrete;

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		delete[] celNum_VitaList_Discrete[i];
	}//end mating class loop

	delete[] celNum_VitaList_Discrete;

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		delete[] maxCelNum_VitaList_Discrete[i];
	}//end mating class loop

	delete[] maxCelNum_VitaList_Discrete;

	for (i = 0; i < max_Num_Bin_Bodylength; i++){
		delete[] celNum_MatingList_Discrete[i];
	}//end mating class loop

	delete[] celNum_MatingList_Discrete;

	for (i = 0; i < max_Num_Bin_Bodylength; i++){
		delete[] maxCelNum_MatingList_Discrete[i];
	}//end mating class loop

	delete[] maxCelNum_MatingList_Discrete;

	delete[] prop_Female_At_Birth;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					for (stg = 0; stg < max_Num_Stg; stg++){
						delete[] P_G[deme][niche][spc][sex][stg];
					}//end rate loop
					delete[] P_G[deme][niche][spc][sex];
				}//end sex loop
				delete[] P_G[deme][niche][spc];
			}//end species loop
			delete[] P_G[deme][niche];
		}//end niche loop
		delete[] P_G[deme];
	}//end deme loop

	delete[] P_G;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Immat_Stg_SexMaturation[spc];
	}//end species loop

	delete[] num_Immat_Stg_SexMaturation;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Stg[spc];
	}//end species loop

	delete[] num_Stg;

	for (spc = 0; spc < num_Spc; spc++){
		delete[] num_Stg_BodyLength_Growth[spc];
	}//end species loop

	delete[] num_Stg_BodyLength_Growth;

	delete randomSeed_Mut;
	delete randomSeed_Trt;

	delete rand_Uni_Real_Mtg;
	delete uni_Real_Mtg;
	delete randomSeed_Mtg;

	delete rand_Uni_Real;
	delete uni_Real;
	delete randomSeed;

	delete rand_Uni_int;
	delete uni_int;
	delete randomSeed_int;

	Deallocate_Memory_Data_Buffer();//de-allocates memory for data buffers for file writing
}//end destructor

void Simul_Proc::Set_Resource_Consistancy(int p_Consistancy_Resource)
{
	if (p_Consistancy_Resource == 0)
	{//there is resource dynamics after burn-in period
		consistancy_Resource = false;
	}
	else
	{//there is no resource dynamics after burn-in period
		consistancy_Resource = true;
	}//end else
}

void Simul_Proc::Proc_Stat(int *p_totNumSimul, int *p_Num_Replicate, int *p_Cnt_Simul)
{
	Display_StatisticalProcessMode();//displays a statistical process mode
	Display_Simul_Setup(p_totNumSimul, p_Num_Replicate, p_Cnt_Simul);//display the simulation settings

	Set_Stats_Trait();//statistical analyses regarding traits
	//Set_Stats_Rsc();//statistical analyses regarding resources
}

const int Simul_Proc::Proc_Simul(int *p_Cnt, int *p_Total_Num_Simul, int *p_Replicate)
{
	//To remove the agent, 1) do not transfer the agent to the next time step, 2) call the function "Remove_Agent()"
	//When an agent is created, set the mating class and traits
	int case_Deme_Niche;

	//gets a spacial design mode
	Get_Deme_Niche_Mode(&case_Deme_Niche);
	//sets simulation output files
	Set_Simul_File_Output();
	//display the simulation settings
	Display_Simul_Setup(p_Total_Num_Simul, p_Replicate, p_Cnt);
	//only simulate resource species before core-species simulation starts
	Burn_In_Rsc(1000);
	//sets the quantity of resource after burn-in period
	Set_Resource_Burn_In();

	//time step
	int t = 0;

	//sets the initial agent
	Set_Initial_Agent();

	//writes initial trait combination data
	Set_Data_Buffer_Trait_Comb_Initial_Population();
	Data_Output_Buffer_Write_Trait_Comb(&t);

	//std::cout << "time steps" << num_Tstep << std::endl;
	//Check_In("time step");

	for (t = 0; t < num_Tstep; t++){
		//the counter for sampling
		counter_Sampling = t % interval_sampling;

		//0. initializes data-out buffers
		Data_Output_Buffer_Initialization(&counter_Sampling, &t);

		//Re-introduction of a core species population
		Set_Reintroduction_Agent(&t);

		//1.0. Setting the density of species
		Set_Spc_Den();

		//resets the resource abundance to the resource abundance after burn-in
		//if the consistancy of the resource clusters is true
		//Reset_Resource_Fixed();

		//1.0. sets the proportion of resource species for each spatial regime
		Set_Prop_Density_Rsc();

		//sets data buffer for resources density at each time step
		Set_Data_Buffer_Density_Tstep(counter_Sampling);
		//sets data buffer for traits at each time step
		Set_Data_Buffer_Trait_Tstep(counter_Sampling);
		//sets data buffer for trait combination
		Set_Data_Buffer_Trait_Comb(&counter_Sampling);
		//sets data buffer for genomes at each time step
		Set_Data_Buffer_Genome_Tstep(counter_Sampling);

		//displays variable values
		//cout << "Simul_Proc: line 1144" << endl;
		//DisplayVariable(&t);

		//switch (case_Deme_Niche){
		//case 0://multiple demes and niches
		//2. reproduction (mating + birth) & movement for new agents
		//Rpd_Mtg_Cls_Mov_Deme_Niche();//random mating within a mating class

		//3. mutation
		//Mut_Bi_Alle_Eq();//bi-alleleic mutation with equal, constant mutation rate

		//4. survival, growth, and movement
		//if (num_RscSpc > 0){
		//Surv_Grw_Trait_Rsc_Mov_Deme_Niche();
		//}//end if

		//break;
		//case 1://multiple demes and a single niche
		//2. reproduction (mating + birth) & movement for new agents
		//Rpd_Mtg_Cls_Mov_Deme();//random mating within a mating class

		//3. mutation
		//Mut_Bi_Alle_Eq();//bi-alleleic mutation with equal, constant mutation rate

		//4. survival, growth, and movement
		//if (num_RscSpc > 0){
		//Surv_Grw_Trait_Rsc_Mov_Deme();
		//}//end if

		//break;
		//case 2://a single deme and multiple niches
		//3. reproduction (mating + birth) & movement for new agents
		//Rpd_Mtg_Cls_Mov_Niche();//random mating within a mating class

		//4. mutation
		//Mut_Bi_Alle_Eq();//bi-alleleic mutation with equal, constant mutation rate

		//5. survival, growth, and movement
		//if (num_RscSpc > 0){
		//Surv_Grw_Trait_Rsc_Mov_Niche();
		//}//end if

		//break;
		//case 3://a single deme and a single niche
		//3. reproduction (mating + birth) & movement for new agents
		//Rpd_Mtg_Cls();//random mating within a mating class

		//4. mutation
		//Mut_Bi_Alle_Eq();//bi-alleleic mutation with equal, constant mutation rate

		//sequence of processes
		//5. survival, growth, and movement
		//Surv_Grw_Trait_Rsc_Discrete();
		//break;
		//default:
		//break;
		//}//end switch

		//Check_In("Before Mating");

		//3. reproduction (mating + birth) & movement for new agents
		//random mating within a mating class
		Rpd_Mtg_Cls_Discrete();

		//verification
		//Check_In("mating done");

		//4. mutation
		Mut_Bi_Alle_Eq();//bi-alleleic mutation with equal, constant mutation rate

		//verification
		//Check_In("mutation done");

		//sequence of processes
		//5. survival, growth, and movement
		Surv_Grw_Trait_Rsc_Discrete();

		//verification
		//Check_In("survival & grow done");

		//1.2. Resource process
		Rsc_Proc();

		//verification
		//Check_In("resource process");

		//if (num_RscSpc > 0){
		//1.2. Resource process
		//Rsc_Proc();
		//}//end if

		//5. updates the agent list. Run this function at the very end of each loop
		Switch_Tick_t();

		//optimizes the counters for memory management
		Opt_Memory_Cnt(&t);

		//calculates statistics
		Data_Output_KolmogorovSmirnov_4D_Mtg_Genotype(&counter_Sampling);
		//Data_Output_KolmogorovSmirnov_2D_Mtg_Bodymass(&counter_Sampling);
		//Data_Output_KolmogorovSmirnov_2D_Mtg_GillRaker(&counter_Sampling);

		//6. writes output data
		Data_Output_Buffer_Write(&counter_Sampling, &t);

		Display_Simul_Status(&t);//display the status of simulation
	}//end time step loop

	Proc_Stat(p_Total_Num_Simul, p_Replicate, p_Cnt);//Processes for statistical analyses

	//DisplayVariable(&t);//displays variable values

	return 0;
}

void Simul_Proc::Set_TimeStep_Reintroduction_Core_Spc(int *p_Status_Re_Introsuction, int *p_TimeStep)
{
	//sets the initial population re-introduction status
	if ((*p_Status_Re_Introsuction) == 1)
	{//re-introduces the initial population 
		status_Reintroduction = true;
	}
	else
	{
		status_Reintroduction = false;
	}//end else


	//time step for re-introducing the initial population
	timeStep_Reintroduction = (*p_TimeStep);
}

void Simul_Proc::Set_Status_Reintroduction_Core_Spc(bool status_Reintroduction)
{
	status_Reintroduction = status_Reintroduction;
}

void Simul_Proc::Set_Reintroduction_Agent(int *p_Current_Tstep)
{
	if (status_Reintroduction == true)
	{//the status of re-introducing a core species is ture
		if ((*p_Current_Tstep) == timeStep_Reintroduction)
		{//if the current time step is the one for re-introducing a core species
			//sets the initial population for re-introducing a core species
			Set_Initial_Agent();
		}//end if
	}//end if
}

void Simul_Proc::Set_Initial_Agent(void){
	//Get the number of agents for the initial population (for each deme/sex/stage/species)
	//Get the initial allele frequency (the number of alleles per locus)
	//int (*deme), (*niche), (*spc), (*sex), (*stg), (*indiv);
	//double tmp_double[8];//a temporary double array
	//int tmp_int[8];//a temporary integer array
	int deme, niche, spc, sex, stg, indiv, end_0;

	for (spc = 0; spc < num_Spc; spc++){
		vector<unsigned short int> gamete_0(num_Locus[spc]);//temporary gamete
		vector<unsigned short int> gamete_1(num_Locus[spc]);//temporary gamete
		vector<int> trait(num_Trait[spc]);



		//(switch (mode_Gene_Trait[spc][5])
		//{
		//case 0:
		//Get_Null_Gamete(gamete_0, &num_Locus[spc], &spc);//sets all alleles to 0
		//Get_Null_Gamete(gamete_1, &num_Locus[spc], &spc);//sets all alleles to 0
		//break;
		//case 1:
		//Get_Full_Gamete(gamete_0, &num_Locus[spc], &spc);//sets all alleles to 1
		//Get_Full_Gamete(gamete_1, &num_Locus[spc], &spc);//sets all alleles to 1
		//break;
		//case 2:
		//Get_Full_Gamete(gamete_0, &num_Locus[spc], &spc);//sets all alleles to 1
		//Get_Null_Gamete(gamete_1, &num_Locus[spc], &spc);//sets all alleles to 0
		//break;
		//case 3:
		//customized randomely drawn bi-allelic frequencies
		//Get_Random_Drawn_Bi_Allelic_Gamete(gamete_0, &num_Locus[spc], &spc, ini_Bi_Allelic_Frequency);
		//Get_Random_Drawn_Bi_Allelic_Gamete(gamete_1, &num_Locus[spc], &spc, ini_Bi_Allelic_Frequency);
		//break;
		//default:
		//Get_Random_Drawn_Bi_Allelic_Gamete(gamete_0, &num_Locus[spc], &spc, ini_Bi_Allelic_Frequency);
		//Get_Random_Drawn_Bi_Allelic_Gamete(gamete_1, &num_Locus[spc], &spc, ini_Bi_Allelic_Frequency);
		//break;
		//}//end switch

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (sex = 0; sex < num_Sex; sex++){
					for (stg = 0; stg < max_Num_Stg; stg++){
						end_0 = initial_Num_Spc[deme][niche][spc][sex][stg];

						for (indiv = 0; indiv < end_0; indiv++){
							Get_Random_Drawn_Bi_Allelic_Gamete(gamete_0, &num_Locus[spc], &spc, ini_Bi_Allelic_Frequency);
							Get_Random_Drawn_Bi_Allelic_Gamete(gamete_1, &num_Locus[spc], &spc, ini_Bi_Allelic_Frequency);

							//adds an agent
							Add_Agent_Unit_IniPop(deme, niche, spc, sex, stg, gamete_0, gamete_1, trait);
						}//end individual loop
					}//end life stage loop
				}//end sex loop
			}//end niche loop
		}//end deme loop
	}//end species loop
}

void Simul_Proc::Remove_Agent(int *p_Deme, int *p_Niche, int *p_Spc, Agent* p_ID, int *p_CelNumM)
{
	//celNum_M[(*p_Spc)]
	Data_Buffer_Agent_Out(p_Deme, p_Niche, (*p_Spc), p_ID);//update data buffer
	agt_M[(*p_Spc)][(*p_CelNumM)] = p_ID;//updates the index number to dead agent list
	(*p_CelNumM)++;//updates the number of cells in the dead agent list
}

void Simul_Proc::Add_Agent_Unit_IniPop(int p_Deme, int p_Niche, int p_Spc, int p_Sex, int p_Stg, vector<unsigned short int> &genome_0_New, vector<unsigned short int> &genome_1_New, vector<int> &trait){
	//Sets traits
	Agent* tmp_ID;//the address of an agent
	Set_Trait(trait, genome_0_New, genome_1_New, p_Spc);
	double tmp_CE;
	//calculate plasticity
	Get_CE_Trait(&tmp_CE, trait, double_Num_Class_Trait_Minus_One[p_Spc], &ce_VB[p_Spc]);

	int tmp_DemNi = (p_Deme * num_Niche) + p_Niche;

	//the total number of alleles controlling bodylength 
	double numAllele_BodyLength = (double)(trt_Loc_Int[p_Spc][0] * num_Position[p_Spc]);

	//temporary age
	//double tmp_Age = ((double)(p_Stg + 1)) / ((double)num_Stg[p_Spc][p_Sex]);
	double tmp_Tot_Num_Stg = (double)num_Stg[p_Spc][p_Sex];
	double tmp_Tot_Num_Stg_BodyLength = num_Stg_BodyLength_Growth[p_Spc][p_Sex];
	double tmp_Age_New = (double)(p_Stg);
	double tmp_BodyLength;
	double tmp_VBC;
	double tmp_VBK_Cumul;
	double tmp_Edd = 1.0;
	double tmp_Ltg;

	if (celNum_M[p_Spc] > 0)
	{//if there is a empty cell in the agent genetic data list
		//new cell number: agt_M[(*p_Spc)][(celNum_M[(*p_Spc)] - 1)]
		tmp_ID = agt_M[p_Spc][(celNum_M[p_Spc] - 1)];
		celNum_M[p_Spc]--;//updates the number of cells in the dead agent list
		tmp_ID->Set_Genome(genome_0_New, genome_1_New, &num_Locus[p_Spc], trait, &num_Trait[p_Spc], &tmp_CE);//sets new genetic data for an agent

		//gets allele dose detetermining bodylength
		tmp_ID->Get_Num_Allele_BodyLength_Double(&tmp_Ltg);
		tmp_Ltg = tmp_Ltg / numAllele_BodyLength;

		//calculates VBC
		Get_VBC_Foraging(&tmp_VBC, &tmp_Ltg, &tmp_Edd, &ltrm, &ltg_VB[p_Spc], &tmp_CE, &ci_VB[p_Spc]);

		//calculates VBK
		Get_Cumulative_VBk(&tmp_VBK_Cumul, &exponent_VonBertallanffy[p_Spc][p_Sex], &tmp_Age_New, &tmp_Tot_Num_Stg_BodyLength);

		//calculate bodylength
		Get_BodyLength_Von_Bertallanffy(&tmp_BodyLength, &bodyLength_Max_Spc[p_Spc], &tmp_VBC, &tmp_VBK_Cumul);

		//set bodylength
		tmp_ID->Set_BodyLength(&tmp_BodyLength);

		//set age
		tmp_ID->Set_Age(&tmp_Age_New);
		
		//sets the constant multiplier for bodylength
		//tmp_ID->Set_Constant_BodyLength_Initial(&bodyLength_Max_Spc[p_Spc], &cost_Reduction_EarlySexMat[p_Spc], &numAllele_BodyLength);

		//sets body length
		//tmp_ID->Set_BodyLength_VonBertallanffy(&exponent_VonBertallanffy[p_Spc][p_Sex], &tmp_Age);

		//sets the bin number for body length
		//tmp_ID->Set_Bin_Number_BodyLength_Unit(&min_BodyLength, &bin_Width_BodyLength[p_Spc]);

		if (tick_t == false)
		{
			if (celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg] < size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])
			{
				agtLst_1[tmp_DemNi][p_Spc][p_Sex][p_Stg][(celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])] = tmp_ID;//updates an agent in the agent list
			}
			else
			{
				agtLst_1[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				agtLst_0[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
			}//end else

			celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
		}
		else
		{
			if (celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg] < size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])
			{
				agtLst_0[tmp_DemNi][p_Spc][p_Sex][p_Stg][(celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])] = tmp_ID;//updates an agent in the agent list
			}
			else
			{
				agtLst_0[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				agtLst_1[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
			}//end else

			celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
		}//end else	
	}
	else
	{
		agt[p_Spc].push_back(new Agent(genome_0_New, genome_1_New, &num_Locus[p_Spc], trait, &num_Trait[p_Spc], &tmp_CE));//adds agent's genetic data
		tmp_ID = agt[p_Spc][(celNum_Agt[p_Spc])];

		//gets allele dose detetermining bodylength
		tmp_ID->Get_Num_Allele_BodyLength_Double(&tmp_Ltg);
		tmp_Ltg = tmp_Ltg / numAllele_BodyLength;

		//calculates VBC
		Get_VBC_Foraging(&tmp_VBC, &tmp_Ltg, &tmp_Edd, &ltrm, &ltg_VB[p_Spc], &tmp_CE, &ci_VB[p_Spc]);

		//calculates VBK
		Get_Cumulative_VBk(&tmp_VBK_Cumul, &exponent_VonBertallanffy[p_Spc][p_Sex], &tmp_Age_New, &tmp_Tot_Num_Stg_BodyLength);

		//calculate bodylength
		Get_BodyLength_Von_Bertallanffy(&tmp_BodyLength, &bodyLength_Max_Spc[p_Spc], &tmp_VBC, &tmp_VBK_Cumul);

		//double tmpMass;
		//Get_BodyMass_From_BodyLength(&tmpMass, &bodyLength_Max_Spc[p_Spc], &p_Spc);
		//std::cout << "Line 1449 max mass " << tmpMass << " max length " << bodyLength_Max_Spc[p_Spc] << std::endl;
		//Get_BodyMass_From_BodyLength(&tmpMass, &tmp_BodyLength, &p_Spc);
		//std::cout << "mass " << tmpMass << " current length " << tmp_BodyLength << std::endl;
		//char tt;
		//std::cin >> tt;

		//set bodylength
		tmp_ID->Set_BodyLength(&tmp_BodyLength);

		//set age
		tmp_ID->Set_Age(&tmp_Age_New);

		//sets the constant multiplier for bodylength
		//tmp_ID->Set_Constant_BodyLength_Initial(&bodyLength_Max_Spc[p_Spc], &cost_Reduction_EarlySexMat[p_Spc], &numAllele_BodyLength);

		//sets body length
		//tmp_ID->Set_BodyLength_VonBertallanffy(&exponent_VonBertallanffy[p_Spc][p_Sex], &tmp_Age);

		//sets the bin number for body length
		//tmp_ID->Set_Bin_Number_BodyLength_Unit(&min_BodyLength, &bin_Width_BodyLength[p_Spc]);

		agt_M[p_Spc].push_back(tmp_ID);//adds a cell in the dead agent list
		celNum_Agt[p_Spc]++;//updates the number of cells in agt_V

		if (tick_t == false)
		{
			if (celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg] < size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]){
				agtLst_1[tmp_DemNi][p_Spc][p_Sex][p_Stg][(celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])] = tmp_ID;//updates an agent in the agent list
			}
			else
			{
				agtLst_1[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				agtLst_0[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
			}//end else

			celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
		}
		else
		{
			if (celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg] < size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])
			{
				agtLst_0[tmp_DemNi][p_Spc][p_Sex][p_Stg][(celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg])] = tmp_ID;//updates an agent in the agent list
			}
			else
			{
				agtLst_0[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				agtLst_1[tmp_DemNi][p_Spc][p_Sex][p_Stg].push_back(tmp_ID);//creates a new cell
				size_AgtLst[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
			}//end else

			celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg]++;//increases the number of living agents
		}//end else		
	}//end else

	//sets the sexual maturation status
	if ((p_Stg) == (num_Stg[p_Spc][p_Sex] - 1))
	{//activates sexual maturation
		tmp_ID->Activate_Status_Sexual_Maturation();
	}
	else
	{//deactivates sexual maturation
		tmp_ID->Reset_Status_Sexual_Maturation();
	}//end else

	//updates the data buffer for agents (Data-IN)
	Data_Buffer_Agent_In(&p_Deme, &p_Niche, p_Spc, tmp_ID);

	//verification
	//double tmp;
	//tmp_ID->Get_BodyLength(&tmp);
	//std::cout << "Line 1660 Simul_Proc::Add_Agent_Unit_IniPop()" << std::endl;
	//std::cout << "maximum bodylength:" << bodyLength_Max_Spc[p_Spc] << std::endl;
	//std::cout << "max " << bodyLength_Max_Spc[p_Spc] << " VBC " << tmp_VBC << " VBK " << tmp_VBK_Cumul << std::endl;
	//std::cout << "bodylength:" << tmp << std::endl;
	//tmp_ID->Get_Age(&tmp);
	//std::cout << "age:" << tmp << std::endl;

	//if (trait[1] > 0)
	//{
	//std::cout << "Gillraker trait IniAgent " << trait[1] << std::endl;
	//Check_In("Line 1660 Simul_Proc::Add_Agent_Unit_IniPop()");
	//}
	//Check_In("Line 1660 Simul_Proc::Add_Agent_Unit_IniPop()");
}

void Simul_Proc::Switch_Tick_t(void)
{
	int deme, niche, spc, sex, stg, end_0;

	if (tick_t == true){
		tick_t = false;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_Spc; spc++){
					for (sex = 0; sex < num_Sex; sex++){
						end_0 = num_Stg[spc][sex];

						for (stg = 0; stg < end_0; stg++){
							celNum_0[deme][niche][spc][sex][stg] = 0;
						}//end life stage loop
					}//end sex loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}
	else{
		tick_t = true;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_Spc; spc++){
					for (sex = 0; sex < num_Sex; sex++){
						end_0 = num_Stg[spc][sex];

						for (stg = 0; stg < end_0; stg++){
							celNum_1[deme][niche][spc][sex][stg] = 0;
						}//end life stage loop
					}//end sex loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}//end else

	for (spc = 0; spc < num_Spc; spc++){
		celNum_New_AgtLst[spc] = 0;
	}//end species loop
}

void Simul_Proc::Transfer_Agent_Unit_Mov_Tick_True(int *p_Deme, int *p_Niche, int *p_Deme_New, int *p_Niche_New, int *p_Spc, int *p_Sex, int *p_Stg, Agent *p_ID)
{
	Data_Buffer_Agent_Out_In(p_Deme, p_Niche, p_Deme_New, p_Niche_New, (*p_Spc), p_ID);//updates data buffer

	if (celNum_1[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)] < size_AgtLst[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)]){
		agtLst_1[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)][(celNum_1[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)])] = p_ID;//updates an agent in the agent list
	}
	else{
		agtLst_1[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)].push_back(p_ID);//creates a new cell
		agtLst_0[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)].push_back(p_ID);//creates a new cell
		size_AgtLst[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)]++;//increases the number of living agents
	}//end else

	celNum_1[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)]++;//increases the number of living agents
}

void Simul_Proc::Transfer_Agent_Unit_Mov_Tick_False(int *p_Deme, int *p_Niche, int *p_Deme_New, int *p_Niche_New, int *p_Spc, int *p_Sex, int *p_Stg, Agent *p_ID)
{
	Data_Buffer_Agent_Out_In(p_Deme, p_Niche, p_Deme_New, p_Niche_New, (*p_Spc), p_ID);//updates data buffer

	if (celNum_0[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)] < size_AgtLst[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)]){
		agtLst_0[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)][(celNum_0[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)])] = p_ID;//updates an agent in the agent list
	}
	else{
		agtLst_0[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)].push_back(p_ID);//creates a new cell
		agtLst_1[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)].push_back(p_ID);//creates a new cell
		size_AgtLst[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)]++;//increases the number of living agents
	}//end else

	celNum_0[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)]++;//increases the number of living agents
}

void Simul_Proc::Transfer_Agent_Unit(vector<Agent*> &p_Agt_Sink, vector<Agent*> &p_Agt_Source, Agent *p_ID, int *p_CelNum, int *p_Size_AgtLst)
{
	//int *p_CelNum = &celNum_1[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)];
	//int *p_Size_AgtLst = &size_AgtLst[(*p_Deme_New)][(*p_Niche_New)][(*p_Spc)][(*p_Sex)][(*p_Stg)];
	//vector<Agent*> &p_Agt_Sink = agtLst_1[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)];
	//vector<Agent*> &p_Agt_Source = agtLst_0[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)];

	if ((*p_CelNum) < (*p_Size_AgtLst))
	{
		p_Agt_Sink[(*p_CelNum)] = p_ID;
		//agtLst_1[(((*p_Deme_New) * num_Niche) + (*p_Niche_New))][(*p_Spc)][(*p_Sex)][(*p_Stg)][(*p_CelNum)] = p_ID;//updates an agent in the agent list
	}
	else
	{
		p_Agt_Sink.push_back(p_ID);//creates a new cell
		p_Agt_Source.push_back(p_ID);//creates a new cell
		(*p_Size_AgtLst)++;//increases the number of living agents
	}//end else

	(*p_CelNum)++;//increases the number of living agents
}

void Simul_Proc::Get_Null_Gamete(vector<unsigned short int> &gamete, int *p_num_Locus, int *p_Spc)
{//generates gamete with allele #0
	int locus;

	for (locus = 0; locus < (*p_num_Locus); locus++)
	{
		if (loc_Trt[(*p_Spc)][locus] == 0)
		{//for upper-bound trait
			gamete[locus] = 0;
		}
		else if (loc_Trt[(*p_Spc)][locus] == 1)
		{//for lower-bound trait
			gamete[locus] = 1;
		}
		else
		{
			gamete[locus] = 0;
		}//end else		
	}//end locus loop
}

void Simul_Proc::Set_Ini_Allele_Frequency(vector<vector<double>> &p_Ini_Bi_Allelic_Frequency)
{//vector<vector<double>> &p_Ini_Bi_Allelic_Frequency: dim[species, locus]
	int nd_0 = (int)p_Ini_Bi_Allelic_Frequency.size();
	int nd_1;
	int i, j;

	for (i = 0; i < nd_0; i++)
	{
		ini_Bi_Allelic_Frequency.push_back(vector<double>());//adds a cell for a species dimension
		nd_1 = (int)p_Ini_Bi_Allelic_Frequency[i].size();//gets the number of loci

		for (j = 0; j < nd_1; j++)
		{
			//sets a bi-allelic frequency
			ini_Bi_Allelic_Frequency[i].push_back(p_Ini_Bi_Allelic_Frequency[i][j]);
		}//end locus loop
	}//end species loop
}

void Simul_Proc::Get_Full_Gamete(vector<unsigned short int> &gamete, int *p_num_Locus, int *p_Spc)
{//generates gamete with allele #0
	int locus;

	for (locus = 0; locus < (*p_num_Locus); locus++){
		if (loc_Trt[(*p_Spc)][locus] == 0)
		{//for upper-bound trait
			gamete[locus] = 1;
		}
		else if (loc_Trt[(*p_Spc)][locus] == 1)
		{//for lower-bound trait
			gamete[locus] = 0;
		}
		else
		{
			gamete[locus] = 1;
		}//end else
	}//end locus loop
}

void Simul_Proc::Get_Random_Drawn_Bi_Allelic_Gamete(vector<unsigned short int> &gamete, int *p_num_Locus, int *p_Spc, vector<vector<double>> &p_Bi_Allelic_Frequency)
{//generates gamete with allele #0
	//vector<double> &p_Bi_Allelic_Frequency, dim[species, locus]: vector containing bi-allelic frequencies (i.e. 0: 0.0, 1:1.0)
	for (int locus = 0; locus < (*p_num_Locus); locus++){
		//draws a uniform random deviate

		if (((*rand_Uni_Real)()) < p_Bi_Allelic_Frequency[(*p_Spc)][locus])
		{
			gamete[locus] = 1;
		}
		else
		{
			gamete[locus] = 0;
		}//end else
	}//end locus loop
}

void Simul_Proc::LotkaVolterra_Competition_Discrete(double *h, double *den_Tgt, double *itr_Grw, double *d_CmtSpc, double *coef_Cmt, int *num_Competing_Species)
{
	//h: unit time step
	//den_Tgt: the density of target species at n
	//itr_Grw: the intrinsic growth rate of the target species
	//d_CmtSpc: dim[species]; the densities of competing species; all numbers are positive
	//coef_Cmt: dim[species]; the coefficients for the densities of competing species; all numbers are positive
	//num_Competing_Species: the number of competing species
	int spc;
	double cmt_Cmp = 0.0;//competition component

	for (spc = 0; spc < (*num_Competing_Species); spc++){
		cmt_Cmp = cmt_Cmp + (coef_Cmt[spc] * d_CmtSpc[spc]);
	}//end competing species loop

	//sets the density of the target species(*den_Tgt) at n + 1
	//Liu & Elaydi (2001), Leslie (1958) and Mickens(1989, 1990, 1994)
	//See Liu & Elaydi (2001) for the references
	(*den_Tgt) = ((1.0 + ((*itr_Grw) * (*h))) * (*den_Tgt)) / (1.0 + ((*h) * cmt_Cmp));
}

void Simul_Proc::Get_Num_Binom_Dist(int *num_Mut_Per_Tstep, int *max_Num_Events, double *p_MutationRate)
{
	//num_Mut_Per_Tstep: the number of mutation events for a species in the population per time step
	//max_Num_Events = (the number of individuals for a given time step) * (the number of loci per individual) * (the number of positions per locus)
	binomial_distribution<int> binom((*max_Num_Events), (*p_MutationRate));//binomial distribution
	(*num_Mut_Per_Tstep) = binom.operator ()((*randomSeed_Mut));//draws a random deviate from binomial distribution
}

void Simul_Proc::Add_Mut_Agent_Unit(int *p_Deme, int *p_Niche, int *p_Spc, vector<unsigned short int> &genome_0_New, vector<unsigned short int> &genome_1_New, vector<int> &trait, int *p_CellNum, int *p_Size_Agt, vector<Agent*> &p_Agt_Sink, vector<Agent*> &p_Agt_Source, int *p_CelNum_M, int *p_celNum_New_AgtLst, bool *p_Status_SexualMaturation)
{
	//Sets traits
	Agent* tmp_ID;
	Set_Trait(trait, genome_0_New, genome_1_New, (*p_Spc));

	double tmp_CE;
	//calculate plasticity
	Get_CE_Trait(&tmp_CE, trait, double_Num_Class_Trait_Minus_One[(*p_Spc)], &ce_VB[(*p_Spc)]);

	if ((*p_CelNum_M) > 0)
	{//if there is a empty cell in the agent genetic data list
		tmp_ID = agt_M[(*p_Spc)][((*p_CelNum_M) - 1)];//new cell number: agt_M[(*p_Spc)][(celNum_M[(*p_Spc)] - 1)]
		(*p_CelNum_M)--;//updates the number of cells in the dead agent list
		tmp_ID->Set_Genome(genome_0_New, genome_1_New, &num_Locus[(*p_Spc)], trait, &num_Trait[(*p_Spc)], &tmp_CE);//sets new genetic data for an agent

		//set bodylength to 0.0
		tmp_ID->Set_BodyLength_Null();

		//set age to 0.0
		tmp_ID->Set_Age_Zero();

		if ((*p_CellNum) < (*p_Size_Agt))
		{
			p_Agt_Sink[(*p_CellNum)] = tmp_ID;//updates an agent in the agent list
		}
		else
		{
			p_Agt_Sink.push_back(tmp_ID);//creates a new cell
			p_Agt_Source.push_back(tmp_ID);//creates a new cell
			(*p_Size_Agt)++;//increases the number of living agents
		}//end else
	}
	else
	{
		agt[(*p_Spc)].push_back(new Agent(genome_0_New, genome_1_New, &num_Locus[(*p_Spc)], trait, &num_Trait[(*p_Spc)], &tmp_CE));//adds agent's genetic data
		tmp_ID = agt[(*p_Spc)][(celNum_Agt[(*p_Spc)])];

		//set bodylength to 0.0
		tmp_ID->Set_BodyLength_Null();

		//set age to 0.0
		tmp_ID->Set_Age_Zero();

		agt_M[(*p_Spc)].push_back(tmp_ID);//adds a cell in the dead agent list
		celNum_Agt[(*p_Spc)]++;//updates the number of cells in agt_V

		if ((*p_CellNum) < (*p_Size_Agt))
		{
			p_Agt_Sink[(*p_CellNum)] = tmp_ID;//updates an agent in the agent list
		}
		else
		{
			p_Agt_Sink.push_back(tmp_ID);//creates a new cell
			p_Agt_Source.push_back(tmp_ID);//creates a new cell
			(*p_Size_Agt)++;//increases the number of living agents
		}//end else
	}//end else

	//resets the sexual maturation status to 'Not sexually matured'
	tmp_ID->Set_Status_Sexual_Maturation(p_Status_SexualMaturation);

	(*p_CellNum)++;//increases the number of living agents

	//add the agent to the new_AgtLst for the pre-zygotic processes
	if ((*p_celNum_New_AgtLst) < size_New_AgtLst[(*p_Spc)])
	{//this is enough cells in new_AgtLst
		new_AgtLst[(*p_Spc)][(*p_celNum_New_AgtLst)] = tmp_ID;//sets a index number in new_AgtLst
		new_AgtLst_Deme[(*p_Spc)][(*p_celNum_New_AgtLst)] = (*p_Deme);
		new_AgtLst_Niche[(*p_Spc)][(*p_celNum_New_AgtLst)] = (*p_Niche);
	}
	else
	{//this is not enough cells in new_AgtLst
		new_AgtLst[(*p_Spc)].push_back(tmp_ID);//adds a new cell in new_AgtLst
		new_AgtLst_Deme[(*p_Spc)].push_back((*p_Deme));//adds a new cell in new_AgtLst
		new_AgtLst_Niche[(*p_Spc)].push_back((*p_Niche));//adds a new cell in new_AgtLst
		size_New_AgtLst[(*p_Spc)]++;//increases the total number of cells in new_AgtLst
	}//end else

	(*p_celNum_New_AgtLst)++;//increases the indicator for new_AgtLst

	//updates the data buffer for agents (Data-IN)
	Data_Buffer_Agent_In(p_Deme, p_Niche, (*p_Spc), tmp_ID);
}

void Simul_Proc::Mut_Bi_Alle_Eq(void)
{
	int spc, tot_Allele, num_Mutations_Tstep, indiv, event_0, locus, position, end_Loc, tmp_NumAgt, numAllele_BodyLength;
	double tmp_CE;
	double randDev, tmp_NumPos, tmp_CelNum_New_AgtLst;
	vector<double> resource_Use(num_RscSpc);
	vector<double> resource_Preference(num_RscSpc);
	Agent *tmp_ID;//temporary agent address

	for (spc = 0; spc < num_Spc; spc++){
		//the total number of alleles controlling bodylength 
		numAllele_BodyLength = (trt_Loc_Int[spc][0] * num_Position[spc]);
		tmp_NumPos = ((double)num_Position[spc]);

		//the number of new agents 
		tmp_CelNum_New_AgtLst = ((double)celNum_New_AgtLst[spc]);
		tmp_NumAgt = celNum_New_AgtLst[spc];

		//verification
		//std::cout << "Line 3795: The number of new agents " << tmp_NumAgt<<std::endl;
		//Check_In("mutation");

		vector<int> trait(num_Trait[spc]);

		tot_Allele = tmp_NumAgt  * num_Position[spc];

		if (tmp_NumAgt > 1){//multiple new agents
			end_Loc = num_Locus[spc];

			for (locus = 0; locus < end_Loc; locus++){
				//gets the number of mutations
				Get_Num_Binom_Dist(&num_Mutations_Tstep, &tot_Allele, &mut_Rat[spc][locus]);

				//verification
				//std::cout << "Line 3795: The number of mutations " << num_Mutations_Tstep << " Locus "<< locus<< std::endl;
				//std::cout << "Line 3795: mutation rate " << mut_Rat[spc][locus] << std::endl;
				//Check_In("mutation");

				if (num_Mutations_Tstep > 0){//if there is a mutation event
					for (event_0 = 0; event_0 < num_Mutations_Tstep; event_0++){
						randDev = (*rand_Uni_Real)();
						indiv = (int)(tmp_CelNum_New_AgtLst * randDev);//randomly chooses an individual
						randDev = (*rand_Uni_Real)();
						position = (int)(tmp_NumPos * randDev);//randomly chooses a position

						//Chosen agent
						tmp_ID = new_AgtLst[spc][indiv];

						//removes old agent data points
						Data_Buffer_Agent_Out(&new_AgtLst_Deme[spc][indiv], &new_AgtLst_Niche[spc][indiv], spc, tmp_ID);

						//Genetic mutation
						tmp_ID->Bi_Allele_Flip(&locus, &position);

						//re-sets trait after mutation
						Set_Trait(trait, spc, tmp_ID);

						//calculate plasticity
						Get_CE_Trait(&tmp_CE, trait, double_Num_Class_Trait_Minus_One[spc], &ce_VB[spc]);

						//re-sets traits
						tmp_ID->Set_Trait(&num_Locus[spc], trait, &num_Trait[spc], &tmp_CE);

						//adds old agent data points
						Data_Buffer_Agent_In(&new_AgtLst_Deme[spc][indiv], &new_AgtLst_Niche[spc][indiv], spc, tmp_ID);

						//verification
						//Check_In("mutation called");
					}//end mutation event loop				
				}//end else
			}//end locus loop
		}
		else if (tmp_NumAgt == 1){//a single new born agent
			indiv = 0;

			for (locus = 0; locus < num_Locus[spc]; locus++){
				//gets the number of mutations
				Get_Num_Binom_Dist(&num_Mutations_Tstep, &tot_Allele, &mut_Rat[spc][locus]);

				if (num_Mutations_Tstep > 0){//if there is a mutation event
					for (event_0 = 0; event_0 < num_Mutations_Tstep; event_0++){
						randDev = (*rand_Uni_Real)();
						position = (int)(tmp_NumPos* randDev);//randomly chooses a position

						//chosen agent
						tmp_ID = new_AgtLst[spc][indiv];

						//removes old agent data points
						Data_Buffer_Agent_Out(&new_AgtLst_Deme[spc][indiv], &new_AgtLst_Niche[spc][indiv], spc, tmp_ID);

						//Genetic mutation
						tmp_ID->Bi_Allele_Flip(&locus, &position);

						//re-sets trait after mutation
						Set_Trait(trait, spc, tmp_ID);

						//calculate plasticity
						Get_CE_Trait(&tmp_CE, trait, double_Num_Class_Trait_Minus_One[spc], &ce_VB[spc]);

						//re-sets traits
						tmp_ID->Set_Trait(&num_Locus[spc], trait, &num_Trait[spc], &tmp_CE);

						//adds old agent data points
						Data_Buffer_Agent_In(&new_AgtLst_Deme[spc][indiv], &new_AgtLst_Niche[spc][indiv], spc, tmp_ID);

						//verification
						//Check_In("mutation called");
					}//end mutation event loop					
				}//end else
			}//end locus loop
		}
		else
		{

		}//end else
	}//end species

	//verification
	//Check_In("mutation done");
}

void Simul_Proc::Local_Aggregate_Prc_Discrete(int *p_Deme, int *p_Niche)
{
	//sets local agent numbers
	Set_CelNum_Spc_Trt_Local_Discrete((*p_Deme), (*p_Niche));
	//sets the number of indivuduals competing to acquire a class of a resource species
	Set_Cmt_Spc_Rsc_Pheotype_Cls_Local_Discrete((*p_Deme), (*p_Niche));
}

void Simul_Proc::Get_Sexual_Maturation_Group_State(double *p_sexMat_Current, double *p_sexMat_Next, bool *null_Sexual_Maturation_Activation_Current, bool *full_Sexual_Maturation_Activation_Current, bool *null_Sexual_Maturation_Activation_Next, bool *full_Sexual_Maturation_Activation_Next)
{//returns the group status of the possibility of sexual maturation
	if ((*p_sexMat_Current) == 0.0)
	{
		(*null_Sexual_Maturation_Activation_Current) = true;
	}
	else
	{
		(*null_Sexual_Maturation_Activation_Current) = false;
	}

	if ((*p_sexMat_Current) == 1.0)
	{
		(*full_Sexual_Maturation_Activation_Current) = true;
	}
	else
	{
		(*full_Sexual_Maturation_Activation_Current) = false;
	}

	if ((*p_sexMat_Next) == 0.0)
	{
		(*null_Sexual_Maturation_Activation_Next) = true;
	}
	else
	{
		(*null_Sexual_Maturation_Activation_Next) = false;
	}

	if ((*p_sexMat_Next) == 1.0)
	{
		(*full_Sexual_Maturation_Activation_Next) = true;
	}
	else
	{
		(*full_Sexual_Maturation_Activation_Next) = false;
	}
}

void Simul_Proc::Rsc_Proc(void)
{
	//LotkaVolterra_Competition_Discrete_Resource(const int *p_RscSpc, const int *p_Deme, const int *p_Niche, const double *h, double *den_Tgt, const double ***itr_Grw, const double ***d_CmtSpc, const double **coef_Cmt)
	//h: unit time step
	//den_Tgt: the density of target species at n
	//itr_Grw: the intrinsic growth rate of the target species
	//d_CmtSpc: dim[species]; the densities of competing species; all numbers are positive
	//coef_Cmt: dim[species]; the coefficients for the densities of competing species; all numbers are positive
	//num_Competing_Species: the number of competing species
	int deme, niche, rsc;

	//Competition & exploitation in Resource
	for (rsc = 0; rsc < num_RscSpc; rsc++){
		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche< num_Niche; niche++){
				LV_Compet_Discrete_Rsc(rsc, deme, niche, unit_T_Step);
			}//end niche loop
		}//end deme loop
	}//end resource species loop
}

void Simul_Proc::Set_Prop_Density_Rsc(void)
{//calculates the proportion of resource species
	int deme, niche, rsc, rCls, end0;
	double tmp_Sum;

	//calculates the proportion of resource species class for each spatial unit 
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			tmp_Sum = 0.0;//initialized the total mass of all resource species

			for (rsc = 0; rsc < num_RscSpc; rsc++){
				end0 = num_PhenoTyp_Rsc[rsc];

				for (rCls = 0; rCls < end0; rCls++){
					tmp_Sum += phen_D_Rsc[deme][niche][rsc][rCls];//initialization
				}//end resource species class loop
			}//end resource loop

			if (tmp_Sum > 0.0)
			{
				for (rsc = 0; rsc < num_RscSpc; rsc++){
					end0 = num_PhenoTyp_Rsc[rsc];

					for (rCls = 0; rCls < end0; rCls++){
						prop_Density_Rsc[deme][niche][rsc][rCls] = phen_D_Rsc[deme][niche][rsc][rCls] / tmp_Sum;//initialization

						//std::cout << prop_Density_Rsc[deme][niche][rsc][rCls] << std::endl;
					}//end resource species class loop
				}//end resource loop
			}
			else
			{
				for (rsc = 0; rsc < num_RscSpc; rsc++){
					end0 = num_PhenoTyp_Rsc[rsc];

					for (rCls = 0; rCls < end0; rCls++){
						prop_Density_Rsc[deme][niche][rsc][rCls] = 0.0;//initialization
					}//end resource species class loop
				}//end resource loop
			}//end else
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Spc_Den(void)
{
	int deme, niche, spc, stg, sex, tmp, tmpStg;

	//calculates the abundance of species except resource species
	if (tick_t == true){
		for (spc = 0; spc < num_Spc; spc++){
			for (deme = 0; deme < num_Deme; deme++){
				for (niche = 0; niche < num_Niche; niche++){
					tmp = 0;//reset the container

					for (sex = 0; sex < num_Sex; sex++){
						tmpStg = num_Stg[spc][sex];//species specific stage

						for (stg = 0; stg < tmpStg; stg++){
							tmp += celNum_0[deme][niche][spc][sex][stg];
						}//end stage loop
					}//end sex loop

					//sets the abundance of a species for a niche
					density_Spc[deme][niche][spc] = (double)tmp;

					//std::cout << density_Spc[deme][niche][spc] << std::endl;
				}//end niche loop
			}//end deme loop
		}//end species loop
	}
	else{
		for (spc = 0; spc < num_Spc; spc++){
			for (deme = 0; deme < num_Deme; deme++){
				for (niche = 0; niche < num_Niche; niche++){
					tmp = 0;//reset the container

					for (sex = 0; sex < num_Sex; sex++){
						tmpStg = num_Stg[spc][sex];
						for (stg = 0; stg < tmpStg; stg++){
							tmp += celNum_1[deme][niche][spc][sex][stg];
						}//end stage loop
					}//end sex loop

					//sets the abundance of a species for a niche
					density_Spc[deme][niche][spc] = (double)tmp;

					//std::cout << density_Spc[deme][niche][spc] << std::endl;
				}//end niche loop
			}//end deme loop
		}//end species loop
	}//end else
}

void Simul_Proc::Set_Data_Rsc_Density_Tstep(void)
{
	int deme, niche, spc;

	//sets the abundance of resource species
	if (num_RscSpc > 0){
		for (spc = 0; spc < num_RscSpc; spc++){
			for (deme = 0; deme < num_Deme; deme++){
				for (niche = 0; niche < num_Niche; niche++){
					density_Rsc[deme][niche][spc] = d_Rsc[deme][niche][spc];
				}//end niche loop
			}//end deme loop
		}//end species loop
	}//end if
}

void Simul_Proc::Set_Data_Buffer_Density_Tstep(int p_Counter_Sampling)
{
	int deme, niche, spc, cls, end0;

	//sets the abundance of resource species
	if (num_RscSpc > 0){
		for (spc = 0; spc < num_RscSpc; spc++)
		{
			for (deme = 0; deme < num_Deme; deme++)
			{
				for (niche = 0; niche < num_Niche; niche++)
				{
					density_Rsc[deme][niche][spc] = d_Rsc[deme][niche][spc];
				}//end niche loop
			}//end deme loop
		}//end species loop

		for (deme = 0; deme < num_Deme; deme++)
		{
			for (niche = 0; niche < num_Niche; niche++)
			{
				for (spc = 0; spc < num_RscSpc; spc++)
				{
					end0 = num_PhenoTyp_Rsc[spc];

					for (cls = 0; cls < end0; cls++)
					{
						data_Buffer_RscCls[p_Counter_Sampling][deme][niche][spc][cls] = phen_D_Rsc[deme][niche][spc][cls];
					}//end resource class loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}//end if
}

void Simul_Proc::Burn_In_Rsc(int p_N_Burn_In)
{
	//p_N_Burn_In: the number of time steps for resource species burn-in
	if (num_RscSpc > 0){
		int t;

		for (t = 0; t < p_N_Burn_In; t++)
		{
			//sets the densities of resource species for each time step
			Set_Data_Rsc_Density_Tstep();
			//1.2. Resource process
			Rsc_Proc();
		}//end time step loop
	}//end if
}

void Simul_Proc::LV_Compet_Discrete_Rsc(int p_RscSpc, int p_Deme, int p_Niche, double h)
{
	//h: unit time step
	//den_Tgt: the density of target species at n
	//itr_Grw: the intrinsic growth rate of the target species
	//d_CmtSpc: dim[deme, niche, species]; the densities of competing species; all numbers are positive
	//ed_CmtSpc: dim[deme, niche, species, resource species]; the effective densities of competing species; all numbers are positive
	//coef_Cmt: dim[species, species]; the coefficients for the densities of competing species; all numbers are positive
	//num_Competing_Species: the number of competing species
	//p_RscSpc: resource species number
	int spc, tmp_Rsc_Target, pheTyp_Cls;//phenotypic class of a resource species
	double tmp_Cmt_Cmp, tmp_Gain_Cmp, tmp, tmpRate, tmp_Density, tmp_Attack_Rate;//competition component
	double tmp_Mut = mut_Rat_Rsc[p_RscSpc];//mutation rate
	int end0 = num_PhenoTyp_Rsc[p_RscSpc];
	double tp = (double)end0;//the number of phenotypes
	int end1 = num_RscSpc + num_Spc;
	int end2 = p_RscSpc + num_Spc;

	//the density of a given resource cluster
	double tmp_Density_Self_Cluster = density_Rsc[(p_Deme)][(p_Niche)][p_RscSpc];
	//population growth loss component
	double cmt_Cmp = 0.0;
	//population growth gain component
	double gain_Cmp = 0.0;

	//competition among resource species
	for (spc = num_Spc; spc < end1; spc++){
		tmp_Rsc_Target = spc - num_Spc;
		//density of  a given resource
		tmp_Density = density_Rsc[(p_Deme)][(p_Niche)][tmp_Rsc_Target];

		//loss
		//functional response
		if (spc != end2)
		{//inter-specific effect
			FunctionalResponse_Rate(&tmpRate, &LV_Loss[p_Deme][p_Niche][end2][spc], &th_Rsc_RscSpc[p_Deme][p_Niche][tmp_Rsc_Target][p_RscSpc], &tmp_Density_Self_Cluster, &q_Rsc_RscSpc[p_Deme][p_Niche][tmp_Rsc_Target][p_RscSpc]);//returns the rate of prey attacked
			cmt_Cmp += (tmpRate * tmp_Density);

			//cout << "line 6359" << endl;
			//cout << "rate " << tmpRate << " Density: " << tmp_Density << " Loss: " << cmt_Cmp << endl;

			//use the attack history to calculate the degree of gain
			tmp_Attack_Rate = LV_Loss[p_Deme][p_Niche][spc][end2];

			if (tmp_Attack_Rate > 0.0)
			{
				FunctionalResponse_Rate(&tmpRate, &tmp_Attack_Rate, &th_Rsc_RscSpc[p_Deme][p_Niche][p_RscSpc][tmp_Rsc_Target], &tmp_Density, &q_Rsc_RscSpc[p_Deme][p_Niche][p_RscSpc][tmp_Rsc_Target]);//returns the rate of prey attacked
				tmpRate = tmpRate * LV_Gain[p_Deme][p_Niche][end2][spc];//reflect the reduction in attack rate to the gain
				gain_Cmp += (tmpRate * tmp_Density);
			}//end if

			//cout << "line 6380" << endl;
			//cout << "rate " << tmpRate << " Resource #1 " << p_RscSpc << " Resource #2 " << tmp_Rsc_Target<<endl;
			//cout << "Density self: " << tmp_Density_Self_Cluster << " Density Target:" << tmp_Density<<endl;
			//cout << "LV Loss: " << LV_Loss[p_Deme][p_Niche][end2][spc] << " LV Gain:" << LV_Gain[p_Deme][p_Niche][end2][spc]<<endl;
			//cout << "Loss: " << cmt_Cmp << " Gain:" << gain_Cmp;
			//char tt;
			//cin>>tt;
		}
		else
		{//intra-specific effect
			//loss
			cmt_Cmp += (LV_Loss[p_Deme][p_Niche][end2][spc] * tmp_Density);
			//cout << "line 6347" << endl;
			//cout << " Self Loss rate: " << LV_Loss[p_Deme][p_Niche][end2][spc] << " Self-Density: " << tmp_Density << endl;
		}//end else			
	}//end competing species loop

	double *tmp_Phen_D_Rsc = phen_D_Rsc[p_Deme][p_Niche][p_RscSpc];

	for (pheTyp_Cls = 0; pheTyp_Cls < end0; pheTyp_Cls++){
		//interaction between agent species and a resource class
		tmp_Cmt_Cmp = cmt_Cmp;
		tmp_Gain_Cmp = gain_Cmp;

		for (spc = 0; spc < num_Spc; spc++){
			tmp_Density = sum_Spc_Rsc[p_Deme][p_Niche][spc][p_RscSpc][pheTyp_Cls];
			//Loss
			tmp_Cmt_Cmp += (LV_Loss[p_Deme][p_Niche][end2][spc] * tmp_Density);
			//gain
			tmp_Gain_Cmp += (LV_Gain[p_Deme][p_Niche][end2][spc] * tmp_Density);

			//for verification
			//if (tmp_Density > 0.0)
			//{
			//cout << "line 6404" << endl;
			//cout << "Resource: " << p_RscSpc << " Phenotype:" << pheTyp_Cls<<" Core Species: "<< spc<<endl;
			//cout << "Density Core Species: " << tmp_Density<<endl;
			//cout << "Culumative loss:" << cmt_Cmp << endl;
			//cout << "Culumative gain:" << gain_Cmp << endl;
			//cout << "Compund loss:" << (LV_Loss[p_Deme][p_Niche][end2][spc] * tmp_Density) << endl;
			//cout << "Compund gain:" << (LV_Gain[p_Deme][p_Niche][end2][spc] * tmp_Density) << endl;
			//cout << "LV Loss: " << LV_Loss[p_Deme][p_Niche][end2][spc] << " LV Gain: " << LV_Gain[p_Deme][p_Niche][end2][spc] << endl;
			//char tt;
			//cin>>tt;
			//}//end if
		}//end species loop

		//sets the density of the target species(*den_Tgt) at n + 1
		//Liu & Elaydi (2001), Leslie (1958) and Mickens(1989, 1990, 1994)
		//See Liu & Elaydi (2001) for the references

		//sets resource species population growth rate
		growthRate_Rsc[pheTyp_Cls] = (1.0 + (intrinsic_Rsc[(p_Deme)][(p_Niche)][(p_RscSpc)] + tmp_Gain_Cmp) * h) / (1.0 + (h* tmp_Cmt_Cmp));
		//sets the density at t-1 
		tmp_Rsc_Cls[pheTyp_Cls] = tmp_Phen_D_Rsc[pheTyp_Cls];
	}//end for loop


	//calculates the density of phenotypes reflecting genetic mutation
	//assuming one mutation event for each genotype
	for (pheTyp_Cls = 0; pheTyp_Cls < end0; pheTyp_Cls++){
		if (pheTyp_Cls > 0)
		{
			if (pheTyp_Cls < (end0 - 1))
			{
				tmp = (growthRate_Rsc[pheTyp_Cls] * (1.0 - tmp_Mut) * tmp_Rsc_Cls[pheTyp_Cls]) + (growthRate_Rsc[(pheTyp_Cls - 1)] * (((double)(pheTyp_Cls - 1)) / tp) * tmp_Mut * tmp_Rsc_Cls[(pheTyp_Cls - 1)]) + (growthRate_Rsc[(pheTyp_Cls + 1)] * (1.0 - (((double)(pheTyp_Cls + 1)) / tp)) * tmp_Mut * tmp_Rsc_Cls[(pheTyp_Cls + 1)]);
			}
			else
			{//pheTyp_Cls == (end0 - 1): Full Mutant
				tmp = (growthRate_Rsc[pheTyp_Cls] * (1.0 - tmp_Mut) * tmp_Rsc_Cls[pheTyp_Cls]) + (growthRate_Rsc[(pheTyp_Cls - 1)] * (((double)(pheTyp_Cls - 1)) / tp) * tmp_Mut * tmp_Rsc_Cls[(pheTyp_Cls - 1)]);
			}//end else
		}
		else
		{//pheTyp_Cls == 0: Wild Type
			tmp = (growthRate_Rsc[pheTyp_Cls] * (1.0 - tmp_Mut) * tmp_Rsc_Cls[pheTyp_Cls]) + (growthRate_Rsc[(pheTyp_Cls + 1)] * (1.0 - (((double)(pheTyp_Cls + 1)) / tp)) * tmp_Mut * tmp_Rsc_Cls[(pheTyp_Cls + 1)]);
		}//end else

		tmp_Phen_D_Rsc[pheTyp_Cls] = tmp;
	}//end for loop

	double *tmp_D_Rsc = &d_Rsc[p_Deme][p_Niche][p_RscSpc];
	//initialization
	(*tmp_D_Rsc) = 0.0;

	//calculates the density of phenotypes reflecting genetic mutation
	for (pheTyp_Cls = 0; pheTyp_Cls < end0; pheTyp_Cls++){
		//calculates the density of phenotypes reflecting genetic mutation
		(*tmp_D_Rsc) += tmp_Phen_D_Rsc[pheTyp_Cls];//updates the total resource density
	}//end for loop

	//verification
	//calculates the density of phenotypes reflecting genetic mutation
	//std::cout << "resource " << p_RscSpc << " density " << (*tmp_D_Rsc) << std::endl;
	//Check_In("ok");
}

void Simul_Proc::Beverton_Holt_DenDep(double *p_previous_Density, double *p_DenDepSurRat, double *p_C, double *p_b_MassIntake_DDS){
	if ((*p_previous_Density) > 0.0)
	{//(*p_b_MassIntake_DDS)
		//for verification
		//cout << "Line 6379" << endl;
		//cout << " Beverton-Holt: C: " << (*p_C) << " Previous density: " << (*p_previous_Density)<<endl;
		//char tt;
		//cin >> tt;

		//with compensation
		(*p_DenDepSurRat) = 1.0 / (1.0 + ((pow(((*p_C) *(*p_previous_Density)), (*p_b_MassIntake_DDS)))));

		//without compensation
		//(*p_DenDepSurRat) = 1.0 / (1.0 + ((*p_C) *(*p_previous_Density)));
	}
	else
	{
		if ((*p_b_MassIntake_DDS) == 0.0)
		{
			//(*p_DenDepSurRat) = 1.0 / (1.0 + 0.0^0.0);
			(*p_DenDepSurRat) = 0.5;
		}
		else
		{
			(*p_DenDepSurRat) = 1.0;
		}//end else
	}//end else
}

void Simul_Proc::Ricker_DenDep(double *p_previous_Density, double *p_DenDepSurRat, double *p_C)
{
	(*p_DenDepSurRat) = exp((-1.0) * (*p_C) * (*p_previous_Density));
}

void Simul_Proc::Set_Gen_Struct(int *p_Num_Locus, int *p_Num_Position, double **p_MutRat, int *p_Num_Trait)
{
	int spc, locus, i;
	num_Locus = new int[num_Spc];
	max_Num_Locus = 0;

	for (spc = 0; spc < num_Spc; spc++){
		num_Locus[spc] = p_Num_Locus[spc];

		if (num_Locus[spc] > max_Num_Locus){
			max_Num_Locus = num_Locus[spc];
		}//end if
	}//end species loop

	num_Position = new int[num_Spc];
	max_Num_Position = 0;

	for (spc = 0; spc < num_Spc; spc++){
		num_Position[spc] = p_Num_Position[spc];

		if (num_Position[spc] > max_Num_Position){
			max_Num_Position = num_Position[spc];
		}//end if
	}//end species loop

	num_Trait = new int[num_Spc];

	max_Num_Trait = 0;

	for (spc = 0; spc < num_Spc; spc++){
		num_Trait[spc] = p_Num_Trait[spc];

		if (num_Trait[spc] > max_Num_Trait){
			max_Num_Trait = num_Trait[spc];
		}//end if
	}//end species loop

	multiplier_Trait_Combination = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		multiplier_Trait_Combination[spc] = new int[m_Num_Traits_Foraging];

		for (i = 0; i < m_Num_Traits_Foraging; i++){
			multiplier_Trait_Combination[spc][i] = 0;
		}//end trait loop
	}//end species loop

	int_Num_Class_Trait = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		int_Num_Class_Trait[spc] = new int[max_Num_Trait];

		for (i = 0; i < max_Num_Trait; i++){
			int_Num_Class_Trait[spc][i] = 1;
		}//end i loop
	}//end species loop

	double_Num_Class_Trait_Minus_One = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		double_Num_Class_Trait_Minus_One[spc] = new double[max_Num_Trait];

		for (i = 0; i < max_Num_Trait; i++){
			double_Num_Class_Trait_Minus_One[spc][i] = (double)int_Num_Class_Trait[spc][i];
		}//end i loop
	}//end species loop

	mut_Rat = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		mut_Rat[spc] = new double[max_Num_Locus];

		for (locus = 0; locus < num_Locus[spc]; locus++){
			mut_Rat[spc][locus] = p_MutRat[spc][locus];
		}//end locus loop
	}//end species loop

	loc_Trt = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		loc_Trt[spc] = new int[max_Num_Locus];

		for (locus = 0; locus <max_Num_Locus; locus++){
			loc_Trt[spc][locus] = (-1);
		}//end locus loop
	}//end species loop

	trt_Loc = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		trt_Loc[spc] = new double[max_Num_Trait];

		for (i = 0; i <max_Num_Trait; i++){
			trt_Loc[spc][i] = 0.0;
		}//end locus loop
	}//end species loop

	trt_Loc_Int = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		trt_Loc_Int[spc] = new int[max_Num_Trait];

		for (i = 0; i <max_Num_Trait; i++){
			trt_Loc_Int[spc][i] = 0;
		}//end locus loop
	}//end species loop

	max_Num_Clas_Trt_Spc = new int[max_Num_Trait];

	num_Clas_Trt_Spc = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		num_Clas_Trt_Spc[spc] = new int[max_Num_Trait];

		for (i = 0; i <max_Num_Trait; i++){
			num_Clas_Trt_Spc[spc][i] = 0;
		}//end locus loop
	}//end species loop

	mul_Dim_Trt_Loc_QTL_Mtg = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		mul_Dim_Trt_Loc_QTL_Mtg[spc] = new double[max_Num_Trait];

		for (i = 0; i < max_Num_Trait; i++){
			mul_Dim_Trt_Loc_QTL_Mtg[spc][i] = 0.0;
		}//end i loop
	}//end species loop

	gen_Dst_Map.resize(num_Spc, vector<double>());

	for (spc = 0; spc < num_Spc; spc++){
		gen_Dst_Map[spc].resize(num_Locus[spc]);

		//the first cell is not used because the number indicates the distance between two loci
		gen_Dst_Map[spc][0] = 0.5;

		if (num_Locus[spc] > 1){
			for (locus = 1; locus < num_Locus[spc]; locus++){
				gen_Dst_Map[spc][locus] = 0.5;
			}//end locus loop
		}//end if
	}//end species loop

	num_Locus_Mor = new int[num_Spc];
}

void Simul_Proc::Set_Simul_Struct(int *****p_Initial_Num_Spc)
{
	int deme, niche, spc, sex, stg;

	initial_Num_Spc = new int****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		initial_Num_Spc[deme] = new int***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			initial_Num_Spc[deme][niche] = new int**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				initial_Num_Spc[deme][niche][spc] = new int*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					initial_Num_Spc[deme][niche][spc][sex] = new int[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						initial_Num_Spc[deme][niche][spc][sex][stg] = p_Initial_Num_Spc[deme][niche][spc][sex][stg];
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	density_Spc = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		density_Spc[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			density_Spc[deme][niche] = new double[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				density_Spc[deme][niche][spc] = 0.0;//initialization
			}//end species loop
		}//end niche loop
	}//end deme loop

	celNum_M = new int[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		celNum_M[spc] = 0;
	}//end species loop

	celNum_Agt = new int[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		celNum_Agt[spc] = 0;
	}//end species loop

	celNum_0 = new int****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		celNum_0[deme] = new int***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			celNum_0[deme][niche] = new int**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				celNum_0[deme][niche][spc] = new int*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					celNum_0[deme][niche][spc][sex] = new int[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						celNum_0[deme][niche][spc][sex][stg] = 0;
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	celNum_1 = new int****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		celNum_1[deme] = new int***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){

			celNum_1[deme][niche] = new int**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				celNum_1[deme][niche][spc] = new int*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					celNum_1[deme][niche][spc][sex] = new int[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						celNum_1[deme][niche][spc][sex][stg] = 0;
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	size_AgtLst = new int****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		size_AgtLst[deme] = new int***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			size_AgtLst[deme][niche] = new int**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				size_AgtLst[deme][niche][spc] = new int*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					size_AgtLst[deme][niche][spc][sex] = new int[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						size_AgtLst[deme][niche][spc][sex][stg] = 0;
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	celNum_New_AgtLst = new int[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		celNum_New_AgtLst[spc] = 0;
	}//end species loop

	size_New_AgtLst = new int[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		size_New_AgtLst[spc] = 0;
	}//end species loop

	//Agent list demography
	agtLst_0.resize(tot_Deme_Niche, vector<vector<vector<vector<Agent*> > > >(num_Spc, vector<vector<vector<Agent*> > >(num_Sex, vector<vector<Agent*> >(max_Num_Stg, vector<Agent*>()))));
	agtLst_1.resize(tot_Deme_Niche, vector<vector<vector<vector<Agent*> > > >(num_Spc, vector<vector<vector<Agent*> > >(num_Sex, vector<vector<Agent*> >(max_Num_Stg, vector<Agent*>()))));

	new_AgtLst.resize(num_Spc, vector<Agent*>());
	new_AgtLst_Deme.resize(num_Spc, vector<int>());
	new_AgtLst_Niche.resize(num_Spc, vector<int>());
	agt_M.resize(num_Spc, vector<Agent*>());

	//Agent Structure
	agt.resize(num_Spc, vector<Agent*>());

	//The parameter controlling steepness in the efficacy of hunting at around the boundaries
	k_Hunt_Bound = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		k_Hunt_Bound[spc] = 0.0;
	}//end species loop

	//The parameter controlling generalist penalty in the efficacy of hunting at around the boundaries
	delta_Hunt_Bound = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		delta_Hunt_Bound[spc] = 0.0;
	}//end species loop

	//Resource choice scaling factor regarding resource body mass
	exponent_Choice_Resource_BodyMass = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		exponent_Choice_Resource_BodyMass[spc] = 1.0;
	}//end species loop
}

void Simul_Proc::Set_Vitality_Struct(int **p_Num_Stg, int **p_Num_Immat_Stg_SexMaturation, double *p_PropFeBrth, double***** p_P, double***** p_G, double *p_MetabolicRate_Biomass_Factor)
{
	int deme, niche, spc, sex, stg;

	//The maximum body mass (g) of a species
	metabolicRate_Biomass_Factor = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		metabolicRate_Biomass_Factor[spc] = p_MetabolicRate_Biomass_Factor[spc];
	}//end species loop

	female_Choosiness_Intensity = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		female_Choosiness_Intensity[spc] = 0.0;
	}//end species loop

	female_Choosiness_Scale = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		female_Choosiness_Scale[spc] = 0.0;
	}//end species loop

	growth_Target_Spc = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		growth_Target_Spc[spc] = new double[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			growth_Target_Spc[spc][sex] = 0.0;
		}//end sex loop
	}//end species loop

	exponent_VonBertallanffy = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		exponent_VonBertallanffy[spc] = new double[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			exponent_VonBertallanffy[spc][sex] = 0.0;
		}//end sex loop
	}//end species loop

	max_Num_Stg = 0;

	num_Stg = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		num_Stg[spc] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			num_Stg[spc][sex] = p_Num_Stg[spc][sex];

			if (num_Stg[spc][sex] > max_Num_Stg)
			{
				max_Num_Stg = num_Stg[spc][sex];
			}//end if
		}//end sex loop
	}//end species loop

	num_Stg_BodyLength_Growth = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		num_Stg_BodyLength_Growth[spc] = new double[num_Sex];
		int tmp_Max_Stg = 0;

		//the number of life stages for bodylength calculation
		//is the maximum number of life stages for a species
		for (sex = 0; sex < num_Sex; sex++){
			if (num_Stg[spc][sex] > tmp_Max_Stg)
			{
				tmp_Max_Stg = num_Stg[spc][sex];
			}//end if
		}//end sex loop

		for (sex = 0; sex < num_Sex; sex++){
			num_Stg_BodyLength_Growth[spc][sex] = tmp_Max_Stg;
		}//end sex loop
	}//end species loop

	num_Immat_Stg_SexMaturation = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		num_Immat_Stg_SexMaturation[spc] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			num_Immat_Stg_SexMaturation[spc][sex] = p_Num_Immat_Stg_SexMaturation[spc][sex];
		}//end sex loop
	}//end species loop

	prop_Female_At_Birth = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		prop_Female_At_Birth[spc] = p_PropFeBrth[spc];
	}//end species loop

	P_G = new double*****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		P_G[deme] = new double****[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			P_G[deme][niche] = new double***[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				P_G[deme][niche][spc] = new double**[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					P_G[deme][niche][spc][sex] = new double*[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						//P (survival rate) + G (survival + growth rate)
						P_G[deme][niche][spc][sex][stg] = new double[2];

						P_G[deme][niche][spc][sex][stg][0] = p_P[deme][niche][spc][sex][stg];
						P_G[deme][niche][spc][sex][stg][1] = p_P[deme][niche][spc][sex][stg] + p_G[deme][niche][spc][sex][stg];
						//cout<<"PG " <<deme<<" "<<niche<<" "<<spc<<" "<<sex<<" "<<stg<<" "<<P_G[deme][niche][spc][sex][stg][0]<<" "<<P_G[deme][niche][spc][sex][stg][1]<<"\n";
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	mode_Forage_Physic_Spc = new int[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		mode_Forage_Physic_Spc[spc] = 0;
	}//end species loop

	ltg_VB = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ltg_VB[spc] = 0.0;
	}//end species loop

	ce_VB = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ce_VB[spc] = 0.0;
	}//end species loop

	ci_VB = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ci_VB[spc] = 0.0;
	}//end species loop

	ltg_VB_Sx = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ltg_VB_Sx[spc] = 0.0;
	}//end species loop

	cv_BodyLength_Growth = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		cv_BodyLength_Growth[spc] = 0.01;
	}//end species loop	

	cost_Reduction_EarlySexMat = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		cost_Reduction_EarlySexMat[spc] = 0.0;
	}//end species loop

	smPlb = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		smPlb[spc] = 0.0;
	}//end species loop

	bodyLength_Max_Spc = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		bodyLength_Max_Spc[spc] = 0.0;
	}//end species loop

	bodyLength_Min_Spc = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		bodyLength_Min_Spc[spc] = 0.0;
	}//end species loop

	bodyMass_Min = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		bodyMass_Min[spc] = 0.0;
	}//end species loop
}

void Simul_Proc::Set_Plasticity(double *p_DDG, double *p_Cost_Reduction_EarlySexMat, double *p_SmPlb)
{
	//sets parameters regarding plasticity
	int spc;

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ddg[spc] = p_DDG[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		cost_Reduction_EarlySexMat[spc] = p_Cost_Reduction_EarlySexMat[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		smPlb[spc] = p_SmPlb[spc];
	}//end species loop
}

void Simul_Proc::Set_Plasticity_Update(bool *p_Mode_Plasticity, double *p_ltg_VB, double *p_ce_VB, double *p_ci_VB, double *p_ltg_VB_Sx, double *p_sigma_BodyLength_Growth)
{
	//sets parameters regarding plasticity
	//plasticity mode
	mode_Plasticity = (*p_Mode_Plasticity);

	//Minimum fraction for producing random deviate with positive value
	ltrm = 0.1;

	int spc;

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ltg_VB[spc] = p_ltg_VB[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ce_VB[spc] = p_ce_VB[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ci_VB[spc] = p_ci_VB[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		ltg_VB_Sx[spc] = p_ltg_VB_Sx[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		cv_BodyLength_Growth[spc] = p_sigma_BodyLength_Growth[spc];
	}//end species loop

	//set minimum body length for a species
	for (spc = 0; spc < num_Spc; spc++)
	{
		//maximum body length
		double tmp_Max_Length = bodyLength_Max_Spc[spc];

		//lgn
		double tmp_Lgn = ltg_VB[spc];

		//ltrn
		double tmp_Ltrn = ltrm;

		double tmp_Age = 0.0;
		double tmp_TimeStep = unit_T_Step;
		double tmp_Tot_Num_Stg_Devidend = num_Stg_BodyLength_Growth[spc][0];
		double tmp_Exponent_VonBertallanffy_BodyLength = exponent_VonBertallanffy[spc][0];
		double tmp_VBK = 0.0;//vbk

		//get vbk
		Get_VBK(&tmp_VBK, &tmp_Exponent_VonBertallanffy_BodyLength, &tmp_Age, &tmp_Tot_Num_Stg_Devidend, &tmp_TimeStep);

		//set minimum body length for a species
		bodyLength_Min_Spc[spc] = tmp_Max_Length * tmp_VBK * (1.0 - ((1.0 - tmp_Ltrn) * tmp_Lgn));

		//verification
		//std::cout << "max " << tmp_Max_Length << " lgn " << tmp_Lgn << " ltrn " << tmp_Ltrn << " TimeStep " << tmp_TimeStep << std::endl;
		//std::cout << " divid " << tmp_Tot_Num_Stg_Devidend << " vbb " << tmp_Exponent_VonBertallanffy_BodyLength << " vbk " << tmp_VBK << std::endl;
		//std::cout << "minimum body length " << bodyLength_Min_Spc[spc] << std::endl;
		//char tt;
		//cin >> tt;

		//set the minimum bodymass
		Get_BodyMass_From_BodyLength(&bodyMass_Min[spc], &bodyLength_Min_Spc[spc], &spc);

		//verification
		//std::cout << "min mass " << bodyMass_Min[spc] << std::endl;
		//std::cout << "max mass " << bodyMass_Max[spc] << std::endl;
		//char tt;
		//cin >> tt;
	}//end species loop
}

void Simul_Proc::Set_Stat_Dst(char * fileName_Output)
{
	string **s_Tag = new string *[1];
	s_Tag[0] = new string[1];

	//File management
	int s_Tag_Col = 1;
	int s_Tag_Row = 1;
	int num_Cat = 1;

	uint32_t tmp_Seed = (uint32_t)rdtsc();//generates a random seed
	//tmp_Seed = 4276579148;//generates a random seed
	randomSeed_Mut = new mt19937((tmp_Seed));//random seed

	s_Tag[0][0] = "Random Seed #0";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	s_Tag[0][0] = "unsigned integer 32";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	csvRW.UInteger32_To_String(&s_Tag[0][0], &tmp_Seed);
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);

	tmp_Seed = (uint32_t)rdtsc();//generates a random seed
	//tmp_Seed = 3648032;//generates a random seed
	randomSeed_Trt = new mt19937((tmp_Seed));//random seed

	s_Tag[0][0] = "Random Seed #1";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	s_Tag[0][0] = "unsigned integer 32";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	csvRW.UInteger32_To_String(&s_Tag[0][0], &tmp_Seed);
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);

	tmp_Seed = (uint32_t)rdtsc();//generates a random seed
	//tmp_Seed = 38503956;//generates a random seed
	randomSeed_Mtg = new mt19937((tmp_Seed));//random seed
	uni_Real_Mtg = new uniform_real_distribution<double>(0.0, 1.0);//uniform real distribution
	rand_Uni_Real_Mtg = new variate_generator<mt19937&, uniform_real_distribution<double> >((*randomSeed_Mtg), (*uni_Real_Mtg));//random variate generator

	s_Tag[0][0] = "Random Seed #2";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	s_Tag[0][0] = "unsigned integer 32";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	csvRW.UInteger32_To_String(&s_Tag[0][0], &tmp_Seed);
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);

	tmp_Seed = (uint32_t)rdtsc();//generates a random seed
	//tmp_Seed = 68794680;//generates a random seed
	randomSeed = new mt19937((tmp_Seed));//random seed
	uni_Real = new uniform_real_distribution<double>(0.0, 1.0);//uniform real distribution
	rand_Uni_Real = new variate_generator<mt19937&, uniform_real_distribution<double> >((*randomSeed), (*uni_Real));//random variate generator

	s_Tag[0][0] = "Random Seed #3";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	s_Tag[0][0] = "unsigned integer 32";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	csvRW.UInteger32_To_String(&s_Tag[0][0], &tmp_Seed);
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);

	tmp_Seed = (uint32_t)rdtsc();//generates a random seed
	//tmp_Seed = 104272604;//generates a random seed
	randomSeed_int = new mt19937((tmp_Seed));//random seed
	uni_int = new uniform_int_distribution<int>(0, 1);//uniform real distribution
	rand_Uni_int = new variate_generator<mt19937&, uniform_int_distribution<int> >((*randomSeed_int), (*uni_int));//random variate generator

	s_Tag[0][0] = "Random Seed #4";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	s_Tag[0][0] = "unsigned integer 32";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	csvRW.UInteger32_To_String(&s_Tag[0][0], &tmp_Seed);
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);

	delete[] s_Tag[0];
	delete[] s_Tag;
}

uint64_t Simul_Proc::rdtsc(void){
	return __rdtsc();
}

void Simul_Proc::Set_Resource(double ***intra_Specific_Interaction_LV, int* p_num_Locus_Rsc, int* p_num_Position_Rsc, double* p_mut_Rat_Rsc, int* p_num_PhenoTyp_Rsc, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc)
{
	//p_D_Spc: dim[deme, niche, resource species], the densities of resources
	//p_num_RscSpc: the number of resource species
	int deme, niche, spc, i;

	for (spc = 0; spc < num_RscSpc; spc++){
		num_Locus_Rsc[spc] = p_num_Locus_Rsc[spc];
	}//end resource species loop

	for (spc = 0; spc < num_RscSpc; spc++){
		num_Position_Rsc[spc] = p_num_Position_Rsc[spc];
	}//end resource species loop

	for (spc = 0; spc < num_RscSpc; spc++){
		mut_Rat_Rsc[spc] = p_mut_Rat_Rsc[spc];
	}//end resource species loop

	tot_Num_Rsc_Cls = 0;

	for (spc = 0; spc < num_RscSpc; spc++){
		num_PhenoTyp_Rsc[spc] = p_num_PhenoTyp_Rsc[spc];
		tot_Num_Rsc_Cls += p_num_PhenoTyp_Rsc[spc];
	}//end resource species loop

	//the mean and standard deviations of the sizes(individual biomass) of phenotypic classes of resource species
	double** p_Trt_RscSpc_Mean_Sd = new double*[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		p_Trt_RscSpc_Mean_Sd[spc] = new double[2];
	}//end resource species loop

	//sets the means and standard deviations of resource species biomass
	Set_Mean_SD_BioMass_Resource(p_Trt_RscSpc_Mean_Sd, p_Max_Mean_BioMass_Rsc, p_Min_Mean_BioMass_Rsc, p_CV_BioMass_Rsc);

	sum_Cmt_Rsc_Spc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		sum_Cmt_Rsc_Spc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			sum_Cmt_Rsc_Spc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				sum_Cmt_Rsc_Spc[deme][niche][spc] = new double[max_Num_PhenTyp_Rsc];//initialization

				for (i = 0; i < max_Num_PhenTyp_Rsc; i++){
					sum_Cmt_Rsc_Spc[deme][niche][spc][i] = 0.0;
				}//end resource species phenotype loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	Set_Biomass_Resource(p_Trt_RscSpc_Mean_Sd);//sets the mean sizes (individual biomass) of phenotypic classes of resource species

	for (spc = 0; spc < num_RscSpc; spc++){
		delete[] p_Trt_RscSpc_Mean_Sd[spc];
	}//end resource species loop

	delete[] p_Trt_RscSpc_Mean_Sd;
}

void Simul_Proc::Set_Ini_Rsc(double ***p_intrinsic_Rsc, double **p_Trt_RscSpc_Mean_Sd, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance)
{
	//sets the initial density for each phenotype of a resource species
	double tmp_Sum, tmp, mean0, sd0, binWidth, totDen, tmp_X, tmp_X_Plus, lower, upper;
	int deme, niche, spc, tnp, end0;

	for (spc = 0; spc < num_RscSpc; spc++){
		binWidth = (6.0 * p_Trt_RscSpc_Mean_Sd[spc][1]) / ((double)num_PhenoTyp_Rsc[spc]);
		boost::math::normal normRscSpxSize(p_Trt_RscSpc_Mean_Sd[spc][0], p_Trt_RscSpc_Mean_Sd[spc][1]);
		mean0 = p_Trt_RscSpc_Mean_Sd[spc][0];
		sd0 = p_Trt_RscSpc_Mean_Sd[spc][1];
		end0 = num_PhenoTyp_Rsc[spc];

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				tmp_Sum = 0.0;//temporary sum
				//Power law: The number of individuals = c*( 1 / (biomass^(alpha) ) )
				//starts simulation with resource species at their carrying capacity without inter-specific interaction
				// LV_Loss[deme][niche][(num_Spc + spc)][(num_Spc + spc)]: intraspecific competition
				//totDen = ((p_intrinsic_Rsc[deme][niche][spc]) / LV_Loss[deme][niche][(num_Spc + spc)][(num_Spc + spc)]) * (p_C_BioMass_Abundance[deme][niche] * (1.0 / (pow(mean0, p_Alpha_BioMass_Abundance[deme][niche]))));//the density of a resource species
				totDen = (p_C_BioMass_Abundance[deme][niche] * (1.0 / (pow(mean0, p_Alpha_BioMass_Abundance[deme][niche]))));//the density of a resource species

				for (tnp = 0; tnp < end0; tnp++){
					//lower bound of a bin
					tmp_X = mean0 - (3.0 * sd0) + binWidth * ((double)tnp);//lower bound of a bin
					//upper bound of a bin
					tmp_X_Plus = tmp_X + binWidth;//upper bound of a bin

					lower = cdf(normRscSpxSize, tmp_X);////lower bound of a bin
					upper = cdf(normRscSpxSize, tmp_X_Plus);//upper bound of a bin

					//sets the density for each phenotype of a resource species
					tmp = totDen * (upper - lower);
					phen_D_Rsc[deme][niche][spc][tnp] = tmp;
					tmp_Sum += tmp;
				}//phenotype loop

				d_Rsc[deme][niche][spc] = tmp_Sum;//the density of a resource species
			}//end niche loop
		}//end deme loop
	}//resource species loop
}

void Simul_Proc::Set_Resource(void)
{
	//p_D_Spc: dim[deme, niche, resource species], the densities of resources
	//p_num_RscSpc: the number of resource species
	int deme, niche, spc, rsc, rCls;

	num_Locus_Rsc = new int[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		num_Locus_Rsc[spc] = 0;
	}//end resource species lop

	num_Position_Rsc = new int[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		num_Position_Rsc[spc] = 0;
	}//end resource species lop

	mut_Rat_Rsc = new double[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		mut_Rat_Rsc[spc] = 0.0;
	}//end resource species lop

	num_PhenoTyp_Rsc = new int[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		num_PhenoTyp_Rsc[spc] = 0;
	}//end resource species lop

	phen_D_Rsc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		phen_D_Rsc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			phen_D_Rsc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				phen_D_Rsc[deme][niche][spc] = new double[max_Num_PhenTyp_Rsc];

				for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
					phen_D_Rsc[deme][niche][spc][rsc] = 0.0;
				}//end phenotype loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	phen_D_Rsc_Fixed = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		phen_D_Rsc_Fixed[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			phen_D_Rsc_Fixed[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				phen_D_Rsc_Fixed[deme][niche][spc] = new double[max_Num_PhenTyp_Rsc];

				for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
					phen_D_Rsc_Fixed[deme][niche][spc][rsc] = 0.0;
				}//end phenotype loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	mean_BioMass_Rsc = new double*[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		mean_BioMass_Rsc[spc] = new double[max_Num_PhenTyp_Rsc];

		for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
			mean_BioMass_Rsc[spc][rsc] = 0.0;
		}//end phenotype loop
	}//end species loop

	bin_Rsc_BioMass = new double**[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		bin_Rsc_BioMass[spc] = new double*[max_Num_PhenTyp_Rsc];

		for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
			bin_Rsc_BioMass[spc][rsc] = new double[2];

			for (rCls = 0; rCls < 2; rCls++)
			{
				bin_Rsc_BioMass[spc][rsc][rCls] = 0.0;
			}//left & right side of a bin loop
		}//end phenotype loop
	}//end species loop

	growthRate_Rsc = new double[max_Num_PhenTyp_Rsc];

	for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
		growthRate_Rsc[rsc] = 0.0;//initialization
	}//end phenotype class loop

	tmp_Rsc_Cls = new double[max_Num_PhenTyp_Rsc];

	for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
		tmp_Rsc_Cls[rsc] = 0.0;//initialization
	}//end phenotype class loop

	d_Rsc = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		d_Rsc[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			d_Rsc[deme][niche] = new double[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				d_Rsc[deme][niche][spc] = 0.0;
			}//end species loop
		}//end niche loop
	}//end deme loop

	prop_Density_Rsc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		prop_Density_Rsc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			prop_Density_Rsc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				prop_Density_Rsc[deme][niche][spc] = new double[max_Num_PhenTyp_Rsc];

				for (rCls = 0; rCls < max_Num_PhenTyp_Rsc; rCls++){
					prop_Density_Rsc[deme][niche][spc][rCls] = 0.0;
				}//end resource phenotype class loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	intrinsic_Rsc = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		intrinsic_Rsc[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			intrinsic_Rsc[deme][niche] = new double[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				intrinsic_Rsc[deme][niche][spc] = 0.0;
			}//end species loop
		}//end niche loop
	}//end deme loop

	density_Rsc = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		density_Rsc[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			density_Rsc[deme][niche] = new double[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				density_Rsc[deme][niche][spc] = 0.0;//initialization
			}//end species loop
		}//end niche loop
	}//end deme loop

	density_Rsc_Fixed = new double**[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		density_Rsc_Fixed[deme] = new double*[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			density_Rsc_Fixed[deme][niche] = new double[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				density_Rsc_Fixed[deme][niche][spc] = 0.0;//initialization
			}//end species loop
		}//end niche loop
	}//end deme loop

	th_Spc_RscSpc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		th_Spc_RscSpc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			th_Spc_RscSpc[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				th_Spc_RscSpc[deme][niche][spc] = new double[num_RscSpc];

				for (rsc = 0; rsc < num_RscSpc; rsc++){
					th_Spc_RscSpc[deme][niche][spc][rsc] = 0.0;
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	q_Spc_RscSpc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		q_Spc_RscSpc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			q_Spc_RscSpc[deme][niche] = new double*[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				q_Spc_RscSpc[deme][niche][spc] = new double[num_RscSpc];

				for (rsc = 0; rsc < num_RscSpc; rsc++){
					q_Spc_RscSpc[deme][niche][spc][rsc] = 0.0;
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	th_Rsc_RscSpc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		th_Rsc_RscSpc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			th_Rsc_RscSpc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				th_Rsc_RscSpc[deme][niche][spc] = new double[num_RscSpc];

				for (rsc = 0; rsc < num_RscSpc; rsc++){
					th_Rsc_RscSpc[deme][niche][spc][rsc] = 0.0;
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	q_Rsc_RscSpc = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		q_Rsc_RscSpc[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			q_Rsc_RscSpc[deme][niche] = new double*[num_RscSpc];

			for (spc = 0; spc < num_RscSpc; spc++){
				q_Rsc_RscSpc[deme][niche][spc] = new double[num_RscSpc];

				for (rsc = 0; rsc < num_RscSpc; rsc++){
					q_Rsc_RscSpc[deme][niche][spc][rsc] = 0.0;
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Lotka_Volterra_Parameters(char * fileName_Output, double *p_b_NutritionalValue, int among_Resource_Interaction, double **neg_Eff_Spc_Rsc_ScalingFactor, double ***p_intrinsic_Rsc, double ***p_Intra_Specific_Interaction_LV, double **a_BiomassRatio_Interaction, double **b_BiomassRatio_Interaction, double **a_BiomassRatio_Interaction_Gain, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double ****p_Stable_Stage_Dist, double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc, double ****p_H_Rsc_RscSpc, double ****p_Qq_Rsc_RscSpc)
{
	//among_Resource_Interaction: controls the interaction among
	//0: No intraction among resource species
	//1: Intraction among resource species

	//p_Intra_Specific_Interaction_LV: Intra-specific-density-dependent effect in the Lotka-Volterra equation
	//dim[deme, niche, sum_Species (core species + resource species)]

	//a_Biomass_Interaction: parameter for the equation determining alpha (alpha = a * (bioMass of a predator / biomass of a prey)^b) in the Lotka-Volterra equation
	//dim[deme, niche]

	//b_Biomass_Interaction: parameter (exponent) for the equation determining alpha (alpha = a * (bioMass of a predator / biomass of a prey)^b) in the Lotka-Volterra equation
	//dim[deme, niche]

	//p_Initial_Trait_State_Spc: the initial state of the body-size trait of a species
	//dim[species]

	//p_Stable_Stage_Dist: stable stage distribution
	//dim[deme, niche, species, stage]
	//the mean and standard deviations of the sizes(individual biomass) of phenotypic classes of resource species
	int deme, niche, spc, spc_0, stg, end_0;

	//sets the mean biomass of a core species at the stable stage distribution
	double * tmp_Mean_BioMass_Spc = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		tmp_Mean_BioMass_Spc[spc] = 0.0;//initialization
		end_0 = num_Stg[spc][1];//the number of life stages for female

		for (stg = 0; stg < end_0; stg++){
			tmp_Mean_BioMass_Spc[spc] += p_Stable_Stage_Dist[0][0][spc][stg] * bodyMass[spc][1][stg][(num_Clas_Trt_Spc[spc][0] - 1)];

			//cout << "Line 7217" << endl;
			//cout << "spc " << spc << " stage " << stg << " dist " << p_Stable_Stage_Dist[0][0][spc][stg] << " bodymass " << bodyMass[spc][1][stg][(p_Initial_BioMass_Trait_State_Spc[spc])];
			//char tt;
			//cin >> tt;
		}//end sex loop
	}//end species loop

	string **s_Tag = new string*[1];
	s_Tag[0] = new string[1];
	s_Tag[0][0] = "(Core Species) Bodymass (g) for Calculating Interaction";

	int s_Tag_Col = 1;
	int s_Tag_Row = 1;
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &s_Tag_Col);
	int num_Cat = 1;
	s_Tag[0][0] = "Core Species";
	csvRW.CSV_Write_Add_String_2D(fileName_Output, s_Tag, &s_Tag_Row, &num_Cat);
	csvRW.CSV_Write_Add_Double_1DD(fileName_Output, tmp_Mean_BioMass_Spc, &num_Spc);

	delete[] s_Tag[0];
	delete[] s_Tag;

	//the mean and standard deviations of the sizes(individual biomass) of phenotypic classes of resource species
	double** p_Trt_RscSpc_Mean_Sd = new double*[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		p_Trt_RscSpc_Mean_Sd[spc] = new double[2];
	}//end resource species loop

	//sets the means and standard deviations of resource species biomass
	Set_Mean_SD_BioMass_Resource(p_Trt_RscSpc_Mean_Sd, p_Max_Mean_BioMass_Rsc, p_Min_Mean_BioMass_Rsc, p_CV_BioMass_Rsc);

	double weight_0, weight_1;

	//sets intrinsic growth rate for resource species
	for (spc = 0; spc < num_RscSpc; spc++){
		//resource species
		weight_0 = p_Trt_RscSpc_Mean_Sd[spc][0];

		//intrinsic rate of increase for resource species
		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				intrinsic_Rsc[deme][niche][spc] = p_intrinsic_Rsc[deme][niche][spc];
			}//end niche loop
		}//end deme loop
	}//end species loop

	//Lotka-Volterra Loss
	for (spc = 0; spc < num_Sum_Spc; spc++){
		if (spc < num_Spc)
		{//for core species
			weight_0 = tmp_Mean_BioMass_Spc[spc];
		}
		else
		{//for resource species
			weight_0 = p_Trt_RscSpc_Mean_Sd[(spc - num_Spc)][0];
		}//end else

		for (spc_0 = 0; spc_0 < num_Sum_Spc; spc_0++){
			if (spc_0 < num_Spc)
			{//for core species
				weight_1 = tmp_Mean_BioMass_Spc[spc_0];
			}
			else
			{//for resource species
				weight_1 = p_Trt_RscSpc_Mean_Sd[(spc_0 - num_Spc)][0];
			}//end else

			//Lotka-Volterra Loss
			for (deme = 0; deme < num_Deme; deme++){
				for (niche = 0; niche < num_Niche; niche++){
					//sets the parameters for the Lotka-Volterra equation 
					if (spc != spc_0)
					{//Inter-specific effect on survival
						if ((spc < num_Spc) && (spc_0 < num_Spc))
						{//interaction of core species - core species
							LV_Loss[deme][niche][spc][spc_0] = a_BiomassRatio_Interaction[deme][niche] * (pow((weight_0 / weight_1), b_BiomassRatio_Interaction[deme][niche]));
						}
						else if ((spc < num_Spc) || (spc_0 < num_Spc))
						{//interaction of core species - resource species
							//This part is also described in foraging function
							if (spc < num_Spc)
							{//the effect of core species on resource species
								LV_Loss[deme][niche][spc][spc_0] = 0.0;
							}
							else
							{//the effect of core species on resource species
								LV_Loss[deme][niche][spc][spc_0] = ((double)max_Num_PhenTyp_Rsc) * neg_Eff_Spc_Rsc_ScalingFactor[deme][niche] * a_BiomassRatio_Interaction[deme][niche] * (pow((weight_0 / weight_1), b_BiomassRatio_Interaction[deme][niche]));
							}//end else
						}
						else
						{//interaction of resource species - resource species
							if (weight_0 > weight_1)
							{//the predator should be larger than prey
								LV_Loss[deme][niche][spc][spc_0] = 0.0;
							}
							else
							{
								LV_Loss[deme][niche][spc][spc_0] = a_BiomassRatio_Interaction[deme][niche] * (pow((weight_0 / weight_1), b_BiomassRatio_Interaction[deme][niche]));
							}//end else
						}//end else
					}
					else
					{//intra-specific effect on survival (spc == spc_0)
						if (spc < num_Spc)
						{
							LV_Loss[deme][niche][spc][spc_0] = 0.0;
						}
						else
						{
							//1. p_intrinsic_Rsc[deme][niche][spc]
							//: intrinsic rate of population growth
							//2. p_Intra_Specific_Interaction_LV[deme][niche][spc]
							//: intra-specific effect on population growth
							//3.(1.0 / (p_C_BioMass_Abundance[deme][niche] * (pow((weight_0), (-p_Alpha_BioMass_Abundance[deme][niche])))))
							//: the population level when intra-specific effect (p_Intra_Specific_Interaction_LV[deme][niche][spc]) is 1.0 and no interaction among resource species
							LV_Loss[deme][niche][spc][spc_0] = p_Intra_Specific_Interaction_LV[deme][niche][spc] * (1.0 / (p_C_BioMass_Abundance[deme][niche] * (pow(((1.0 / weight_0)), (p_Alpha_BioMass_Abundance[deme][niche])))));
						}//end else
					}//end else

					//cout << "Line 7332  spc #1:" << spc << "  wt0: " << weight_0 << " spc # 2: " << spc_0 << "  wt1: " << weight_1 << "  N_LV: " << LV_Loss[deme][niche][spc][spc_0] << endl;
					//cout <<"Intra-specific effect"<< p_Intra_Specific_Interaction_LV[deme][niche][spc] << "     " << p_C_BioMass_Abundance[deme][niche] << endl;
					//cout << "Inter-specific Negative exponent: " << b_BiomassRatio_Interaction[deme][niche] << "Inter-specific Negative slope: " << a_BiomassRatio_Interaction[deme][niche] << endl;
					//char tt;
					//cin >> tt;
				}//end niche loop
			}//end deme loop
		}//end spc_0 loop
	}//end species loop

	//Lotka-Volterra Gain
	for (spc = 0; spc < num_Sum_Spc; spc++){
		if (spc < num_Spc)
		{//for core species
			weight_0 = tmp_Mean_BioMass_Spc[spc];
		}
		else
		{//for resource species
			weight_0 = p_Trt_RscSpc_Mean_Sd[(spc - num_Spc)][0];
		}//end else

		//Lotka-Volterra gain
		for (spc_0 = 0; spc_0 < num_Sum_Spc; spc_0++){
			if (spc_0 < num_Spc)
			{//for core species
				weight_1 = tmp_Mean_BioMass_Spc[spc_0];
			}
			else
			{//for resource species
				weight_1 = p_Trt_RscSpc_Mean_Sd[(spc_0 - num_Spc)][0];
			}//end else

			for (deme = 0; deme < num_Deme; deme++){
				for (niche = 0; niche < num_Niche; niche++){
					//sets the parameters for the Lotka-Volterra equation 
					if (spc != spc_0)
					{//Inter-specific effect on survival
						if ((spc < num_Spc) && (spc_0 < num_Spc))
						{//interaction of core species - core species
							if (weight_0 < weight_1)
							{
								LV_Gain[deme][niche][spc][spc_0] = 0.0;
							}
							else
							{
								LV_Gain[deme][niche][spc][spc_0] = a_BiomassRatio_Interaction_Gain[deme][niche];
							}
						}
						else if ((spc < num_Spc) || (spc_0 < num_Spc))
						{//interaction of core species - resource species
							if (weight_0 < weight_1)
							{
								LV_Gain[deme][niche][spc][spc_0] = 0.0;
							}
							else
							{
								if (spc < num_Spc)
								{//Gain by a core species from a resource species (this is described by energy gain in density-dependent survival)
									//LV_Gain[deme][niche][spc][spc_0] = a_BiomassRatio_Interaction_Gain[deme][niche] * (pow((weight_1 / weight_0), b_BiomassRatio_Interaction_Gain[deme][niche]));
									LV_Gain[deme][niche][spc][spc_0] = 0.0;
								}
								else
								{//Gain by a resource species from a core species
									//LV_Gain[deme][niche][spc][spc_0] = a_BiomassRatio_Interaction_Gain[deme][niche] * (pow((weight_1 / weight_0), b_BiomassRatio_Interaction_Gain[deme][niche]));
									LV_Gain[deme][niche][spc][spc_0] = 0.0;
								}//end else
							}//end else
						}
						else
						{//interaction of resource species - resource species
							if (weight_0 < weight_1)
							{//the predator should be larger than prey
								LV_Gain[deme][niche][spc][spc_0] = 0.0;
							}
							else
							{
								LV_Gain[deme][niche][spc][spc_0] = a_BiomassRatio_Interaction_Gain[deme][niche];
							}//end else
						}//end else
					}
					else
					{//intra-specific effect
						LV_Gain[deme][niche][spc][spc_0] = 0.0;
					}//end else

					//cout << "Line 6444 " <<spc<< "  wt0: " << weight_0 <<" "<<spc_0<< "  wt1: " << weight_1 << "  P_LV: " << LV_Gain[deme][niche][spc][spc_0] << endl;
				}//end niche loop
			}//end deme loop
		}//end spc_0 loop
	}//end species loop

	//cout << "Line 6381: Type a character" << endl;
	//char tc;
	//cin >> tc;

	//nutritional value of prey
	for (spc = 0; spc < num_Spc; spc++){
		weight_0 = tmp_Mean_BioMass_Spc[spc];

		for (spc_0 = 0; spc_0 < num_Sum_Spc; spc_0++){
			if (spc_0 < num_Spc)
			{//for core species
				weight_1 = tmp_Mean_BioMass_Spc[spc_0];
			}
			else
			{//for resource species
				weight_1 = p_Trt_RscSpc_Mean_Sd[(spc_0 - num_Spc)][0];
			}//end else

			//nutritional value of prey
			if (p_b_NutritionalValue[spc] == 0.0)
			{
				k_NutritionValue[spc][spc_0] = 1.0;
			}
			else
			{
				k_NutritionValue[spc][spc_0] = pow((weight_1 / weight_0), p_b_NutritionalValue[spc]);
			}
		}//end spc_0 loop
	}//end species loop

	//sets the initial density for each phenotype of a resource species
	Set_Ini_Rsc(intrinsic_Rsc, p_Trt_RscSpc_Mean_Sd, p_Alpha_BioMass_Abundance, p_C_BioMass_Abundance);
	//sets functional response for resource
	Set_Functional_Response_Rsc(p_H_Rsc_RscSpc, p_Qq_Rsc_RscSpc, p_Alpha_BioMass_Abundance, p_C_BioMass_Abundance, p_Trt_RscSpc_Mean_Sd);
	//sets functional response for core species
	Set_Functional_Response_Spc(p_Alpha_BioMass_Abundance, p_C_BioMass_Abundance, p_Trt_RscSpc_Mean_Sd);

	delete[] tmp_Mean_BioMass_Spc;

	for (spc = 0; spc < num_RscSpc; spc++){
		delete[] p_Trt_RscSpc_Mean_Sd[spc];
	}//end resource species loop

	delete[] p_Trt_RscSpc_Mean_Sd;
}

void Simul_Proc::Set_Spc_Interaction(double *****p_Cd, double ***p_b_MassIntake_DDS, double ****h_Spc_RscSpc, double ****qq_Spc_RscSpc, double *p_K_Hunt_Bound, double *p_Delta_Hunt_Bound, double *p_Exponent_Choice_Resource_BodyMass, int *p_Mode_Forage_Physic_Spc)
{//sets parameters regarding species interaction
	//dim[competing species(num_Sum_Spc), competing species(num_Sum_Spc)]; matrix of coefficients for competition in a Lotka-Volterra equation
	int spc, spc_0, deme, niche, sex, stg;

	//parameter for density dependence
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					for (stg = 0; stg < max_Num_Stg; stg++){
						Cd[deme][niche][spc][sex][stg] = p_Cd[deme][niche][spc][sex][stg];
						//cout << " Line 7536: cd" << p_Cd[deme][niche][spc][sex][stg]<<endl;
						//char ee;
						//cin >> ee;
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	int tmp_Stg;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			tmp_Stg = num_Stg[spc][sex];

			for (stg = 0; stg < tmp_Stg; stg++){
				b_MassIntake_DDS[spc][sex][stg] = p_b_MassIntake_DDS[spc][sex][stg];
				//cout << "sp " << spc << " sex " << sex << "  stg " << stg << "  " << b_MassIntake_DDS[spc][sex][stg];
			}//end species loop
		}//end niche loop
	}//end deme loop

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (spc_0 = 0; spc_0 < num_RscSpc; spc_0++){
					th_Spc_RscSpc[deme][niche][spc][spc_0] = h_Spc_RscSpc[spc][deme][niche][spc_0];
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (spc_0 = 0; spc_0 < num_RscSpc; spc_0++){
					q_Spc_RscSpc[deme][niche][spc][spc_0] = qq_Spc_RscSpc[deme][niche][spc][spc_0];
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	for (spc = 0; spc < num_Spc; spc++){
		k_Hunt_Bound[spc] = p_K_Hunt_Bound[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		delta_Hunt_Bound[spc] = p_Delta_Hunt_Bound[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		exponent_Choice_Resource_BodyMass[spc] = p_Exponent_Choice_Resource_BodyMass[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		mode_Forage_Physic_Spc[spc] = p_Mode_Forage_Physic_Spc[spc];
	}//end species loop
}

void Simul_Proc::Set_Spc_Interaction(void)
{//sets parameters regarding species interaction
	//dim[competing species(num_Sum_Spc), competing species(num_Sum_Spc)]; matrix of coefficients for competition in a Lotka-Volterra equation
	int spc, spc_0, deme, niche, sex, stg;

	LV_Loss = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		LV_Loss[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			LV_Loss[deme][niche] = new double*[num_Sum_Spc];

			for (spc = 0; spc < num_Sum_Spc; spc++){
				LV_Loss[deme][niche][spc] = new double[num_Sum_Spc];

				for (spc_0 = 0; spc_0 < num_Sum_Spc; spc_0++){
					LV_Loss[deme][niche][spc][spc_0] = 0.0;
				}//end spc_0 loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	LV_Gain = new double***[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		LV_Gain[deme] = new double**[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			LV_Gain[deme][niche] = new double*[num_Sum_Spc];

			for (spc = 0; spc < num_Sum_Spc; spc++){
				LV_Gain[deme][niche][spc] = new double[num_Sum_Spc];

				for (spc_0 = 0; spc_0 < num_Sum_Spc; spc_0++){
					LV_Gain[deme][niche][spc][spc_0] = 0.0;
				}//end spc_0 loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	k_NutritionValue = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		k_NutritionValue[spc] = new double[num_Sum_Spc];

		for (spc_0 = 0; spc_0 < num_Sum_Spc; spc_0++){
			k_NutritionValue[spc][spc_0] = 0.0;
		}//end all species loop
	}//end species loop

	//parameter for density dependence
	Cd = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		Cd[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			Cd[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				Cd[deme][niche][spc] = new double*[num_Sex];

				for (sex = 0; sex < num_Sex; sex++){
					Cd[deme][niche][spc][sex] = new double[max_Num_Stg];

					for (stg = 0; stg < max_Num_Stg; stg++){
						Cd[deme][niche][spc][sex][stg] = 0.0;
					}//end life stage loop
				}//end sex loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	b_MassIntake_DDS = new double**[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		b_MassIntake_DDS[spc] = new double*[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			b_MassIntake_DDS[spc][sex] = new double[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				b_MassIntake_DDS[spc][sex][stg] = 0.0;
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Resource_Burn_In(void)
{
	int deme, niche, spc, rsc;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				density_Rsc_Fixed[deme][niche][spc] = 0.0;//initialization
			}//end species loop
		}//end niche loop
	}//end deme loop

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
					phen_D_Rsc_Fixed[deme][niche][spc][rsc] = phen_D_Rsc[deme][niche][spc][rsc];
					density_Rsc_Fixed[deme][niche][spc] += phen_D_Rsc[deme][niche][spc][rsc];
				}//end phenotype loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Reset_Resource_Fixed(void)
{
	if (consistancy_Resource == true)
	{
		int deme, niche, spc, rsc;

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					for (rsc = 0; rsc < max_Num_PhenTyp_Rsc; rsc++){
						phen_D_Rsc[deme][niche][spc][rsc] = phen_D_Rsc_Fixed[deme][niche][spc][rsc];
					}//end phenotype loop
				}//end species loop
			}//end niche loop
		}//end deme loop

		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				for (spc = 0; spc < num_RscSpc; spc++){
					density_Rsc[deme][niche][spc] = density_Rsc_Fixed[deme][niche][spc];//initialization
				}//end species loop
			}//end niche loop
		}//end deme loop
	}//end if
}

void Simul_Proc::Set_Movement(double *p_Mov_Patch, double **p_Mov_Niche)
{
	int niche_0, niche_1;

	if (num_Deme > 1){
		mov_Patch = (*p_Mov_Patch);
	}
	else{
		mov_Patch = 0.0;
	}//end else

	mov_Patch_half = 0.5 * mov_Patch;

	if (num_Niche > 1){//multiple niche
		for (niche_0 = 0; niche_0 < num_Niche; niche_0++){
			mov_Niche_Cumul[niche_0][0] = p_Mov_Niche[niche_0][0];
			mov_Niche[niche_0][0] = p_Mov_Niche[niche_0][0];

			for (niche_1 = 1; niche_1 < num_Niche; niche_1++){
				mov_Niche[niche_0][niche_1] = p_Mov_Niche[niche_0][niche_1];
				mov_Niche_Cumul[niche_0][niche_1] = mov_Niche_Cumul[niche_0][(niche_1 - 1)] + p_Mov_Niche[niche_0][niche_1];
			}//end niche_1 loop
		}//end niche_0 loop
	}
	else{//only one niche
		mov_Niche[0][0] = 1.0;
		mov_Niche_Cumul[0][0] = 1.0;
	}//end else
}

void Simul_Proc::Set_Movement(void)
{
	int niche_0, niche_1;
	mov_Patch = 0.0;

	mov_Patch_half = 0.5 * mov_Patch;

	mov_Niche_Cumul = new double*[num_Niche];

	for (niche_0 = 0; niche_0 < num_Niche; niche_0++){
		mov_Niche_Cumul[niche_0] = new double[num_Niche];
	}//end niche loop

	mov_Niche = new double*[num_Niche];

	for (niche_0 = 0; niche_0 < num_Niche; niche_0++){
		mov_Niche[niche_0] = new double[num_Niche];
	}//end niche loop

	if (num_Niche > 1){//multiple niche
		for (niche_0 = 0; niche_0 < num_Niche; niche_0++){
			mov_Niche_Cumul[niche_0][0] = 0.0;
			mov_Niche[niche_0][0] = 0.0;

			for (niche_1 = 1; niche_1 < num_Niche; niche_1++){
				mov_Niche_Cumul[niche_0][niche_1] = 0.0;
				mov_Niche[niche_0][niche_1] = 0.0;
			}//end niche_1 loop
		}//end niche_0 loop
	}
	else{//only one niche
		mov_Niche_Cumul[0][0] = 1.0;
		mov_Niche[0][0] = 1.0;
	}//end else
}

void Simul_Proc::DisplayVariable(int *tStep)
{
	int deme, niche, spc, sex, stg, demNi;

	cout << "\n" << "Time step[" << (*tStep) << "]" << "\n";

	for (spc = 0; spc < num_Spc; spc++){
		cout << "The total number of used agent cells " << spc << " is " << celNum_Agt[spc] << "\n";
	}

	for (spc = 0; spc < num_Spc; spc++){
		cout << "The total number of agent cells in the library[" << spc << "] is " << agt[spc].size() << "\n";
	}

	for (spc = 0; spc < num_Spc; spc++){
		cout << "The total number of dead agent cells in the library[" << spc << "] is " << agt_M[spc].size() << "\n";
	}

	for (spc = 0; spc < num_Spc; spc++){
		cout << "(tick = true) The total number of male adult agents of species " << spc << " is " << celNum_0[0][0][spc][0][(num_Stg[spc][0] - 1)] << "\n";
	}

	for (spc = 0; spc < num_Spc; spc++){
		cout << "(tick = true) The total number of female adult agents of species " << spc << " is " << celNum_0[0][0][spc][1][(num_Stg[spc][1] - 1)] << "\n";
	}

	for (spc = 0; spc < num_Spc; spc++){
		cout << "(tick = false) The total number of male adult agents of species " << spc << " is " << celNum_1[0][0][spc][0][(num_Stg[spc][0] - 1)] << "\n";
	}

	for (spc = 0; spc < num_Spc; spc++){
		cout << "(tick = false) The total number of female adult agents of species " << spc << " is " << celNum_1[0][0][spc][1][(num_Stg[spc][1] - 1)] << "\n";
	}

	for (spc = 0; spc < num_RscSpc; spc++){
		cout << "The total mass of agents of resource species " << spc << " is " << d_Rsc[0][0][spc] << "\n";
	}

	int tmp_Stg;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			demNi = (deme * num_Niche) + niche;

			for (spc = 0; spc < num_Spc; spc++){
				for (sex = 0; sex < num_Sex; sex++){
					tmp_Stg = num_Stg[spc][sex];

					for (stg = 0; stg < tmp_Stg; stg++){
						cout << "agt index list_0 " << spc << " at deme " << deme << " niche " << niche << " sex " << sex << " stage " << stg << " is " << agtLst_0[demNi][spc][sex][stg].size() << "\n";
						cout << "agt index list_1 " << spc << " at deme " << deme << " niche " << niche << " sex " << sex << " stage " << stg << " is " << agtLst_1[demNi][spc][sex][stg].size() << "\n";
						cout << "agt index list Size " << spc << " at deme " << deme << " niche " << niche << " sex " << sex << " stage " << stg << " is " << size_AgtLst[deme][niche][spc][sex][stg] << "\n";
					}//end stage loop
				}//end sex loop				
			}//end species loop
		}//end niche loop
	}//end deme loop

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				cout << "Density of species " << spc << " at deme " << deme << " niche " << niche << " is " << density_Spc[deme][niche][spc] << "\n";
			}//end species loop
		}//end niche loop
	}//end deme loop

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				cout << "Density of species " << spc << " at deme " << deme << " niche " << niche << " is " << density_Rsc[deme][niche][spc] << "\n";
			}//end species loop
		}//end niche loop
	}//end deme loop

	cout << "Type a character to proceed" << endl;
	char tt;
	cin >> tt;
}

void Simul_Proc::Set_Data_Output_Buffer(int** p_Class_Trait)
{
	int spc, trt, i, j, k, l, m, end_0;

	for (spc = 0; spc < num_Spc; spc++){
		for (trt = 0; trt < num_Trait[spc]; trt++){
			int_Num_Class_Trait[spc][trt] = p_Class_Trait[spc][trt];
		}//end trait loop
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		for (trt = 0; trt < num_Trait[spc]; trt++){
			double_Num_Class_Trait_Minus_One[spc][trt] = (double)(p_Class_Trait[spc][trt] - 1);
		}//end trait loop
	}//end species loop

	tmp_Data_Buffer_Trait = new int ***[tot_Deme_Niche];

	for (i = 0; i < tot_Deme_Niche; i++)
	{
		tmp_Data_Buffer_Trait[i] = new int **[num_Spc];

		for (j = 0; j < num_Spc; j++)
		{//max_Num_Clas_Trt
			tmp_Data_Buffer_Trait[i][j] = new int *[max_Num_Trait];

			for (k = 0; k < max_Num_Trait; k++)
			{
				tmp_Data_Buffer_Trait[i][j][k] = new int[max_Num_Clas_Trt];

				for (l = 0; l < max_Num_Clas_Trt; l++)
				{
					tmp_Data_Buffer_Trait[i][j][k][l] = 0;
				}
			}
		}
	}

	max_Num_Trait_Combination = 0;

	num_Trait_Combination = new int[num_Spc];
	int tmp_Trait_Comb_Spc = 0;//temporary sum

	for (spc = 0; spc < num_Spc; spc++){
		end_0 = m_Num_Traits_Foraging - 1;//the number of traits
		k = 0;
		m = 1;//initialization for counting the combination of traits

		for (trt = end_0; trt >= 0; trt--){
			//cout << "trt " << trt << " m " << m << endl;
			//char tt;
			//cin >> tt;

			multiplier_Trait_Combination[spc][trt] = m;//sets the multiplier for the trait combination
			k += m * (num_Clas_Trt_Spc[spc][trt] - 1);
			m *= num_Clas_Trt_Spc[spc][trt];
		}//end trait loop

		num_Trait_Combination[spc] = k + 1;
		tmp_Trait_Comb_Spc += num_Trait_Combination[spc];

		if (num_Trait_Combination[spc] > max_Num_Trait_Combination)
		{
			max_Num_Trait_Combination = num_Trait_Combination[spc];
		}//end if
	}//end species loop

	tot_Num_Cel_Trait_Comb = num_Deme * num_Niche * tmp_Trait_Comb_Spc;//the total number of cells for trait combination per sampling time step

	tmp_Data_Buffer_Trait_Foraging_Combination = new int ***[num_Deme];

	for (i = 0; i < num_Deme; i++){
		tmp_Data_Buffer_Trait_Foraging_Combination[i] = new int **[num_Niche];

		for (j = 0; j < num_Niche; j++){
			tmp_Data_Buffer_Trait_Foraging_Combination[i][j] = new int *[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				tmp_Data_Buffer_Trait_Foraging_Combination[i][j][spc] = new int[max_Num_Trait_Combination];

				for (l = 0; l < max_Num_Trait_Combination; l++){
					tmp_Data_Buffer_Trait_Foraging_Combination[i][j][spc][l] = 0;//initialization
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_Trait_Combination = new int ***[num_Deme];

	for (i = 0; i < num_Deme; i++){
		data_Buffer_Trait_Combination[i] = new int **[num_Niche];

		for (j = 0; j < num_Niche; j++){
			data_Buffer_Trait_Combination[i][j] = new int *[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				data_Buffer_Trait_Combination[i][j][spc] = new int[max_Num_Trait_Combination];

				for (l = 0; l < max_Num_Trait_Combination; l++){
					data_Buffer_Trait_Combination[i][j][spc][l] = 0;//initialization
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	int tmp_Max_Trt = max_Num_Clas_Trt;

	data_Buffer_Bodysize_Mtg = new int ****[num_Deme];

	for (i = 0; i < num_Deme; i++){
		data_Buffer_Bodysize_Mtg[i] = new int ***[num_Niche];

		for (j = 0; j < num_Niche; j++){
			data_Buffer_Bodysize_Mtg[i][j] = new int **[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				data_Buffer_Bodysize_Mtg[i][j][spc] = new int *[max_Num_Clas_Trt];

				for (l = 0; l < max_Num_Clas_Trt; l++){
					data_Buffer_Bodysize_Mtg[i][j][spc][l] = new int[max_Num_Clas_Trt];//initialization

					for (m = 0; m < max_Num_Clas_Trt; m++){
						data_Buffer_Bodysize_Mtg[i][j][spc][l][m] = 0;//initialization
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_Bodysize_Genotype_Mtg = new int ****[num_Deme];

	for (i = 0; i < num_Deme; i++){
		data_Buffer_Bodysize_Genotype_Mtg[i] = new int ***[num_Niche];

		for (j = 0; j < num_Niche; j++){
			data_Buffer_Bodysize_Genotype_Mtg[i][j] = new int **[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				data_Buffer_Bodysize_Genotype_Mtg[i][j][spc] = new int *[max_Num_Clas_Trt];

				for (l = 0; l < max_Num_Clas_Trt; l++){
					data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l] = new int[max_Num_Clas_Trt];//initialization

					for (m = 0; m < max_Num_Clas_Trt; m++){
						data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l][m] = 0;//initialization
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_Gillraker_Genotype_Mtg = new int ****[num_Deme];

	for (i = 0; i < num_Deme; i++){
		data_Buffer_Gillraker_Genotype_Mtg[i] = new int ***[num_Niche];

		for (j = 0; j < num_Niche; j++){
			data_Buffer_Gillraker_Genotype_Mtg[i][j] = new int **[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				data_Buffer_Gillraker_Genotype_Mtg[i][j][spc] = new int *[max_Num_Clas_Trt];

				for (l = 0; l < max_Num_Clas_Trt; l++){
					data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l] = new int[max_Num_Clas_Trt];//initialization

					for (m = 0; m < max_Num_Clas_Trt; m++){
						data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l][m] = 0;//initialization
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_Trait = new int ****[interval_sampling];

	for (i = 0; i < interval_sampling; i++){
		data_Buffer_Trait[i] = new int ***[tot_Deme_Niche];

		for (j = 0; j < tot_Deme_Niche; j++){
			data_Buffer_Trait[i][j] = new int **[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				data_Buffer_Trait[i][j][spc] = new int *[max_Num_Trait];

				for (trt = 0; trt < max_Num_Trait; trt++){
					data_Buffer_Trait[i][j][spc][trt] = new int[max_Num_Clas_Trt];
				}//end trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	int n, o;
	tmp_Max_Trt = max_Num_Clas_Trt;

	tmp_Data_Buffer_Mtg_Trait = new double ******[num_Deme];

	for (i = 0; i < num_Deme; i++)
	{
		tmp_Data_Buffer_Mtg_Trait[i] = new double*****[num_Niche];

		for (j = 0; j < num_Niche; j++)
		{
			tmp_Data_Buffer_Mtg_Trait[i][j] = new double****[num_Spc];

			for (k = 0; k < num_Spc; k++)
			{
				tmp_Data_Buffer_Mtg_Trait[i][j][k] = new double***[tmp_Max_Trt];

				for (l = 0; l < tmp_Max_Trt; l++)
				{
					tmp_Data_Buffer_Mtg_Trait[i][j][k][l] = new double**[tmp_Max_Trt];

					for (m = 0; m < tmp_Max_Trt; m++)
					{
						tmp_Data_Buffer_Mtg_Trait[i][j][k][l][m] = new double*[tmp_Max_Trt];

						for (n = 0; n < tmp_Max_Trt; n++)
						{
							tmp_Data_Buffer_Mtg_Trait[i][j][k][l][m][n] = new double[tmp_Max_Trt];

							for (o = 0; o < tmp_Max_Trt; o++)
							{
								tmp_Data_Buffer_Mtg_Trait[i][j][k][l][m][n][o] = 0.0;
							}//end female lower bound loop
						}//end female upper bound loop
					}//end male lower bound loop
				}//end male upper bound loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	exp_Data_Buffer_Mtg_Trait = new double***[tmp_Max_Trt];

	for (l = 0; l < tmp_Max_Trt; l++)
	{
		exp_Data_Buffer_Mtg_Trait[l] = new double**[tmp_Max_Trt];

		for (m = 0; m < tmp_Max_Trt; m++)
		{
			exp_Data_Buffer_Mtg_Trait[l][m] = new double*[tmp_Max_Trt];

			for (n = 0; n < tmp_Max_Trt; n++)
			{
				exp_Data_Buffer_Mtg_Trait[l][m][n] = new double[tmp_Max_Trt];

				for (o = 0; o < tmp_Max_Trt; o++)
				{
					exp_Data_Buffer_Mtg_Trait[l][m][n][o] = 0.0;
				}//end female lower bound loop
			}//end female upper bound loop
		}//end male lower bound loop
	}//end male upper bound loop

	data_Buffer_Mtg_Trait = new double***[tmp_Max_Trt];

	for (l = 0; l < tmp_Max_Trt; l++)
	{
		data_Buffer_Mtg_Trait[l] = new double**[tmp_Max_Trt];

		for (m = 0; m < tmp_Max_Trt; m++)
		{
			data_Buffer_Mtg_Trait[l][m] = new double*[tmp_Max_Trt];

			for (n = 0; n < tmp_Max_Trt; n++)
			{
				data_Buffer_Mtg_Trait[l][m][n] = new double[tmp_Max_Trt];

				for (o = 0; o < tmp_Max_Trt; o++)
				{
					data_Buffer_Mtg_Trait[l][m][n][o] = 0.0;
				}//end female lower bound loop
			}//end female upper bound loop
		}//end male lower bound loop
	}//end male upper bound loop

	num_Param_KolmogorovSmirnov = 7;//the number of parameters for Kolmogorov-Smirnov test

	data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype = new double ***[num_Deme];

	for (i = 0; i < num_Deme; i++)
	{
		data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i] = new double**[num_Niche];

		for (j = 0; j < num_Niche; j++)
		{
			data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i][j] = new double*[num_Spc];

			for (k = 0; k < num_Spc; k++)
			{
				data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i][j][k] = new double[num_Param_KolmogorovSmirnov];

				for (l = 0; l < num_Param_KolmogorovSmirnov; l++)
				{
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[i][j][k][l] = 0.0;
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass = new double ***[num_Deme];

	for (i = 0; i < num_Deme; i++)
	{
		data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i] = new double**[num_Niche];

		for (j = 0; j < num_Niche; j++)
		{
			data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i][j] = new double*[num_Spc];

			for (k = 0; k < num_Spc; k++)
			{
				data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i][j][k] = new double[num_Param_KolmogorovSmirnov];

				for (l = 0; l < num_Param_KolmogorovSmirnov; l++)
				{
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[i][j][k][l] = 0.0;
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker = new double ***[num_Deme];

	for (i = 0; i < num_Deme; i++)
	{
		data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i] = new double**[num_Niche];

		for (j = 0; j < num_Niche; j++)
		{
			data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i][j] = new double*[num_Spc];

			for (k = 0; k < num_Spc; k++)
			{
				data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i][j][k] = new double[num_Param_KolmogorovSmirnov];

				for (l = 0; l < num_Param_KolmogorovSmirnov; l++)
				{
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[i][j][k][l] = 0.0;
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	data_Buffer_RscCls = new double ****[interval_sampling];

	for (i = 0; i < interval_sampling; i++)
	{
		data_Buffer_RscCls[i] = new double ***[num_Deme];

		for (j = 0; j < num_Deme; j++)
		{
			data_Buffer_RscCls[i][j] = new double **[num_Niche];

			for (k = 0; k < num_Niche; k++)
			{
				data_Buffer_RscCls[i][j][k] = new double *[num_RscSpc];

				for (l = 0; l < num_RscSpc; l++)
				{
					data_Buffer_RscCls[i][j][k][l] = new double[max_Num_PhenTyp_Rsc];

					for (m = 0; m < max_Num_PhenTyp_Rsc; m++)
					{
						data_Buffer_RscCls[i][j][k][l][m] = 0.0;
					}//end resource class loop
				}//end resource cluster loop
			}//end niche loop
		}//end j loop
	}//end i loop

	//Genetic data buffer for each time step
	tmp_Data_Buffer_Allele = new int ***[tot_Deme_Niche_Spc];

	for (i = 0; i < tot_Deme_Niche_Spc; i++){
		tmp_Data_Buffer_Allele[i] = new int **[max_Num_Locus];

		for (j = 0; j < max_Num_Locus; j++){
			tmp_Data_Buffer_Allele[i][j] = new int *[max_Num_Allele];

			for (k = 0; k < max_Num_Allele; k++){
				tmp_Data_Buffer_Allele[i][j][k] = new int[max_Num_Allele];

				for (l = 0; l < max_Num_Allele; l++){
					tmp_Data_Buffer_Allele[i][j][k][l] = 0;
				}//end l loop
			}//end k loop
		}//end j loop
	}//end i loop

	//Genetic data buffer for each sampling interval
	data_Buffer_Allele = new int****[interval_sampling];

	for (i = 0; i < interval_sampling; i++){
		data_Buffer_Allele[i] = new int***[tot_Deme_Niche_Spc];

		for (j = 0; j < tot_Deme_Niche_Spc; j++){
			data_Buffer_Allele[i][j] = new int**[max_Num_Locus];

			for (k = 0; k < max_Num_Locus; k++){
				data_Buffer_Allele[i][j][k] = new int*[max_Num_Allele];

				for (l = 0; l < max_Num_Allele; l++){
					data_Buffer_Allele[i][j][k][l] = new int[max_Num_Allele];

					for (m = 0; m < max_Num_Allele; m++){
						data_Buffer_Allele[i][j][k][l][m] = 0;
					}//end l loop
				}//end k loop
			}//end j loop
		}//end deme loop
	}//end sampling interval loop
}

void Simul_Proc::Data_Output_Buffer_Initialization(int *p_Counter_Sampling, int *p_CrntTstep)
{
	//p_CrntTstep: the time step that the data buffer is initialized
	if ((*p_Counter_Sampling) == 0){
		int i, j, spc, k, l, m, n, o, end_0;

		//sets a key time step for output-data buffering
		tstep_Key_Buffer_Output = (*p_CrntTstep);

		for (i = 0; i < interval_sampling; i++){
			for (j = 0; j < tot_Deme_Niche_Spc; j++){
				spc = (j % num_Niche_Num_Spc) % num_Spc;//species
				end_0 = num_Locus[spc];

				for (int k = 0; k < end_0; k++){
					for (l = 0; l < max_Num_Allele; l++){
						for (m = 0; m < max_Num_Allele; m++){
							data_Buffer_Allele[i][j][k][l][m] = 0;
						}//end l loop
					}//end k loop
				}//end j loop
			}//end deme loop
		}//end sampling interval loop

		if (num_RscSpc > 0){
			//Density data buffer initialization
			for (i = 0; i < interval_sampling; i++){
				for (j = 0; j < num_Deme; j++){
					for (k = 0; k < num_Niche; k++){
						for (spc = 0; spc < num_RscSpc; spc++){
							end_0 = num_PhenoTyp_Rsc[spc];

							for (l = 0; l < end_0; l++)
							{
								data_Buffer_RscCls[i][j][k][spc][l] = 0.0;
							}//end resource class loop
						}//end species loop
					}//end niche loop	
				}//end niche loop
			}//end deme loop

		}//end if

		int tmp_Max_Trt = max_Num_Clas_Trt;

		for (i = 0; i < num_Deme; i++)
		{
			for (j = 0; j < num_Niche; j++)
			{
				for (k = 0; k < num_Spc; k++)
				{
					for (l = 0; l < tmp_Max_Trt; l++)
					{
						for (m = 0; m < tmp_Max_Trt; m++)
						{
							for (n = 0; n < tmp_Max_Trt; n++)
							{
								for (o = 0; o < tmp_Max_Trt; o++)
								{
									tmp_Data_Buffer_Mtg_Trait[i][j][k][l][m][n][o] = 0.0;
								}//end female lower bound loop
							}//end female upper bound loop
						}//end male lower bound loop
					}//end male upper bound loop
				}//end species loop
			}//end niche loop
		}//end deme loop

		tmp_Max_Trt = max_Num_Bodymass_Cls;

		for (i = 0; i < num_Deme; i++)
		{
			for (j = 0; j < num_Niche; j++)
			{
				for (k = 0; k < num_Spc; k++)
				{
					for (l = 0; l< tmp_Max_Trt; l++)
					{
						for (m = 0; m < tmp_Max_Trt; m++)
						{
							tmp_Data_Buffer_Mtg_Bodymass[i][j][k][l][m] = 0.0;
						}//end male bodymass class loop
					}//end male bodymass class loop
				}//end species loop
			}//end niche loop
		}//end deme loop

		tmp_Max_Trt = max_Num_GillRaker_Cls;

		for (i = 0; i < num_Deme; i++)
		{
			for (j = 0; j < num_Niche; j++)
			{
				for (k = 0; k < num_Spc; k++)
				{
					for (l = 0; l < tmp_Max_Trt; l++)
					{
						for (m = 0; m < tmp_Max_Trt; m++)
						{
							tmp_Data_Buffer_Mtg_GillRaker[i][j][k][l][m] = 0.0;
						}//end male bodymass class loop
					}//end male bodymass class loop
				}//end species loop
			}//end niche loop
		}//end deme loop

	}//end if
}

void Simul_Proc::Data_Output_Buffer_Set_Cell_Trait(void)
{
	int  j, spc, trt, cls;
	tot_Num_Cel_Trait_Class = 0;//the total number of cells in the trait-output-data buffer

	for (j = 0; j < tot_Deme_Niche; j++){
		for (spc = 0; spc < num_Spc; spc++){
			for (trt = 0; trt < num_Trait[spc]; trt++){
				for (cls = 0; cls < int_Num_Class_Trait[spc][trt]; cls++){
					//increases the total number of cells in the trait-output-data buffer
					tot_Num_Cel_Trait_Class++;
				}//end class loop
			}//end trait loop
		}//end species loop
	}//end niche loop
}


void Simul_Proc::Data_Output_Buffer_Set_Cell_KolmogorovSmirnov(void)
{
	int i, j, k, l;
	tot_Num_Cel_KolmogorovSmirnov = 0;

	for (i = 0; i < num_Deme; i++)
	{
		for (j = 0; j < num_Niche; j++)
		{
			for (k = 0; k < num_Spc; k++)
			{
				for (l = 0; l < num_Param_KolmogorovSmirnov; l++)
				{
					tot_Num_Cel_KolmogorovSmirnov++;//the total number of cells in the Kolmogorov-Smirnov-test-output-data buffer
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Data_Output_Buffer_Set_Cell_Genome(void)
{
	int j, k, l, m, spc, end_0;
	tot_Num_Cel_Genome = 0;//the total number of cells in the trait-output-data buffer

	//genome data buffer initialization
	for (j = 0; j < tot_Deme_Niche_Spc; j++){
		spc = (j % num_Niche_Num_Spc) % num_Spc;//species
		end_0 = num_Locus[spc];

		for (k = 0; k < end_0; k++){
			for (l = 0; l < max_Num_Allele; l++){
				for (m = l; m < max_Num_Allele; m++){
					tot_Num_Cel_Genome++;
				}//end l loop
			}//end k loop
		}//end j loop
	}//end deme loop
}

void Simul_Proc::Set_File_Name(string string_Simul_Counter)
{
	string string_CSV = ".csv";

	/////////////////////////////////
	//sets the file name for traits//
	fileName_Trait_Str = "tr" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for trait combinations//
	fileName_Trait_Comb_Str = "tc" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mate trait combinations//
	fileName_Bodysize_Mtg_Str = "mtg_U_Ph" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mate trait combinations//
	fileName_Bodysize_Mtg_Adt_Str = "mtg_U_Ph_Sum" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mate trait combinations//
	fileName_Bodysize_Genotype_Mtg_Str = "mtg_U_Gn" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mate trait combinations//
	fileName_Bodysize_Genotype_Mtg_Adt_Str = "mtg_U_Gn_Sum" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mate trait combinations//
	fileName_GillRaker_Genotype_Mtg_Str = "mtg_L_Gn" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mate trait combinations//
	fileName_GillRaker_Genotype_Mtg_Adt_Str = "mtg_L_Gn_Sum" + string_Simul_Counter + string_CSV;

	//////////////////////////////////
	//sets the file name for genomes//
	fileName_Genome_Str = "gn" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mating pair genotypes//
	fileName_KolmogorovSmirnov_Mtg_Genotype_Str = "kG" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mating pair bodymass//
	fileName_KolmogorovSmirnov_Mtg_Bodymass_Str = "kB" + string_Simul_Counter + string_CSV;

	/////////////////////////////////
	//sets the file name for mating pair bodymass//
	fileName_KolmogorovSmirnov_Mtg_GillRaker_Str = "kR" + string_Simul_Counter + string_CSV;

	////////////////////////////////////
	//sets the file name for densities//
	fileName_Density_Str = "rc" + string_Simul_Counter + string_CSV;
}

void Simul_Proc::Set_Simul_File_Output(void)
{
	int size_String;

	/////////////////////////////////
	//sets the file name for traits//
	size_String = (int)fileName_Trait_Str.length();
	fileName_Trait = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Trait_Str, fileName_Trait);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Trait);//creates the file for traits

	//sets the number of cells for data output
	Data_Output_Buffer_Set_Cell_Trait();

	/////////////////////////////////
	//sets the file name for trait combinations//
	size_String = (int)fileName_Trait_Comb_Str.length();
	fileName_Trait_Comb = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Trait_Comb_Str, fileName_Trait_Comb);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Trait_Comb);//creates the file for traits

	/////////////////////////////////
	//sets the file name for mate pair bodysize//
	size_String = (int)fileName_Bodysize_Mtg_Str.length();
	fileName_Bodysize_Mtg = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Bodysize_Mtg_Str, fileName_Bodysize_Mtg);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Bodysize_Mtg);//creates the file for traits

	/////////////////////////////////
	//sets the file name for mate pair bodysize//
	size_String = (int)fileName_Bodysize_Mtg_Adt_Str.length();
	fileName_Bodysize_Mtg_Adt = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Bodysize_Mtg_Adt_Str, fileName_Bodysize_Mtg_Adt);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Bodysize_Mtg_Adt);//creates the file for traits

	/////////////////////////////////
	//sets the file name for trait combinations//
	size_String = (int)fileName_Bodysize_Genotype_Mtg_Str.length();
	fileName_Bodysize_Genotype_Mtg = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Bodysize_Genotype_Mtg_Str, fileName_Bodysize_Genotype_Mtg);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Bodysize_Genotype_Mtg);//creates the file for traits

	/////////////////////////////////
	//sets the file name for trait combinations//
	size_String = (int)fileName_Bodysize_Genotype_Mtg_Adt_Str.length();
	fileName_Bodysize_Genotype_Mtg_Adt = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Bodysize_Genotype_Mtg_Adt_Str, fileName_Bodysize_Genotype_Mtg_Adt);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Bodysize_Genotype_Mtg_Adt);//creates the file for traits

	/////////////////////////////////
	//sets the file name for trait combinations//
	size_String = (int)fileName_GillRaker_Genotype_Mtg_Str.length();
	fileName_GillRaker_Genotype_Mtg = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_GillRaker_Genotype_Mtg_Str, fileName_GillRaker_Genotype_Mtg);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_GillRaker_Genotype_Mtg);//creates the file for traits

	/////////////////////////////////
	//sets the file name for trait combinations//
	size_String = (int)fileName_GillRaker_Genotype_Mtg_Adt_Str.length();
	fileName_GillRaker_Genotype_Mtg_Adt = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_GillRaker_Genotype_Mtg_Adt_Str, fileName_GillRaker_Genotype_Mtg_Adt);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_GillRaker_Genotype_Mtg_Adt);//creates the file for traits

	//////////////////////////////////
	//sets the file name for genomes//
	size_String = (int)fileName_Genome_Str.length();
	fileName_Genome = new char[(size_String + 1)];//one more cell for ending character array

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Genome_Str, fileName_Genome);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Genome);//creates the file for traits

	//sets the number of cells for data output
	Data_Output_Buffer_Set_Cell_Genome();

	/////////////////////////////////
	//sets the file name for mating pair genotypes//
	size_String = (int)fileName_KolmogorovSmirnov_Mtg_Genotype_Str.length();
	fileName_KolmogorovSmirnov_Mtg_Genotype = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_KolmogorovSmirnov_Mtg_Genotype_Str, fileName_KolmogorovSmirnov_Mtg_Genotype);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_KolmogorovSmirnov_Mtg_Genotype);//creates the file for traits

	//sets the number of cells for data output
	Data_Output_Buffer_Set_Cell_KolmogorovSmirnov();

	/////////////////////////////////
	//sets the file name for mating pair bodymass//
	size_String = (int)fileName_KolmogorovSmirnov_Mtg_Bodymass_Str.length();
	fileName_KolmogorovSmirnov_Mtg_Bodymass = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_KolmogorovSmirnov_Mtg_Bodymass_Str, fileName_KolmogorovSmirnov_Mtg_Bodymass);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_KolmogorovSmirnov_Mtg_Bodymass);//creates the file for traits

	/////////////////////////////////
	//sets the file name for mating pair bodymass//
	size_String = (int)fileName_KolmogorovSmirnov_Mtg_GillRaker_Str.length();
	fileName_KolmogorovSmirnov_Mtg_GillRaker = new char[(size_String + 1)];//one more cell for ending character array	

	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_KolmogorovSmirnov_Mtg_GillRaker_Str, fileName_KolmogorovSmirnov_Mtg_GillRaker);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_KolmogorovSmirnov_Mtg_GillRaker);//creates the file for traits

	////////////////////////////////////
	//sets the file name for densities//
	size_String = (int)fileName_Density_Str.length();
	fileName_Density = new char[(size_String + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	csvRW.Copy_String_To_Char_Arr(fileName_Density_Str, fileName_Density);//copy string and paste on char* 
	csvRW.CSV_Create(fileName_Density);//creates the file for densities

	//tmp_fileName = "RcBm" + string_Simul_Counter + string_CSV;
	//size_String = (int)tmp_fileName.length();
	//fileName_Resource_Class_Bodymass = new char[(size_String + 1)];//one more cell for ending character array
	//the number of cells in char* should be (the length of string + 1)
	//csvRW.Copy_String_To_Char_Arr(tmp_fileName, fileName_Resource_Class_Bodymass);//copy string and paste on char* 
	//csvRW.CSV_Create(fileName_Resource_Class_Bodymass);//creates the file for densities

	status_Simul_File_Creation = true;//file creation status; true: simulation files are created ,false: files are not created
	Set_File_Column_Header();//sets column-headers for output files
}

void Simul_Proc::Data_Output_Buffer_Write(int *p_Counter_Sampling, int *p_Current_Tstep)
{
	if ((*p_Counter_Sampling) == (interval_sampling - 1)){
		Data_Output_Buffer_Write_Trait();//writes trait output data
		Data_Output_Buffer_Write_Trait_Comb(p_Current_Tstep);
		Data_Output_Buffer_Write_Genome();//writes genome output data

		if (num_RscSpc > 0){
			Data_Output_Buffer_Write_Density();//writes resource density output data
		}//end if
	}//end if

	if ((*p_Counter_Sampling) == 0){
		//writes mate pair bodylength data
		Data_Output_Buffer_Write_Bodylength_Mate_Pair(p_Current_Tstep);

		Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_Genotype(p_Current_Tstep);
		//Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_Bodymass(p_Current_Tstep);
		//Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_GillRaker(p_Current_Tstep);
	}//end if
}

void Simul_Proc::Data_Output_Buffer_Write_Trait(void)
{
	int i, j, spc, trt, cls, numRow, numCol, counter_Column, end_0, end_1;

	////////////////////////////
	//Add data to trait output//
	numRow = interval_sampling;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Trait_Class;

	for (i = 0; i < interval_sampling; i++){
		counter_Column = 0;
		data_Buffer_Trait_2D[i][counter_Column] = i + tstep_Key_Buffer_Output;
		counter_Column++;

		for (j = 0; j < tot_Deme_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = num_Trait[spc];

				for (trt = 0; trt < end_0; trt++){
					end_1 = int_Num_Class_Trait[spc][trt];

					for (cls = 0; cls < end_1; cls++){
						data_Buffer_Trait_2D[i][counter_Column] = data_Buffer_Trait[i][j][spc][trt][cls];

						//initialize data container
						data_Buffer_Trait[i][j][spc][trt][cls] = 0;
						counter_Column++;
					}//end class loop
				}//end trait loop
			}//end species loop
		}//end deme_niche loop
	}//end sampling interval loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Trait, data_Buffer_Trait_2D, &numRow, &numCol);
}

void Simul_Proc::Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_Genotype(int *p_Current_Tstep)
{
	int i, deme, niche, stat, spc, numRow, numCol, counter_Column;

	////////////////////////////
	//Add data to trait output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_KolmogorovSmirnov;
	double **str_Tmp;//buffer for trait output
	str_Tmp = new double*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new double[numCol];
	}//end row loop

	counter_Column = 0;
	str_Tmp[0][counter_Column] = ((double)(*p_Current_Tstep));//sets the current timestep
	counter_Column++;

	for (deme = 0; deme < num_Deme; deme++)
	{
		for (niche = 0; niche < num_Niche; niche++)
		{
			for (spc = 0; spc < num_Spc; spc++)
			{
				for (stat = 0; stat < num_Param_KolmogorovSmirnov; stat++)
				{
					str_Tmp[0][counter_Column] = data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][stat];
					counter_Column++;
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end i loop

	csvRW.CSV_Write_Add_Double_2D(fileName_KolmogorovSmirnov_Mtg_Genotype, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_Bodymass(int *p_Current_Tstep)
{
	int i, deme, niche, stat, spc, numRow, numCol, counter_Column;

	////////////////////////////
	//Add data to trait output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_KolmogorovSmirnov;
	double **str_Tmp;//buffer for trait output
	str_Tmp = new double*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new double[numCol];
	}//end row loop

	counter_Column = 0;
	str_Tmp[0][counter_Column] = ((double)(*p_Current_Tstep));//sets the current timestep
	counter_Column++;

	for (deme = 0; deme < num_Deme; deme++)
	{
		for (niche = 0; niche < num_Niche; niche++)
		{
			for (spc = 0; spc < num_Spc; spc++)
			{
				for (stat = 0; stat < num_Param_KolmogorovSmirnov; stat++)
				{
					str_Tmp[0][counter_Column] = data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][stat];
					counter_Column++;
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end i loop

	csvRW.CSV_Write_Add_Double_2D(fileName_KolmogorovSmirnov_Mtg_Bodymass, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Data_Output_Buffer_Write_KolmogorovSmirnov_Mtg_GillRaker(int *p_Current_Tstep)
{
	int i, deme, niche, stat, spc, numRow, numCol, counter_Column;

	////////////////////////////
	//Add data to trait output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_KolmogorovSmirnov;
	double **str_Tmp;//buffer for trait output
	str_Tmp = new double*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new double[numCol];
	}//end row loop

	counter_Column = 0;
	str_Tmp[0][counter_Column] = ((double)(*p_Current_Tstep));//sets the current timestep
	counter_Column++;

	for (deme = 0; deme < num_Deme; deme++)
	{
		for (niche = 0; niche < num_Niche; niche++)
		{
			for (spc = 0; spc < num_Spc; spc++)
			{
				for (stat = 0; stat < num_Param_KolmogorovSmirnov; stat++)
				{
					str_Tmp[0][counter_Column] = data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][stat];
					counter_Column++;
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end i loop

	csvRW.CSV_Write_Add_Double_2D(fileName_KolmogorovSmirnov_Mtg_GillRaker, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Data_Output_Buffer_Write_Trait_Comb(int *p_Current_Tstep)
{
	int i, j, spc, trt, numRow, numCol, counter_Column, end_0;

	////////////////////////////
	//Add data to trait output//
	numRow = 1;//the total number of cells in the trait-output-data buffer (Do not change this)
	numCol = 1 + tot_Num_Cel_Trait_Comb;

	counter_Column = 0;
	data_Buffer_Trait_Combination_2D[0][counter_Column] = (*p_Current_Tstep);//sets the current timestep
	counter_Column++;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = num_Trait_Combination[spc];

				for (trt = 0; trt < end_0; trt++){
					data_Buffer_Trait_Combination_2D[0][counter_Column] = data_Buffer_Trait_Combination[i][j][spc][trt];
					counter_Column++;
				}//end trait loop
			}//end species loop
		}//end niche loop
	}//end i loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Trait_Comb, data_Buffer_Trait_Combination_2D, &numRow, &numCol);
}

void Simul_Proc::Data_Output_Buffer_Write_Genome(void)
{
	int i, j, spc, loc, k, l, numRow, numCol, counter_Column, end_0;

	////////////////////////////
	//Add data to trait output//
	numRow = interval_sampling;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Genome;

	for (i = 0; i < interval_sampling; i++){
		counter_Column = 0;
		data_Buffer_Allele_2D[i][counter_Column] = i + tstep_Key_Buffer_Output;
		counter_Column++;

		for (j = 0; j < tot_Deme_Niche_Spc; j++){
			spc = (j % num_Niche_Num_Spc) % num_Spc;
			end_0 = num_Locus[spc];

			for (loc = 0; loc < end_0; loc++){
				for (k = 0; k < max_Num_Allele; k++){
					for (l = k; l < max_Num_Allele; l++){
						data_Buffer_Allele_2D[i][counter_Column] = data_Buffer_Allele[i][j][loc][k][l];
						counter_Column++;
					}//end l loop
				}//end trait loop
			}//end niche loop
		}//end deme loop
	}//end sampling interval loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Genome, data_Buffer_Allele_2D, &numRow, &numCol);
}

void Simul_Proc::Data_Output_Buffer_Write_Density(void)
{
	int i, j, k, spc, cls, end0, numRow, numCol, counter_Column;
	//sets a key time step for output-data buffering

	////////////////////////////
	//Add data to trait output//
	numRow = interval_sampling;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * tot_Num_Rsc_Cls);

	for (i = 0; i < interval_sampling; i++){
		counter_Column = 0;//initialization
		data_Buffer_RscCls_2D[i][counter_Column] = (double)(i + tstep_Key_Buffer_Output);
		counter_Column++;

		for (j = 0; j < num_Deme; j++){
			for (k = 0; k < num_Niche; k++){
				for (spc = 0; spc < num_RscSpc; spc++){
					end0 = num_PhenoTyp_Rsc[spc];

					for (cls = 0; cls < end0; cls++)
					{
						data_Buffer_RscCls_2D[i][counter_Column] = data_Buffer_RscCls[i][j][k][spc][cls];
						counter_Column++;
					}//end resource class loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}//end time step loop

	csvRW.CSV_Write_Add_Double_2D(fileName_Density, data_Buffer_RscCls_2D, &numRow, &numCol);
}

void Simul_Proc::Set_File_Column_Header(void)
{
	//sets column-headers for trait output files
	Set_File_Column_Header_Trait();

	//sets column-headers for trait combinationoutput files
	Set_File_Column_Header_Trait_Comb();

	//sets column-headers for mate pair bodysize files
	Set_File_Column_Header_Bodysize_Mtg();

	//sets column-headers for mate pair bodysize genotype files
	Set_File_Column_Header_Bodysize_Genotype_Mtg();

	//sets column-headers for mate pair gillraker genotype files
	Set_File_Column_Header_GillRaker_Genotype_Mtg();


	//sets column-headers for genome output files
	Set_File_Column_Header_Genome();

	//sets column-headers for density output files
	Set_File_Column_Header_Density();

	Set_File_Column_Header_KolmogorovSmirnov(fileName_KolmogorovSmirnov_Mtg_Genotype);//sets column-headers for Kolmogorov-Smirnov test
	Set_File_Column_Header_KolmogorovSmirnov(fileName_KolmogorovSmirnov_Mtg_Bodymass);//sets column-headers for Kolmogorov-Smirnov test
	Set_File_Column_Header_KolmogorovSmirnov(fileName_KolmogorovSmirnov_Mtg_GillRaker);//sets column-headers for Kolmogorov-Smirnov test
}

void Simul_Proc::Set_File_Column_Header_KolmogorovSmirnov(char *p_FileName)
{
	int i, numRow, numCol;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_stats = "P";

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_KolmogorovSmirnov;//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;
	int deme, niche, spc, stat;

	for (deme = 0; deme < num_Deme; deme++)
	{
		for (niche = 0; niche < num_Niche; niche++)
		{
			for (spc = 0; spc < num_Spc; spc++)
			{
				for (stat = 0; stat < num_Param_KolmogorovSmirnov; stat++)
				{
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_stats + std::to_string(stat);
					counter_Column++;//increases the row counter
				}//end statistics loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(p_FileName, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Set_File_Column_Header_Trait(void)
{
	int i, j, spc, numRow, numCol, end_0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_Trait = "T";
	string str_TraitClass = "K";
	int s_Deme, s_Niche, trt, end_1, cls;

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Trait_Class;//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	for (j = 0; j < tot_Deme_Niche; j++){
		s_Deme = (int)(((double)j) / ((double)num_Niche));//deme
		s_Niche = j % num_Niche;//niche

		for (spc = 0; spc < num_Spc; spc++){
			end_0 = num_Trait[spc];

			for (trt = 0; trt < end_0; trt++){
				end_1 = int_Num_Class_Trait[spc][trt];

				for (cls = 0; cls < end_1; cls++){
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(s_Deme) + str_Niche + std::to_string(s_Niche) + str_Species + std::to_string(spc) + str_Trait + std::to_string(trt) + str_TraitClass + std::to_string(cls);
					counter_Column++;//increases the row counter
				}//end class loop
			}//end trait loop
		}//end species loop
	}//end deme_niche loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Trait, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;

	numRow = interval_sampling;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Trait_Class;
	data_Buffer_Trait_2D = new int*[numRow];

	for (i = 0; i < numRow; i++){
		data_Buffer_Trait_2D[i] = new int[numCol];
	}//end row loop
}

void Simul_Proc::Set_File_Column_Header_Trait_Value(vector<double> &tmp_TraitValue)
{
	int j, spc, end_0;
	int  trt, end_1, cls;
	double tmp;

	for (j = 0; j < tot_Deme_Niche; j++){
		//s_Deme = (int)(((double)j) / ((double)num_Niche));//deme
		//s_Niche = j % num_Niche;//niche

		for (spc = 0; spc < num_Spc; spc++){
			end_0 = num_Trait[spc];

			for (trt = 0; trt < end_0; trt++){
				end_1 = int_Num_Class_Trait[spc][trt];

				if (end_1 > 1)
				{
					for (cls = 0; cls < end_1; cls++){
						tmp = ((double)cls) / ((double)(end_1 - 1));
						tmp_TraitValue.push_back(tmp);
					}//end class loop
				}
				else
				{
					for (cls = 0; cls < end_1; cls++){
						tmp = 0.0;
						tmp_TraitValue.push_back(tmp);
					}//end class loop
				}//end else
			}//end trait loop
		}//end species loop
	}//end deme_niche loop
}

void Simul_Proc::Set_File_Column_Header_Trait_Single_Value(vector<double> &tmp_TraitValue, int *p_NumTraits, int *p_Num_Class_Trait)
{
	int j, spc, end_0;
	int  trt, end_1, cls;
	double tmp;

	for (j = 0; j < tot_Deme_Niche; j++){
		//s_Deme = (int)(((double)j) / ((double)num_Niche));//deme
		//s_Niche = j % num_Niche;//niche

		for (spc = 0; spc < num_Spc; spc++){
			end_0 = (*p_NumTraits);

			for (trt = 0; trt < end_0; trt++){
				end_1 = (*p_Num_Class_Trait);

				if (end_1 > 1)
				{
					for (cls = 0; cls < end_1; cls++){
						tmp = ((double)cls) / ((double)(end_1 - 1));
						tmp_TraitValue.push_back(tmp);
					}//end class loop
				}
				else
				{
					for (cls = 0; cls < end_1; cls++){
						tmp = 0.0;
						tmp_TraitValue.push_back(tmp);
					}//end class loop
				}//end else
			}//end trait loop
		}//end species loop
	}//end deme_niche loop
}

void Simul_Proc::Set_File_Column_Header_Trait_Comb(void)
{
	int i, j, spc, numRow, numCol, end_0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_Trait_Comb = "B";

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Trait_Comb;//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	int deme, niche, tmp_Remainder, trait, end_1;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = num_Trait_Combination[spc];
				end_1 = m_Num_Traits_Foraging;

				for (i = 0; i < end_0; i++){
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Trait_Comb;
					tmp_Remainder = i;//initializes the temporary remainder

					for (j = 0; j < end_1; j++){
						trait = tmp_Remainder / multiplier_Trait_Combination[spc][j];
						str_Tmp[0][counter_Column] += "|" + std::to_string(trait);
						tmp_Remainder = tmp_Remainder % multiplier_Trait_Combination[spc][j];
					}//end trait loop

					counter_Column++;
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Trait_Comb, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;

	numRow = 1;//the total number of cells in the trait-output-data buffer (Do not change this)
	numCol = 1 + tot_Num_Cel_Trait_Comb;
	data_Buffer_Trait_Combination_2D = new int*[numRow];

	for (i = 0; i < numRow; i++){
		data_Buffer_Trait_Combination_2D[i] = new int[numCol];
	}//end row loop
}

void Simul_Proc::Set_File_Column_Header_Bodysize_Mtg(void)
{
	int i, j, spc, numRow, numCol, end_0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_Bodysize_Male = "UPM";
	string str_Bodysize_Female = "UPF";

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * num_Spc * max_Num_Clas_Trt * max_Num_Clas_Trt);//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	int deme, niche, end_1;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = max_Num_Clas_Trt;
				end_1 = max_Num_Clas_Trt;

				for (i = 0; i < end_0; i++){
					for (j = 0; j < end_1; j++){
						str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Bodysize_Male + std::to_string(i) + str_Bodysize_Female + std::to_string(j);
						counter_Column++;//increase the counter
					}//end trait loop
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Bodysize_Mtg, str_Tmp, &numRow, &numCol);
	
	data_Buffer_Bodysize_Mtg_2D = new int*[numRow];

	for (i = 0; i < numRow; i++){
		data_Buffer_Bodysize_Mtg_2D[i] = new int[numCol];
	}//end row loop
	
	///////////////////////
	//summary data header//
	///////////////////////
	//trait data buffer initialization
	counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = max_Num_Clas_Trt;

				for (i = 0; i < end_0; i++){
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Bodysize_Male + std::to_string(i);
					counter_Column++;//increase the counter
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	int tmp_Num_Col = counter_Column;
	csvRW.CSV_Write_Add_String_2D(fileName_Bodysize_Mtg_Adt, str_Tmp, &numRow, &tmp_Num_Col);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Set_File_Column_Header_Bodysize_Genotype_Mtg(void)
{
	int i, j, spc, numRow, numCol, end_0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_Bodysize_Male = "UGM";
	string str_Bodysize_Female = "UGF";

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * num_Spc * max_Num_Clas_Trt * max_Num_Clas_Trt);//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	int deme, niche, end_1;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = max_Num_Clas_Trt;
				end_1 = max_Num_Clas_Trt;

				for (i = 0; i < end_0; i++){
					for (j = 0; j < end_1; j++){
						str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Bodysize_Male + std::to_string(i) + str_Bodysize_Female + std::to_string(j);
						counter_Column++;//increase the counter
					}//end trait loop
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Bodysize_Genotype_Mtg, str_Tmp, &numRow, &numCol);

	///////////////////////
	//summary data header//
	///////////////////////
	//trait data buffer initialization
	counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = max_Num_Clas_Trt;

				for (i = 0; i < end_0; i++){
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Bodysize_Male + std::to_string(i);
					counter_Column++;//increase the counter
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	int tmp_Num_Col = counter_Column;
	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Bodysize_Genotype_Mtg_Adt, str_Tmp, &numRow, &tmp_Num_Col);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Set_File_Column_Header_GillRaker_Genotype_Mtg(void)
{
	int i, j, spc, numRow, numCol, end_0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_Bodysize_Male = "LGM";
	string str_Bodysize_Female = "LGF";

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * num_Spc * max_Num_Clas_Trt * max_Num_Clas_Trt);//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	int deme, niche, end_1;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = max_Num_Clas_Trt;
				end_1 = max_Num_Clas_Trt;

				for (i = 0; i < end_0; i++){
					for (j = 0; j < end_1; j++){
						str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Bodysize_Male + std::to_string(i) + str_Bodysize_Female + std::to_string(j);
						counter_Column++;//increase the counter
					}//end trait loop
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_GillRaker_Genotype_Mtg, str_Tmp, &numRow, &numCol);

	//trait data buffer initialization
	counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				end_0 = max_Num_Clas_Trt;
				end_1 = max_Num_Clas_Trt;

				for (i = 0; i < end_0; i++){
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(deme) + str_Niche + std::to_string(niche) + str_Species + std::to_string(spc) + str_Bodysize_Male + std::to_string(i);
					counter_Column++;//increase the counter
				}//end trait combination loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	int tmp_Num_col = counter_Column;
	csvRW.CSV_Write_Add_String_2D(fileName_GillRaker_Genotype_Mtg_Adt, str_Tmp, &numRow, &tmp_Num_col);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Set_File_Column_Header_Genome(void)
{
	int i, j, spc, loc, k, l, numRow, numCol, end_0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_Species = "S";
	string str_Locus = "L";
	string str_Position = "P";
	int s_Deme, s_Niche;

	////////////////////////////
	//Add data to a genotype output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Genome;//add a column for time step

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	for (j = 0; j < tot_Deme_Niche_Spc; j++){
		s_Deme = (int)((((double)j) / ((double)num_Niche_Num_Spc)));//deme
		s_Niche = (int)(((double)(j % num_Niche_Num_Spc)) / ((double)num_Spc));//niche
		spc = (j % num_Niche_Num_Spc) % num_Spc;//species
		end_0 = num_Locus[spc];

		for (loc = 0; loc < end_0; loc++){
			for (k = 0; k < max_Num_Allele; k++){
				for (l = k; l < max_Num_Allele; l++){
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(s_Deme) + str_Niche + std::to_string(s_Niche) + str_Species + std::to_string(spc) + str_Locus + std::to_string(loc) + str_Position + std::to_string(k) + str_Position + std::to_string(l);
					counter_Column++;//increases the row counter
				}//end l loop
			}//end trait loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Genome, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;

	numRow = interval_sampling;//the total number of cells in the trait-output-data buffer
	numCol = 1 + tot_Num_Cel_Genome;
	data_Buffer_Allele_2D = new int*[numRow];

	for (i = 0; i < numRow; i++){
		data_Buffer_Allele_2D[i] = new int[numCol];
	}//end row loop
}

void Simul_Proc::Set_File_Column_Header_Density(void)
{
	int i, j, k, rsc, cls, numRow, numCol, totCls, end0;
	string str_Deme = "D";
	string str_Niche = "N";
	string str_ResourceSpecies = "R";
	string str_ResourceClass = "C";

	totCls = 0;//the total number of resource classes

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		totCls += num_PhenoTyp_Rsc[rsc];//adds the number of resource classes
	}//end resource loop

	////////////////////////////
	//Add data to resource class output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * totCls);

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//trait data buffer initialization
	int counter_Column = 0;
	str_Tmp[0][counter_Column] = "Timestep";//sets an index cell
	counter_Column++;
	string tmp_Str;

	for (j = 0; j < num_Deme; j++){
		for (k = 0; k < num_Niche; k++){
			for (rsc = 0; rsc < num_RscSpc; rsc++){
				end0 = num_PhenoTyp_Rsc[rsc];//the number of resource classes

				for (cls = 0; cls < end0; cls++)
				{
					str_Tmp[0][counter_Column] = str_Deme + std::to_string(j) + str_Niche + std::to_string(j) + str_ResourceSpecies + std::to_string(rsc) + str_ResourceClass + std::to_string(cls);
					counter_Column++;//increases the row counter
				}//end resource class loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//writes the column header for trait-output file
	csvRW.CSV_Write_Add_String_2D(fileName_Density, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;

	//Memory allocation
	numRow = interval_sampling;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * tot_Num_Rsc_Cls);
	data_Buffer_RscCls_2D = new double*[numRow];

	for (i = 0; i < numRow; i++){
		data_Buffer_RscCls_2D[i] = new double[numCol];
	}//end row loop
}

void Simul_Proc::Set_File_Column_Header_Density_Value(vector<double> &tmp_Bodymass)
{
	int j, k, rsc, cls, end0;

	////////////////////////////
	for (j = 0; j < num_Deme; j++){
		for (k = 0; k < num_Niche; k++){
			for (rsc = 0; rsc < num_RscSpc; rsc++){
				end0 = num_PhenoTyp_Rsc[rsc];//the number of resource classes

				for (cls = 0; cls < end0; cls++)
				{
					tmp_Bodymass.push_back(mean_BioMass_Rsc[rsc][cls]);
				}//end resource class loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_File_Resource_Bodymass(char *p_FileName_Setting)
{
	int i, rsc, cls, numRow, numCol, totCls, end0;
	string str_ResourceSpecies = "R";
	string str_ResourceClass = "C";

	totCls = 0;//the total number of resource classes

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		totCls += num_PhenoTyp_Rsc[rsc];//adds the number of resource classes
	}//end resource loop

	////////////////////////////
	//Add data to resource class output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = totCls;

	string ** str_Tmp;//buffer for trait output
	str_Tmp = new string*[numRow];

	for (i = 0; i < numRow; i++){
		str_Tmp[i] = new string[numCol];
	}//end row loop

	//write the category name
	int tmp_Num_Row = 1;
	int tmp_Num_Col = 1;

	str_Tmp[0][0] = "Resource Representative Bodymass";
	csvRW.CSV_Write_Add_String_2D(p_FileName_Setting, str_Tmp, &tmp_Num_Row, &tmp_Num_Col);

	//resource bodymass data buffer initialization
	int counter_Column = 0;
	string tmp_Str;

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		end0 = num_PhenoTyp_Rsc[rsc];//the number of resource classes

		for (cls = 0; cls < end0; cls++)
		{
			str_Tmp[0][counter_Column] = str_ResourceSpecies + std::to_string(rsc) + str_ResourceClass + std::to_string(cls);
			counter_Column++;//increases the row counter
		}//end resource class loop
	}//end species loop

	//writes the column header for resource bodymass-output file
	csvRW.CSV_Write_Add_String_2D(p_FileName_Setting, str_Tmp, &numRow, &numCol);

	counter_Column = 0;

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		end0 = num_PhenoTyp_Rsc[rsc];//the number of resource classes

		for (cls = 0; cls < end0; cls++)
		{
			str_Tmp[0][counter_Column] = std::to_string(mean_BioMass_Rsc[rsc][cls]);
			counter_Column++;//increases the row counter
		}//end resource class loop
	}//end species loop

	//writes the column header for resource bodymass-output file
	csvRW.CSV_Write_Add_String_2D(p_FileName_Setting, str_Tmp, &numRow, &numCol);

	//de-allocate memory
	for (i = 0; i < numRow; i++){
		delete[] str_Tmp[i];
	}//end row loop

	delete[] str_Tmp;
}

void Simul_Proc::Set_CelNum_Spc_Trt_Local_Discrete(int deme, int niche)
{
	//(Mode_Trait_Rsc == 0):The number of resource-use traits = the number of resource types
	int spc, trt0, trt1, end0, end1;

	for (spc = 0; spc < num_Spc; spc++)
	{
		//counts individuals sorted into phenotypes by using vitaList
		Set_VitaList_CelNum_Discrete(deme, niche, spc);

		//bodylength bin
		end0 = num_Bin_Bodylength[spc];
		//gillraker genotype
		end1 = num_Clas_Trt_Spc[spc][1];

		for (trt0 = 0; trt0 < end0; trt0++){
			for (trt1 = 0; trt1 < end1; trt1++){
				//sets the number of agents for a trait type for species
				celNum_Spc_Trt_Local_Discrete[spc][trt0][trt1] = celNum_VitaList_Discrete[trt0][trt1];

				//verification
				//if (celNum_Spc_Trt_Local_Discrete[spc][trt0][trt1] > 0)
				//{
				//std::cout << "spc " << spc << " trt0 " << trt0 << " trt1 " << trt1 << " abundance " << celNum_Spc_Trt_Local_Discrete[spc][trt0][trt1] << std::endl;
				//std::cout << "spc " << spc << " trt0 " << trt0 << " trt1 " << trt1 << " vitaList abundance " << celNum_VitaList_Discrete[trt0][trt1] << std::endl;
				//}//end if
			}//end lower trait loop
		}//end upper trait loop
	}//end species loop

	//verification
	//Check_In("Set_CelNum_Spc_Trt_Local_Discrete() done");
}

void Simul_Proc::Get_FunctionalResponse_Esp(double **p_Frsp_Rate, double **p_Esp, double **p_Density_Rsc, int p_Deme, int p_Niche, int p_Spc)
{
	//sets functional response (rate) (p_Frsp_Rate) of a core species to resource size classes

	//p_Frsp_Rate: functional response rate
	//dim[resource, size class]

	//p_Esp: efficiency of foraging a resource by a core species
	//dim[resource, size class]

	//p_Density_Rsc: resource density
	//dim[resource, size class]

	//p_Deme: deme
	//p_Niche: niche
	//p_Spc: core species

	int rsc, cls, end0;
	double tmp_Th_Spc_RscSpc, tmp_Q_Spc_RscSpc;

	for (rsc = 0; rsc < num_RscSpc; rsc++)
	{
		end0 = num_PhenoTyp_Rsc[rsc];
		tmp_Th_Spc_RscSpc = th_Spc_RscSpc[p_Deme][p_Niche][p_Spc][rsc];
		tmp_Q_Spc_RscSpc = q_Spc_RscSpc[p_Deme][p_Niche][p_Spc][rsc];

		for (cls = 0; cls < end0; cls++)
		{
			//sets the functional response of a species to a resource species
			FunctionalResponse_Rate(&p_Frsp_Rate[rsc][cls], &p_Esp[rsc][cls], &tmp_Th_Spc_RscSpc, &p_Density_Rsc[rsc][cls], &tmp_Q_Spc_RscSpc);
		}//end resource size class loop
	}//end resource cluster loop

	//verification
	//for (rsc = 0; rsc < num_RscSpc; rsc++)
	//{
	//for (cls = 0; cls < end0; cls++)
	//{
	//std::cout << "resource " << rsc << " class " << cls << " resource density " << p_Density_Rsc[rsc][cls] << std::endl;
	//}//end resource size class loop
	//}//end resource cluster loop

	//for (rsc = 0; rsc < num_RscSpc; rsc++)
	//{
	//for (cls = 0; cls < end0; cls++)
	//{
	//std::cout << "resource " << rsc << " class " << cls << " esp " << p_Esp[rsc][cls] << std::endl;
	//}//end resource size class loop
	//}//end resource cluster loop

	//for (rsc = 0; rsc < num_RscSpc; rsc++)
	//{		
	//for (cls = 0; cls < end0; cls++)
	//{
	//std::cout << "resource " << rsc << " class " << cls << " functional response " << p_Frsp_Rate[rsc][cls] << std::endl;
	//}//end resource size class loop
	//}//end resource cluster loop

	//Check_In("Line 5507: Get_FunctionalResponse_Esp");
}

void Simul_Proc::Get_ForagingTime_Normalization_Constant_Discrete(double *normalizationConstant, double **p_Frsp, int p_Deme, int p_Niche, int p_Spc)
{
	int rsc, cls, end0, tmp_Rsc;
	int numSpc_Tmp = num_Spc;
	double tmp_NormalizationConstant = 0.0;
	double tmp_Importance_Mean_Biomass_Rsc = 0.0;

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		end0 = num_PhenoTyp_Rsc[rsc];
		tmp_Rsc = numSpc_Tmp + rsc;

		for (cls = 0; cls < end0; cls++){

			if (mean_BioMass_Rsc[rsc][cls] > 0.0)
			{
				tmp_Importance_Mean_Biomass_Rsc = pow(mean_BioMass_Rsc[rsc][cls], exponent_Choice_Resource_BodyMass[p_Spc]);
			}
			else
			{
				tmp_Importance_Mean_Biomass_Rsc = 0.0;
			}//end else

			//Relative abundance of a class of a resource species (phen_D_Rsc[deme][niche][rsc][rCls] / tmp_Tot_Mass_Rsc)
			//calculates the sum of total foraging time
			tmp_NormalizationConstant += p_Frsp[rsc][cls] * k_NutritionValue[p_Spc][tmp_Rsc] * tmp_Importance_Mean_Biomass_Rsc * prop_Density_Rsc[p_Deme][p_Niche][rsc][cls];//initialization
		}//end resource species class loop
	}//end resource loop

	(*normalizationConstant) = tmp_NormalizationConstant;
}

void Simul_Proc::Set_Data_Buffer_Trait_Tstep(int p_Counter_Sampling)
{
	//(Mode_Trait_Rsc == 0):The number of resource-use traits = the number of resource types
	int deme, niche, demNi, spc, trt, i, end_0, end_1;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			demNi = (deme * num_Niche) + niche;

			for (spc = 0; spc < num_Spc; spc++){
				end_0 = num_Trait[spc];

				for (trt = 0; trt < end_0; trt++){
					end_1 = int_Num_Class_Trait[spc][trt];

					for (i = 0; i < end_1; i++){
						data_Buffer_Trait[p_Counter_Sampling][demNi][spc][trt][i] = tmp_Data_Buffer_Trait[demNi][spc][trt][i];
					}//end trait class loop
				}//end trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Data_Buffer_Trait_Comb(int *p_Counter_Sampling)
{
	//(Mode_Trait_Rsc == 0):The number of resource-use traits = the number of resource types
	if ((*p_Counter_Sampling) == (interval_sampling - 1)){
		int i, j, k, l, spc;

		for (i = 0; i < num_Deme; i++){
			for (j = 0; j < num_Niche; j++){
				for (spc = 0; spc < num_Spc; spc++){
					k = num_Trait_Combination[spc];

					for (l = 0; l < k; l++){
						data_Buffer_Trait_Combination[i][j][spc][l] = tmp_Data_Buffer_Trait_Foraging_Combination[i][j][spc][l];//initialization
					}//end lower-bound trait loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}//end if
}

void Simul_Proc::Set_Data_Buffer_Trait_Comb_Initial_Population(void)
{
	//(Mode_Trait_Rsc == 0):The number of resource-use traits = the number of resource types
	int i, j, k, l, spc;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				k = num_Trait_Combination[spc];

				for (l = 0; l < k; l++){
					data_Buffer_Trait_Combination[i][j][spc][l] = tmp_Data_Buffer_Trait_Foraging_Combination[i][j][spc][l];//initialization
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Data_Buffer_Genome_Tstep(int p_Counter_Sampling)
{
	//(Mode_Trait_Rsc == 0):The number of resource-use traits = the number of resource types
	int deme, niche, demNiSp, spc, loc, i, j, end_0;

	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				demNiSp = (deme * num_Niche_Num_Spc) + (niche * num_Spc) + spc;
				end_0 = num_Locus[spc];

				for (loc = 0; loc < end_0; loc++){
					for (i = 0; i < max_Num_Allele; i++){
						for (j = i; j < max_Num_Allele; j++){
							data_Buffer_Allele[p_Counter_Sampling][demNiSp][loc][i][j] = tmp_Data_Buffer_Allele[demNiSp][loc][i][j];
						}//end j loop
					}//end i loop
				}//end locus loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Gen_Trait(int **p_Loc_Trt, bool p_Switch_Assortative_Mating, double *p_Female_Choosiness_Scale, double *p_Female_Choosiness_Intensity, double *p_MetabolicRate_Biomass_Factor)
{
	int spc, stg, sex, loc, i, j;

	for (spc = 0; spc < num_Spc; spc++){
		for (i = 0; i < max_Num_Trait; i++){
			trt_Loc[spc][i] = 0.0;
		}//end locus loop
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		for (i = 0; i < max_Num_Trait; i++){
			trt_Loc_Int[spc][i] = 0;
		}//end locus loop
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		for (loc = 0; loc < num_Locus[spc]; loc++){
			loc_Trt[spc][loc] = p_Loc_Trt[spc][loc];
			trt_Loc[spc][p_Loc_Trt[spc][loc]] += 1.0;
			trt_Loc_Int[spc][p_Loc_Trt[spc][loc]] += 1;
		}//end locus loop
	}//end species loop

	//sets the number of possible classes for each trait
	//It was assumed that all loci are di-allelic
	//Each allele controling a trait contributes equally to the trait
	max_Num_Clas_Trt = 0;

	for (i = 0; i < max_Num_Trait; i++){
		max_Num_Clas_Trt_Spc[i] = 0;
	}//end trait loop

	for (spc = 0; spc < num_Spc; spc++){
		for (i = 0; i < num_Trait[spc]; i++){
			num_Clas_Trt_Spc[spc][i] = (trt_Loc_Int[spc][i] * num_Position[spc]) + 1;

			if (num_Clas_Trt_Spc[spc][i] > max_Num_Clas_Trt_Spc[i]){
				max_Num_Clas_Trt_Spc[i] = num_Clas_Trt_Spc[spc][i];
			}//end if

			if (num_Clas_Trt_Spc[spc][i] > max_Num_Clas_Trt)
			{
				max_Num_Clas_Trt = num_Clas_Trt_Spc[spc][i];
			}//end if
		}//end locus loop
	}//end species loop

	//the maximum number of genotypes for gillraker trait
	max_Num_Genotype_GillRaker = max_Num_Clas_Trt_Spc[1];

	esp_Discrete = new double*[num_RscSpc];

	for (int rsc = 0; rsc < num_RscSpc; rsc++)
	{
		esp_Discrete[rsc] = new double[max_Num_PhenTyp_Rsc];

		for (int cls = 0; cls < max_Num_PhenTyp_Rsc; cls++)
		{
			//initialization
			esp_Discrete[rsc][cls] = 0.0;
		}//end resource size class loop
	}//end resource cluster loop

	frsp = new double*[num_RscSpc];

	for (int rsc = 0; rsc < num_RscSpc; rsc++)
	{
		frsp[rsc] = new double[max_Num_PhenTyp_Rsc];

		for (int cls = 0; cls < max_Num_PhenTyp_Rsc; cls++)
		{
			//initialization
			frsp[rsc][cls] = 0.0;
		}//end resource size class loop
	}//end resource cluster loop

	ft_Discrete = new double*[num_RscSpc];

	for (int rsc = 0; rsc < num_RscSpc; rsc++)
	{
		ft_Discrete[rsc] = new double[max_Num_PhenTyp_Rsc];

		for (int cls = 0; cls < max_Num_PhenTyp_Rsc; cls++)
		{
			//initialization
			ft_Discrete[rsc][cls] = 0.0;
		}//end resource size class loop
	}//end resource cluster loop

	scalingFactor_BodyLength_BodyMass_Conversion = new double[num_Spc];//dim[species]: the maximum bound of the upper-bound trait
	exponent_BodyLength_BodyMass_Conversion = new double[num_Spc];//dim[species]: the maximum bound of the upper-bound trait

	upper_Bound_Max = new double[num_Spc];//dim[species]: the maximum bound of the upper-bound trait
	upper_Bound_Min = new double[num_Spc];//dim[species]: the minimum bound of the upper-bound trait
	lower_Bound_Max = new double[num_Spc];//dim[species]: the maximum bound of the lower-bound trait
	lower_Bound_Min = new double[num_Spc];//dim[species]: the minimum bound of the lower-bound trait

	exponent_Upper_Bound = new double[num_Spc];//dim[species]: the maximum bound of the upper-bound trait
	exponent_Lower_Bound = new double[num_Spc];//dim[species]: the minimum bound of the lower-bound trait

	max_Num_Bodymass_Cls = 0;
	int tmp_Num = 0;

	//The body mass (g) of a trait at a life stage
	bodyMass = new double***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		bodyMass[spc] = new double**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			bodyMass[spc][sex] = new double*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				bodyMass[spc][sex][stg] = new double[max_Num_Clas_Trt_Spc[0]];

				for (i = 0; i < max_Num_Clas_Trt_Spc[0]; i++){
					bodyMass[spc][sex][stg][i] = 0.0;
				}//end upper-bound trait loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	//The body length of a trait at a life stage
	bodyLength = new double***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		bodyLength[spc] = new double**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			bodyLength[spc][sex] = new double*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				bodyLength[spc][sex][stg] = new double[max_Num_Clas_Trt_Spc[0]];

				for (i = 0; i < max_Num_Clas_Trt_Spc[0]; i++){
					bodyLength[spc][sex][stg][i] = 0.0;
				}//end upper-bound trait loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	//The defaul degree of the development in body length of a trait at a life stage
	degree_Dev_Bodylength_Default = new double***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		degree_Dev_Bodylength_Default[spc] = new double**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			degree_Dev_Bodylength_Default[spc][sex] = new double*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				degree_Dev_Bodylength_Default[spc][sex][stg] = new double[max_Num_Clas_Trt_Spc[0]];

				for (i = 0; i < max_Num_Clas_Trt_Spc[0]; i++){
					degree_Dev_Bodylength_Default[spc][sex][stg][i] = 0.0;
				}//end upper-bound trait loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	//The gill raker trait at a life stage
	gillRaker = new double***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		gillRaker[spc] = new double**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			gillRaker[spc][sex] = new double*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				gillRaker[spc][sex][stg] = new double[max_Num_Clas_Trt_Spc[1]];

				for (i = 0; i < max_Num_Clas_Trt_Spc[1]; i++){
					gillRaker[spc][sex][stg][i] = 0.0;
				}//end upper-bound trait loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	//The body length of a trait at a life stage
	early_Sexual_Maturation = new double***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		early_Sexual_Maturation[spc] = new double**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			early_Sexual_Maturation[spc][sex] = new double*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++){
				early_Sexual_Maturation[spc][sex][stg] = new double[max_Num_Clas_Trt_Spc[0]];

				for (i = 0; i < max_Num_Clas_Trt_Spc[0]; i++){
					early_Sexual_Maturation[spc][sex][stg][i] = 0.0;
				}//end upper-bound trait loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	//The maximum body mass (g)
	bodyMass_Max = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		bodyMass_Max[spc] = 0.0;
	}//end species loop

	//the number of bins for body length for each species
	num_Bin_Bodylength = new int[num_Spc];

	//scaling factor for the effect of available resource mass 
	//on the development of bodylength
	ddg = new double[num_Spc];

	//the maximum number of bins for bodylength trait
	max_Num_Bin_Bodylength = 0;

	for (spc = 0; spc < num_Spc; spc++){
		num_Bin_Bodylength[spc] = 250;

		if (num_Bin_Bodylength[spc] > max_Num_Bin_Bodylength)
		{//sets the maximum number of bins for bodylength trait
			max_Num_Bin_Bodylength = num_Bin_Bodylength[spc];
		}//end if
	}//end species loop

	//the width of a bin for body length for each species
	bin_Width_BodyLength = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		bin_Width_BodyLength[spc] = 0.0;
	}//end species loop

	//sets the multiplier for creating dimension for traits
	for (spc = 0; spc < num_Spc; spc++){

		switch (mode_Gene_Trait[spc][1]){
		case 0://assumes
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL
			for (i = 0; i < num_Trait[spc]; i++){
				mul_Dim_Trt_Loc_QTL_Mtg[spc][i] = (trt_Loc[spc][i] * ((double)num_Position[spc]));
			}//end i loop
			break;
		case 1://1: Randome mating with a niche
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL
			for (i = 0; i < num_Trait[spc]; i++){
				mul_Dim_Trt_Loc_QTL_Mtg[spc][i] = 0.0;
			}//end i loop
			break;
		case 2://assumes
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL
			for (i = 0; i < num_Trait[spc]; i++){
				mul_Dim_Trt_Loc_QTL_Mtg[spc][i] = (trt_Loc[spc][i] * ((double)num_Position[spc]));
			}//end i loop
			break;
		default://assumes
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL
			for (i = 0; i < num_Trait[spc]; i++){
				mul_Dim_Trt_Loc_QTL_Mtg[spc][i] = (trt_Loc[spc][i] * ((double)num_Position[spc]));
			}//end i loop
			break;
		}//end switch

	}//end species loop

	//sets the number of loci used for determining the morphology of species
	for (spc = 0; spc < num_Spc; spc++){
		j = 0;//initializes the counter

		switch (mode_Gene_Trait[spc][3]){
		case 0://Trait-Resource use determined by Morphology
			//0: low-pass filter (mouth size (body-size)) + high-pass filter (gill)
			//These traits affect the efficiency of using resouce in the defined range of size.

			for (i = 0; i < num_Locus[spc]; i++)
			{
				if (loc_Trt[spc][i] < 2)
				{//if the locus controls morphological traits
					j++;//increases the counter
				}//end if
			}//end i loop

			num_Locus_Mor[spc] = j;//sets the number of loci used for determining the morphology of species
			break;
		case 2://Trait-Resource use determined by Morphology
			//0: low-pass filter (mouth size (body-size)) + high-pass filter (gill)
			//These traits affect the efficiency of using resouce in the defined range of size.

			for (i = 0; i < num_Locus[spc]; i++)
			{
				if (loc_Trt[spc][i] < 2)
				{//if the locus controls morphological traits
					j++;//increases the counter
				}//end if
			}//end i loop

			num_Locus_Mor[spc] = j;//sets the number of loci used for determining the morphology of species
			break;
		default://Trait-Resource use determined by Morphology
			//0: low-pass filter (mouth size (body-size)) + high-pass filter (gill)
			//These traits affect the efficiency of using resouce in the defined range of size.

			for (i = 0; i < num_Locus[spc]; i++)
			{
				if (loc_Trt[spc][i] < 2)
				{//if the locus controls morphological traits
					j++;//increases the counter
				}//end if
			}//end i loop

			num_Locus_Mor[spc] = j;//sets the number of loci used for determining the morphology of species
			break;
		}//end switch
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){

		switch (mode_Gene_Trait[spc][1]){
		case 0://assumes that only the trait #0 determines mating classes
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL

			//determine the greater number of life stages
			if (num_Stg[spc][0] < num_Stg[spc][1])
			{//more life stages for female
				sex = 1;
			}
			else
			{
				sex = 0;
			}//end else

			break;
		case 1://assumes Randome mating with a niche
			break;
		case 2://assumes that only the trait #0 determines mating classes
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL

			//determine the greater number of life stages
			if (num_Stg[spc][0] < num_Stg[spc][1])
			{//more life stages for female
				sex = 1;
			}
			else
			{
				sex = 0;
			}//end else
			break;
		default://assumes that only the trait #0 determines mating classes
			//1) bi-allelic loci,
			//2) two traits sets the higher and lower limit if the size of prey (resource)
			//3) QTL

			//determine the greater number of life stages
			if (num_Stg[spc][0] < num_Stg[spc][1])
			{//more life stages for female
				sex = 1;
			}
			else
			{
				sex = 0;
			}//end else

			break;
			break;
		}//end switch
	}//end species loop

	//set genetics-driven assortative mating mode
	mode_Assortative_Mating = p_Switch_Assortative_Mating;

	for (spc = 0; spc < num_Spc; spc++)
	{
		female_Choosiness_Intensity[spc] = p_Female_Choosiness_Intensity[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		female_Choosiness_Scale[spc] = p_Female_Choosiness_Scale[spc];
	}//end species loop

	size_MatingList_Discrete = new int*[max_Num_Bin_Bodylength];

	for (i = 0; i < max_Num_Bin_Bodylength; i++)
	{
		size_MatingList_Discrete[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			size_MatingList_Discrete[i][sex] = 0;
		}//end sex loop
	}//end mating class loop

	max_Num_MateSearchLargeStep_Discrete = 0;
	num_MateSearchLargeStep_Discrete = new int[num_Spc];

	for (i = 0; i < num_Spc; i++)
	{
		num_MateSearchLargeStep_Discrete[i] = (int)(((double)num_Bin_Bodylength[i]) / 5.0);

		if (num_MateSearchLargeStep_Discrete[i] < 1)
		{
			num_MateSearchLargeStep_Discrete[i] = 1;
		}//end if

		if (num_MateSearchLargeStep_Discrete[i] > max_Num_MateSearchLargeStep_Discrete)
		{
			max_Num_MateSearchLargeStep_Discrete = num_MateSearchLargeStep_Discrete[i];
		}//end if
	}//end species loop

	double unit_Large_search_Step;

	mateSearchLargeStep_Discrete = new double*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		unit_Large_search_Step = 1.0 / ((double)(num_MateSearchLargeStep_Discrete[spc] + 1));

		mateSearchLargeStep_Discrete[spc] = new double[max_Num_MateSearchLargeStep_Discrete];
		mateSearchLargeStep_Discrete[spc][0] = unit_Large_search_Step;

		if (num_MateSearchLargeStep_Discrete[spc] > 1)
		{
			int tmp_End = num_MateSearchLargeStep_Discrete[spc];

			for (i = 1; i < tmp_End; i++)
			{
				mateSearchLargeStep_Discrete[spc][i] = mateSearchLargeStep_Discrete[spc][(i - 1)] + unit_Large_search_Step;
			}//end large mate-search-step loop
		}//end if
	}//end species loop

	celNum_MatingList_Discrete = new int*[max_Num_Bin_Bodylength];

	for (i = 0; i < max_Num_Bin_Bodylength; i++){
		celNum_MatingList_Discrete[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			celNum_MatingList_Discrete[i][sex] = 0;
		}//end sex loop
	}//end mating class loop

	maxCelNum_MatingList_Discrete = new int*[max_Num_Bin_Bodylength];

	for (i = 0; i < max_Num_Bin_Bodylength; i++){
		maxCelNum_MatingList_Discrete[i] = new int[num_Sex];

		for (sex = 0; sex < num_Sex; sex++){
			maxCelNum_MatingList_Discrete[i][sex] = 0;
		}//end sex loop
	}//end mating class loop

	matingList_Discrete.resize(max_Num_Bin_Bodylength, vector<vector<Agent*> >(num_Sex, vector<Agent*>()));

	//sets data container for discrete vital list
	Set_Container_Vital_List_Discrete();
}

void Simul_Proc::Set_Container_Vital_List_Discrete(void)
{
	int i, j, k;
	/////////////////////////////
	//containers for vital list//
	/////////////////////////////

	//the maximum number of bins for bodylength
	//for vital list (Temporary vector for survival processes)
	//the number of bins for bodylength: num_Bin_Bodylength
	cls_UpperBound_Spc_Discrete = max_Num_Bin_Bodylength;
	//the maximum number of genotypes for gillraker trait
	//for vital list (Temporary vector for survival processes)
	cls_LowerBound_Spc_Discrete = max_Num_Genotype_GillRaker;

	size_VitaList_Discrete = new int*[cls_UpperBound_Spc_Discrete];

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		size_VitaList_Discrete[i] = new int[cls_LowerBound_Spc_Discrete];

		for (j = 0; j < cls_LowerBound_Spc_Discrete; j++){
			size_VitaList_Discrete[i][j] = 0;
		}//end sex loop
	}//end mating class loop

	celNum_VitaList_Discrete = new int*[cls_UpperBound_Spc_Discrete];

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		celNum_VitaList_Discrete[i] = new int[cls_LowerBound_Spc_Discrete];

		for (j = 0; j < cls_LowerBound_Spc_Discrete; j++){
			celNum_VitaList_Discrete[i][j] = 0;
		}//end sex loop
	}//end mating class loop

	maxCelNum_VitaList_Discrete = new int*[cls_UpperBound_Spc_Discrete];

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		maxCelNum_VitaList_Discrete[i] = new int[cls_LowerBound_Spc_Discrete];

		for (j = 0; j < cls_LowerBound_Spc_Discrete; j++){
			maxCelNum_VitaList_Discrete[i][j] = 0;
		}//end sex loop
	}//end mating class loop

	celNum_Spc_Trt_Local_Discrete = new int**[num_Spc];

	for (i = 0; i < num_Spc; i++)
	{
		celNum_Spc_Trt_Local_Discrete[i] = new int*[cls_UpperBound_Spc_Discrete];

		for (j = 0; j < cls_UpperBound_Spc_Discrete; j++)
		{
			celNum_Spc_Trt_Local_Discrete[i][j] = new int[cls_LowerBound_Spc_Discrete];

			for (k = 0; k < cls_LowerBound_Spc_Discrete; k++)
			{
				celNum_Spc_Trt_Local_Discrete[i][j][k] = 0;
			}//end lower foraging bound loop
		}//end upper foraging bound loop
	}//end species loop

	vitaList_Discrete.resize(cls_UpperBound_Spc_Discrete, std::vector<std::vector<Agent*> >(cls_LowerBound_Spc_Discrete, std::vector<Agent*>()));
}

void Simul_Proc::Get_Gamete_No_Linkage(vector<unsigned short int> &gamete, int p_Spc, Agent *p_ID)
{
	//gamete is a 1D unsigned short int array for a new gamete
	//It assumes that all loci are not linked
	//p_AgtLibIndex: index number for an agent in agt[][]

	//gets a genome from an agent
	unsigned short int** genome = p_ID->Get_Genome();
	int locus, chromatid;
	int end_Loc = num_Locus[p_Spc];

	for (locus = 0; locus < end_Loc; locus++){
		chromatid = (*rand_Uni_int)();//temporarily stores a uniform random deviate in gamete
		//cout<<"\n"<<"uni int rand "<<chromatid<<"\n";
		gamete[locus] = genome[locus][chromatid];
	}//end locus loop
}

void Simul_Proc::Get_Gamete_Linkage(vector<unsigned short int> &gamete, int *p_Spc, int *p_AgtLibIndex)
{
	//gamete is a 1D unsigned short int array for a new gamete
	//It assumes that all loci are not linked
	//p_AgtLibIndex: index number for an agent in agt[][]

	//gamete is a 1D unsigned short int array for a new gamete
	//Gene_Dst_Map is the 1D double array containing the genetic distances between two loci
	//Maximum genetic distance = 0.5, Minimum genetic distance = 0.0
	//the number of cells in Gene_Dst_Map is ((*p_num_Locus) -1)

	//gets a genome from an agent
	unsigned short int** genome = agt[(*p_Spc)][(*p_AgtLibIndex)]->Get_Genome();
	double ranDev = (*rand_Uni_Real)();//temporarily stores a uniform random deviate in gamete
	int chromatid = 0;
	int locus;

	if (ranDev < 0.5){
		chromatid = 1;//chromatid number
	}//end if

	gamete[0] = genome[0][chromatid];//the first locus

	if (num_Locus[(*p_Spc)] > 1){
		for (locus = 1; locus < num_Locus[(*p_Spc)]; locus++){
			ranDev = (*rand_Uni_Real)();//temporarily stores a uniform random deviate in gamete

			if (ranDev < gen_Dst_Map[(*p_Spc)][locus]){//switch the chromatid
				chromatid = abs((chromatid - 1));//switch the chromatid 0 -> 1, 1 -> 0
			}//end if

			gamete[locus] = genome[locus][chromatid];
		}//end locus loop
	}//end if
}

void Simul_Proc::Set_Trait(vector<int> &trait, vector<unsigned short int> &gamete_0, vector<unsigned short int> &gamete_1, int p_Spc)
{
	int i;

	//initializes traits
	int end_0 = num_Trait[p_Spc];

	for (i = 0; i < end_0; i++){
		trait[i] = 0;//initializes traits
	}//end i loop

	//assumes that all loci are QTL
	//1) bi-allelic loci,
	//2) two traits sets the higher and lower limit if the size of prey (resource)
	//3) QTL
	end_0 = num_Locus[p_Spc];
	int *tmp_Loc_Trt = loc_Trt[p_Spc];

	for (i = 0; i < end_0; i++){
		//calculates traits
		trait[tmp_Loc_Trt[i]] += (gamete_0[i] + gamete_1[i]);
	}//end locus loop
}

void Simul_Proc::Set_Trait(vector<int> &trait, int p_Spc, Agent *p_ID)
{
	int i, j;
	//initializes traits
	int end_0 = num_Trait[p_Spc];
	int end_1 = num_Position[p_Spc];

	//gets a genome
	unsigned short int** genome = p_ID->Get_Genome();

	for (i = 0; i < end_0; i++){
		trait[i] = 0;//initializes traits
	}//end i loop

	//assumes that all loci are QTL
	//1) bi-allelic loci,
	//2) two traits sets the higher and lower limit if the size of prey (resource)
	//3) QTL
	end_0 = num_Locus[p_Spc];
	int *tmp_Loc_Trt = loc_Trt[p_Spc];

	for (i = 0; i < end_0; i++){
		for (j = 0; j < end_1; j++){
			//calculates traits
			trait[tmp_Loc_Trt[i]] += genome[i][j];
		}//end position loop
	}//end locus loop
}

void Simul_Proc::Get_Deme_Niche_Mode(int *p_Deme_Niche_Mode)
{
	if (num_Deme > 1){
		if (num_Niche > 1){//multiple demes and niches
			(*p_Deme_Niche_Mode) = 0;
		}
		else{//multiple demes and a single niche
			(*p_Deme_Niche_Mode) = 1;
		}//end else
	}
	else{
		if (num_Niche > 1){//a single deme and multiple niches
			(*p_Deme_Niche_Mode) = 2;
		}
		else{//a single deme and a single niche
			(*p_Deme_Niche_Mode) = 3;
		}//end else
	}//end else
}

void Simul_Proc::Data_Buffer_Agent_In(int *p_Deme, int *p_Niche, int p_Spc, Agent *p_ID){
	//p_Spc: species
	int i, allele_0, allele_1, end_0;
	int demNiSp = ((*p_Deme) * num_Niche_Num_Spc) + ((*p_Niche) * num_Spc) + p_Spc;
	//p_Index: the index number of the agent
	unsigned short int **tmp_Genome = p_ID->Get_Genome();//gets the genome
	end_0 = num_Locus[p_Spc];

	//sets the data buffer for alleles
	for (i = 0; i < end_0; i++){
		allele_0 = tmp_Genome[i][0];
		allele_1 = tmp_Genome[i][1];

		//allele_0 should be less than allele_1
		if (allele_0 > allele_1)
		{
			tmp_Data_Buffer_Allele[demNiSp][i][allele_1][allele_0]++;
		}
		else
		{
			tmp_Data_Buffer_Allele[demNiSp][i][allele_0][allele_1]++;
		}//end else
	}//end i loop


	demNiSp = ((*p_Deme) * num_Niche) + (*p_Niche);
	end_0 = num_Trait[p_Spc];
	int traitType = 0;//initialization
	int *tmp_Trait = p_ID->Get_Trait();//gets the trait

	for (i = 0; i < end_0; i++){
		allele_0 = tmp_Trait[i];
		tmp_Data_Buffer_Trait[demNiSp][p_Spc][i][allele_0]++;

		if (i < m_Num_Traits_Foraging)
		{
			traitType += allele_0 * multiplier_Trait_Combination[p_Spc][i];//calculates the trait type
		}//end if		
	}//end trait loop

	//updates the foraging trait combination statistics
	tmp_Data_Buffer_Trait_Foraging_Combination[(*p_Deme)][(*p_Niche)][(p_Spc)][traitType]++;
}

void Simul_Proc::Data_Buffer_Agent_Out(int *p_Deme, int *p_Niche, int p_Spc, Agent *p_ID){
	//p_Spc: species
	//p_Index: the index number of the agent
	int i, allele_0, allele_1, end_0;
	int demNiSp = ((*p_Deme) * num_Niche_Num_Spc) + ((*p_Niche) * num_Spc) + p_Spc;
	unsigned short int **tmp_Genome = p_ID->Get_Genome();//gets the genome
	end_0 = num_Locus[p_Spc];

	//sets the data buffer for alleles
	for (i = 0; i < end_0; i++){
		allele_0 = tmp_Genome[i][0];
		allele_1 = tmp_Genome[i][1];

		//allele_0 should be less than allele_1
		if (allele_0 > allele_1){
			tmp_Data_Buffer_Allele[demNiSp][i][allele_1][allele_0]--;
		}
		else
		{
			tmp_Data_Buffer_Allele[demNiSp][i][allele_0][allele_1]--;
		}//end else
	}//end i loop

	end_0 = num_Trait[p_Spc];;
	demNiSp = ((*p_Deme) * num_Niche) + (*p_Niche);
	int traitType = 0;
	int *tmp_Trait = p_ID->Get_Trait();//gets the trait

	//sets the data buffer for traits
	for (i = 0; i < end_0; i++){
		allele_0 = tmp_Trait[i];
		tmp_Data_Buffer_Trait[demNiSp][p_Spc][i][allele_0]--;

		if (i < m_Num_Traits_Foraging)
		{
			traitType += allele_0 * multiplier_Trait_Combination[p_Spc][i];//calculates the trait type
		}//end if
	}//end i loop

	tmp_Data_Buffer_Trait_Foraging_Combination[(*p_Deme)][(*p_Niche)][(p_Spc)][traitType]--;//updates the trait type
}

void Simul_Proc::Data_Buffer_Agent_Out_In(int *p_Deme, int *p_Niche, int *p_Deme_New, int *p_Niche_New, int p_Spc, Agent *p_ID){
	//p_Spc: species
	//p_Index: the index number of the agent
	int i, allele_0, allele_1, end_0;
	int demNiSp = ((*p_Deme) * num_Niche_Num_Spc) + ((*p_Niche) * num_Spc) + p_Spc;
	int demNiSp_New = ((*p_Deme_New) * num_Niche_Num_Spc) + ((*p_Niche_New) * num_Spc) + p_Spc;
	unsigned short int **tmp_Genome = p_ID->Get_Genome();//gets the genome
	end_0 = num_Locus[p_Spc];

	//sets the data buffer for alleles
	for (i = 0; i < end_0; i++){
		allele_0 = tmp_Genome[i][0];
		allele_1 = tmp_Genome[i][1];

		//allele_0 should be less than allele_1
		if (allele_0 > allele_1){
			tmp_Data_Buffer_Allele[demNiSp][i][allele_1][allele_0]--;
			tmp_Data_Buffer_Allele[demNiSp_New][i][allele_1][allele_0]++;
		}
		else
		{
			tmp_Data_Buffer_Allele[demNiSp][i][allele_0][allele_1]--;
			tmp_Data_Buffer_Allele[demNiSp_New][i][allele_0][allele_1]++;
		}//end else
	}//end i loop

	end_0 = num_Trait[p_Spc];
	demNiSp = ((*p_Deme) * num_Niche) + (*p_Niche);
	demNiSp_New = ((*p_Deme_New) * num_Niche) + (*p_Niche_New);
	int traitType = 0;
	int *tmp_Trait = p_ID->Get_Trait();//gets the trait

	//sets the data buffer for traits
	for (i = 0; i < end_0; i++){
		allele_0 = tmp_Trait[i];
		tmp_Data_Buffer_Trait[demNiSp][p_Spc][i][allele_0]--;
		tmp_Data_Buffer_Trait[demNiSp_New][p_Spc][i][allele_0]++;

		if (i < m_Num_Traits_Foraging)
		{
			traitType += allele_0 * multiplier_Trait_Combination[p_Spc][i];//calculates the trait type
		}//end if
	}//end i loop

	//updates trait combination
	tmp_Data_Buffer_Trait_Foraging_Combination[(*p_Deme)][(*p_Niche)][(p_Spc)][traitType]--;//updates the trait type
	tmp_Data_Buffer_Trait_Foraging_Combination[(*p_Deme_New)][(*p_Niche_New)][(p_Spc)][traitType]++;//updates the trait type
}

void Simul_Proc::FunctionalResponse_Rate(double *huntingRate, double *a, double *th, double *denPrey, double *q)
{
	//returns the number of prey attacked
	//the density of hunted prey = (a * Ttot * (N^(q))) / (1.0 + (a * Ttot * (th) * (N^(q))))
	//denPreyHunted: the density of hunted prey
	//denPrey: the density of prey
	//a: attack rate
	//Ttot (=1.0): The total amount of time for hunting
	//th: the fraction of time used for handling prey during hunting
	//q: Type I: q = 1.0, Type II: q = 1.0, Type III: q = 2.0

	if ((*denPrey) > 0.0)
	{
		if ((*q) == 1.0)
		{//functional response II
			(*huntingRate) = (*a) / (1.0 + ((*a) * (*denPrey) * (*th)));
		}
		else
		{
			(*huntingRate) = ((*a) * (pow((*denPrey), ((*q) - 1.0)))) / (1.0 + (((*a) * pow((*denPrey), (*q))) * (*th)));
		}//end else
	}
	else
	{//no prey
		(*huntingRate) = 0.0;
	}//end else
}

void Simul_Proc::Get_Resource_Preference_Sergie_Rsc(vector<double> &trait, int p_Num_Physical_Trait)
{
	//returns resource preference for each resource
	//The number of resource types (num_RscSpc) > 1
	//The number of resource types = the number of traits (num) - the number of physical traits + 1
	//Each trait value ranges between 0 and 1
	//p_Num_Physical_Trait: the number of physical traits
	//the number of traits = the number of physical traits + the number of preference traits 

	//calculate the sum of trait values
	int i;
	//the number of traits = the number of physical traits + the number of preference traits
	int end_0 = p_Num_Physical_Trait + num_RscSpc - 1;
	double sum_Preference = 0.0;

	//reads traits determining preference
	for (i = p_Num_Physical_Trait; i < end_0; i++){
		//sets from the preference to the second resource
		sum_Preference += trait[i];
	}//end trait loop

	end_0 = num_RscSpc;

	if (sum_Preference > 1.0){
		//sets the preference for the last resource
		trait[p_Num_Physical_Trait] = 0.0;

		//sets the preference for other resource
		for (i = p_Num_Physical_Trait; i < end_0; i++){
			trait[i] /= sum_Preference;
		}//end trait loop
	}
	else{
		//sets the preference for the last resource
		trait[p_Num_Physical_Trait] = 1.0 - sum_Preference;
	}//end else
}

void Simul_Proc::ESP_Sigmoid_Func(double *p_Hunt_Efficiency, double *p_x, double *p_UpperBound, double *p_LowerBound, double *p_K, double *p_Delta)
{
	//Function for describing the decrease in the efficiency of hunting at around the upper and lower size boundaries
	//Error function (erf) was used to have a tent function-like behavior with sigmoidal decrease in the efficacy of hunting at around the boundaries.
	//p_Delta: This controls the degree of disadvantage imposed by the difference between the upper-bound trait and the lower-bound trait
	if ((*p_UpperBound) > (*p_LowerBound))
	{
		double log_Upperbound = log10((*p_UpperBound));
		double log_LowerBound = log10((*p_LowerBound));
		double log_X = log10((*p_x));
		double z = (log_Upperbound + log_LowerBound) / 2.0;//z = (upper bound + Lower bound) / 2
		double c;
		double penalty_LowerBound_Trait;//Penalty by the lower-bound trait

		if ((*p_x) > (*p_LowerBound))
		{
			penalty_LowerBound_Trait = pow((*p_LowerBound) / (*p_x), (*p_Delta));
		}
		else
		{
			penalty_LowerBound_Trait = 1.0;
		}//else

		//Normalization factor
		c = ((2.0 + (exp((-(*p_K)) * (z - log_LowerBound))) + (exp((-(*p_K)) * (z - log_Upperbound)))) / (exp((-(*p_K)) * (z - log_Upperbound))));//Scaling factor to make the maximum efficacy at the middele point (= ((upper bound + lower bound) / 2))  1.

		//cout << "Line 6649: c: " << c << endl;
		//calculates the attack rate by using compound logistic function
		(*p_Hunt_Efficiency) = penalty_LowerBound_Trait * c * (1.0 / (1.0 + exp(((-(*p_K))*(log_X - log_LowerBound))))) * (1.0 - (1.0 / (1.0 + exp(((-(*p_K))*(log_X - log_Upperbound))))));
	}
	else
	{
		(*p_Hunt_Efficiency) = 0.0;
	}//end else
}

void Simul_Proc::ESP_Norm_Func(double *p_Hunt_Efficiency, double *p_Mu, double *p_Delta, double *binWidth_Rsc, double *bin_Left, double *bin_Right)
{
	//Function for describing the decrease in the efficiency of hunting at around the upper and lower size boundaries
	//Error function (erf) was used to have a tent function-like behavior with sigmoidal decrease in the efficacy of hunting at around the boundaries.
	//double tmp_0 = ((*p_x) - (*p_Mu)) / (*p_Delta);//standarization
	(*p_Hunt_Efficiency) = ((boost::math::cdf((*std_Norm_Dist), (((*bin_Right) - (*p_Mu)) / (*p_Delta)))) - (boost::math::cdf((*std_Norm_Dist), (((*bin_Left) - (*p_Mu)) / (*p_Delta))))) / ((boost::math::cdf((*std_Norm_Dist), ((*binWidth_Rsc) / (*p_Delta)))) - (boost::math::cdf((*std_Norm_Dist), 0.0)));
}

void Simul_Proc::Set_Biomass_Resource(double **p_Trt_RscSpc_Mean_Sd)
{
	int rsc, cls0, end0;
	double mu, sd, min_Size_Rsc_Tmp, binWidth_Rsc, bin_Left_Rsc, bin_Right_Rsc;

	vector<boost::math::normal*> trt_RscSpc_Norm;//dim[resource species]; Normal distributions of resource species traits

	//sets the Normal distributions of resource species traits
	for (rsc = 0; rsc < num_RscSpc; rsc++){
		trt_RscSpc_Norm.push_back(new boost::math::normal(p_Trt_RscSpc_Mean_Sd[rsc][0], p_Trt_RscSpc_Mean_Sd[rsc][1]));
	}//end resource species loop

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		mu = trt_RscSpc_Norm[rsc]->mean();//mean resource species size
		sd = trt_RscSpc_Norm[rsc]->standard_deviation();//standard deviation resource species size
		//The maximum resource species size
		//max_Size_Rsc_Tmp = mu + (3.0 * sd);
		//The minimum resource species size
		min_Size_Rsc_Tmp = mu - (3.0 * sd);

		//The width of a bin for resource species size
		binWidth_Rsc = (6.0 * sd) / ((double)num_PhenoTyp_Rsc[rsc]);

		end0 = num_PhenoTyp_Rsc[rsc];

		for (cls0 = 0; cls0 <end0; cls0++){
			//The left end of a resource species bin in the histogram
			bin_Left_Rsc = min_Size_Rsc_Tmp + ((double)cls0) * binWidth_Rsc;
			//The right end of a resource species bin in the histogram
			bin_Right_Rsc = bin_Left_Rsc + binWidth_Rsc;

			//sets the mean sizes (individual biomass) of phenotypic classes of resource species
			mean_BioMass_Rsc[rsc][cls0] = (bin_Left_Rsc + bin_Right_Rsc) / 2.0;
		}//end resource species phenotype loop
	}//end resource species loop

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		delete trt_RscSpc_Norm[rsc];
	}//end resource species loop
}

void Simul_Proc::Set_Mean_SD_BioMass_Resource(double** p_Trt_RscSpc_Mean_Sd, double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc)
{
	if (num_RscSpc > 1)
	{//Multiple resource species
		int rsc;
		//Multiplier for the mean biomass of a species
		double multiplier_Biomass = pow((p_Max_Mean_BioMass_Rsc / p_Min_Mean_BioMass_Rsc), (1.0 / ((double)(num_RscSpc - 1))));
		double tmp_Multiplier = 1.0;

		for (rsc = 0; rsc < num_RscSpc; rsc++){
			//sets the mean of biomass of a resource species
			p_Trt_RscSpc_Mean_Sd[rsc][0] = p_Min_Mean_BioMass_Rsc * tmp_Multiplier;
			//sets the standard deviation of biomass of a resource species
			p_Trt_RscSpc_Mean_Sd[rsc][1] = p_CV_BioMass_Rsc * p_Trt_RscSpc_Mean_Sd[rsc][0];
			//updates the multiplier
			tmp_Multiplier *= multiplier_Biomass;

			//verification
			//std::cout << "Line 7115 Resource[" << rsc << "] mean mass: " << p_Trt_RscSpc_Mean_Sd[rsc][0] << std::endl;
			//char tt;
			//std::cin >> tt;
		}//end resource species loop
	}
	else
	{//One resource species
		p_Trt_RscSpc_Mean_Sd[0][0] = p_Min_Mean_BioMass_Rsc;
		p_Trt_RscSpc_Mean_Sd[0][1] = p_CV_BioMass_Rsc * p_Min_Mean_BioMass_Rsc;
	}//end else
}

void Simul_Proc::Set_Body_Length_Mass_SexMat(double *max_Max_Body_Length, double *min_Min_Body_Length, double *bodyLength_Max, double *bodyLength_Min, double *p_Exp_Length_To_Growth, double *p_Cst_Length_To_Growth)
{
	//sets the efficacy of the use of prey affected by prey size (ESP)
	int spc, sex, stg, tmpStg, trt0, end4, end0;
	double tmp_Max_trait;
	double *tmpGrowth_Length_State = new double[max_Num_Stg];

	for (spc = 0; spc < num_Spc; spc++)
	{
		max_Max_Body_Length[spc] = 0.0;
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		min_Min_Body_Length[spc] = bodyLength_Max[spc];
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++)
	{
		bodyLength_Max_Spc[spc] = bodyLength_Max[spc];

		//std::cout << "max body length " << bodyLength_Max_Spc[spc] << std::endl;
		//char tt;
		//std::cin >> tt;
	}//end species loop

	//sets bodylength-bodymass conversion factors
	//(constant)
	for (spc = 0; spc < num_Spc; spc++)
	{
		scalingFactor_BodyLength_BodyMass_Conversion[spc] = pow(10.0, p_Cst_Length_To_Growth[spc]);
	}//end species loop

	//sets bodylength-bodymass conversion factors 
	//(exponent)
	for (spc = 0; spc < num_Spc; spc++)
	{
		exponent_BodyLength_BodyMass_Conversion[spc] = p_Exp_Length_To_Growth[spc];
	}//end species loop

	double constant_Length_Mass_Conversion = 0.0;

	//sets the body length and body mass
	for (spc = 0; spc < num_Spc; spc++){
		//constant for converting body length to bodymass
		constant_Length_Mass_Conversion = pow(10.0, p_Cst_Length_To_Growth[spc]);

		//sets the maximum number of life stages
		int stg_Max = num_Stg[spc][0];

		for (sex = 0; sex < num_Sex; sex++)
		{
			if (num_Stg[spc][sex] > stg_Max)
			{
				stg_Max = num_Stg[spc][sex];
			}//end if
		}//end if

		for (sex = 0; sex < num_Sex; sex++){
			end4 = num_Stg[spc][sex];

			//maturation in body length
			for (stg = 0; stg < end4; stg++){
				tmpStg = stg + 1;

				Get_Von_Bertalaffy_Growth_Length(&tmpGrowth_Length_State[stg], &tmpStg, &stg_Max, &growth_Target_Spc[spc][sex]);//returns the degree of growth by using von Bertalaffy function

				//verification
				//cout << "Line 10617" << endl;
				//cout << "Life stage: " << stg << " Von Bertallanffy Growth: " << tmpGrowth_Length_State[stg] << endl;
				//char tt;
				//cin >> tt;
			}//end life stage loop

			if (mode_Forage_Physic_Spc[spc] == 0)
			{
				end0 = num_Clas_Trt_Spc[spc][0];

				for (trt0 = 0; trt0 < end0; trt0++){
					//The maximum bound for a species
					if (end0 > 1)
					{
						tmp_Max_trait = ((double)trt0) / ((double)(num_Clas_Trt_Spc[spc][0] - 1));
					}
					else
					{
						tmp_Max_trait = 0.0;
					}//end else

					//sets the body masses for each trait
					for (stg = 0; stg < end4; stg++){
						switch (mode_Gene_Trait[spc][4])
						{
						case 0:
							bodyLength[spc][sex][stg][trt0] = (bodyLength_Min[spc] * (pow((bodyLength_Max[spc] / bodyLength_Min[spc]), tmp_Max_trait))) * tmpGrowth_Length_State[stg];//reflect the growth state
							break;
						case 1:
							bodyLength[spc][sex][stg][trt0] = tmpGrowth_Length_State[stg] * (bodyLength_Min[spc] + ((bodyLength_Max[spc] - bodyLength_Min[spc]) * tmp_Max_trait));//reflect the growth state							
							break;
						default:
							bodyLength[spc][sex][stg][trt0] = (bodyLength_Min[spc] * (pow((bodyLength_Max[spc] / bodyLength_Min[spc]), tmp_Max_trait))) * tmpGrowth_Length_State[stg];//reflect the growth state
							break;
						}//end switch

						//sets the defaul degree of the development in bodylength 
						degree_Dev_Bodylength_Default[spc][sex][stg][trt0] = tmpGrowth_Length_State[stg];

						//Body length to bodymass conversion
						bodyMass[spc][sex][stg][trt0] = constant_Length_Mass_Conversion * (pow(bodyLength[spc][sex][stg][trt0], p_Exp_Length_To_Growth[spc]));//reflect the growth state

						//updates the minimum observed body length
						if (bodyLength[spc][sex][stg][trt0] < min_Min_Body_Length[spc])
						{
							min_Min_Body_Length[spc] = bodyLength[spc][sex][stg][trt0];
						}//end if

						//updates the maximum observed body length
						if (bodyLength[spc][sex][stg][trt0] > max_Max_Body_Length[spc])
						{
							max_Max_Body_Length[spc] = bodyLength[spc][sex][stg][trt0];
						}//end if

						//sets immature sexual maturation pattern
						switch (mode_Gene_Trait[spc][4])
						{
						case 0:
							Get_Chance_Sexual_Maturation(&early_Sexual_Maturation[spc][sex][stg][trt0], &spc, &stg, &sex, &tmp_Max_trait, &num_Immat_Stg_SexMaturation[spc][sex]);
							break;
						case 1:
							Get_Chance_Sexual_Maturation(&early_Sexual_Maturation[spc][sex][stg][trt0], &spc, &stg, &sex, &tmp_Max_trait, &num_Immat_Stg_SexMaturation[spc][sex]);
							break;
						default:
							Get_Chance_Sexual_Maturation(&early_Sexual_Maturation[spc][sex][stg][trt0], &spc, &stg, &sex, &tmp_Max_trait, &num_Immat_Stg_SexMaturation[spc][sex]);
							break;
						}//end switch

						//cout << "Line 10474" << endl;
						//cout << endl << "spc[" << spc << "]sex[" << sex << "]stg[" << stg << "]trt0[" << trt0 << "] Body mass: " << bodyMass[spc][sex][stg][trt0] << endl;
						//cout << endl << "spc[" << spc << "]sex[" << sex << "]stg[" << stg << "]trt0[" << trt0 << "] Early sex maturation: " << early_Sexual_Maturation[spc][sex][stg][trt0] << endl;
						//char tt;
						//cin >> tt;
						//cout << " Growth state: " << tmpGrowth_State[stg] << endl;
					}//end life stage loop
				}//end upper limit trait loop
			}
			else if (mode_Forage_Physic_Spc[spc] == 1)
			{

			}
			else
			{
			}//end else
		}//end sex loop
	}//end species loop

	//sets the bin width for bodylength
	Set_Bin_Width_BodyLength(bodyLength_Max);

	for (spc = 0; spc < num_Spc; spc++)
	{
		//default setting
		Get_BodyMass_From_BodyLength(&bodyMass_Max[spc], &bodyLength_Max_Spc[spc], &spc);
		//std::cout << "line 6752: maximum body mass" << bodyMass_Max[spc] << " spc " << spc << std::endl;
	}//end species loop

	delete[] tmpGrowth_Length_State;
}

void Simul_Proc::Set_Foraging_Trait_Bound(double *p_Upper_Bound_Min, double *p_Lower_Bound_Max, double *p_Lower_Bound_Min)
{
	int spc;

	for (spc = 0; spc < num_Spc; spc++)
	{
		//upper-bound max
		upper_Bound_Max[spc] = bodyMass_Max[spc] * 0.1;
		//lower-bound max
		lower_Bound_Max[spc] = p_Lower_Bound_Max[spc];
		//upper-bound min
		upper_Bound_Min[spc] = p_Upper_Bound_Min[spc];
		//lower-bound min
		lower_Bound_Min[spc] = p_Lower_Bound_Min[spc];
	}//end species loop

	//For verification
	//for (spc = 0; spc < num_Spc; spc++)
	//{
	//cout << " Line 10839" << endl;
	//cout << "Up Max " << upper_Bound_Max[spc] << " Up Min " << upper_Bound_Min[spc] << " Low Max " << lower_Bound_Max[spc] << " Low Min" << lower_Bound_Min[spc] << endl;
	//char tt;
	//cin >> tt;
	//}//end species loop
}

void Simul_Proc::Set_ESP_BodyMass(double p_Max_Mean_BioMass_Rsc, double p_Min_Mean_BioMass_Rsc, double p_CV_BioMass_Rsc, double *bodyLength_Max, double *bodyLength_Min, double *p_Exp_Length_To_Growth, double *p_Cst_Length_To_Growth, double *p_Upper_Bound_Min, double *p_Lower_Bound_Max, double *p_Lower_Bound_Min, double *p_Exponent_Upper_Bound, double *p_Exponent_Lower_Bound)
{
	//sets the efficacy of the use of prey affected by prey size (ESP)
	int spc, rsc, cls0;
	double min_Size_Rsc_Tmp, mu, sd, binWidth_Rsc;

	//sets the exponent for the upper-foraging bound trait
	for (spc = 0; spc < num_Spc; spc++)
	{
		exponent_Upper_Bound[spc] = p_Exponent_Upper_Bound[spc];
	}//end species loop

	//sets the exponent for the lower-foraging bound trait
	for (spc = 0; spc < num_Spc; spc++)
	{
		exponent_Lower_Bound[spc] = p_Exponent_Lower_Bound[spc];
	}//end species loop

	double *tmpGrowth_Length_State = new double[max_Num_Stg];

	//the mean and standard deviations of the sizes(individual biomass) of phenotypic classes of resource species
	double** p_Trt_RscSpc_Mean_Sd = new double*[num_RscSpc];

	for (spc = 0; spc < num_RscSpc; spc++){
		p_Trt_RscSpc_Mean_Sd[spc] = new double[2];
	}//end resource species loop

	double *max_Max_Body_Length = new double[num_Spc];//the maximum observed body length
	double *min_Min_Body_Length = new double[num_Spc];//the minimum observed body length

	//sets the means and standard deviations of resource species biomass
	Set_Mean_SD_BioMass_Resource(p_Trt_RscSpc_Mean_Sd, p_Max_Mean_BioMass_Rsc, p_Min_Mean_BioMass_Rsc, p_CV_BioMass_Rsc);

	vector<boost::math::normal*> trt_RscSpc_Norm;//dim[resource species]; Normal distributions of resource species traits

	//sets the Normal distributions of resource species traits
	for (rsc = 0; rsc < num_RscSpc; rsc++){
		trt_RscSpc_Norm.push_back(new boost::math::normal(p_Trt_RscSpc_Mean_Sd[rsc][0], p_Trt_RscSpc_Mean_Sd[rsc][1]));
	}//end resource species loop

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		mu = trt_RscSpc_Norm[rsc]->mean();//mean resource species size
		sd = trt_RscSpc_Norm[rsc]->standard_deviation();//standard deviation resource species size
		//The maximum resource species size
		//max_Size_Rsc_Tmp = mu + (3.0 * sd);
		//The minimum resource species size
		min_Size_Rsc_Tmp = mu - (3.0 * sd);

		//The width of a bin for resource species size
		binWidth_Rsc = (6.0 * sd) / ((double)num_PhenoTyp_Rsc[rsc]);

		for (cls0 = 0; cls0 < num_PhenoTyp_Rsc[rsc]; cls0++){
			//The left end of a resource species bin in the histogram
			bin_Rsc_BioMass[rsc][cls0][0] = min_Size_Rsc_Tmp + ((double)cls0) * binWidth_Rsc;
			//The right end of a resource species bin in the histogram
			bin_Rsc_BioMass[rsc][cls0][1] = bin_Rsc_BioMass[rsc][cls0][0] + binWidth_Rsc;

			//std::cout << "rsc " << rsc << " cls " << cls0 << " left " << bin_Rsc_BioMass[rsc][cls0][0] << " right " << bin_Rsc_BioMass[rsc][cls0][1] << std::endl;
		}//end resource species phenotype loop
	}//end resource species loop

	//sets the body length and body mass
	Set_Body_Length_Mass_SexMat(max_Max_Body_Length, min_Min_Body_Length, bodyLength_Max, bodyLength_Min, p_Exp_Length_To_Growth, p_Cst_Length_To_Growth);

	//sets foraging trait bounds
	Set_Foraging_Trait_Bound(p_Upper_Bound_Min, p_Lower_Bound_Max, p_Lower_Bound_Min);

	int deme, niche, rCls;

	sum_Spc_Rsc = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++){
		sum_Spc_Rsc[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++){
			sum_Spc_Rsc[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++){
				sum_Spc_Rsc[deme][niche][spc] = new double*[num_RscSpc];

				for (rsc = 0; rsc < num_RscSpc; rsc++){
					sum_Spc_Rsc[deme][niche][spc][rsc] = new double[max_Num_PhenTyp_Rsc];

					for (rCls = 0; rCls < max_Num_PhenTyp_Rsc; rCls++){
						sum_Spc_Rsc[deme][niche][spc][rsc][rCls] = 0.0;
					}//end resource phenotype loop
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	delete[] max_Max_Body_Length;
	delete[] min_Min_Body_Length;

	delete[] tmpGrowth_Length_State;

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		delete trt_RscSpc_Norm[rsc];
	}//end resource species loop

	for (spc = 0; spc < num_RscSpc; spc++){
		delete[] p_Trt_RscSpc_Mean_Sd[spc];
	}//end resource species loop

	delete[] p_Trt_RscSpc_Mean_Sd;

	Set_Data_Buffer_Mtg_Bodymass();//sets data buffer for mating with respect to bodymass
	Set_Data_Buffer_Mtg_GillRaker();//sets data buffer for mating with respect to gill-raker

	//tests foraging efficiency for species for resource
	//Test_ESP_Discrete();
}

void Simul_Proc::Set_Data_Buffer_Mtg_Bodymass(void)
{
	int spc, stg, sex, trt0, trtM, trtF;

	num_Bodymass_Cls = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		num_Bodymass_Cls[spc] = new int[num_Sex];
	}//end species loop

	//max_Num_Stg 
	index_Trait_Bodymass = new int ***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		index_Trait_Bodymass[spc] = new int**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++)
		{
			index_Trait_Bodymass[spc][sex] = new int*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++)
			{
				index_Trait_Bodymass[spc][sex][stg] = new int[max_Num_Clas_Trt_Spc[0]];

				for (trt0 = 1; trt0 < max_Num_Clas_Trt_Spc[0]; trt0++)
				{
					index_Trait_Bodymass[spc][sex][stg][trt0] = (-1);
				}//end trait 0 loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	max_Num_Bodymass_Cls = 0;

	Data_Manipulation dM;//class for sorting data

	//sorts data
	for (spc = 0; spc < num_Spc; spc++)
	{
		for (sex = 0; sex < num_Sex; sex++)
		{
			dM.Sort_Double_Two_D(bodyMass[spc][sex], &num_Stg[spc][sex], &num_Clas_Trt_Spc[spc][0], index_Trait_Bodymass[spc][sex], &num_Bodymass_Cls[spc][sex]);
		}//end sex loop
	}//end species loop

	//sets the maximum number of bodymass classes
	for (spc = 0; spc < num_Spc; spc++)
	{
		for (sex = 0; sex < num_Sex; sex++)
		{
			if (num_Bodymass_Cls[spc][sex] > max_Num_Bodymass_Cls)
			{
				max_Num_Bodymass_Cls = num_Bodymass_Cls[spc][sex];
			}//end if
		}//end sex loop
	}//end species loop

	data_Buffer_Mtg_Bodymass = new double*[max_Num_Bodymass_Cls];

	for (trtM = 0; trtM < max_Num_Bodymass_Cls; trtM++)
	{
		data_Buffer_Mtg_Bodymass[trtM] = new double[max_Num_Bodymass_Cls];

		for (trtF = 0; trtF < max_Num_Bodymass_Cls; trtF++)
		{
			data_Buffer_Mtg_Bodymass[trtM][trtF] = 0.0;
		}//end female bodymass class loop
	}//end male bodymass class loop

	exp_Data_Buffer_Mtg_Bodymass = new double*[max_Num_Bodymass_Cls];

	for (trtM = 0; trtM < max_Num_Bodymass_Cls; trtM++)
	{
		exp_Data_Buffer_Mtg_Bodymass[trtM] = new double[max_Num_Bodymass_Cls];

		for (trtF = 0; trtF < max_Num_Bodymass_Cls; trtF++)
		{
			exp_Data_Buffer_Mtg_Bodymass[trtM][trtF] = 0.0;
		}//end female bodymass class loop
	}//end male bodymass class loop

	int deme, niche;

	tmp_Data_Buffer_Mtg_Bodymass = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++)
	{
		tmp_Data_Buffer_Mtg_Bodymass[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++)
		{
			tmp_Data_Buffer_Mtg_Bodymass[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++)
			{
				tmp_Data_Buffer_Mtg_Bodymass[deme][niche][spc] = new double*[max_Num_Bodymass_Cls];

				for (trtM = 0; trtM < max_Num_Bodymass_Cls; trtM++)
				{
					tmp_Data_Buffer_Mtg_Bodymass[deme][niche][spc][trtM] = new double[max_Num_Bodymass_Cls];

					for (trtF = 0; trtF < max_Num_Bodymass_Cls; trtF++)
					{
						tmp_Data_Buffer_Mtg_Bodymass[deme][niche][spc][trtM][trtF] = 0.0;
					}//end male bodymass class loop
				}//end male bodymass class loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Deallocate_Memory_Data_Buffer_Mtg_Bodymass(void)
{
	int spc, stg, sex, trtM;

	for (spc = 0; spc < num_Spc; spc++)
	{
		delete[] num_Bodymass_Cls[spc];
	}//end species loop

	delete[] num_Bodymass_Cls;

	for (spc = 0; spc < num_Spc; spc++)
	{
		for (sex = 0; sex < num_Sex; sex++)
		{
			for (stg = 0; stg < max_Num_Stg; stg++)
			{
				delete[] index_Trait_Bodymass[spc][sex][stg];
			}//end life stage loop

			delete[] index_Trait_Bodymass[spc][sex];
		}//end sex loop

		delete[] index_Trait_Bodymass[spc];
	}//end species loop

	delete[] index_Trait_Bodymass;

	for (trtM = 0; trtM < max_Num_Bodymass_Cls; trtM++)
	{
		delete[] data_Buffer_Mtg_Bodymass[trtM];
	}//end male bodymass class loop

	delete[] data_Buffer_Mtg_Bodymass;

	for (trtM = 0; trtM < max_Num_Bodymass_Cls; trtM++)
	{
		delete[] exp_Data_Buffer_Mtg_Bodymass[trtM];
	}//end male bodymass class loop

	delete[] exp_Data_Buffer_Mtg_Bodymass;

	int deme, niche;

	for (deme = 0; deme < num_Deme; deme++)
	{
		for (niche = 0; niche < num_Niche; niche++)
		{
			for (spc = 0; spc < num_Spc; spc++)
			{
				for (trtM = 0; trtM < max_Num_Bodymass_Cls; trtM++)
				{
					delete[] tmp_Data_Buffer_Mtg_Bodymass[deme][niche][spc][trtM];
				}//end male bodymass class loop

				delete[] tmp_Data_Buffer_Mtg_Bodymass[deme][niche][spc];
			}//end species loop

			delete[] tmp_Data_Buffer_Mtg_Bodymass[deme][niche];
		}//end niche loop

		delete[] tmp_Data_Buffer_Mtg_Bodymass[deme];
	}//end deme loop

	delete[] tmp_Data_Buffer_Mtg_Bodymass;
}

void Simul_Proc::Set_Data_Buffer_Mtg_GillRaker(void)
{
	int spc, stg, sex, trt0, trtM, trtF;

	num_GillRaker_Cls = new int*[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		num_GillRaker_Cls[spc] = new int[num_Sex];
	}//end species loop

	//max_Num_Stg 
	index_Trait_GillRaker = new int ***[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		index_Trait_GillRaker[spc] = new int**[num_Sex];

		for (sex = 0; sex < num_Sex; sex++)
		{
			index_Trait_GillRaker[spc][sex] = new int*[max_Num_Stg];

			for (stg = 0; stg < max_Num_Stg; stg++)
			{
				index_Trait_GillRaker[spc][sex][stg] = new int[max_Num_Clas_Trt_Spc[1]];

				for (trt0 = 1; trt0 < max_Num_Clas_Trt_Spc[1]; trt0++)
				{
					index_Trait_GillRaker[spc][sex][stg][trt0] = (-1);
				}//end trait 0 loop
			}//end life stage loop
		}//end sex loop
	}//end species loop

	max_Num_GillRaker_Cls = 0;

	Data_Manipulation dM;//class for sorting data

	//sorts data
	for (spc = 0; spc < num_Spc; spc++)
	{
		for (sex = 0; sex < num_Sex; sex++)
		{
			dM.Sort_Double_Two_D(gillRaker[spc][sex], &num_Stg[spc][sex], &num_Clas_Trt_Spc[spc][1], index_Trait_GillRaker[spc][sex], &num_GillRaker_Cls[spc][sex]);
		}//end sex loop
	}//end species loop

	//sets the maximum number of bodymass classes
	for (spc = 0; spc < num_Spc; spc++)
	{
		for (sex = 0; sex < num_Sex; sex++)
		{
			if (num_GillRaker_Cls[spc][sex] > max_Num_GillRaker_Cls)
			{
				max_Num_GillRaker_Cls = num_GillRaker_Cls[spc][sex];
			}//end if
		}//end sex loop
	}//end species loop

	data_Buffer_Mtg_GillRaker = new double*[max_Num_GillRaker_Cls];

	for (trtM = 0; trtM < max_Num_GillRaker_Cls; trtM++)
	{
		data_Buffer_Mtg_GillRaker[trtM] = new double[max_Num_GillRaker_Cls];

		for (trtF = 0; trtF < max_Num_GillRaker_Cls; trtF++)
		{
			data_Buffer_Mtg_GillRaker[trtM][trtF] = 0.0;
		}//end female bodymass class loop
	}//end male bodymass class loop

	exp_Data_Buffer_Mtg_GillRaker = new double*[max_Num_GillRaker_Cls];

	for (trtM = 0; trtM < max_Num_GillRaker_Cls; trtM++)
	{
		exp_Data_Buffer_Mtg_GillRaker[trtM] = new double[max_Num_GillRaker_Cls];

		for (trtF = 0; trtF < max_Num_GillRaker_Cls; trtF++)
		{
			exp_Data_Buffer_Mtg_GillRaker[trtM][trtF] = 0.0;
		}//end female bodymass class loop
	}//end male bodymass class loop

	int deme, niche;

	tmp_Data_Buffer_Mtg_GillRaker = new double****[num_Deme];

	for (deme = 0; deme < num_Deme; deme++)
	{
		tmp_Data_Buffer_Mtg_GillRaker[deme] = new double***[num_Niche];

		for (niche = 0; niche < num_Niche; niche++)
		{
			tmp_Data_Buffer_Mtg_GillRaker[deme][niche] = new double**[num_Spc];

			for (spc = 0; spc < num_Spc; spc++)
			{
				tmp_Data_Buffer_Mtg_GillRaker[deme][niche][spc] = new double*[max_Num_GillRaker_Cls];

				for (trtM = 0; trtM < max_Num_GillRaker_Cls; trtM++)
				{
					tmp_Data_Buffer_Mtg_GillRaker[deme][niche][spc][trtM] = new double[max_Num_GillRaker_Cls];

					for (trtF = 0; trtF < max_Num_GillRaker_Cls; trtF++)
					{
						tmp_Data_Buffer_Mtg_GillRaker[deme][niche][spc][trtM][trtF] = 0.0;
					}//end male bodymass class loop
				}//end male bodymass class loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Deallocate_Memory_Data_Buffer_Mtg_GillRaker(void)
{
	int spc, stg, sex, trtM;

	for (spc = 0; spc < num_Spc; spc++)
	{
		delete[] num_GillRaker_Cls[spc];
	}//end species loop

	delete[] num_GillRaker_Cls;

	for (spc = 0; spc < num_Spc; spc++)
	{
		for (sex = 0; sex < num_Sex; sex++)
		{
			for (stg = 0; stg < max_Num_Stg; stg++)
			{
				delete[] index_Trait_GillRaker[spc][sex][stg];
			}//end life stage loop

			delete[] index_Trait_GillRaker[spc][sex];
		}//end sex loop

		delete[] index_Trait_GillRaker[spc];
	}//end species loop

	delete[] index_Trait_GillRaker;

	for (trtM = 0; trtM < max_Num_GillRaker_Cls; trtM++)
	{
		delete[] data_Buffer_Mtg_GillRaker[trtM];
	}//end male bodymass class loop

	delete[] data_Buffer_Mtg_GillRaker;

	for (trtM = 0; trtM < max_Num_GillRaker_Cls; trtM++)
	{
		delete[] exp_Data_Buffer_Mtg_GillRaker[trtM];
	}//end male bodymass class loop

	delete[] exp_Data_Buffer_Mtg_GillRaker;

	int deme, niche;

	for (deme = 0; deme < num_Deme; deme++)
	{
		for (niche = 0; niche < num_Niche; niche++)
		{
			for (spc = 0; spc < num_Spc; spc++)
			{
				for (trtM = 0; trtM < max_Num_GillRaker_Cls; trtM++)
				{
					delete[] tmp_Data_Buffer_Mtg_GillRaker[deme][niche][spc][trtM];
				}//end male bodymass class loop

				delete[] tmp_Data_Buffer_Mtg_GillRaker[deme][niche][spc];
			}//end species loop

			delete[] tmp_Data_Buffer_Mtg_GillRaker[deme][niche];
		}//end niche loop

		delete[] tmp_Data_Buffer_Mtg_GillRaker[deme];
	}//end deme loop

	delete[] tmp_Data_Buffer_Mtg_GillRaker;
}

void Simul_Proc::Set_VitaList_CelNum_Discrete(int p_Deme, int p_Niche, int p_Spc)
{
	int deme_Niche = p_Deme * num_Niche + p_Niche;
	int sex, stg, i, bin_Bodylength, gillRaker_Genotype, trt_Lower, end0, end1, sex_End, stg_End;

	//bodylength
	double tmp_Bin_Width_BodyLength = bin_Width_BodyLength[p_Spc];

	//exponent (-k) for von Bertallanffy function
	double tmp_Exponent_Von_bertallanffy;

	int tmp_Size_Bin = num_Bin_Bodylength[p_Spc];
	double tmp_BodyLength, tmp_BodyLength_Previous, tmp_Age, tmp_Num_Allele_BodyLength, tmp_VBC, tmp_VBK;

	//von Bertallanffy function
	double tmp_Ltg = ltg_VB[p_Spc];
	double tmp_Ce;
	double tmp_Ci = ci_VB[p_Spc];
	double tmp_Edd = 1.0;

	//the total number of alleles controlling bodylength 
	double numAllele_BodyLength = (double)(trt_Loc_Int[p_Spc][0] * num_Position[p_Spc]);

	//time step
	double tmp_TimeStep = unit_T_Step;

	//the total number of life stages for calculating bodylength
	double tmp_Tot_Num_Stg_Devidend;

	//the maximum asymptotic bodylength
	double tmp_Max_Asymptotic_Bodylength = bodyLength_Max_Spc[p_Spc];

	//Minimum fraction for producing random deviate with positive value
	double tmp_Ltrm = ltrm;

	//initializes the counter for vitaList
	sex_End = num_Sex;
	end0 = cls_UpperBound_Spc_Discrete;
	end1 = cls_LowerBound_Spc_Discrete;

	for (bin_Bodylength = 0; bin_Bodylength < end0; bin_Bodylength++){
		for (trt_Lower = 0; trt_Lower < end1; trt_Lower++){
			celNum_VitaList_Discrete[bin_Bodylength][trt_Lower] = 0;
		}//end lower trait loop
	}//end upper trait loop

	//agent memory address
	Agent *tmp_ID;
	vector<Agent*> tmp_cat;

	if (tick_t == true)
	{//for even sequences
		for (sex = 0; sex < sex_End; sex++)
		{
			tmp_Tot_Num_Stg_Devidend = num_Stg_BodyLength_Growth[p_Spc][sex];
			stg_End = num_Stg[p_Spc][sex];
			tmp_Exponent_Von_bertallanffy = exponent_VonBertallanffy[p_Spc][sex];

			for (stg = 0; stg < stg_End; stg++)
			{
				end0 = celNum_0[p_Deme][p_Niche][p_Spc][sex][stg];

				if (end0 > 0)
				{
					tmp_cat = agtLst_0[deme_Niche][p_Spc][sex][stg];

					for (i = 0; i < end0; i++)
					{
						//agent memory location
						tmp_ID = tmp_cat[i];

						//get bodylength
						tmp_ID->Get_BodyLength(&tmp_BodyLength_Previous);

						//get age
						tmp_ID->Get_Age(&tmp_Age);

						//get the effect of environment on bodylength growth
						tmp_ID->Get_CE(&tmp_Ce);

						//get the number of allele controling bodylength
						tmp_ID->Get_Num_Allele_BodyLength_Double(&tmp_Num_Allele_BodyLength);

						//allele dose controlling bodylength
						tmp_Num_Allele_BodyLength = (tmp_Num_Allele_BodyLength / numAllele_BodyLength);

						//calculates VBC
						Get_VBC_Foraging(&tmp_VBC, &tmp_Num_Allele_BodyLength, &tmp_Edd, &tmp_Ltrm, &tmp_Ltg, &tmp_Ce, &tmp_Ci);

						//std::cout << "edd " << tmp_Edd << " allele dose " << tmp_Num_Allele_BodyLength << std::endl;
						//std::cout << "ltg " << tmp_Ltg << " ce " << tmp_Ce << std::endl;
						//std::cout << "ci " << tmp_Ci << std::endl;

						//calculates VBK
						Get_VBK(&tmp_VBK, &tmp_Exponent_Von_bertallanffy, &tmp_Age, &tmp_Tot_Num_Stg_Devidend, &tmp_TimeStep);

						//std::cout << "exponent " << tmp_Exponent_Von_bertallanffy << " age " << tmp_Age << std::endl;
						//std::cout << "total number of life stages for body length calculation" << tmp_Tot_Num_Stg_Devidend << std::endl;
						//std::cout << "timestep " << tmp_TimeStep << std::endl;

						//calculate bodylength for foraging
						Get_BodyLength_Von_Bertallanffy_Foraging(&tmp_BodyLength, &tmp_BodyLength_Previous, &tmp_Max_Asymptotic_Bodylength, &tmp_VBC, &tmp_VBK);

						//calculate the bin number for bodylength
						Convert_BodyLength_To_Bin_Number(&bin_Bodylength, &tmp_BodyLength, &tmp_Bin_Width_BodyLength, &tmp_Size_Bin);

						//set the bin number for bodylength
						tmp_ID->Set_Bin_Number_BodyLength_Foraging(&bin_Bodylength);

						//gets the gillraker genotype traits of an agent
						tmp_ID->Get_GillRaker_Genotype(&gillRaker_Genotype);


						//increases the counter
						//std::cout << "bodylength " << tmp_BodyLength << std::endl;
						//std::cout << "VBK " << tmp_VBK << " VBC " << tmp_VBC << std::endl;
						//std::cout << "bodylength bin " << bin_Bodylength << " gillraker genotype " << gillRaker_Genotype<<std::endl;
						//Check_In("Line 12408: Simul_Proc::Set_VitaList_CelNum_Discrete()");

						celNum_VitaList_Discrete[bin_Bodylength][gillRaker_Genotype]++;

						//verification
						//std::cout << "foraging bodylength " << tmp_BodyLength << std::endl;
						//std::cout << "bin bodylength " << bin_Bodylength << " bin gillraker " << gillRaker_Genotype << std::endl;
						//std::cout << "sex " << sex << " life stage " << stg << std::endl;
						//Check_In("Line 12408: Simul_Proc::Set_VitaList_CelNum_Discrete()");
					}//end individual loop
				}//if there is a living agent
			}//end life stage loop
		}//end sex loop
	}
	else
	{//for odd sequences
		for (sex = 0; sex < sex_End; sex++)
		{
			tmp_Tot_Num_Stg_Devidend = num_Stg_BodyLength_Growth[p_Spc][sex];
			stg_End = num_Stg[p_Spc][sex];
			tmp_Exponent_Von_bertallanffy = exponent_VonBertallanffy[p_Spc][sex];

			for (stg = 0; stg < stg_End; stg++)
			{
				end0 = celNum_1[p_Deme][p_Niche][p_Spc][sex][stg];

				if (end0 > 0)
				{
					tmp_cat = agtLst_1[deme_Niche][p_Spc][sex][stg];

					for (i = 0; i < end0; i++)
					{
						//agent memory location
						tmp_ID = tmp_cat[i];

						//get bodylength
						tmp_ID->Get_BodyLength(&tmp_BodyLength_Previous);

						//get age
						tmp_ID->Get_Age(&tmp_Age);

						//get the effect of environment on bodylength growth
						tmp_ID->Get_CE(&tmp_Ce);

						//get the number of allele controling bodylength
						tmp_ID->Get_Num_Allele_BodyLength_Double(&tmp_Num_Allele_BodyLength);

						//allele dose controlling bodylength
						tmp_Num_Allele_BodyLength = (tmp_Num_Allele_BodyLength / numAllele_BodyLength);

						//calculates VBC
						Get_VBC_Foraging(&tmp_VBC, &tmp_Num_Allele_BodyLength, &tmp_Edd, &tmp_Ltrm, &tmp_Ltg, &tmp_Ce, &tmp_Ci);

						//calculates VBK
						Get_VBK(&tmp_VBK, &tmp_Exponent_Von_bertallanffy, &tmp_Age, &tmp_Tot_Num_Stg_Devidend, &tmp_TimeStep);

						//calculate bodylength for foraging
						Get_BodyLength_Von_Bertallanffy_Foraging(&tmp_BodyLength, &tmp_BodyLength_Previous, &tmp_Max_Asymptotic_Bodylength, &tmp_VBC, &tmp_VBK);

						//calculate the bin number for bodylength
						Convert_BodyLength_To_Bin_Number(&bin_Bodylength, &tmp_BodyLength, &tmp_Bin_Width_BodyLength, &tmp_Size_Bin);

						//set the bin number for bodylength
						tmp_ID->Set_Bin_Number_BodyLength_Foraging(&bin_Bodylength);

						//gets the gillraker genotype traits of an agent
						tmp_ID->Get_GillRaker_Genotype(&gillRaker_Genotype);

						//increases the counter
						celNum_VitaList_Discrete[bin_Bodylength][gillRaker_Genotype]++;

						//verification
						//std::cout << "foraging bodylength " << tmp_BodyLength << std::endl;
						///std::cout << "bin bodylength " << bin_Bodylength << " bin gillraker " << gillRaker_Genotype << std::endl;
						//std::cout << "sex " << sex << " life stage " << stg << std::endl;
						//Check_In("Line 12465: Simul_Proc::Set_VitaList_CelNum_Discrete()");
					}//end individual loop
				}//if there is a living agent
			}//end life stage loop
		}//end sex loop
	}//end else
}

void Simul_Proc::Get_Bin_Number_Degree_Of_Development_Body_Length(int *bin_Number, double *p_DegreeOfDevelopment_Current, double *p_Unit_Development_Foraging, double *p_Bin_Width, double *constant_VonBertallanffy, double *exponent_VonBertallanffy)
{//returns the bin number for bodylength for foraging
	//double growthState = ((*p_DegreeOfDevelopment_Current) + (*p_Unit_Development_Foraging));
	//double bodyLength = (*constant_VonBertallanffy) * (1.0 - (exp(((-(*exponent_VonBertallanffy)) * growthState))));
	//double bodyLength = ((*constant_VonBertallanffy) * (1.0 - (exp(((-(*exponent_VonBertallanffy)) * ((*p_DegreeOfDevelopment_Current) + (*p_Unit_Development_Foraging)))))));
	//(*bin_Number) = (int)(bodyLength / (*p_Bin_Width));
	(*bin_Number) = (int)(((*constant_VonBertallanffy) * (1.0 - (exp((((*exponent_VonBertallanffy)) * ((*p_DegreeOfDevelopment_Current) + (*p_Unit_Development_Foraging))))))) / (*p_Bin_Width));
}

void Simul_Proc::Opt_Memory_Cnt(int *p_CrntTime)
{
	if (((*p_CrntTime) % 20) == 0)
	{//every 10 time steps
		//Memory management for discrete temporary data container
		Opt_Memory_Cnt_Discrete();
	}//end if
}

void Simul_Proc::Opt_Memory_Cnt_Discrete(void)
{
	int i, j;

	for (i = 0; i < cls_UpperBound_Spc_Discrete; i++){
		for (j = 0; j < cls_LowerBound_Spc_Discrete; j++){
			if ((maxCelNum_VitaList_Discrete[i][j] * 10) < (size_VitaList_Discrete[i][j] * 8))
			{
				//reduces the number of cells in a data container
				vitaList_Discrete[i][j].resize(maxCelNum_VitaList_Discrete[i][j]);
				size_VitaList_Discrete[i][j] = maxCelNum_VitaList_Discrete[i][j];
			}//end if

			maxCelNum_VitaList_Discrete[i][j] = 0;
		}//end lower-foraging bound loop
	}//end upper-foraging bound loop
}

void Simul_Proc::Set_Vita_Memory_Cnt_Discrete(void)
{
	int end0 = cls_UpperBound_Spc_Discrete;
	int end1 = cls_LowerBound_Spc_Discrete;
	int i, j;

	for (i = 0; i < end0; i++){
		for (j = 0; j < end1; j++){
			if (celNum_VitaList_Discrete[i][j] > maxCelNum_VitaList_Discrete[i][j])
			{
				maxCelNum_VitaList_Discrete[i][j] = celNum_VitaList_Discrete[i][j];
			}//end if
		}//end sex loop
	}//end mating class loop
}

void Simul_Proc::Set_Ini_Cumul_Indiv_Rsc_Cls(double **p_Sum_Cmt_Rsc_Spc, double ***p_Sum_Spc_Rsc)
{//initializes the number of individuals foraging resource classes
	int rsc, cls, spc;

	for (rsc = 0; rsc < num_RscSpc; rsc++){
		for (cls = 0; cls < max_Num_PhenTyp_Rsc; cls++){
			p_Sum_Cmt_Rsc_Spc[rsc][cls] = 0.0;
		}//end resource species phenotype loop
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		for (rsc = 0; rsc < num_RscSpc; rsc++){
			for (cls = 0; cls < max_Num_PhenTyp_Rsc; cls++){
				p_Sum_Spc_Rsc[spc][rsc][cls] = 0.0;
			}//end resource species phenotype loop
		}//end resource species loop
	}//end species loop
}

void Simul_Proc::Set_Cmt_Spc_Rsc_Pheotype_Cls_Local_Discrete(int deme, int niche)
{
	//For verification
	//double tot_Indiv = 0.0;

	//initializes the number of individuals foraging resource classes
	Set_Ini_Cumul_Indiv_Rsc_Cls(sum_Cmt_Rsc_Spc[deme][niche], sum_Spc_Rsc[deme][niche]);

	int spc, rsc, phenoTypCls, trt0, trt1, end0, end1, end2;
	int tmp_NumSpc = num_Spc;
	double tmp_Num_Indiv, tmp;

	for (spc = 0; spc < num_Spc; spc++){
		end0 = num_Bin_Bodylength[spc];//bodylength bin
		end1 = num_Clas_Trt_Spc[spc][1];//lower limit loop end number

		for (trt0 = 0; trt0 < end0; trt0++)
		{
			for (trt1 = 0; trt1 < end1; trt1++)
			{
				if (celNum_Spc_Trt_Local_Discrete[spc][trt0][trt1] > 0)
				{//if there is an individual
					//gets esp_Discrete (foraging efficiency)
					//std::cout << "line 7706 " << std::endl;
					//std::cout << "cumul " << std::endl;
					Get_ESP_Discrete(esp_Discrete, &spc, &trt0, &trt1);
					//Check_In("Get_ESP_Discrete()");

					//gets frsp (functional response)
					Get_FunctionalResponse_Esp(frsp, esp_Discrete, phen_D_Rsc[deme][niche], deme, niche, spc);
					//Check_In("Get_FunctionalResponse_Esp()");

					//gets foraging time for resource size classes
					Get_Foraging_Time_Functional_Response(ft_Discrete, frsp, deme, niche, spc);
					//Check_In("Get_Foraging_Time_Functional_Response()");

					//gets the number of individuals in a trait class 
					tmp_Num_Indiv = ((double)celNum_Spc_Trt_Local_Discrete[spc][trt0][trt1]);

					for (rsc = 0; rsc < num_RscSpc; rsc++){
						end2 = num_PhenoTyp_Rsc[rsc];

						for (phenoTypCls = 0; phenoTypCls < end2; phenoTypCls++){
							//efficacy of hunting: esp[spc][stg][trt0][trt1][rsc][phenoTypCls]
							//fraction of foraging time: tmp_Prop_ForagingTime
							//the number of agents:  tmp_Num_Indiv
							tmp = frsp[rsc][phenoTypCls] * ft_Discrete[rsc][phenoTypCls] * tmp_Num_Indiv;
							sum_Spc_Rsc[deme][niche][spc][rsc][phenoTypCls] += tmp;
							sum_Cmt_Rsc_Spc[deme][niche][rsc][phenoTypCls] += tmp;

							//For verification
							//tot_Indiv += tmp;

							//if ((spc == 0) && (stg == 0) && (trt0 == (end0 - 1) && (trt1 == 0)))
							//{
							//cout << " Line 11777" << endl;
							//cout << "Spc: " << spc << " stg: " << stg << " trt0: " << trt0 << " trt1: " << trt1 << " rsc: " << rsc << " cls: " << phenoTypCls << endl;
							//cout << " Proportion of foraging time: " << tmp_Prop_ForagingTime << endl;
							//cout << "Total number of individuals: " << tmp_Num_Indiv << endl;
							//char tt;
							//cin >> tt;
							//}//end if

						}//end 
					}//end resource loop
				}//end if
			}//end lower trait loop
		}//end upper trait loop
	}//end species loop

	//verification
	//for (spc = 0; spc < num_Spc; spc++)
	//{
	//for (rsc = 0; rsc < num_RscSpc; rsc++){
	//end2 = num_PhenoTyp_Rsc[rsc];

	//for (phenoTypCls = 0; phenoTypCls < end2; phenoTypCls++){
	//std::cout << "resource " << rsc << " class " << phenoTypCls << std::endl;
	//std::cout << "species competition" << sum_Spc_Rsc[deme][niche][spc][rsc][phenoTypCls] << "competition" << sum_Cmt_Rsc_Spc[deme][niche][rsc][phenoTypCls] << std::endl;
	//}//end 
	//}//end resource loop
	//}//end species loop
}

void Simul_Proc::Get_Von_Bertalaffy_Growth_Weight(double *GrowthState, int *current_LifeStage, int *total_LifeStage, double *final_GrowthState, double *exp_Length_To_Weight)
{
	//exp_Length_To_Weight: exponent for converting the degree of maturation in length to that in weight
	//returns the degree of growth by using von Bertalaffy function
	(*GrowthState) = (pow((1.0 - exp((log((1.0 - (*final_GrowthState)))) * (((double)(*current_LifeStage) / (double)(*total_LifeStage))))), (*exp_Length_To_Weight)));
}

void Simul_Proc::Get_Von_Bertalaffy_Growth_Length(double *GrowthState, int *current_LifeStage, int *total_LifeStage, double *final_GrowthState)
{
	//returns the degree of growth by using von Bertalaffy function
	(*GrowthState) = (1.0 - exp((log((1.0 - (*final_GrowthState)))) * (((double)(*current_LifeStage) / (double)(*total_LifeStage)))));
}

void Simul_Proc::Set_ForagingTrait(double **p_Growth_Target_Spc)
{
	int spc, sex;

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			growth_Target_Spc[spc][sex] = p_Growth_Target_Spc[spc][sex];
		}//end sex loop
	}//end species loop

	for (spc = 0; spc < num_Spc; spc++){
		for (sex = 0; sex < num_Sex; sex++){
			exponent_VonBertallanffy[spc][sex] = log((1.0 - growth_Target_Spc[spc][sex]));
		}//end sex loop
	}//end species loop
}

void Simul_Proc::Get_CDF_4D_Discrete_Self(double ****map, int numCela, int numCelb, int numCelc, int numCeld)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j, k, l;

	//1. Initial process
	//cumul_Map[0][0][0][0] = map[0][0][0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		map[i][0][0][0] = map[i][0][0][0] + map[(i - 1)][0][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		map[0][i][0][0] = map[0][i][0][0] + map[0][(i - 1)][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelc; i++)
	{
		map[0][0][i][0] = map[0][0][i][0] + map[0][0][(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCeld; i++)
	{
		map[0][0][0][i] = map[0][0][0][i] + map[0][0][0][(i - 1)];
	}//end dimention 1 loop


	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			map[i][j][0][0] = map[i][j][0][0] + map[(i - 1)][j][0][0] + map[i][(j - 1)][0][0] - map[(i - 1)][(j - 1)][0][0];
		}
	}//end dimention 1 loop

	//Dimension 1,3
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			map[i][0][k][0] = map[i][0][k][0] + map[(i - 1)][0][k][0] + map[i][0][(k - 1)][0] - map[(i - 1)][0][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,4
	for (i = 1; i < numCela; i++)
	{
		for (l = 1; l < numCeld; l++)
		{
			map[i][0][0][l] = map[i][0][0][l] + map[(i - 1)][0][0][l] + map[i][0][0][(l - 1)] - map[(i - 1)][0][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			map[0][j][k][0] = map[0][j][k][0] + map[0][(j - 1)][k][0] + map[0][j][(k - 1)][0] - map[0][(j - 1)][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,4
	for (j = 1; j < numCelb; j++)
	{
		for (l = 1; l < numCeld; l++)
		{
			map[0][j][0][l] = map[0][j][0][l] + map[0][(j - 1)][0][l] + map[0][j][0][(l - 1)] - map[0][(j - 1)][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 3,4
	for (k = 1; k < numCelc; k++)
	{
		for (l = 1; l < numCeld; l++)
		{
			map[0][0][k][l] = map[0][0][k][l] + map[0][0][(k - 1)][l] + map[0][0][k][(l - 1)] - map[0][0][(k - 1)][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 3 process
	//Dimension 1,2,3
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				map[i][j][k][0] = map[i][j][k][0] + map[(i - 1)][j][k][0] + map[i][(j - 1)][k][0] + map[i][j][(k - 1)][0] - map[(i - 1)][(j - 1)][k][0] - map[(i - 1)][j][(k - 1)][0] - map[i][(j - 1)][(k - 1)][0] + map[(i - 1)][(j - 1)][(k - 1)][0];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,2,4
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (l = 1; l < numCeld; l++)
			{
				map[i][j][0][l] = map[i][j][0][l] + map[(i - 1)][j][0][l] + map[i][(j - 1)][0][l] + map[i][j][0][(l - 1)] - map[(i - 1)][(j - 1)][0][l] - map[(i - 1)][j][0][(l - 1)] - map[i][(j - 1)][0][(l - 1)] + map[(i - 1)][(j - 1)][0][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,3,4
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				map[i][0][k][l] = map[i][0][k][l] + map[(i - 1)][0][k][l] + map[i][0][(k - 1)][l] + map[i][0][k][(l - 1)] - map[(i - 1)][0][(k - 1)][l] - map[(i - 1)][0][k][(l - 1)] - map[i][0][(k - 1)][(l - 1)] + map[(i - 1)][0][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3,4
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				map[0][j][k][l] = map[0][j][k][l] + map[0][(j - 1)][k][l] + map[0][j][(k - 1)][l] + map[0][j][k][(l - 1)] - map[0][(j - 1)][(k - 1)][l] - map[0][(j - 1)][k][(l - 1)] - map[0][j][(k - 1)][(l - 1)] + map[0][(j - 1)][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 4 process
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				for (l = 1; l < numCeld; l++)
				{
					map[i][j][k][l] = map[i][j][k][l] + map[(i - 1)][j][k][l] + map[i][(j - 1)][k][l] + map[i][j][(k - 1)][l] + map[i][j][k][(l - 1)] + (map[i][(j - 1)][(k - 1)][(l - 1)] + map[(i - 1)][j][(k - 1)][(l - 1)] + map[(i - 1)][(j - 1)][k][(l - 1)] + map[(i - 1)][(j - 1)][(k - 1)][l])
						- (map[(i - 1)][(j - 1)][k][l] + map[(i - 1)][j][(k - 1)][l] + map[(i - 1)][j][k][(l - 1)] + map[i][(j - 1)][(k - 1)][l] + map[i][(j - 1)][k][(l - 1)] + map[i][j][(k - 1)][(l - 1)]) - map[(i - 1)][(j - 1)][(k - 1)][(l - 1)];
				}//end dimension 4 loop
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop
}

void Simul_Proc::Get_CDF_2D_Discrete_Self(double **map, int numCela, int numCelb)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j;

	//1. Initial process
	//cumul_Map[0][0][0][0] = map[0][0][0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		map[i][0] = map[i][0] + map[(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		map[0][i] = map[0][i] + map[0][(i - 1)];
	}//end dimention 1 loop

	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			map[i][j] = map[i][j] + map[(i - 1)][j] + map[i][(j - 1)] - map[(i - 1)][(j - 1)];
		}//end dimention 2 loop
	}//end dimention 1 loop
}

void Simul_Proc::Get_CDF_4D_Discrete(double ****cumul_Map, double ****map, int numCela, int numCelb, int numCelc, int numCeld)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j, k, l;

	//1. Initial process
	cumul_Map[0][0][0][0] = map[0][0][0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		cumul_Map[i][0][0][0] = map[i][0][0][0] + cumul_Map[(i - 1)][0][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		cumul_Map[0][i][0][0] = map[0][i][0][0] + cumul_Map[0][(i - 1)][0][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelc; i++)
	{
		cumul_Map[0][0][i][0] = map[0][0][i][0] + cumul_Map[0][0][(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCeld; i++)
	{
		cumul_Map[0][0][0][i] = map[0][0][0][i] + cumul_Map[0][0][0][(i - 1)];
	}//end dimention 1 loop


	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			cumul_Map[i][j][0][0] = map[i][j][0][0] + cumul_Map[(i - 1)][j][0][0] + cumul_Map[i][(j - 1)][0][0] - cumul_Map[(i - 1)][(j - 1)][0][0];
		}
	}//end dimention 1 loop

	//Dimension 1,3
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			cumul_Map[i][0][k][0] = map[i][0][k][0] + cumul_Map[(i - 1)][0][k][0] + cumul_Map[i][0][(k - 1)][0] - cumul_Map[(i - 1)][0][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,4
	for (i = 1; i < numCela; i++)
	{
		for (l = 1; l < numCeld; l++)
		{
			cumul_Map[i][0][0][l] = map[i][0][0][l] + cumul_Map[(i - 1)][0][0][l] + cumul_Map[i][0][0][(l - 1)] - cumul_Map[(i - 1)][0][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			cumul_Map[0][j][k][0] = map[0][j][k][0] + cumul_Map[0][(j - 1)][k][0] + cumul_Map[0][j][(k - 1)][0] - cumul_Map[0][(j - 1)][(k - 1)][0];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,4
	for (j = 1; j < numCelb; j++)
	{
		for (l = 1; l < numCeld; l++)
		{
			cumul_Map[0][j][0][l] = map[0][j][0][l] + cumul_Map[0][(j - 1)][0][l] + cumul_Map[0][j][0][(l - 1)] - cumul_Map[0][(j - 1)][0][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 3,4
	for (k = 1; k < numCelc; k++)
	{
		for (l = 1; l < numCeld; l++)
		{
			cumul_Map[0][0][k][l] = map[0][0][k][l] + cumul_Map[0][0][(k - 1)][l] + cumul_Map[0][0][k][(l - 1)] - cumul_Map[0][0][(k - 1)][(l - 1)];
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 3 process
	//Dimension 1,2,3
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				cumul_Map[i][j][k][0] = map[i][j][k][0] + cumul_Map[(i - 1)][j][k][0] + cumul_Map[i][(j - 1)][k][0] + cumul_Map[i][j][(k - 1)][0] - cumul_Map[(i - 1)][(j - 1)][k][0] - cumul_Map[(i - 1)][j][(k - 1)][0] - cumul_Map[i][(j - 1)][(k - 1)][0] + cumul_Map[(i - 1)][(j - 1)][(k - 1)][0];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,2,4
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (l = 1; l < numCeld; l++)
			{
				cumul_Map[i][j][0][l] = map[i][j][0][l] + cumul_Map[(i - 1)][j][0][l] + cumul_Map[i][(j - 1)][0][l] + cumul_Map[i][j][0][(l - 1)] - cumul_Map[(i - 1)][(j - 1)][0][l] - cumul_Map[(i - 1)][j][0][(l - 1)] - cumul_Map[i][(j - 1)][0][(l - 1)] + cumul_Map[(i - 1)][(j - 1)][0][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 1,3,4
	for (i = 1; i < numCela; i++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				cumul_Map[i][0][k][l] = map[i][0][k][l] + cumul_Map[(i - 1)][0][k][l] + cumul_Map[i][0][(k - 1)][l] + cumul_Map[i][0][k][(l - 1)] - cumul_Map[(i - 1)][0][(k - 1)][l] - cumul_Map[(i - 1)][0][k][(l - 1)] - cumul_Map[i][0][(k - 1)][(l - 1)] + cumul_Map[(i - 1)][0][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//Dimension 2,3,4
	for (j = 1; j < numCelb; j++)
	{
		for (k = 1; k < numCelc; k++)
		{
			for (l = 1; l < numCeld; l++)
			{
				cumul_Map[0][j][k][l] = map[0][j][k][l] + cumul_Map[0][(j - 1)][k][l] + cumul_Map[0][j][(k - 1)][l] + cumul_Map[0][j][k][(l - 1)] - cumul_Map[0][(j - 1)][(k - 1)][l] - cumul_Map[0][(j - 1)][k][(l - 1)] - cumul_Map[0][j][(k - 1)][(l - 1)] + cumul_Map[0][(j - 1)][(k - 1)][(l - 1)];
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop

	//4. Dimension 4 process
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			for (k = 1; k < numCelc; k++)
			{
				for (l = 1; l < numCeld; l++)
				{
					cumul_Map[i][j][k][l] = map[i][j][k][l] + cumul_Map[(i - 1)][j][k][l] + cumul_Map[i][(j - 1)][k][l] + cumul_Map[i][j][(k - 1)][l] + cumul_Map[i][j][k][(l - 1)] + (cumul_Map[i][(j - 1)][(k - 1)][(l - 1)] + cumul_Map[(i - 1)][j][(k - 1)][(l - 1)] + cumul_Map[(i - 1)][(j - 1)][k][(l - 1)] + cumul_Map[(i - 1)][(j - 1)][(k - 1)][l])
						- (cumul_Map[(i - 1)][(j - 1)][k][l] + cumul_Map[(i - 1)][j][(k - 1)][l] + cumul_Map[(i - 1)][j][k][(l - 1)] + cumul_Map[i][(j - 1)][(k - 1)][l] + cumul_Map[i][(j - 1)][k][(l - 1)] + cumul_Map[i][j][(k - 1)][(l - 1)]) - cumul_Map[(i - 1)][(j - 1)][(k - 1)][(l - 1)];
				}//end dimension 4 loop
			}//end dimension 3 loop
		}//end dimension 2 loop
	}//end dimention 1 loop
}

void Simul_Proc::Get_CDF_2D_Discrete(double **cumul_Map, double **map, int numCela, int numCelb)
{
	//Gets the cumulative 4 dimensional discrete cumulative distribution
	//cumul_Map: the map of 4-dimensional cumulative distribution
	//map: the map of 4-dimensional descrete probability distribution
	//int numCela: the number of classes in the dimension 1
	//int numCelb: the number of classes in the dimension 2
	//int numCelc: the number of classes in the dimension 3
	//int numCeld: the number of classes in the dimension 4
	int i, j;

	//1. Initial process
	cumul_Map[0][0] = map[0][0];

	//2. Dimension 1 process
	for (i = 1; i < numCela; i++)
	{
		cumul_Map[i][0] = map[i][0] + cumul_Map[(i - 1)][0];
	}//end dimention 1 loop

	for (i = 1; i < numCelb; i++)
	{
		cumul_Map[0][i] = map[0][i] + cumul_Map[0][(i - 1)];
	}//end dimention 1 loop


	//3. Dimension 2 process
	//Dimension 1,2
	for (i = 1; i < numCela; i++)
	{
		for (j = 1; j < numCelb; j++)
		{
			cumul_Map[i][j] = map[i][j] + cumul_Map[(i - 1)][j] + cumul_Map[i][(j - 1)] - cumul_Map[(i - 1)][(j - 1)];
		}//end dimention 2 loop
	}//end dimention 1 loop
}

void Simul_Proc::Cdf_4D_Observed_Expected(double ****mat, double ****mat_Exp, int numCls, double *num_Observations)
{
	//numCls: the number of classes for each dimension
	//mat: the map of 4-dimensional descrete probability distribution
	//mat_Exp: expected map
	double totSum;

	//calculates expected frequencies
	Get_Exp_Freq_4D_Discrete(mat_Exp, &totSum, mat, numCls);//calculates the expected frequency
	//calculates cdf of expected pdf
	Get_CDF_4D_Discrete_Self(mat_Exp, numCls, numCls, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	//convert real values to fractions
	Get_Fraction_Dim_4_Discrete(mat, totSum, numCls);//convert elements into fractions by using the total sum
	//calculates cdf of observed pdf
	Get_CDF_4D_Discrete_Self(mat, numCls, numCls, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	(*num_Observations) = totSum;//sets the total number of observations
	//Verification
	//cout << "1-D Expected value: " << (4.0) << " Observed value: " << map[0][0][0][(numCeld - 1)] << endl;
	//cout << "2-D Expected value: " << (4.0*4.0) << " Observed value: " << map[0][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[0][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][0][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][0] << endl;
	//cout << "4-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "4-D Expected valuefor 3 3 3 2: " << (4.0*4.0*4.0*3.0) << " Observed value: " << map[3][3][3][2] << endl;
	//cout << "Success!"<<endl;
}

void Simul_Proc::Cdf_2D_Observed_Expected(double **mat, double **mat_Exp, int numCls, double *num_Observations)
{
	//numCls: the number of classes for each dimension
	//mat: the map of 4-dimensional descrete probability distribution
	//mat_Exp: expected map
	double totSum;

	//calculates expected frequencies
	Get_Exp_Freq_2D_Discrete(mat_Exp, &totSum, mat, numCls);//calculates the expected frequency
	//calculates cdf of expected pdf
	Get_CDF_2D_Discrete_Self(mat_Exp, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	//convert real values to fractions
	Get_Fraction_Dim_2_Discrete(mat, totSum, numCls);//convert elements into fractions by using the total sum
	//calculates cdf of observed pdf
	Get_CDF_2D_Discrete_Self(mat, numCls, numCls);//Gets the cumulative 4 dimensional discrete cumulative distribution

	(*num_Observations) = totSum;//sets the total number of observations

	//Verification
	//cout << "1-D Expected value: " << (4.0) << " Observed value: " << map[0][0][0][(numCeld - 1)] << endl;
	//cout << "2-D Expected value: " << (4.0*4.0) << " Observed value: " << map[0][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[0][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][0][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][0][(numCeld - 1)] << endl;
	//cout << "3-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][0] << endl;
	//cout << "4-D Expected value: " << (4.0*4.0*4.0*4.0) << " Observed value: " << map[(numCela - 1)][(numCelb - 1)][(numCelc - 1)][(numCeld - 1)] << endl;
	//cout << "4-D Expected valuefor 3 3 3 2: " << (4.0*4.0*4.0*3.0) << " Observed value: " << map[3][3][3][2] << endl;
	//cout << "Success!"<<endl;
}

void Simul_Proc::Get_Exp_Freq_4D_Discrete(double ****mat_Exp, double *p_TotSum, double ****mat, int nc)
{
	//double ****mat_Exp: expected frequencies
	//double *p_TotSum: the total sum
	//double ****mat: raw matrix
	//int nc: the number of classes for each dimension

	//calculates the expected frequency
	int i, j, k, l;
	const int dim = 4;
	double **frequency = new double *[dim];//expected frequency for each dimension
	double totSum = 0.0;//The total sum of elements

	//memory allocation
	for (i = 0; i < dim; i++)
	{
		frequency[i] = new double[nc];
	}//end i loop

	Get_Freq_Dim_4_Discrete(mat, &totSum, nc, frequency);//gets the expected frequency for each dimension

	(*p_TotSum) = totSum;//sets the total sum

	//sets the expected frequency
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			for (k = 0; k < nc; k++)
			{
				for (l = 0; l < nc; l++)
				{
					mat_Exp[i][j][k][l] = frequency[0][i] * frequency[1][j] * frequency[2][k] * frequency[3][l];
				}//end 4d loop
			}//end 3d loop
		}//end 2d loop
	}//end 1d loop

	//memory deallocation
	for (i = 0; i < dim; i++)
	{
		delete[] frequency[i];
	}//end i loop

	delete[] frequency;
}

void Simul_Proc::Get_Exp_Freq_2D_Discrete(double **mat_Exp, double *p_TotSum, double **mat, int nc)
{
	//double ****mat_Exp: expected frequencies
	//double *p_TotSum: the total sum
	//double ****mat: raw matrix
	//int nc: the number of classes for each dimension

	//calculates the expected frequency
	int i, j;
	const int dim = 2;
	double **frequency = new double *[dim];//expected frequency for each dimension
	double totSum = 0.0;//The total sum of elements

	//memory allocation
	for (i = 0; i < dim; i++)
	{
		frequency[i] = new double[nc];
	}//end i loop

	Get_Freq_Dim_2_Discrete(mat, &totSum, nc, frequency);//gets the expected frequency for each dimension

	(*p_TotSum) = totSum;//sets the total sum

	//sets the expected frequency
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			mat_Exp[i][j] = frequency[0][i] * frequency[1][j];
		}//end 2d loop
	}//end 1d loop

	//memory deallocation
	for (i = 0; i < dim; i++)
	{
		delete[] frequency[i];
	}//end i loop

	delete[] frequency;
}

void Simul_Proc::Get_Fraction_Dim_4_Discrete(double ****mat, double p_TotSum, int nc)
{
	//double ****mat: 4d matrix
	//double * p_TotSum: Pre-calculated Total sum
	//int nc: the number of classes in each dimension
	int i, j, k, l;

	if (p_TotSum != 0.0)
	{
		//calculates the sums
		for (i = 0; i < nc; i++)
		{
			for (j = 0; j < nc; j++)
			{
				for (k = 0; k < nc; k++)
				{
					for (l = 0; l < nc; l++)
					{
						mat[i][j][k][l] /= p_TotSum;
					}//end 4 d loop
				}//end 3 d loop
			}//end 2 d loop
		}//end 1 d loop
	}//end if
}

void Simul_Proc::Get_Fraction_Dim_2_Discrete(double **mat, double p_TotSum, int nc)
{
	//double ****mat: 4d matrix
	//double * p_TotSum: Pre-calculated Total sum
	//int nc: the number of classes in each dimension
	int i, j;

	if (p_TotSum != 0.0)
	{
		//calculates the sums
		for (i = 0; i < nc; i++)
		{
			for (j = 0; j < nc; j++)
			{
				mat[i][j] /= p_TotSum;
			}//end 2 d loop
		}//end 1 d loop
	}//end if
}

void Simul_Proc::Get_Freq_Dim_4_Discrete(double ****mat, double *p_TotSum, int nc, double **frq)
{
	//double ****mat: 4d matrix
	//double * p_TotSum: Total sum
	//int nc: the number of classes in each dimension
	//double *frq: dim[dimentions (=4), class ] frequencies in each dimension
	int i, j, k, l;
	const int dim = 4;
	double tmp;

	//initialization
	for (j = 0; j < dim; j++)
	{
		for (i = 0; i < nc; i++)
		{
			frq[j][i] = 0.0;
		}//end class loop
	}//end dimension loop

	//calculates the sums
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			for (k = 0; k < nc; k++)
			{
				for (l = 0; l < nc; l++)
				{
					tmp = mat[i][j][k][l];
					frq[0][i] += tmp;
					frq[1][j] += tmp;
					frq[2][k] += tmp;
					frq[3][l] += tmp;
				}//end 4 d loop
			}//end 3 d loop
		}//end 2 d loop
	}//end 1 d loop

	double totSum = 0.0;

	//calculates the total sum
	for (i = 0; i < nc; i++)
	{
		totSum += frq[0][i];
	}//end 1 d loop

	(*p_TotSum) = totSum;//sets the total sum

	//calculates the class frequencies for each dimension

	if (totSum > 0.0)
	{
		for (j = 0; j < dim; j++)
		{
			for (i = 0; i < nc; i++)
			{
				frq[j][i] /= totSum;
			}//end class loop
		}//end dimension loop
	}//end if
}

void Simul_Proc::Get_Freq_Dim_2_Discrete(double **mat, double *p_TotSum, int nc, double **frq)
{
	//double **mat: 2d matrix
	//double * p_TotSum: Total sum
	//int nc: the number of classes in each dimension
	//double *frq: dim[dimentions (=2), class ] frequencies in each dimension
	int i, j;
	const int dim = 2;
	double tmp;

	//initialization
	for (j = 0; j < dim; j++)
	{
		for (i = 0; i < nc; i++)
		{
			frq[j][i] = 0.0;
		}//end class loop
	}//end dimension loop

	//calculates the sums
	for (i = 0; i < nc; i++)
	{
		for (j = 0; j < nc; j++)
		{
			tmp = mat[i][j];
			frq[0][i] += tmp;
			frq[1][j] += tmp;
		}//end 2 d loop
	}//end 1 d loop

	double totSum = 0.0;

	//calculates the total sum
	for (i = 0; i < nc; i++)
	{
		totSum += frq[0][i];
	}//end 1 d loop

	(*p_TotSum) = totSum;//sets the total sum

	//calculates the class frequencies for each dimension

	if (totSum > 0.0)
	{
		for (j = 0; j < dim; j++)
		{
			for (i = 0; i < nc; i++)
			{
				frq[j][i] /= totSum;
			}//end class loop
		}//end dimension loop
	}//end if
}

void Simul_Proc::Get_Kolmogorov_Smirnov_Test_SampleSize(double *p_AdjustedN, double N0, double N1)
{
	if ((N0 > 10.0) && (N1 > 10.0))
	{
		(*p_AdjustedN) = (N0 * N1) / (N0 + N1);
	}
	else
	{
		(*p_AdjustedN) = N0;
	}//end else
}

void Simul_Proc::Get_Kolmogorov_Smirnov_Test_Z_Inf(double *Z_Inf, double *Z, double D, double N0)
{
	(*Z) = (pow(N0, 0.5)) * D;
	double tmp = (1.0 - (0.53 * (pow(N0, (-0.9)))));

	if (tmp != 0.0)
	{
		(*Z_Inf) = (*Z) / tmp;
	}
	else
	{
		(*Z_Inf) = std::numeric_limits<double>::infinity();
	}//end else
}

void Simul_Proc::Get_Kolmogorov_Smirnov_Test_P_Value(double *p_PValue, double *p_PValue_Peacock, double *Z_Inf)
{
	//Kolmogorov-Smirnov distribution
	//p_PValue_Peacock: p-value calculation shown in Peacock (1983)
	//KolmogorovDist ks;
	//double tmp_Cdf = ks.K((*p_N), (*D));
	//(*p_PValue) = 1.0 - tmp_Cdf;
	double tmp_Cdf = cdf(std_Norm_KS_Test, (*Z_Inf));
	(*p_PValue) = 1.0 - tmp_Cdf;
	(*p_PValue_Peacock) = 2.0 * (exp((-2.0 * (pow(((*Z_Inf) - 0.5), 2.0)))));
}

void Simul_Proc::ReArrange_4D_Mat(double ****mat_Rearranged, double ****mat, int nCls, int *s)
{
	//nCls: the number of classes for each dimension
	//s: vector indicating the sequence order 0: increasing order , 1: decreasing order
	//s[0]: 1st dimension order, s[1]: second dimension order, s[2]: third dimension order, s[3]:fourth dimension order
	const int dim = 4;
	int *counter = new int[dim];//counter
	int *start = new int[dim];//start point
	int *cursor = new int[dim];//current position for a rearrange matrix cursor
	int *increment = new int[dim];//increment for iteration for a rearranged matrix

	for (int i = 0; i < dim; i++)
	{
		if (s[i] == 0)
		{//increasing order
			start[i] = 0;
			increment[i] = 1;//increment
		}
		else
		{
			start[i] = (nCls - 1);
			increment[i] = (-1);//increment
		}//else
	}//end dimension loop

	cursor[0] = start[0];
	counter[0] = 0;
	while (counter[0] < nCls)
	{
		cursor[1] = start[1];
		counter[1] = 0;
		while (counter[1] < nCls)
		{
			cursor[2] = start[2];
			counter[2] = 0;
			while (counter[2] < nCls)
			{
				cursor[3] = start[3];
				counter[3] = 0;
				while (counter[3] < nCls)
				{//sets a rearranged matrix
					mat_Rearranged[cursor[0]][cursor[1]][cursor[2]][cursor[3]] = mat[counter[0]][counter[1]][counter[2]][counter[3]];
					cursor[3] += increment[3];
					counter[3]++;
				}//

				cursor[2] += increment[2];
				counter[2]++;
			}//

			cursor[1] += increment[1];
			counter[1]++;
		}//

		cursor[0] += increment[0];
		counter[0]++;
	}//

	delete[] start;
	delete[] cursor;
	delete[] increment;
	delete[] counter;
}

void Simul_Proc::ReArrange_2D_Mat(double **mat_Rearranged, double **mat, int nCls, int *s)
{
	//nCls: the number of classes for each dimension
	//s: vector indicating the sequence order 0: increasing order , 1: decreasing order
	//s[0]: 1st dimension order, s[1]: second dimension order, s[2]: third dimension order, s[3]:fourth dimension order
	const int dim = 2;
	int *counter = new int[dim];//counter
	int *start = new int[dim];//start point
	int *cursor = new int[dim];//current position for a rearrange matrix cursor
	int *increment = new int[dim];//increment for iteration for a rearranged matrix

	for (int i = 0; i < dim; i++)
	{
		if (s[i] == 0)
		{//increasing order
			start[i] = 0;
			increment[i] = 1;//increment
		}
		else
		{
			start[i] = (nCls - 1);
			increment[i] = (-1);//increment
		}//else
	}//end dimension loop

	cursor[0] = start[0];
	counter[0] = 0;

	while (counter[0] < nCls)
	{
		cursor[1] = start[1];
		counter[1] = 0;
		while (counter[1] < nCls)
		{
			mat_Rearranged[cursor[0]][cursor[1]] = mat[counter[0]][counter[1]];
			cursor[1] += increment[1];
			counter[1]++;
		}//

		cursor[0] += increment[0];
		counter[0]++;
	}//

	delete[] start;
	delete[] cursor;
	delete[] increment;
	delete[] counter;
}

void Simul_Proc::Get_MaxDiff_4D_Mat(double *p_Max, double ****mat0, double ****mat1, int nCls)
{
	int i, j, k, l;
	double tmp_Max = 0.0;

	//calculates the sums
	for (i = 0; i < nCls; i++)
	{
		for (j = 0; j < nCls; j++)
		{
			for (k = 0; k < nCls; k++)
			{
				for (l = 0; l < nCls; l++)
				{
					if ((abs((mat0[i][j][k][l] - mat1[i][j][k][l]))) > tmp_Max)
					{
						tmp_Max = (abs((mat0[i][j][k][l] - mat1[i][j][k][l])));
					}//end if
				}//end 4 d loop
			}//end 3 d loop
		}//end 2 d loop
	}//end 1 d loop

	(*p_Max) = tmp_Max;//sets the maximum difference
}

void Simul_Proc::Get_MaxDiff_2D_Mat(double *p_Max, double **mat0, double **mat1, int nCls)
{
	int i, j;
	double tmp_Max = 0.0;

	//calculates the sums
	for (i = 0; i < nCls; i++)
	{
		for (j = 0; j < nCls; j++)
		{
			if ((abs((mat0[i][j] - mat1[i][j]))) > tmp_Max)
			{
				tmp_Max = (abs((mat0[i][j] - mat1[i][j])));
			}//end if
		}//end 2 d loop
	}//end 1 d loop

	(*p_Max) = tmp_Max;//sets the maximum difference
}

void Simul_Proc::Data_Output_KolmogorovSmirnov_2D_Mtg_GillRaker(int *p_Counter_Sampling)
{
	if ((*p_Counter_Sampling) == 0)
	{
		int deme, niche, spc;
		int *sequenceOrder = new int[2];//sequence order for 4D matrix
		double D_Tot, D, Z_Inf, Z_Raw, p_Value, p_Value_PeaCock, tot_Num_Observations;//statistics for Kolmogorov_Smirnov_Test
		//sample size for Kolmogorov_Smirnov_Test
		double adjusted_Sample_Size;

		for (deme = 0; deme < num_Deme; deme++)
		{
			for (niche = 0; niche < num_Niche; niche++)
			{
				for (spc = 0; spc < num_Spc; spc++)
				{
					D_Tot = 0.0;//initialization of D

					//4d permutation
					for (sequenceOrder[0] = 0; sequenceOrder[0] < 2; sequenceOrder[0]++)
					{
						for (sequenceOrder[1] = 0; sequenceOrder[1] < 2; sequenceOrder[1]++)
						{
							//data_Buffer_Mtg_Trait:Re-arranged raw cdf
							//exp_Data_Buffer_Mtg_Trait: expected pdf of the Re-arranged raw cdf
							ReArrange_2D_Mat(data_Buffer_Mtg_GillRaker, tmp_Data_Buffer_Mtg_GillRaker[deme][niche][spc], max_Num_GillRaker_Cls, sequenceOrder);//Re-arrange 4D matrix in a specific order
							Cdf_2D_Observed_Expected(data_Buffer_Mtg_GillRaker, exp_Data_Buffer_Mtg_GillRaker, max_Num_GillRaker_Cls, &tot_Num_Observations);//calculates the cumulative probability density for 4-D observed values and their expected values
							Get_MaxDiff_2D_Mat(&D, data_Buffer_Mtg_GillRaker, exp_Data_Buffer_Mtg_GillRaker, max_Num_GillRaker_Cls);//finds the maximum difference in 4-D matrix

							if (D > D_Tot)
							{
								D_Tot = D;
							}//end if
						}//end dimension 1 loop
					}//end dimension 1 loop

					//To print out:  p_Value, D_Tot, Z, tot_Num_Observations, adjusted_Sample_Size
					Get_Kolmogorov_Smirnov_Test_SampleSize(&adjusted_Sample_Size, tot_Num_Observations, tot_Num_Observations);//calculates the sample size for Kolmogorov_Smirnov_Test
					Get_Kolmogorov_Smirnov_Test_Z_Inf(&Z_Inf, &Z_Raw, D_Tot, adjusted_Sample_Size);//calculates the infinite Z for Kolmogorov_Smirnov_Test
					Get_Kolmogorov_Smirnov_Test_P_Value(&p_Value, &p_Value_PeaCock, &Z_Inf);//calculates the infinite Z for Kolmogorov_Smirnov_Test

					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][0] = Z_Inf;
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][1] = Z_Raw;
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][2] = D_Tot;
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][3] = p_Value;
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][4] = p_Value_PeaCock;
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][5] = tot_Num_Observations;
					data_Buffer_Kolmogorov_Smirnov_Mtg_GillRaker[deme][niche][spc][6] = adjusted_Sample_Size;
				}//end species loop
			}//end niche
		}//end deme

		delete[] sequenceOrder;
	}//end if
}

void Simul_Proc::Data_Output_KolmogorovSmirnov_2D_Mtg_Bodymass(int *p_Counter_Sampling)
{
	if ((*p_Counter_Sampling) == 0)
	{
		int deme, niche, spc;
		int *sequenceOrder = new int[2];//sequence order for 4D matrix
		double D_Tot, D, Z_Inf, Z_Raw, p_Value, p_Value_PeaCock, tot_Num_Observations;//statistics for Kolmogorov_Smirnov_Test
		//sample size for Kolmogorov_Smirnov_Test
		double adjusted_Sample_Size;

		for (deme = 0; deme < num_Deme; deme++)
		{
			for (niche = 0; niche < num_Niche; niche++)
			{
				for (spc = 0; spc < num_Spc; spc++)
				{
					D_Tot = 0.0;//initialization of D

					//4d permutation
					for (sequenceOrder[0] = 0; sequenceOrder[0] < 2; sequenceOrder[0]++)
					{
						for (sequenceOrder[1] = 0; sequenceOrder[1] < 2; sequenceOrder[1]++)
						{
							//data_Buffer_Mtg_Trait:Re-arranged raw cdf
							//exp_Data_Buffer_Mtg_Trait: expected pdf of the Re-arranged raw cdf
							ReArrange_2D_Mat(data_Buffer_Mtg_Bodymass, tmp_Data_Buffer_Mtg_Bodymass[deme][niche][spc], max_Num_Bodymass_Cls, sequenceOrder);//Re-arrange 4D matrix in a specific order
							Cdf_2D_Observed_Expected(data_Buffer_Mtg_Bodymass, exp_Data_Buffer_Mtg_Bodymass, max_Num_Bodymass_Cls, &tot_Num_Observations);//calculates the cumulative probability density for 4-D observed values and their expected values
							Get_MaxDiff_2D_Mat(&D, data_Buffer_Mtg_Bodymass, exp_Data_Buffer_Mtg_Bodymass, max_Num_Bodymass_Cls);//finds the maximum difference in 4-D matrix

							if (D > D_Tot)
							{
								D_Tot = D;
							}//end if
						}//end dimension 2 loop
					}//end dimension 1 loop

					//To print out:  p_Value, D_Tot, Z, tot_Num_Observations, adjusted_Sample_Size
					Get_Kolmogorov_Smirnov_Test_SampleSize(&adjusted_Sample_Size, tot_Num_Observations, tot_Num_Observations);//calculates the sample size for Kolmogorov_Smirnov_Test
					Get_Kolmogorov_Smirnov_Test_Z_Inf(&Z_Inf, &Z_Raw, D_Tot, adjusted_Sample_Size);//calculates the infinite Z for Kolmogorov_Smirnov_Test
					Get_Kolmogorov_Smirnov_Test_P_Value(&p_Value, &p_Value_PeaCock, &Z_Inf);//calculates the infinite Z for Kolmogorov_Smirnov_Test

					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][0] = Z_Inf;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][1] = Z_Raw;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][2] = D_Tot;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][3] = p_Value;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][4] = p_Value_PeaCock;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][5] = tot_Num_Observations;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Bodymass[deme][niche][spc][6] = adjusted_Sample_Size;
				}//end species loop
			}//end niche
		}//end deme

		delete[] sequenceOrder;
	}//end if
}

void Simul_Proc::Data_Output_KolmogorovSmirnov_4D_Mtg_Genotype(int *p_Counter_Sampling)
{
	if ((*p_Counter_Sampling) == 0)
	{
		int deme, niche, spc;
		int *sequenceOrder = new int[4];//sequence order for 4D matrix
		double D_Tot, D, Z_Inf, Z_Raw, p_Value, p_Value_PeaCock, tot_Num_Observations;//statistics for Kolmogorov_Smirnov_Test
		//sample size for Kolmogorov_Smirnov_Test
		double adjusted_Sample_Size;

		for (deme = 0; deme < num_Deme; deme++)
		{
			for (niche = 0; niche < num_Niche; niche++)
			{
				for (spc = 0; spc < num_Spc; spc++)
				{
					D_Tot = 0.0;//initialization of D

					//4d permutation
					for (sequenceOrder[0] = 0; sequenceOrder[0] < 2; sequenceOrder[0]++)
					{
						for (sequenceOrder[1] = 0; sequenceOrder[1] < 2; sequenceOrder[1]++)
						{
							for (sequenceOrder[2] = 0; sequenceOrder[2] < 2; sequenceOrder[2]++)
							{
								for (sequenceOrder[3] = 0; sequenceOrder[3] < 2; sequenceOrder[3]++)
								{
									//data_Buffer_Mtg_Trait:Re-arranged raw cdf
									//exp_Data_Buffer_Mtg_Trait: expected pdf of the Re-arranged raw cdf
									ReArrange_4D_Mat(data_Buffer_Mtg_Trait, tmp_Data_Buffer_Mtg_Trait[deme][niche][spc], max_Num_Clas_Trt, sequenceOrder);//Re-arrange 4D matrix in a specific order
									Cdf_4D_Observed_Expected(data_Buffer_Mtg_Trait, exp_Data_Buffer_Mtg_Trait, max_Num_Clas_Trt, &tot_Num_Observations);//calculates the cumulative probability density for 4-D observed values and their expected values
									Get_MaxDiff_4D_Mat(&D, data_Buffer_Mtg_Trait, exp_Data_Buffer_Mtg_Trait, max_Num_Clas_Trt);//finds the maximum difference in 4-D matrix

									if (D > D_Tot)
									{
										D_Tot = D;
									}//end if
								}//end dimension 1 loop
							}//end dimension 1 loop
						}//end dimension 1 loop
					}//end dimension 1 loop

					//To print out:  p_Value, D_Tot, Z, tot_Num_Observations, adjusted_Sample_Size
					Get_Kolmogorov_Smirnov_Test_SampleSize(&adjusted_Sample_Size, tot_Num_Observations, tot_Num_Observations);//calculates the sample size for Kolmogorov_Smirnov_Test
					Get_Kolmogorov_Smirnov_Test_Z_Inf(&Z_Inf, &Z_Raw, D_Tot, adjusted_Sample_Size);//calculates the infinite Z for Kolmogorov_Smirnov_Test
					Get_Kolmogorov_Smirnov_Test_P_Value(&p_Value, &p_Value_PeaCock, &Z_Inf);//calculates the infinite Z for Kolmogorov_Smirnov_Test

					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][0] = Z_Inf;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][1] = Z_Raw;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][2] = D_Tot;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][3] = p_Value;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][4] = p_Value_PeaCock;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][5] = tot_Num_Observations;
					data_Buffer_Kolmogorov_Smirnov_Mtg_Genotype[deme][niche][spc][6] = adjusted_Sample_Size;
				}//end species loop
			}//end niche
		}//end deme

		delete[] sequenceOrder;
	}//end if
}

void Simul_Proc::Display_Simul_Status(int *p_tStep)
{
	if (((*p_tStep) % 1000) == 0)
	{
		cout << "Timestep " << (*p_tStep) << endl;
		cout << "The number of agent cells" << endl;

		int spc;

		for (spc = 0; spc < num_Spc; spc++)
		{
			cout << "Core species #" << spc << " : " << celNum_Agt[spc] << endl;
		}//end species loop
	}//end if
}

void Simul_Proc::Display_Simul_Setup(int *p_totNumSimul, int *p_Num_Replicate, int *p_Cnt_Simul)
{
	cout << endl << "Simulation: " << (*p_Cnt_Simul) << "  / " << ((*p_totNumSimul) - 1) << "  Replicate: " << (*p_Num_Replicate) << endl;
}

void Simul_Proc::Display_StatisticalProcessMode(void)
{
	cout << "Statistical Process Mode";
}

void Simul_Proc::Get_Chance_Sexual_Maturation(double *p_Chance_Sexual_Maturation_stage, int * p_Spc, int *p_Current_Lifestage, int *p_Sex, double *p_Allele_Dosage, int *p_Num_Stg_Immat_Sex_Mat)
{
	if ((*p_Current_Lifestage) < (num_Stg[(*p_Spc)][(*p_Sex)] - 1 - (*p_Num_Stg_Immat_Sex_Mat)))
	{//sexually-immature life stages having no chance for reproduction
		(*p_Chance_Sexual_Maturation_stage) = 0.0;
	}
	else if ((*p_Current_Lifestage) == (num_Stg[(*p_Spc)][(*p_Sex)] - 1))
	{//the last life stage
		(*p_Chance_Sexual_Maturation_stage) = 1.0;
	}
	else
	{//life stages having a chance for reproduction
		//Full (1) allele was assumed to suppress sexual maturation
		(*p_Chance_Sexual_Maturation_stage) = 1.0 - (pow((*p_Allele_Dosage), (1.0 / ((double)(*p_Num_Stg_Immat_Sex_Mat)))));
	}//end else
}

void Simul_Proc::Get_Mean_Fertility_Predicted_By_Bodymass(double *p_Fertility, double *p_Bodymass)
{
	//double slope = 542.0;
	double exponent = 0.893;
	//(*p_Fertility) = slope * (pow((*p_Bodymass), exponent));
	(*p_Fertility) = (pow((*p_Bodymass), exponent));
}

void Simul_Proc::Get_Offsprinf_Convergence_Factor_From_FemaleOffspring(double *p_Factor, double *p_Prop_Females_At_Birth)
{
	if ((*p_Prop_Females_At_Birth) == 0.0)
	{
		(*p_Factor) = 0.0;
	}
	else
	{
		(*p_Factor) = 1.0 + ((1.0 - (*p_Prop_Females_At_Birth)) / (*p_Prop_Females_At_Birth));
	}//end else
}

void Simul_Proc::Set_Fertility(double**** p_F, double**** p_F_CV, double *p_Prop_Female_At_Birth)
{
	int deme, niche, spc, stg, i;
	int chosen_Trait = 0;

	proportion_Female_At_Birth = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		proportion_Female_At_Birth[spc] = p_Prop_Female_At_Birth[spc];
	}//end species loop

	double *max_F = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		max_F[spc] = 0.0;
	}//end species loop

	//calculates the maximum thumber of female off spring derive from a female adult
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (stg = 0; stg < max_Num_Stg; stg++){
					if (p_F[deme][niche][spc][stg] > max_F[spc])
					{
						max_F[spc] = p_F[deme][niche][spc][stg];
					}//end if
				}//end life stage loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	maximum_Fertility = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		maximum_Fertility[spc] = max_F[spc];
	}//end species loop

	double *max_F_CV = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		max_F_CV[spc] = 0.0;
	}//end species loop

	//calculates the maximum thumber of female off spring derive from a female adult
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (stg = 0; stg < max_Num_Stg; stg++){
					if (p_F_CV[deme][niche][spc][stg] > max_F_CV[spc])
					{
						max_F_CV[spc] = p_F_CV[deme][niche][spc][stg];
					}//end if
				}//end life stage loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	maximum_Fertility_CV = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		maximum_Fertility_CV[spc] = max_F_CV[spc];
	}//end species loop

	//The body mass (g) of a trait at a life stage
	double ***Num_Female_Offspring = new double**[num_Spc];

	for (spc = 0; spc < num_Spc; spc++){
		Num_Female_Offspring[spc] = new double*[max_Num_Stg];

		for (stg = 0; stg < max_Num_Stg; stg++){
			Num_Female_Offspring[spc][stg] = new double[max_Num_Clas_Trt_Spc[0]];

			for (i = 0; i < max_Num_Clas_Trt_Spc[chosen_Trait]; i++){
				Num_Female_Offspring[spc][stg][i] = 0.0;
			}//end upper-bound trait loop
		}//end life stage loop
	}//end species loop

	//the maximum expected fertility
	double *max_Fertility = new double[num_Spc];

	for (spc = 0; spc < num_Spc; spc++)
	{
		max_Fertility[spc] = 0.0;
	}//end species loop

	int end0, end1;

	for (spc = 0; spc < num_Spc; spc++){
		end0 = num_Stg[spc][1];//female life stages

		for (stg = 0; stg < end0; stg++){
			end1 = max_Num_Clas_Trt_Spc[chosen_Trait];

			for (i = 0; i < end1; i++){
				Get_Mean_Fertility_Predicted_By_Bodymass(&Num_Female_Offspring[spc][stg][i], &bodyMass[spc][1][stg][i]);

				if (Num_Female_Offspring[spc][stg][i] > max_Fertility[spc])
				{
					max_Fertility[spc] = Num_Female_Offspring[spc][stg][i];
				}//end if
			}//end upper-bound trait loop
		}//end life stage loop
	}//end species loop

	double sexPropFactor = 0.0;//factor for coverting the number of female offsprings to the number of male anf female offsprings

	//deallocate memory
	for (spc = 0; spc < num_Spc; spc++){
		for (stg = 0; stg < max_Num_Stg; stg++){
			delete[] Num_Female_Offspring[spc][stg];
		}//end life stage loop

		delete[] Num_Female_Offspring[spc];
	}//end species loop

	delete[] Num_Female_Offspring;

	delete[] max_Fertility;
	delete[] max_F_CV;
	delete[] max_F;
}

void Simul_Proc::Set_Functional_Response_Rsc(double ****p_H_Rsc_RscSpc, double ****p_Qq_Rsc_RscSpc, double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double **p_Trt_RscSpc_Mean_Sd)
{
	int deme, niche, spc, rsc;
	double tmp_Abundance;//temporary abundance of resource species

	//sets handling time
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				for (rsc = 0; rsc < num_RscSpc; rsc++){
					tmp_Abundance = (p_C_BioMass_Abundance[deme][niche] * (pow((1.0 / p_Trt_RscSpc_Mean_Sd[rsc][0]), p_Alpha_BioMass_Abundance[deme][niche])));
					th_Rsc_RscSpc[deme][niche][spc][rsc] = p_H_Rsc_RscSpc[deme][niche][spc][rsc] * (1.0 / tmp_Abundance);
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	//sets functional response mode
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_RscSpc; spc++){
				for (rsc = 0; rsc < num_RscSpc; rsc++){
					q_Rsc_RscSpc[deme][niche][spc][rsc] = p_Qq_Rsc_RscSpc[deme][niche][spc][rsc];
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Set_Functional_Response_Spc(double **p_Alpha_BioMass_Abundance, double **p_C_BioMass_Abundance, double **p_Trt_RscSpc_Mean_Sd)
{
	int deme, niche, spc, rsc;
	double tmp_Abundance;//temporary abundance of resource species

	//sets handling time
	for (deme = 0; deme < num_Deme; deme++){
		for (niche = 0; niche < num_Niche; niche++){
			for (spc = 0; spc < num_Spc; spc++){
				for (rsc = 0; rsc < num_RscSpc; rsc++){
					tmp_Abundance = (p_C_BioMass_Abundance[deme][niche] * (pow((1.0 / p_Trt_RscSpc_Mean_Sd[rsc][0]), p_Alpha_BioMass_Abundance[deme][niche])));
					tmp_Abundance = tmp_Abundance / ((double)max_Num_PhenTyp_Rsc);
					th_Spc_RscSpc[deme][niche][spc][rsc] = th_Spc_RscSpc[deme][niche][spc][rsc] * (1.0 / tmp_Abundance);
				}//end resource species loop
			}//end species loop
		}//end niche loop
	}//end deme loop
}

void Simul_Proc::Deallocate_Memory_Simul_FileName(void)
{
	if (status_Simul_File_Creation == true)
	{
		delete[] fileName_KolmogorovSmirnov_Mtg_GillRaker;
		delete[] fileName_KolmogorovSmirnov_Mtg_Bodymass;
		delete[] fileName_KolmogorovSmirnov_Mtg_Genotype;
		delete[] fileName_Trait_Comb;
		delete[] fileName_Bodysize_Mtg;
		delete[] fileName_Bodysize_Mtg_Adt;
		delete[] fileName_Bodysize_Genotype_Mtg;
		delete[] fileName_Bodysize_Genotype_Mtg_Adt;
		delete[] fileName_GillRaker_Genotype_Mtg;
		delete[] fileName_GillRaker_Genotype_Mtg_Adt;
		delete[] fileName_Trait;
		delete[] fileName_Genome;
	}//end if
}

void Simul_Proc::Set_Stats_Trait(void)
{
	vector<double> tmp_Trait_Value;//trait values for columns in trait output file
	Set_File_Column_Header_Trait_Value(tmp_Trait_Value);//sets trait values for output files

	//the number of data partitions per row
	int tmp_Num_Segments = num_Deme * num_Niche * max_Num_Trait;
	int num_Header_Rows = 1;
	int num_Header_Columns = 1;
	int tmp_Sort_Order = 0;//row sorted
	Batch_Data_Proc datProc;//an object for batch process

	//calculates means and standard deviations, and sets the output file
	string fileName_Mu;//a new file name for the batch data
	fileName_Mu = datProc.Batch_Mean_Var_2D_Value_Frequency_Raw(tmp_Trait_Value, fileName_Trait_Str, tmp_Num_Segments, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);

	//calculates the first-degree difference
	//datProc.Batch_Delta_2D_Value(fileName_Mu, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);

	//calculates the second-degree difference
	//datProc.Batch_Delta_Delta_2D_Value(fileName_Mu, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);

	//for single trait data files
	int tmp_Num_Traits = 1;
	tmp_Num_Segments = num_Deme * num_Niche * tmp_Num_Traits;
	int tmp_Num_Class_Trait = max_Num_Clas_Trt;//the number of classes for a trait

	//trait values for columns in a single trait output file
	vector<double> tmp_Trait_Value_Single;	
	Set_File_Column_Header_Trait_Single_Value(tmp_Trait_Value_Single, &tmp_Num_Traits, &tmp_Num_Class_Trait);//sets trait values for output files
	//bodysize phenotypes for mates
	fileName_Mu = datProc.Batch_Mean_Var_2D_Value_Frequency_Raw(tmp_Trait_Value_Single, fileName_Bodysize_Mtg_Adt_Str, tmp_Num_Segments, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);
	//bodysize genotypes for mates
	fileName_Mu = datProc.Batch_Mean_Var_2D_Value_Frequency_Raw(tmp_Trait_Value_Single, fileName_Bodysize_Genotype_Mtg_Adt_Str, tmp_Num_Segments, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);
	//gillraker genotypes for mates
	fileName_Mu = datProc.Batch_Mean_Var_2D_Value_Frequency_Raw(tmp_Trait_Value_Single, fileName_GillRaker_Genotype_Mtg_Adt_Str, tmp_Num_Segments, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);
}

void Simul_Proc::Set_Stats_Rsc(void)
{
	vector<double> tmp_Rsc_Bodymass;//trait values for columns in trait output file
	Set_File_Column_Header_Density_Value(tmp_Rsc_Bodymass);//sets bodymass for resource classes

	//the number of partitions for calculating means and standard deviations
	int tmp_Num_Segments = num_Deme * num_Niche * num_RscSpc;
	int tmp_Tot_Num_Segments = num_Deme * num_Niche;
	int num_Header_Rows = 1;
	int num_Header_Columns = 1;
	int tmp_Sort_Order = 0;//row sorted
	string additional_Tag;//additional tag for the file name
	Batch_Data_Proc datProc;//an object for batch process

	//calculates product of abundance and unit biomass
	string fileName_TotalMass;//file name for the total mass for each resource size class 
	fileName_TotalMass = datProc.Batch_Product_Value_Frequency(tmp_Rsc_Bodymass, fileName_Density_Str, num_Header_Rows, num_Header_Columns);

	//calculates the first difference
	//datProc.Batch_Delta_2D_Value(fileName_TotalMass, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);

	//calculates the second difference
	//datProc.Batch_Delta_Delta_2D_Value(fileName_TotalMass, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);

	//calculates means and standard deviations, and sets the output file
	//For each resource-size cluster 
	additional_Tag = "cl";//additional tag for the file name
	datProc.Batch_Mean_Var_2D_Value_Frequency_Raw_Add_Char(tmp_Rsc_Bodymass, fileName_Density_Str, additional_Tag, tmp_Num_Segments, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);
	//calculates means and standard deviations, and sets the output file
	//For all resource clusters 
	additional_Tag = "al";//additional tag for the file name
	datProc.Batch_Mean_Var_2D_Value_Frequency_Raw_Add_Char(tmp_Rsc_Bodymass, fileName_Density_Str, additional_Tag, tmp_Tot_Num_Segments, num_Header_Rows, num_Header_Columns, tmp_Sort_Order);
}

void Simul_Proc::Deallocate_Memory_Data_Buffer(void)
{
	if (status_Simul_File_Creation == true)
	{
		int i;

		//de-allocate memory
		for (i = 0; i < interval_sampling; i++){
			delete[] data_Buffer_Allele_2D[i];
		}//end row loop

		delete[] data_Buffer_Allele_2D;

		//de-allocate memory
		for (i = 0; i < interval_sampling; i++){
			delete[] data_Buffer_RscCls_2D[i];
		}//end row loop

		delete[] data_Buffer_RscCls_2D;

		//de-allocate memory
		for (i = 0; i < interval_sampling; i++){
			delete[] data_Buffer_Trait_2D[i];
		}//end row loop

		delete[] data_Buffer_Trait_2D;

		//de-allocate memory
		for (i = 0; i < 1; i++){
			delete[] data_Buffer_Trait_Combination_2D[i];
		}//end row loop

		delete[] data_Buffer_Trait_Combination_2D;

		//de-allocate memory
		for (i = 0; i < 1; i++){
			delete[] data_Buffer_Bodysize_Mtg_2D[i];
		}//end row loop

		delete[] data_Buffer_Bodysize_Mtg_2D;
	}//end if
}

void Simul_Proc::Get_Bin_Number_BodyLength(int *p_bodyLength_Bin, double *p_Current_BodyLength, double *p_Max_BodyLength, double *p_Min_BodyLength, int *p_Num_Bin)
{
	//bodylength bin width
	double bin_Width = ((*p_Max_BodyLength) - (*p_Min_BodyLength)) / ((double)(*p_Num_Bin));
	//current body length bin
	(*p_bodyLength_Bin) = (int)(((*p_Current_BodyLength) - (*p_Min_BodyLength)) / bin_Width);
}

void Simul_Proc::Set_Bin_Width_BodyLength(double *p_BodyLength_Max)
{
	//sets the bin width for bodylength
	int spc;
	double bodyLength_Min = 0.0;

	//std::cout << std::endl << std::endl << "Simul_Proc::Set_Bin_Width_BodyLength" << std::endl << std::endl;

	for (spc = 0; spc < num_Spc; spc++)
	{
		bin_Width_BodyLength[spc] = (p_BodyLength_Max[spc] - bodyLength_Min) / ((double)num_Bin_Bodylength[spc]);
		//std::cout << "Bodylength bin width Species " << spc << ": " << bin_Width_BodyLength[spc] << std::endl;
		//char tt;
		//std::cin >> tt;
	}//end species loop
}

void Simul_Proc::Test_ESP_Discrete(void)
{
	//The magnitude of resource covered by the upper-limit trait (the maximum resource size for the upper-limit trait / the minimum resource size for the upper-limit trait)
	for (int spc = 0; spc < num_Spc; spc++){
		//the number of bins for bodylength
		int end0 = num_Bin_Bodylength[spc];//bodylength bin
		int end1 = num_Clas_Trt_Spc[spc][1];//gillraker genotype

		for (int bin_BodyLength = 0; bin_BodyLength < end0; bin_BodyLength++)
		{
			for (int trt1 = 0; trt1 < end1; trt1++)
			{
				Get_ESP_Discrete(esp_Discrete, &spc, &bin_BodyLength, &trt1);

				//verification
				for (int rsc = 0; rsc < num_RscSpc; rsc++)
				{
					for (int cls = 0; cls < max_Num_PhenTyp_Rsc; cls++)
					{
						std::cout << " rsc " << rsc << " cls " << cls << std::endl;
						std::cout << "ESP Discrete: " << esp_Discrete[rsc][cls] << std::endl;
					}//end resource size class loop
				}//end resource cluster loop

				std::cout << "Species " << spc << " bodylength " << bin_BodyLength << " gillRaker " << trt1 << std::endl;
				char tt;
				std::cin >> tt;
			}//lower-foraging bound loop
		}//end bodylength bin loop		
	}//end species loop
}

void Simul_Proc::Get_BodyLength_From_Bin_Number(double *p_BodyLength, int *p_Bin_Number, int *p_Species)
{//returns bodylength from a bin number

	//bodylength
	(*p_BodyLength) = ((double)(*p_Bin_Number)) * bin_Width_BodyLength[(*p_Species)];
}

void Simul_Proc::Get_BodyMass_From_Bin_Number(double *p_BodyMass, int *p_Bin_Number, int *p_Species)
{//returns bodylength from a bin number
	//the maximum bodylength

	double tmp_BodyLength;
	//calculates bodylength
	Get_BodyLength_From_Bin_Number(&tmp_BodyLength, p_Bin_Number, p_Species);
	//calculates bodymass
	(*p_BodyMass) = (scalingFactor_BodyLength_BodyMass_Conversion[(*p_Species)] * (pow(tmp_BodyLength, exponent_BodyLength_BodyMass_Conversion[(*p_Species)])));

	//std::cout << "bodylength " << tmp_BodyLength<<std::endl;
}

void Simul_Proc::Get_GillRaker_Value_From_BodyLength(double *p_GillRaker_Value, double *p_BodyLength, int *p_Num_Allele_GillRaker, int *p_Species, double *p_Exponent_Lower_Bound)
{//returns gillraker value from genetic information and bodylength
	//the maximum bodylength
	//double maximum_BodyLength = num_Bin_Bodylength[(*p_Species)] * bin_Width_BodyLength[(*p_Species)];
	//degree of development in bodylength
	//double degree_Development_BodyLength = (*p_BodyLength) / maximum_BodyLength;
	//genetic potential of development in gillraker
	//double degree_Potential_GillRaker = (*p_Num_Allele_GillRaker) / ((double)(num_Clas_Trt_Spc[(*p_Species)][1] - 1));

	//the difference between the minimum and the maximum
	//resource size covered by the gillraker trait
	double lbd = (lower_Bound_Max[(*p_Species)] - lower_Bound_Min[(*p_Species)]);

	//genetic effect
	double tmp_Effect_Allele = (1.0 - ((*p_Num_Allele_GillRaker) / ((double)(num_Clas_Trt_Spc[(*p_Species)][1] - 1))));
	//genetif effect on lower-foraging bound
	//tmp_Effect_Allele = pow(tmp_Effect_Allele, (1.0 - (*p_Exponent_Lower_Bound)));

	//bodysize effect
	//double bodyLength_Max = (num_Bin_Bodylength[(*p_Species)] * bin_Width_BodyLength[(*p_Species)]);
	//body length devidend

	//double tmp_ScalingFactor = 5.0;
	//double tmp_Effect_BodyLength = ((*p_BodyLength) + ((*p_Exponent_Lower_Bound) * bodyLength_Max)) / ((1.0 + (*p_Exponent_Lower_Bound)) * bodyLength_Max);
	//double tmp_Effect_BodyLength = (((*p_BodyLength) ) / ((num_Bin_Bodylength[(*p_Species)] * bin_Width_BodyLength[(*p_Species)]) - (*p_Ltb_Minimum)));

	//bodymass
	double tmp_BodyMass;

	//get bodymass
	Get_BodyMass_From_BodyLength(&tmp_BodyMass, p_BodyLength, p_Species);

	//calculate bodymass difference
	tmp_BodyMass = tmp_BodyMass - bodyMass_Min[(*p_Species)];

	//prevent a negative value
	if (tmp_BodyMass < 0.0)
	{
		tmp_BodyMass = 0.0;
	}//end if

	//the degree of relative growth in bodymass
	double tmp_Effect_BodyMass = tmp_BodyMass / (bodyMass_Max[(*p_Species)] - bodyMass_Min[(*p_Species)]);

	//bodysize effect on lower-foraging-bound
	if (tmp_Effect_BodyMass > 0.0){
		tmp_Effect_BodyMass = pow(tmp_Effect_BodyMass, (*p_Exponent_Lower_Bound));
	}
	else{
		if ((*p_Exponent_Lower_Bound) > 0.0){
			tmp_Effect_BodyMass = 0.0;
		}
		else{
			tmp_Effect_BodyMass = 1.0;
		}//end else
	}//end if
	

	//candidate function
	//double tmp_Effect_BodyMass = (tmp_BodyMass + ((*p_Exponent_Lower_Bound) * (bodyMass_Max[(*p_Species)] - bodyMass_Min[(*p_Species)]))) / ((1.0 + (*p_Exponent_Lower_Bound)) * (bodyMass_Max[(*p_Species)] - bodyMass_Min[(*p_Species)]));
	//double tmp_Effect_BodyLength = (tmp_BodyMass + ((*p_Exponent_Lower_Bound) * (bodyMass_Max[(*p_Species)] - bodyMass_Min[(*p_Species)]))) / ((1.0 + (*p_Exponent_Lower_Bound)) * (bodyMass_Max[(*p_Species)] - bodyMass_Min[(*p_Species)]));

	//if (tmp_Effect_BodyLength > 0.0)
	//{
	//tmp_Effect_BodyLength = pow(tmp_Effect_BodyLength, 2.0);
	//}
	//else
	//{
	//tmp_Effect_BodyLength = 0.0;
	//}

	//old: gillraker value indicating resource size
	//(*p_GillRaker_Value) = lower_Bound_Min[(*p_Species)] + lbd * (pow((((pow(lbd, exponent_a)) - 1.0) / (lbd - 1.0)), (*p_Exponent_Lower_Bound)));

	(*p_GillRaker_Value) = lower_Bound_Min[(*p_Species)] + (lbd * tmp_Effect_Allele * tmp_Effect_BodyMass); 
	
	//std::cout << "Line 9748 bodymass " << tmp_BodyMass;
	//std::cout << "Max diff " << (bodyMass_Max[(*p_Species)] - bodyMass_Min[(*p_Species)]) << std::endl;
	//std::cout << "bodymass max " << bodyMass_Max[(*p_Species)] << std::endl;
	//std::cout << "bodymass min " << bodyMass_Min[(*p_Species)] << std::endl;
	//std::cout << "modification factor " << (*p_Exponent_Lower_Bound) << std::endl;
	//std::cout << "effect body length " << tmp_Effect_BodyMass << std::endl;
	//std::cout << "lower bound max " << lower_Bound_Max[(*p_Species)] << std::endl;
	//std::cout << "lower bound min " << lower_Bound_Min[(*p_Species)] << std::endl;
	//std::cout << "lower bound diff " << lbd << std::endl;
	//std::cout << "genetic effect " << tmp_Effect_Allele << std::endl;
	//std::cout << "Gillraker value " << (*p_GillRaker_Value) << std::endl;
	//char tt;
	//std::cin >> tt;
}

void Simul_Proc::Get_Upper_Bound_Value_From_BodyLength(double *p_Upper_Bound_Value, double *p_BodyLength, int *p_Species, double *p_Exponent_Upper_Bound)
{
	//the maximum bodylength
	//double maximum_BodyLength = num_Bin_Bodylength[(*p_Species)] * bin_Width_BodyLength[(*p_Species)];
	//degree of development in bodylength
	//double degree_Development_BodyLength = (*p_BodyLength) / maximum_BodyLength;
	//the difference between the maximum and the minimum resource size 
	//covered by the upper-bount trait (gape size)
	//double diff = upper_Bound_Max[(*p_Species)] - upper_Bound_Min[(*p_Species)];

	//upper-bound trait value (resource size at the upper-foraging bound)
	//(*p_Upper_Bound_Value) = upper_Bound_Min[(*p_Species)] + diff * (pow(degree_Development_BodyLength, (*p_Exponent_Upper_Bound)));

	//upper-bound trait value (resource size at the upper-foraging bound)
	//(*p_Upper_Bound_Value) = upper_Bound_Min[(*p_Species)] + (upper_Bound_Max[(*p_Species)] - upper_Bound_Min[(*p_Species)]) * (pow((((*p_BodyLength) - (*p_Ltb_Minimum)) / ((num_Bin_Bodylength[(*p_Species)] * bin_Width_BodyLength[(*p_Species)]) - (*p_Ltb_Minimum))), (*p_Exponent_Upper_Bound)));

	double tmp_BodyMass;
	Get_BodyMass_From_BodyLength(&tmp_BodyMass, p_BodyLength, p_Species);

	//upper-bound trait value (resource size at the upper-foraging bound)
	(*p_Upper_Bound_Value) = 0.01 * tmp_BodyMass;
}

void Simul_Proc::Get_ESP_Discrete(double **p_Esp_Discrete, int *p_Spc, int *p_Bin_BodyLength, int *p_GillRaker_Genotype)
{
	//double Ltb_Minimum;//the minimum body length at age 0
	double tmp_BodyLength, upper_Bound_Tmp, lower_Bound_Tmp, esp_Left_Bin, esp_Right_Bin;
	int rsc, cls, end0;

	//core species
	int spc = (*p_Spc);

	//returns bodylength from a bin number
	Get_BodyLength_From_Bin_Number(&tmp_BodyLength, p_Bin_BodyLength, &spc);

	//returns upper-bound-foraging trait value from bodylength
	//upper-bound-foraging trait value: max_Size_Spc_Tmp
	Get_Upper_Bound_Value_From_BodyLength(&upper_Bound_Tmp, &tmp_BodyLength, &spc, &exponent_Upper_Bound[spc]);

	//returns gillraker value from genetic information and bodylength
	//lower-foraging bound trait value: min_Size_Spc_Tmp
	Get_GillRaker_Value_From_BodyLength(&lower_Bound_Tmp, &tmp_BodyLength, p_GillRaker_Genotype, &spc, &exponent_Lower_Bound[spc]);

	if (upper_Bound_Tmp > lower_Bound_Tmp)
	{
		for (rsc = 0; rsc < num_RscSpc; rsc++)
		{
			end0 = num_PhenoTyp_Rsc[rsc];

			for (cls = 0; cls < end0; cls++)
			{
				//calculates the ESP at the left end of a bin (esp_Left)
				ESP_Sigmoid_Func(&esp_Left_Bin, &bin_Rsc_BioMass[rsc][cls][0], &upper_Bound_Tmp, &lower_Bound_Tmp, &k_Hunt_Bound[spc], &delta_Hunt_Bound[spc]);
				//calculates the ESP at the right end of a bin (esp_Right)
				ESP_Sigmoid_Func(&esp_Right_Bin, &bin_Rsc_BioMass[rsc][cls][1], &upper_Bound_Tmp, &lower_Bound_Tmp, &k_Hunt_Bound[spc], &delta_Hunt_Bound[spc]);
				p_Esp_Discrete[rsc][cls] = (esp_Left_Bin + esp_Right_Bin) / 2.0;
			}//end resource species phenotype loop
		}//end resource species loop
	}
	else
	{
		//foraging efficiency is 0
		for (rsc = 0; rsc < num_RscSpc; rsc++)
		{
			end0 = num_PhenoTyp_Rsc[rsc];

			for (cls = 0; cls < end0; cls++)
			{
				p_Esp_Discrete[rsc][cls] = 0.0;
			}//end resource species phenotype loop
		}//end resource species loop
	}//end else

	//verification
	//std::cout << "Line 9217" << std::endl;
	//std::cout << "Bodylength " << tmp_BodyLength << std::endl;
	//std::cout << "Upperbound " << upper_Bound_Tmp << std::endl;
	//std::cout << "lowerbound " << lower_Bound_Tmp << std::endl;//p_GillRaker_Genotype
	//std::cout << "Body length bin " << (*p_Bin_BodyLength) << std::endl;//p_GillRaker_Genotype
	//std::cout << "Gillraker genotype " << (*p_GillRaker_Genotype) << std::endl;//p_GillRaker_Genotype
	//std::cout << " steepness: " << k_Hunt_Bound[spc] << std::endl;
	//std::cout << "Generalist penalty: " << delta_Hunt_Bound[spc] << std::endl;
	//char tt;
	//std::cin >> tt;

	//for (rsc = 0; rsc < num_RscSpc; rsc++)
	//{
	//end0 = num_PhenoTyp_Rsc[rsc];
	//for (cls = 0; cls < end0; cls++)
	//{
	//std::cout << "rsc " << rsc << " class " << cls << " resource mass right " << bin_Rsc_BioMass[rsc][cls][0] << std::endl;
	//std::cout << "rsc " << rsc << " class " << cls << " resource mass left " << bin_Rsc_BioMass[rsc][cls][1] << std::endl;
	//std::cout << "rsc " << rsc << " class " << cls << " foraging efficiency " << p_Esp_Discrete[rsc][cls] << std::endl;
	//}//end resource species phenotype loop
	//}//end resource species loop

	//Check_In("Simul_Proc::Get_ESP_Discrete()");
}

void Simul_Proc::Surv_Grw_Trait_Rsc_Discrete(void)
{
	int deme, niche, spc, sex, stg, indiv, tmp_Stg, numStg, trt0, trt1, end0, end1, end2;
	int tmp_Tot_Num_Allele_BodyLength;
	double tmp_Num_Allele_BodyLength;
	double ranDev, tmp_Density, tmp_Sur, tmp_Dev, tmp_BodyLength, tmp_MetabolicRate_Biomass_Factor, t_CD;
	double tmp_Metabolic_Rate, tmp_BinWidth, tmp_Gd_a, tmp_Gd_b, tmp_Exponent_MetabilicRate_BodyMass;//probability of early (immature) sexual maturation
	double tmp_EDD, tmp_DDS, tmp_DDG, tmp_ltg_VB_Sx, tmp_Exponent_VonBertallanffy_BodyLength, tmp_SxIniB;
	bool status_Sexual_Maturation;
	double PG[2];

	//von Bertallanffy function
	double tmp_Ltg;
	double tmp_Ce;
	double tmp_Ci;
	double tmp_VBC, tmp_VBK;

	//time step
	double tmp_TimeStep = unit_T_Step;

	//age
	double tmp_Age;

	//stage
	double tmp_Tot_Num_Stg_Double;

	//the maximum asymptotic bodylength
	double tmp_Max_Asymptotic_Bodylength;
	//the minimum body length
	double tmp_Min_Bodylength;
	//the number of allele controling bodylength
	double tmp_Tot_Num_Allele_BodyLength_Double;
	//the number of allele controling bodylength
	double tmp_BodyLength_Growth;
	//mean bodylength growth
	double tmp_Mu_BodyLength_growth;
	//coefficient variation bodylength growth
	double tmp_CoefficientVariation_BodyLength_Growth;
	//standard deviation in body length growth
	double tmp_StandardDeviation_BodyLength_Growth;

	//log-normal distribution
	double tmp_M;
	//log-normal distribution
	double tmp_V;

	//Sexual maturation initiation
	int tmp_First_Age_Reproduction_Initiation, tmp_Last_Age_Reproduction_Initiation;
	double tmp_Inverse_Immature_Reprodive_Age = 1.0;
	double tmp_Tot_Num_Stg_Devidend;

	//Minimum fraction for producing random deviate with positive value
	double tmp_Ltrm = ltrm;

	//baseline bodylength growth
	double tmp_Baseline_Growth_BodyLength;

	//bodylength calculation
	//the total number of life stages for calculating bodylength
	Agent* tmp_ID;

	if (tick_t == true)
	{
		for (deme = 0; deme < num_Deme; deme++)
		{
			for (niche = 0; niche < num_Niche; niche++)
			{
				Local_Aggregate_Prc_Discrete(&deme, &niche);//Local-level Aggregate Process

				//verification
				//Check_In("Local_Aggregate_Prc_Discrete done");

				for (spc = 0; spc < num_Spc; spc++)
				{
					//von Bertallanffy function
					tmp_Ltg = ltg_VB[spc];
					//tmp_Ce = ce_VB[spc];
					tmp_Ci = ci_VB[spc];

					//the maximum asymptotic bodylength
					tmp_Max_Asymptotic_Bodylength = bodyLength_Max_Spc[spc];
					//the minimum body length 
					tmp_Min_Bodylength = bodyLength_Min_Spc[spc];
					//sd bodylength growth
					tmp_CoefficientVariation_BodyLength_Growth = cv_BodyLength_Growth[spc];

					//the number of bins for body length for each species
					end0 = num_Bin_Bodylength[spc];
					//the number of alleles determining bodylength
					tmp_Tot_Num_Allele_BodyLength = (trt_Loc_Int[spc][0] * num_Position[spc]);
					//double
					tmp_Tot_Num_Allele_BodyLength_Double = (double)tmp_Tot_Num_Allele_BodyLength;
					//end number: gillraker genotype loop
					end1 = num_Clas_Trt_Spc[spc][1];
					//exponent for bodymass metabolic rate conversion
					tmp_MetabolicRate_Biomass_Factor = metabolicRate_Biomass_Factor[spc];
					//bodylength bin width
					tmp_BinWidth = bin_Width_BodyLength[spc];
					//for bodylength-bodymass conversion
					tmp_Gd_a = scalingFactor_BodyLength_BodyMass_Conversion[spc];
					//for bodylength-bodymass conversion
					tmp_Gd_b = exponent_BodyLength_BodyMass_Conversion[spc];
					//for bioMass-Metabolic rate conversion
					tmp_Exponent_MetabilicRate_BodyMass = metabolicRate_Biomass_Factor[spc];
					//scaling factor for the effect of available resource mass 
					//on the development of bodylength
					tmp_DDG = ddg[spc];
					//the range of variance in the degree of development in bodylength per time step
					//tmp_Range_Degree_Development_BodyLength = lku[spc] - lkl[spc];
					//the minimum degree of development in bodylength per time step
					//tmp_Min_Degree_Development_BodyLength = lkl[spc];
					//the exponent for controlling the effect of 
					//the available resource mass on the initiation of sexual activities
					tmp_ltg_VB_Sx = ltg_VB_Sx[spc];

					for (sex = 0; sex < num_Sex; sex++)
					{
						//for bodylength calculation
						tmp_Tot_Num_Stg_Devidend = num_Stg_BodyLength_Growth[spc][sex];

						//Sexual maturation initiation
						tmp_First_Age_Reproduction_Initiation = (num_Stg[spc][sex] - num_Immat_Stg_SexMaturation[spc][sex] - 1);
						tmp_Last_Age_Reproduction_Initiation = (num_Stg[spc][sex] - 1);

						if (num_Immat_Stg_SexMaturation[spc][sex] > 0)
						{
							tmp_Inverse_Immature_Reprodive_Age = 1.0 / ((double)num_Immat_Stg_SexMaturation[spc][sex]);
						}//end if

						//The exponent for von Bertallanffy function
						//dim[species, sex];
						tmp_Exponent_VonBertallanffy_BodyLength = exponent_VonBertallanffy[spc][sex];
						//the number of life stages
						numStg = num_Stg[spc][sex];
						tmp_Tot_Num_Stg_Double = (double)numStg;

						vector<vector<Agent*>> &tmp_Agt_Source = agtLst_0[((deme * num_Niche) + niche)][spc][sex];
						vector<vector<Agent*>> &tmp_Agt_Sink = agtLst_1[((deme * num_Niche) + niche)][spc][sex];

						for (stg = 0; stg < numStg; stg++)
						{
							if (celNum_0[deme][niche][spc][sex][stg] > 0)
							{//if there is an agent
								//chance of survival at the current life stage
								tmp_Sur = P_G[deme][niche][spc][sex][stg][0];
								//chance of development and survival
								tmp_Dev = P_G[deme][niche][spc][sex][stg][1];
								//scaling factor for density dependent survival
								t_CD = Cd[deme][niche][spc][sex][stg];

								int *tmp_CelNum = &celNum_1[deme][niche][spc][sex][stg];//current cell number
								int *tmp_Size_Agt = &size_AgtLst[deme][niche][spc][sex][stg];//available cell size

								if (stg < (numStg - 1))
								{//Immature life stages
									tmp_Stg = stg + 1;
								}
								else
								{//Adult life stage
									tmp_Stg = stg;
								}//end else

								int *tmp_CelNum_Growth = &celNum_1[deme][niche][spc][sex][tmp_Stg];//growth cell number
								int *tmp_Size_Agt_Growth = &size_AgtLst[deme][niche][spc][sex][tmp_Stg];//available cell size

								//sorts individuals into phenotypes by using vitaList
								Set_VitaList_Discrete(deme, niche, spc, sex, stg);

								//verification
								//Check_In("Set_VitaList_Discrete() : true");

								for (trt0 = 0; trt0 < end0; trt0++)
								{
									for (trt1 = 0; trt1 < end1; trt1++)
									{
										if (celNum_VitaList_Discrete[trt0][trt1] > 0)
										{
											end2 = celNum_VitaList_Discrete[trt0][trt1];
											//std::cout << "the number of agents " << end2 << std::endl;

											//temporary vital list for discrete traits
											vector<Agent*> &tmp_VitaList = vitaList_Discrete[trt0][trt1];

											//gets esp_Discrete (foraging efficiency)
											Get_ESP_Discrete(esp_Discrete, &spc, &trt0, &trt1);

											//gets frsp (functional response)
											Get_FunctionalResponse_Esp(frsp, esp_Discrete, phen_D_Rsc[deme][niche], deme, niche, spc);

											//gets foraging time
											Get_Foraging_Time_Functional_Response(ft_Discrete, frsp, deme, niche, spc);

											//gets the total available resource mass for an individual
											Get_Available_Resource_Mass_Individual_Discrete(&tmp_Density, frsp, ft_Discrete, phen_D_Rsc[deme][niche], sum_Cmt_Rsc_Spc[deme][niche], spc);

											//gets the metabolic rate
											Get_Basic_Metabolic_Rate_From_Bin_BodyLength(&tmp_Metabolic_Rate, &tmp_BinWidth, &trt0, &tmp_Gd_a, &tmp_Gd_b, &tmp_Exponent_MetabilicRate_BodyMass);

											//gets edd (tmp_EDD)
											//scaling factor for the effect of available resource mass 
											//on the development of bodylength
											Get_EDD(&tmp_EDD, &tmp_Density, &tmp_Metabolic_Rate, &tmp_DDG, &t_CD);

											//std::cout<<" sex "<< sex << "stg " << stg << " trt0 " << trt0 << " trt1 " << trt1 << std::endl;
											//std::cout << "metabolic rate " << std::setprecision(5) << tmp_Metabolic_Rate << std::endl;
											//std::cout << "edd " << std::setprecision(5) << tmp_EDD << std::endl;
											//std::cout << "density " << std::setprecision(5) << tmp_Density << std::endl;
											//std::cout << "metabolic rate " << std::setprecision(5) << tmp_Metabolic_Rate << std::endl;
											//std::cout << "edd " << std::setprecision(5) << tmp_EDD << std::endl;
											//std::cout << "density " << std::setprecision(5) << tmp_Density << std::endl;
											//Check_In("Line 14854: Simul_Proc::Surv_Grw_Trait_Rsc_Discrete()");

											//returns the degree of development in body length (tmp_LKE)
											//Get_LKE(&tmp_LKE, &tmp_EDD, &tmp_Range_Degree_Development_BodyLength, &tmp_Min_Degree_Development_BodyLength);

											for (indiv = 0; indiv < end2; indiv++)
											{
												//ID setup
												tmp_ID = tmp_VitaList[indiv];
												//get previous bodylength
												tmp_ID->Get_BodyLength(&tmp_BodyLength);
												//get age
												tmp_ID->Get_Age(&tmp_Age);
												//get the effect of environment on bodylength growth
												tmp_ID->Get_CE(&tmp_Ce);
												//get the number of allele controlling bodylength
												tmp_ID->Get_Num_Allele_BodyLength_Double(&tmp_Num_Allele_BodyLength);
												//calculates allele dose (ltg) controlling bodylength
												tmp_Num_Allele_BodyLength = tmp_Num_Allele_BodyLength / tmp_Tot_Num_Allele_BodyLength_Double;

												//calculates VBK
												Get_VBK(&tmp_VBK, &tmp_Exponent_VonBertallanffy_BodyLength, &tmp_Age, &tmp_Tot_Num_Stg_Devidend, &tmp_TimeStep);

												//calculates baseline bodylength growth (tmp_Baseline_Growth_BodyLength)
												Get_Baseline_Growth_BodyLength(&tmp_Baseline_Growth_BodyLength, &tmp_Max_Asymptotic_Bodylength, &tmp_Ltg, &tmp_VBK);

												//calculates VBC
												Get_VBC(&tmp_VBC, &tmp_Num_Allele_BodyLength, &tmp_EDD, &tmp_Ltrm, &tmp_Ltg, &tmp_Ce, &tmp_Ci);

												//calculate mean bodylength growth
												Get_BodyLength_Von_Bertallanffy(&tmp_Mu_BodyLength_growth, &tmp_Max_Asymptotic_Bodylength, &tmp_VBC, &tmp_VBK);

												tmp_StandardDeviation_BodyLength_Growth = tmp_Mu_BodyLength_growth * tmp_CoefficientVariation_BodyLength_Growth;

												//converts mu and sd for normal distribution to m and v for log-normal distribution
												Convert_Mu_SD_NormDist_To_M_V_LogNormDist(&tmp_M, &tmp_V, &tmp_Mu_BodyLength_growth, &tmp_StandardDeviation_BodyLength_Growth);

												//draws a random deviate for growth
												ranDev = 0.0;
												//gets the value between 0.0015 & 0.9985 (3 * sigma)
												while ((ranDev < QUANTILE_FOUR_SIGMA_LEFT) || (ranDev > QUANTILE_FOUR_SIGMA_RIGHT))
												{
													ranDev = (*rand_Uni_Real_Mtg)();
												}//end while

												//calculates random bodylength growth
												Get_Random_Deviate_Log_Normal_Dist(&tmp_BodyLength_Growth, &ranDev, &tmp_M, &tmp_V);

												//calculate bodylength
												tmp_BodyLength += (tmp_Baseline_Growth_BodyLength + tmp_BodyLength_Growth);

												//metabolic rate (tmp_Metabolic_Rate) from bodylength
												Get_Metabolic_Rate_From_BodyLength(&tmp_Metabolic_Rate, &tmp_BodyLength, &tmp_Gd_a, &tmp_Gd_b, &tmp_Exponent_MetabilicRate_BodyMass);

												//density dependent survival rate (tmp_DDS) from metabolic rate 
												Get_DDS_MetabolicRate_Rci(&tmp_DDS, &t_CD, &tmp_Density, &tmp_Metabolic_Rate);

												//calculates vital rates 
												//i.e. survival & developmental rates
												//survival rate
												PG[0] = tmp_Sur * tmp_DDS;
												//growth rate
												PG[1] = tmp_Dev * tmp_DDS;

												//std::cout << "metabolic rate " << std::setprecision(5) << tmp_Metabolic_Rate << std::endl;
												//std::cout << "edd " << std::setprecision(5) << tmp_EDD << std::endl;
												//std::cout << "VBC " << std::setprecision(5) << tmp_VBC << std::endl;
												//std::cout << "VBK " << std::setprecision(5) << tmp_VBK << std::endl;
												//std::cout << "Max Bodylength " << std::setprecision(5) << tmp_Max_Asymptotic_Bodylength << std::endl;
												//std::cout << "Mean Bodylength " << std::setprecision(5) << tmp_Mu_BodyLength_growth << std::endl;
												//std::cout << "CV Bodylength " << std::setprecision(5) << tmp_CoefficientVariation_BodyLength_Growth << std::endl;
												//std::cout << "SD Bodylength " << std::setprecision(5) << tmp_StandardDeviation_BodyLength_Growth << std::endl;
												//std::cout << "M " << std::setprecision(5) << tmp_M << std::endl;
												//std::cout << "V " << std::setprecision(5) << tmp_V << std::endl;
												//std::cout << "Bodylength " << std::setprecision(5) << tmp_BodyLength << std::endl;
												//std::cout << "dds " << std::setprecision(5) << tmp_DDS << std::endl;
												//std::cout << "PG0 " << std::setprecision(5) << PG[0] << std::endl;
												//std::cout << "PG1 " << std::setprecision(5) << PG[1] << std::endl;
												//std::cout << "randev " << std::setprecision(7) << ranDev << std::endl;												
												//Check_In("Line 14854: Simul_Proc::Surv_Grw_Trait_Rsc_Discrete()");

												//draws a random deviate for a vital process
												ranDev = (*rand_Uni_Real)();//draws a uniform random deviate for survival

												//vitality process
												if (ranDev >= PG[1])
												{//agent dies
													//removes an agent
													Remove_Agent(&deme, &niche, &spc, tmp_VitaList[indiv], &celNum_M[spc]);
													//std::cout << "Agent dies" << std::endl;
												}
												else
												{//agent survives
													//sets new bodylength
													tmp_ID->Set_BodyLength(&tmp_BodyLength);
													//increase age
													tmp_ID->Increase_Age(&tmp_TimeStep);

													if (ranDev < PG[0])
													{//an agent survives and stays at the same stage

														Transfer_Agent_Unit(tmp_Agt_Sink[stg], tmp_Agt_Source[stg], tmp_VitaList[indiv], tmp_CelNum, tmp_Size_Agt);
														//std::cout << "Agent survives" << std::endl;
													}
													else
													{//an agent survives and grows to the next stage
														Transfer_Agent_Unit(tmp_Agt_Sink[tmp_Stg], tmp_Agt_Source[tmp_Stg], tmp_VitaList[indiv], tmp_CelNum_Growth, tmp_Size_Agt_Growth);
														//std::cout << "Agent develops" << std::endl;
													}//end else

													//checks the sexual maturation status
													tmp_ID->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

													//std::cout << "sexual maturation " << status_Sexual_Maturation  << std::endl;
													//Check_In("Sexual maturation");

													//if else for sexual maturation
													if (status_Sexual_Maturation == false)
													{//if the individual is sexually not matured
														//determines the initiation of sexual activities
														Get_SxIniB(tmp_ID, &tmp_SxIniB, &tmp_ltg_VB_Sx, &tmp_EDD, &tmp_Ce, &tmp_Ci, &tmp_Num_Allele_BodyLength, &tmp_Stg, &tmp_First_Age_Reproduction_Initiation, &tmp_Last_Age_Reproduction_Initiation, &tmp_Inverse_Immature_Reprodive_Age);
													}//end if
												}//end else
											}//end individual loop
										}//end if	
									}//end gillraker trait
								}//end upper trait loop
							}//end if							
						}//end stage loop
					}//end sex loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}
	else
	{
		for (deme = 0; deme < num_Deme; deme++)
		{
			for (niche = 0; niche < num_Niche; niche++)
			{
				Local_Aggregate_Prc_Discrete(&deme, &niche);//Local-level Aggregate Process

				//verification
				//Check_In("Local_Aggregate_Prc_Discrete done");

				for (spc = 0; spc < num_Spc; spc++)
				{
					//von Bertallanffy function
					tmp_Ltg = ltg_VB[spc];
					//tmp_Ce = ce_VB[spc];
					tmp_Ci = ci_VB[spc];

					//the maximum asymptotic bodylength
					tmp_Max_Asymptotic_Bodylength = bodyLength_Max_Spc[spc];
					//the minimum body length 
					tmp_Min_Bodylength = bodyLength_Min_Spc[spc];
					//sd bodylength growth
					tmp_CoefficientVariation_BodyLength_Growth = cv_BodyLength_Growth[spc];

					//the number of bins for body length for each species
					end0 = num_Bin_Bodylength[spc];
					//the number of alleles determining bodylength
					tmp_Tot_Num_Allele_BodyLength = (trt_Loc_Int[spc][0] * num_Position[spc]);
					//double
					tmp_Tot_Num_Allele_BodyLength_Double = (double)tmp_Tot_Num_Allele_BodyLength;
					//end number: gillraker genotype loop
					end1 = num_Clas_Trt_Spc[spc][1];
					//exponent for bodymass metabolic rate conversion
					tmp_MetabolicRate_Biomass_Factor = metabolicRate_Biomass_Factor[spc];
					//bodylength bin width
					tmp_BinWidth = bin_Width_BodyLength[spc];
					//for bodylength-bodymass conversion
					tmp_Gd_a = scalingFactor_BodyLength_BodyMass_Conversion[spc];
					//for bodylength-bodymass conversion
					tmp_Gd_b = exponent_BodyLength_BodyMass_Conversion[spc];
					//for bioMass-Metabolic rate conversion
					tmp_Exponent_MetabilicRate_BodyMass = metabolicRate_Biomass_Factor[spc];
					//scaling factor for the effect of available resource mass 
					//on the development of bodylength
					tmp_DDG = ddg[spc];
					//the range of variance in the degree of development in bodylength per time step
					//tmp_Range_Degree_Development_BodyLength = lku[spc] - lkl[spc];
					//the minimum degree of development in bodylength per time step
					//tmp_Min_Degree_Development_BodyLength = lkl[spc];
					//the exponent for controlling the effect of 
					//the available resource mass on the initiation of sexual activities
					tmp_ltg_VB_Sx = ltg_VB_Sx[spc];

					for (sex = 0; sex < num_Sex; sex++)
					{
						//for bodylength calculation
						tmp_Tot_Num_Stg_Devidend = num_Stg_BodyLength_Growth[spc][sex];

						//Sexual maturation initiation
						tmp_First_Age_Reproduction_Initiation = (num_Stg[spc][sex] - num_Immat_Stg_SexMaturation[spc][sex] - 1);
						tmp_Last_Age_Reproduction_Initiation = (num_Stg[spc][sex] - 1);

						if (num_Immat_Stg_SexMaturation[spc][sex] > 0)
						{
							tmp_Inverse_Immature_Reprodive_Age = 1.0 / ((double)num_Immat_Stg_SexMaturation[spc][sex]);
						}//end if

						//The exponent for von Bertallanffy function
						//dim[species, sex];
						tmp_Exponent_VonBertallanffy_BodyLength = exponent_VonBertallanffy[spc][sex];
						//the number of life stages
						numStg = num_Stg[spc][sex];
						tmp_Tot_Num_Stg_Double = (double)numStg;

						vector<vector<Agent*>> &tmp_Agt_Source = agtLst_1[((deme * num_Niche) + niche)][spc][sex];
						vector<vector<Agent*>> &tmp_Agt_Sink = agtLst_0[((deme * num_Niche) + niche)][spc][sex];

						for (stg = 0; stg < numStg; stg++)
						{
							if (celNum_1[deme][niche][spc][sex][stg] > 0)
							{//if there is an agent
								//chance of survival at the current life stage
								tmp_Sur = P_G[deme][niche][spc][sex][stg][0];
								//chance of development and survival
								tmp_Dev = P_G[deme][niche][spc][sex][stg][1];
								//scaling factor for density dependent survival
								t_CD = Cd[deme][niche][spc][sex][stg];

								int *tmp_CelNum = &celNum_0[deme][niche][spc][sex][stg];//current cell number
								int *tmp_Size_Agt = &size_AgtLst[deme][niche][spc][sex][stg];//available cell size

								if (stg < (numStg - 1))
								{//Immature life stages
									tmp_Stg = stg + 1;
								}
								else
								{//Adult life stage
									tmp_Stg = stg;
								}//end else

								int *tmp_CelNum_Growth = &celNum_0[deme][niche][spc][sex][tmp_Stg];//growth cell number
								int *tmp_Size_Agt_Growth = &size_AgtLst[deme][niche][spc][sex][tmp_Stg];//available cell size

								//sorts individuals into phenotypes by using vitaList
								Set_VitaList_Discrete(deme, niche, spc, sex, stg);

								//verification
								//Check_In("Set_VitaList_Discrete() : true");

								for (trt0 = 0; trt0 < end0; trt0++)
								{
									for (trt1 = 0; trt1 < end1; trt1++)
									{
										if (celNum_VitaList_Discrete[trt0][trt1] > 0)
										{
											end2 = celNum_VitaList_Discrete[trt0][trt1];
											//std::cout << "the number of agents " << end2 << std::endl;

											//temporary vital list for discrete traits
											vector<Agent*> &tmp_VitaList = vitaList_Discrete[trt0][trt1];

											//gets esp_Discrete (foraging efficiency)
											Get_ESP_Discrete(esp_Discrete, &spc, &trt0, &trt1);

											//gets frsp (functional response)
											Get_FunctionalResponse_Esp(frsp, esp_Discrete, phen_D_Rsc[deme][niche], deme, niche, spc);

											//gets foraging time
											Get_Foraging_Time_Functional_Response(ft_Discrete, frsp, deme, niche, spc);

											//gets the total available resource mass for an individual
											Get_Available_Resource_Mass_Individual_Discrete(&tmp_Density, frsp, ft_Discrete, phen_D_Rsc[deme][niche], sum_Cmt_Rsc_Spc[deme][niche], spc);

											//gets the metabolic rate
											Get_Basic_Metabolic_Rate_From_Bin_BodyLength(&tmp_Metabolic_Rate, &tmp_BinWidth, &trt0, &tmp_Gd_a, &tmp_Gd_b, &tmp_Exponent_MetabilicRate_BodyMass);

											//gets edd (tmp_EDD)
											//scaling factor for the effect of available resource mass 
											//on the development of bodylength
											Get_EDD(&tmp_EDD, &tmp_Density, &tmp_Metabolic_Rate, &tmp_DDG, &t_CD);

											//returns the degree of development in body length (tmp_LKE)
											//Get_LKE(&tmp_LKE, &tmp_EDD, &tmp_Range_Degree_Development_BodyLength, &tmp_Min_Degree_Development_BodyLength);

											for (indiv = 0; indiv < end2; indiv++)
											{
												//ID setup
												tmp_ID = tmp_VitaList[indiv];
												//get previous bodylength
												tmp_ID->Get_BodyLength(&tmp_BodyLength);
												//get age
												tmp_ID->Get_Age(&tmp_Age);
												//get the effect of environment on bodylength growth
												tmp_ID->Get_CE(&tmp_Ce);
												//get the number of allele controlling bodylength
												tmp_ID->Get_Num_Allele_BodyLength_Double(&tmp_Num_Allele_BodyLength);
												//calculates allele dose (ltg) controlling bodylength
												tmp_Num_Allele_BodyLength = tmp_Num_Allele_BodyLength / tmp_Tot_Num_Allele_BodyLength_Double;

												//calculates VBK
												Get_VBK(&tmp_VBK, &tmp_Exponent_VonBertallanffy_BodyLength, &tmp_Age, &tmp_Tot_Num_Stg_Devidend, &tmp_TimeStep);

												//calculates baseline bodylength growth (tmp_Baseline_Growth_BodyLength)
												Get_Baseline_Growth_BodyLength(&tmp_Baseline_Growth_BodyLength, &tmp_Max_Asymptotic_Bodylength, &tmp_Ltg, &tmp_VBK);

												//calculates VBC
												Get_VBC(&tmp_VBC, &tmp_Num_Allele_BodyLength, &tmp_EDD, &tmp_Ltrm, &tmp_Ltg, &tmp_Ce, &tmp_Ci);

												//calculate mean bodylength growth
												Get_BodyLength_Von_Bertallanffy(&tmp_Mu_BodyLength_growth, &tmp_Max_Asymptotic_Bodylength, &tmp_VBC, &tmp_VBK);

												tmp_StandardDeviation_BodyLength_Growth = tmp_Mu_BodyLength_growth * tmp_CoefficientVariation_BodyLength_Growth;

												//converts mu and sd for normal distribution to m and v for log-normal distribution
												Convert_Mu_SD_NormDist_To_M_V_LogNormDist(&tmp_M, &tmp_V, &tmp_Mu_BodyLength_growth, &tmp_StandardDeviation_BodyLength_Growth);

												//draws a random deviate for growth
												ranDev = 0.0;
												//gets the value between 0.0015 & 0.9985 (3 * sigma)
												while ((ranDev < QUANTILE_FOUR_SIGMA_LEFT) || (ranDev > QUANTILE_FOUR_SIGMA_RIGHT))
												{
													ranDev = (*rand_Uni_Real_Mtg)();
												}//end while

												//calculates random bodylength growth
												Get_Random_Deviate_Log_Normal_Dist(&tmp_BodyLength_Growth, &ranDev, &tmp_M, &tmp_V);

												//calculate bodylength
												tmp_BodyLength += (tmp_Baseline_Growth_BodyLength + tmp_BodyLength_Growth);

												//metabolic rate (tmp_Metabolic_Rate) from bodylength
												Get_Metabolic_Rate_From_BodyLength(&tmp_Metabolic_Rate, &tmp_BodyLength, &tmp_Gd_a, &tmp_Gd_b, &tmp_Exponent_MetabilicRate_BodyMass);

												//density dependent survival rate (tmp_DDS) from metabolic rate 
												Get_DDS_MetabolicRate_Rci(&tmp_DDS, &t_CD, &tmp_Density, &tmp_Metabolic_Rate);

												//calculates vital rates 
												//i.e. survival & developmental rates
												//survival rate
												PG[0] = tmp_Sur * tmp_DDS;
												//growth rate
												PG[1] = tmp_Dev * tmp_DDS;

												//draws a random deviate for a vital process
												ranDev = (*rand_Uni_Real)();//draws a uniform random deviate for survival

												//std::cout << "Bodylength " << std::setprecision(5) << tmp_BodyLength << std::endl;
												//std::cout << "metabolic rate " << std::setprecision(5) << tmp_Metabolic_Rate << std::endl;
												//std::cout << "dds " << std::setprecision(5) << tmp_DDS << std::endl;
												//std::cout << "PG0 " << std::setprecision(5) << PG[0] << std::endl;
												//std::cout << "PG1 " << std::setprecision(5) << PG[1] << std::endl;
												//std::cout << "randev " << std::setprecision(7) << ranDev << std::endl;
												//Check_In("Line 14854: Simul_Proc::Surv_Grw_Trait_Rsc_Discrete()");

												//vitality process
												if (ranDev >= PG[1])
												{//agent dies
													//removes an agent
													Remove_Agent(&deme, &niche, &spc, tmp_VitaList[indiv], &celNum_M[spc]);
													//std::cout << "Agent dies" << std::endl;
												}
												else
												{//agent survives
													//sets new bodylength
													tmp_ID->Set_BodyLength(&tmp_BodyLength);
													//increase age
													tmp_ID->Increase_Age(&tmp_TimeStep);

													if (ranDev < PG[0])
													{//an agent survives and stays at the same stage

														Transfer_Agent_Unit(tmp_Agt_Sink[stg], tmp_Agt_Source[stg], tmp_VitaList[indiv], tmp_CelNum, tmp_Size_Agt);
														//std::cout << "Agent survives" << std::endl;
													}
													else
													{//an agent survives and grows to the next stage
														Transfer_Agent_Unit(tmp_Agt_Sink[tmp_Stg], tmp_Agt_Source[tmp_Stg], tmp_VitaList[indiv], tmp_CelNum_Growth, tmp_Size_Agt_Growth);
														//std::cout << "Agent develops" << std::endl;
													}//end else

													//checks the sexual maturation status
													tmp_ID->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

													//std::cout << "sexual maturation " << status_Sexual_Maturation  << std::endl;

													//if else for sexual maturation
													if (status_Sexual_Maturation == false)
													{//if the individual is sexually not matured
														//determines the initiation of sexual activities
														Get_SxIniB(tmp_ID, &tmp_SxIniB, &tmp_ltg_VB_Sx, &tmp_EDD, &tmp_Ce, &tmp_Ci, &tmp_Num_Allele_BodyLength, &tmp_Stg, &tmp_First_Age_Reproduction_Initiation, &tmp_Last_Age_Reproduction_Initiation, &tmp_Inverse_Immature_Reprodive_Age);
													}//end if
												}//end else
											}//end individual loop
										}//end if	
									}//end gillraker trait
								}//end upper trait loop
							}//end if							
						}//end stage loop
					}//end sex loop
				}//end species loop
			}//end niche loop
		}//end deme loop
	}//end else
}

void Simul_Proc::Set_VitaList_Discrete(int p_Deme, int p_Niche, int p_Spc, int p_Sex, int p_Stg)
{
	int deme_Niche = p_Deme * num_Niche + p_Niche;
	int i, trt_Upper, trt_Lower, end0, end1, tmp_Cel;
	//temprary agent data pointer
	Agent* tmp_ID;

	//initializes the counter for vitaList
	end0 = cls_UpperBound_Spc_Discrete;
	end1 = cls_LowerBound_Spc_Discrete;

	for (trt_Upper = 0; trt_Upper < end0; trt_Upper++){
		for (trt_Lower = 0; trt_Lower < end1; trt_Lower++){
			celNum_VitaList_Discrete[trt_Upper][trt_Lower] = 0;
		}//end lower trait loop
	}//end upper trait loop

	if (tick_t == true)
	{//for even sequences
		end0 = celNum_0[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg];
		vector<Agent*> &tmp_Agt = agtLst_0[deme_Niche][p_Spc][p_Sex][p_Stg];

		if (end0 > 0){
			for (i = 0; i < end0; i++){
				//gets the traits of an agent
				tmp_ID = tmp_Agt[i];
				//gets bodylength bin number
				tmp_ID->Get_Bin_Number_BodyLength_Foraging(&trt_Upper);
				//gets gillraker genotype
				tmp_ID->Get_GillRaker_Genotype(&trt_Lower);

				//gets the current cell number
				tmp_Cel = celNum_VitaList_Discrete[trt_Upper][trt_Lower];
				celNum_VitaList_Discrete[trt_Upper][trt_Lower]++;//increases the counter

				if (tmp_Cel < size_VitaList_Discrete[trt_Upper][trt_Lower])
				{//if there is an empty cell
					vitaList_Discrete[trt_Upper][trt_Lower][tmp_Cel] = tmp_ID;
				}
				else
				{//if there is no empty cell
					vitaList_Discrete[trt_Upper][trt_Lower].push_back(tmp_ID);
					size_VitaList_Discrete[trt_Upper][trt_Lower]++;
				}//end else
			}//end individual loop
		}//if there is a living agent
	}
	else
	{//for odd sequences
		end0 = celNum_1[p_Deme][p_Niche][p_Spc][p_Sex][p_Stg];
		vector<Agent*> &tmp_Agt = agtLst_1[deme_Niche][p_Spc][p_Sex][p_Stg];

		if (end0 > 0){
			for (i = 0; i < end0; i++){
				//gets the traits of an agent
				tmp_ID = tmp_Agt[i];
				//gets bodylength bin number
				tmp_ID->Get_Bin_Number_BodyLength_Foraging(&trt_Upper);
				//gets gillraker genotype
				tmp_ID->Get_GillRaker_Genotype(&trt_Lower);

				//gets the current cell number
				tmp_Cel = celNum_VitaList_Discrete[trt_Upper][trt_Lower];
				celNum_VitaList_Discrete[trt_Upper][trt_Lower]++;//increases the counter

				if (tmp_Cel < size_VitaList_Discrete[trt_Upper][trt_Lower])
				{//if there is an empty cell
					vitaList_Discrete[trt_Upper][trt_Lower][tmp_Cel] = tmp_ID;
				}
				else
				{//if there is no empty cell
					vitaList_Discrete[trt_Upper][trt_Lower].push_back(tmp_ID);
					size_VitaList_Discrete[trt_Upper][trt_Lower]++;
				}//end else
			}//end individual loop
		}//if there is a living agent
	}//end else

	//sets the counters for memory management regarding survival
	Set_Vita_Memory_Cnt_Discrete();

	//verification
	//std::cout << "sex " << p_Sex << " life stage " << p_Stg << std::endl;

	//for (i = 0; i < cls_UpperBound_Spc_Discrete; i++)
	//{
	//for (int j = 0; j < cls_LowerBound_Spc_Discrete; j++)
	//{
	//if (celNum_VitaList_Discrete[i][j] > 0)
	//{
	//std::cout << "trt0 " << i << " trt1 " << j << std::endl;
	//std::cout << "num agents " << celNum_VitaList_Discrete[i][j] << std::endl;
	//std::cout << "maximum num agents " << maxCelNum_VitaList_Discrete[i][j] << std::endl;
	//}//end if
	//}//end j
	//}//end i

	//Check_In("Line 15255: Set_VitaList_Discrete()");
}

void Simul_Proc::Get_Foraging_Time_Functional_Response(double **p_foraging_Time, double **p_Frsp, int p_Deme, int p_Niche, int p_Spc)
{
	//p_foraging_Time: the proportion of time for foraging a resource size class
	//dim[resource cluster, size class]
	double tmp_Normaization_Constant, tmp_Nutritional_Value;//normalization factor for foraging time

	//sets the normalization constant for calculating the fraction of foraging time for a class of a resourcespecies
	Get_ForagingTime_Normalization_Constant_Discrete(&tmp_Normaization_Constant, p_Frsp, p_Deme, p_Niche, p_Spc);

	int rsc, cls, end0, end1;
	int tmp_NumSpc = num_Spc;
	end0 = num_RscSpc;
	double tmp_Scaled_Resource_BodyMass = 0.0;

	if (tmp_Normaization_Constant > 0.0)
	{
		for (rsc = 0; rsc < end0; rsc++){
			end1 = num_PhenoTyp_Rsc[rsc];
			//nutritional value to a species
			tmp_Nutritional_Value = k_NutritionValue[p_Spc][(tmp_NumSpc + rsc)];

			for (cls = 0; cls < end1; cls++){
				//calculates the fraction of foraging time for each class of a resource species with the sum of total foraging time
				Get_Prop_ForagingTime_Discrete(&p_foraging_Time[rsc][cls], &tmp_Scaled_Resource_BodyMass, p_Frsp, &tmp_Normaization_Constant, &tmp_Nutritional_Value, &p_Deme, &p_Niche, &p_Spc, &rsc, &cls, &tmp_NumSpc);
			}//end 
		}//end resource loop
	}
	else
	{//if the normalization constant is 0.0
		for (rsc = 0; rsc < end0; rsc++){
			end1 = num_PhenoTyp_Rsc[rsc];

			for (cls = 0; cls < end1; cls++){
				p_foraging_Time[rsc][cls] = 0.0;
			}//end 
		}//end resource loop
	}//end else

	//verification
	//for (rsc = 0; rsc < end0; rsc++){
	//end1 = num_PhenoTyp_Rsc[rsc];

	//for (cls = 0; cls < end1; cls++){		
	//cout << "resource " << rsc << " class " << cls << " foraging time " << p_foraging_Time[rsc][cls] << std::endl;
	//}//end 
	//}//end resource loop

	//Check_In("Line 15307: Get_Foraging_Time_Functional_Response()");
}

void Simul_Proc::Get_Prop_ForagingTime_Discrete(double *p_Prop_ForagingTime, double *p_Scaled_Resource_Mass, double **p_FunctionalResponse, double *normalizationConstant, double *p_Nutritional_Value, int *p_Deme, int *p_Niche, int *p_Spc, int *p_Rsc, int *p_rCls, int *p_Num_Spc)
{
	if (mean_BioMass_Rsc[(*p_Rsc)][(*p_rCls)] > 0.0)
	{
		(*p_Scaled_Resource_Mass) = pow(mean_BioMass_Rsc[(*p_Rsc)][(*p_rCls)], exponent_Choice_Resource_BodyMass[(*p_Spc)]);
	}
	else
	{
		(*p_Scaled_Resource_Mass) = 0.0;
	}//end else

	(*p_Prop_ForagingTime) = (p_FunctionalResponse[(*p_Rsc)][(*p_rCls)] * (*p_Nutritional_Value) * (*p_Scaled_Resource_Mass) * prop_Density_Rsc[(*p_Deme)][(*p_Niche)][(*p_Rsc)][(*p_rCls)]) / (*normalizationConstant);
}

void Simul_Proc::Get_Available_Resource_Mass_Individual_Discrete(double *p_Available_Mass_For_Indiv, double **p_Frsp, double **p_Ft_Discrete, double **p_Density_Resource, double **p_Sum_Cmt_Rsc_Spc, int spc)
{
	//returns the total available mass for each agent
	//p_Available_Mass_For_Indiv: the total available mass for an individual
	//p_Frsp: functional response for a given trait combination at a niche
	//p_Ft_Discrete: the proportion of foraging time for a given trait combination at a niche
	//p_Density_Resource: the density of resource at a niche
	//p_Sum_Cmt_Rsc_Spc: the number of individuals competing for a resource size class
	//spc: core species

	int rsc, cls, end1;
	int end0 = num_RscSpc;
	double tmp, tmp_Nutritional_Value;

	//the total biomass consumed by an individual
	double tmp_Rsc_BioMass = 0.0;//initialization
	int tmp_Rsc;

	for (rsc = 0; rsc < end0; rsc++){
		end1 = num_PhenoTyp_Rsc[rsc];
		tmp_Rsc = rsc + num_Spc;
		tmp_Nutritional_Value = k_NutritionValue[spc][tmp_Rsc];

		for (cls = 0; cls < end1; cls++){
			tmp = p_Sum_Cmt_Rsc_Spc[rsc][cls];

			if (tmp > 0.0)
			{
				tmp_Rsc_BioMass += ((mean_BioMass_Rsc[rsc][cls] * p_Density_Resource[rsc][cls]) * tmp_Nutritional_Value * p_Frsp[rsc][cls] * p_Ft_Discrete[rsc][cls]) / tmp;
			}//end if
		}//end resource species phenotype loop
	}//end resource loop loop

	(*p_Available_Mass_For_Indiv) = tmp_Rsc_BioMass;//sets the total biomass consumed by an individual
}

void Simul_Proc::Get_Basic_Metabolic_Rate_From_Bin_BodyLength(double *p_Maximum_Metabolic_Rate, double *p_Bin_Width_BodyLength, int *p_Bin_Number_BodyLength, double *p_Gd_a, double *p_Gd_b, double *p_Exponent_MetabilicRate_BodyMass)
{
	//double bodyLength = ((double)(*p_Bin_Number_BodyLength)) * (*p_Bin_Width_BodyLength);
	//double bodyMass = (*p_Gd_a) * (pow(bodyLength, (*p_Gd_b)));
	//(*p_Maximum_Metabolic_Rate) = pow(bodyMass, (*p_Exponent_MetabilicRate_BodyMass));

	(*p_Maximum_Metabolic_Rate) = pow(((*p_Gd_a) * (pow((((double)(*p_Bin_Number_BodyLength)) * (*p_Bin_Width_BodyLength)), (*p_Gd_b)))), (*p_Exponent_MetabilicRate_BodyMass));
}

void Simul_Proc::Get_Metabolic_Rate_From_BodyLength(double *p_Metabolic_Rate, double *p_BodyLength, double *p_Gd_a, double *p_Gd_b, double *p_Exponent_MetabilicRate_BodyMass)
{
	//returns the metabolic rate by using bodylength
	//double bodyLength = ((double)(*p_Bin_Number_BodyLength)) * (*p_Bin_Width_BodyLength);
	//double bodyMass = (*p_Gd_a) * (pow(bodyLength, (*p_Gd_b)));
	//(*p_Maximum_Metabolic_Rate) = pow(bodyMass, (*p_Exponent_MetabilicRate_BodyMass));

	(*p_Metabolic_Rate) = pow(((*p_Gd_a) * (pow((*p_BodyLength), (*p_Gd_b)))), (*p_Exponent_MetabilicRate_BodyMass));
}

void Simul_Proc::Get_EDD(double *p_Edd, double *p_Rci, double *p_Metabolic_Rate, double *p_Ddg, double *p_Ddc)
{//returns edd for reflecting resource availability to development and survival
	//p_Edd: edd
	//p_Rci: available resource mass for an individual
	//p_Metabolic_Rate: metabolic rate
	//p_Ddc: scaling factor for the effect of available resource mass on the development in bodylength
	//double unit = (*p_Rci) / ((*p_Metabolic_Rate) * (*p_Ddc));
	(*p_Edd) = exp(((*p_Ddg) * (*p_Rci) / ((*p_Metabolic_Rate))));
}

void Simul_Proc::Get_LKE(double *p_LKE, double *p_EDD, double *p_LKRN, double *p_LKN)
{//returns the degree of development in body length
	//p_LKE: the degree of development in bodylength per time step
	//p_LKRN: the range of the degree of development in bodylength per time step
	//p_LKN: the minimum degree of development in bodylength per time step
	//p_EDD: the effect of available resource on the development in bodylength
	(*p_LKE) = (*p_LKN) + ((*p_LKRN) * (*p_EDD));
}

void Simul_Proc::Get_DDS_MetabolicRate_Rci(double *p_DDS, double *p_Ddc, double *p_Rci, double *p_MetabolicRate)
{//returns density-dependent-survival rate
	//by using the metabolic rate and available resource mass
	(*p_DDS) = 0.0;

	if ((*p_Rci) > 0.0)
	{
		(*p_DDS) = (*p_Rci) / ((*p_Rci) + ((*p_Ddc) * (*p_MetabolicRate)));
	}//end if
}

void Simul_Proc::Mtg_Unit_Discrete(int p_Deme, int p_Niche, int p_Spc)
{
	//male_List is a part of agt
	//num_Male is the actual number of males in the male_List
	//female_List is a part of agt
	//num_Female is the actual number of females in the female_List
	//p_Spc: pointer for the target species	

	//resets memory
	Set_Rpd_Memory_Cnt_Discrete();

	//For deciding the activation of reproduction
	int new_Born_Stage = 0;//assumes that the only stage for the new borns is 0
	bool *Reproduction_Activation = new bool[num_Sex];

	for (int sex = 0; sex < num_Sex; sex++)
	{
		//the number of immature life stages having chance of early reproduction
		//num_Immat_Stg_SexMaturation[spc][sex];
		if (new_Born_Stage >= (num_Stg[p_Spc][sex] - num_Immat_Stg_SexMaturation[p_Spc][sex] - 1))
		{
			Reproduction_Activation[sex] = true;
		}
		else
		{
			Reproduction_Activation[sex] = false;
		}
	}//end sex loop

	bool sampling_Status = false;//sampling status

	if (counter_Sampling == (interval_sampling - 1))
	{
		sampling_Status = true;
	}//end if

	//sets mates by bodylength
	Set_Mates_By_BodyLength(p_Deme, p_Niche, p_Spc);

	int i, j, matingClass_Female, chosen_Male, sex, fertility;
	double real_rand, tmp_Num_Male;//uniform random deviate
	vector<unsigned short int> gamete_0(num_Locus[p_Spc]);
	vector<unsigned short int> gamete_1(num_Locus[p_Spc]);
	//all traits
	vector<int> trait(num_Trait[p_Spc]);

	//the proportion of females at birth
	double propFemaleAtBirth = prop_Female_At_Birth[p_Spc];

	//mating & birth
	//the number of bodylength bins
	int end_0 = num_Bin_Bodylength[p_Spc];
	int end_1;

	//the number of bodylength bins
	//int end_Trt_Female = num_Bin_Bodylength[p_Spc];
	int k;
	double sum_Male_Mates;
	double *cumul_Prob_Male_Mate = new double[end_0];//the cumulative probability of a male being chosen by a female
	int *tmp_mateSearchLargeStep = new int[num_MateSearchLargeStep_Discrete[p_Spc]];
	int mateSearchStartPoint;
	int mateSearchStartPoint_Last = (num_MateSearchLargeStep_Discrete[p_Spc] - 1);
	double *tmp_mateSearchLargeStep_Val = new double[num_MateSearchLargeStep_Discrete[p_Spc]];
	int *tmp_CelNum_M = &celNum_M[p_Spc];
	int * tmp_Cel_Num_New_AgtLst = &celNum_New_AgtLst[p_Spc];
	int end_2 = num_MateSearchLargeStep_Discrete[p_Spc];

	for (i = 0; i < end_2; i++)
	{
		tmp_mateSearchLargeStep_Val[i] = mateSearchLargeStep_Discrete[p_Spc][i];
	}//end large-step search loop

	int **tmp_CelNum;
	int **tmp_Size_Agt;

	if (tick_t == true)
	{
		vector<vector<vector<Agent*>>> &tmp_Agt_Sink = agtLst_1[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		vector<vector<vector<Agent*>>> &tmp_Agt_Source = agtLst_0[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		tmp_CelNum = celNum_1[p_Deme][p_Niche][p_Spc];//growth cell number;	
		tmp_Size_Agt = size_AgtLst[p_Deme][p_Niche][p_Spc];//available cell size;
		double tmp_Mean_Fertility, tmp_SD_Fertility, tmp_BodyMass_Female;

		for (matingClass_Female = 0; matingClass_Female < end_0; matingClass_Female++)
		{
			//female trait
			//tmp_Mating_Trait_Female = matingClass_Female % end_Trt_Female;
			//calculates the life stage
			//stg_Female = (matingClass_Female - tmp_Mating_Trait_Female) / end_Trt_Female;

			end_1 = celNum_MatingList_Discrete[matingClass_Female][1];

			//end_1 > 0
			if (end_1 > 0)
			{//if there is a female in a mating group
				//returns bodymass from a bin number
				Get_BodyMass_From_Bin_Number(&tmp_BodyMass_Female, &matingClass_Female, &p_Spc);

				//calculates mean and sd for fertility
				Get_Mean_SD_Fertility_From_BodyMass(&tmp_Mean_Fertility, &tmp_SD_Fertility, &tmp_BodyMass_Female, &p_Spc);

				//std::cout << "tmp_Mean_Fertility" << tmp_Mean_Fertility << std::endl;
				//std::cout << "tmp_SD_Fertility" << tmp_SD_Fertility << std::endl;
				//Check_In("debug in");

				if (tmp_SD_Fertility > 0.0)
				{
					//calculates the probability of mating
					Set_Cumul_Prob_Choice_Trait_Freq_BodyLength(cumul_Prob_Male_Mate, &sum_Male_Mates, p_Spc, &matingClass_Female, tmp_mateSearchLargeStep);

					if (sum_Male_Mates > 0.0)
					{
						//normal fertility function reflecting the effect of the body size
						boost::math::normal tmp_NormDist(tmp_Mean_Fertility, tmp_SD_Fertility);

						//sets the upper & lower fertility bounds
						//Get_Bound_Fertility(&tmp_Fertility_Upper_Bound, &tmp_Fertility_Lower_Bound, &tmp_Mean_Fertility, &tmp_SD_Fertility);

						//std::cout << "Fertility upper bound " << tmp_Fertility_Upper_Bound << std::endl;
						//std::cout << "Fertility lower bound " << tmp_Fertility_Lower_Bound << std::endl;

						//std::cout << "Mean fertility " << tmp_Mean_Fertility << std::endl;
						//std::cout << "SD fertility " << tmp_SD_Fertility << std::endl;

						//std::cout << "Female bodymass " << tmp_BodyMass_Female << std::endl;
						//std::cout << "Female mating class " << matingClass_Female << std::endl;

						//verification
						//Check_In("Line 15526: Mtg_Unit_Discrete(): Before individual mate pick loop: Tick true");

						//female individual loop
						for (i = 0; i < end_1; i++)
						{

							real_rand = 0.0;
							//gets the value between 0.0015 & 0.9985
							while ((real_rand < QUANTILE_FOUR_SIGMA_LEFT) || (real_rand > QUANTILE_FOUR_SIGMA_RIGHT))
							{
								real_rand = (*rand_Uni_Real_Mtg)();
							}//end while

							//calculates quantile of the normal-distribution
							fertility = boost::math::iround((boost::math::quantile(tmp_NormDist, real_rand)));

							//checks fertility
							//Check_Fertility(&fertility, &tmp_Fertility_Upper_Bound, &tmp_Fertility_Lower_Bound);

							//std::cout << "fertility " << fertility << endl;

							//verification
							//Check_In("Line 15540: Check fertility: Tick true");

							if (fertility > 0)
							{
								//draws a uniform random deviate to choose a male mate
								real_rand = (*rand_Uni_Real_Mtg)();

								//Large-step-mate search
								mateSearchStartPoint = 0;

								//Large-step-mate search
								for (k = mateSearchStartPoint_Last; k > (-1); k--)
								{
									if (real_rand > tmp_mateSearchLargeStep_Val[k])
									{
										//finds the starting point for a mate search
										mateSearchStartPoint = tmp_mateSearchLargeStep[k];
										break;
									}//end if
								}//end large-step-mate search loop

								tmp_Num_Male = 0.0;

								//choose a group of males
								for (k = mateSearchStartPoint; k < end_0; k++)
								{
									if (real_rand < cumul_Prob_Male_Mate[k])
									{
										tmp_Num_Male = (double)celNum_MatingList_Discrete[k][0];//the number of males in
										break;
									}//end if
								}//end male mate loop

								//verification
								//std::cout << "Mating group " << k << std::endl;
								//std::cout << "the number of individuals in the Mating group " << tmp_Num_Male << std::endl;
								//Check_In("Line 15575: chooses a mating group: Tick true");

								if (tmp_Num_Male > 0.0)
								{//if there is a male in a chosen mating group
									chosen_Male = (int)(tmp_Num_Male * ((*rand_Uni_Real_Mtg)()));//chooses a male in the group

									//verification
									//cout << "Line 1530 " << "fert " << fertility << " chosen male " << chosen_Male << std::endl;
									//std::cout << " Tot " << tmp_Num_Male << " searc " << mateSearchStartPoint << endl;
									//std::cout << "chosen male " << chosen_Male << std::endl;
									//Check_In("Line 15583: chosen male: Tick true");

									for (j = 0; j < fertility; j++)
									{
										//a gamete from male
										Get_Gamete_No_Linkage(gamete_0, p_Spc, matingList_Discrete[k][0][chosen_Male]);
										//a gamete from female
										Get_Gamete_No_Linkage(gamete_1, p_Spc, matingList_Discrete[matingClass_Female][1][i]);

										//sex determination
										Get_Sex_New_Born(&sex, &propFemaleAtBirth);

										//adds a new agent
										Add_Mut_Agent_Unit(&p_Deme, &p_Niche, &p_Spc, gamete_0, gamete_1, trait, &tmp_CelNum[sex][new_Born_Stage], &tmp_Size_Agt[sex][new_Born_Stage], tmp_Agt_Sink[sex][new_Born_Stage], tmp_Agt_Source[sex][new_Born_Stage], tmp_CelNum_M, tmp_Cel_Num_New_AgtLst, &Reproduction_Activation[sex]);
									}//end fertility loop
								}//end if
							}//end if
						}//end female loop
					}//end if males
				}//end if fertility				
			}//if there is a female
		}//end mating class loop
	}
	else
	{//tick_t == false
		vector<vector<vector<Agent*>>> &tmp_Agt_Sink = agtLst_0[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		vector<vector<vector<Agent*>>> &tmp_Agt_Source = agtLst_1[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		tmp_CelNum = celNum_0[p_Deme][p_Niche][p_Spc];//growth cell number;	
		tmp_Size_Agt = size_AgtLst[p_Deme][p_Niche][p_Spc];//available cell size;
		double tmp_Mean_Fertility, tmp_SD_Fertility, tmp_BodyMass_Female;

		for (matingClass_Female = 0; matingClass_Female < end_0; matingClass_Female++)
		{
			//female trait
			//tmp_Mating_Trait_Female = matingClass_Female % end_Trt_Female;
			//calculates the life stage
			//stg_Female = (matingClass_Female - tmp_Mating_Trait_Female) / end_Trt_Female;

			end_1 = celNum_MatingList_Discrete[matingClass_Female][1];

			if (end_1 > 0)
			{//if there is a female in a mating group
				//returns bodymass from a bin number
				Get_BodyMass_From_Bin_Number(&tmp_BodyMass_Female, &matingClass_Female, &p_Spc);

				//calculates mean and sd for fertility
				Get_Mean_SD_Fertility_From_BodyMass(&tmp_Mean_Fertility, &tmp_SD_Fertility, &tmp_BodyMass_Female, &p_Spc);

				if (tmp_SD_Fertility > 0.0)
				{
					//calculates the probability of mating
					Set_Cumul_Prob_Choice_Trait_Freq_BodyLength(cumul_Prob_Male_Mate, &sum_Male_Mates, p_Spc, &matingClass_Female, tmp_mateSearchLargeStep);

					if (sum_Male_Mates > 0.0)
					{
						//normal fertility function reflecting the effect of the body size
						boost::math::normal tmp_NormDist(tmp_Mean_Fertility, tmp_SD_Fertility);

						//sets the upper & lower fertility bounds
						//Get_Bound_Fertility(&tmp_Fertility_Upper_Bound, &tmp_Fertility_Lower_Bound, &tmp_Mean_Fertility, &tmp_SD_Fertility);

						//female loop
						for (i = 0; i < end_1; i++)
						{
							real_rand = 0.0;
							//gets the value between 0.0015 & 0.9985 (3 * sigma)
							while ((real_rand < QUANTILE_FOUR_SIGMA_LEFT) || (real_rand > QUANTILE_FOUR_SIGMA_RIGHT))
							{
								real_rand = (*rand_Uni_Real_Mtg)();
							}//end while

							//calculates quantile of the normal-distribution
							fertility = boost::math::iround((boost::math::quantile(tmp_NormDist, real_rand)));

							//checks fertility
							//Check_Fertility(&fertility, &tmp_Fertility_Upper_Bound, &tmp_Fertility_Lower_Bound);

							if (fertility > 0)
							{
								//draws a uniform random deviate to choose a male mate
								real_rand = (*rand_Uni_Real_Mtg)();

								//Large-step-mate search
								mateSearchStartPoint = 0;

								//Large-step-mate search
								for (k = mateSearchStartPoint_Last; k > (-1); k--)
								{
									if (real_rand > tmp_mateSearchLargeStep_Val[k])
									{
										//finds the starting point for a mate search
										mateSearchStartPoint = tmp_mateSearchLargeStep[k];
										break;
									}//end if
								}//end large-step-mate search loop

								tmp_Num_Male = 0.0;

								//choose a group of males
								for (k = mateSearchStartPoint; k < end_0; k++)
								{
									if (real_rand < cumul_Prob_Male_Mate[k])
									{
										tmp_Num_Male = (double)celNum_MatingList_Discrete[k][0];//the number of males in
										break;
									}//end if
								}//end male mate loop							

								if (tmp_Num_Male > 0.0)
								{//if there is a male in a chosen mating group
									chosen_Male = (int)(tmp_Num_Male * ((*rand_Uni_Real_Mtg)()));//chooses a male in the group
									//cout << "Line 1530 " << "fert " << fertility << " chosen male " << chosen_Male << " precal " << (tmp_Num_Male * real_rand) << " Rand  " << real_rand << " Tot " << tmp_Num_Male << " searc " << mateSearchStartPoint << endl;

									for (j = 0; j < fertility; j++)
									{
										//a gamete from male
										Get_Gamete_No_Linkage(gamete_0, p_Spc, matingList_Discrete[k][0][chosen_Male]);
										//a gamete from female
										Get_Gamete_No_Linkage(gamete_1, p_Spc, matingList_Discrete[matingClass_Female][1][i]);

										//sex determination
										Get_Sex_New_Born(&sex, &propFemaleAtBirth);

										//adds a new agent
										Add_Mut_Agent_Unit(&p_Deme, &p_Niche, &p_Spc, gamete_0, gamete_1, trait, &tmp_CelNum[sex][new_Born_Stage], &tmp_Size_Agt[sex][new_Born_Stage], tmp_Agt_Sink[sex][new_Born_Stage], tmp_Agt_Source[sex][new_Born_Stage], tmp_CelNum_M, tmp_Cel_Num_New_AgtLst, &Reproduction_Activation[sex]);
									}//end fertility loop
								}//end if
							}//end if
						}//end female loop
					}//end if
				}//end if fertility				
			}//if there is a female
		}//end mating class loop		
	}//end else

	delete[] tmp_mateSearchLargeStep_Val;
	delete[] tmp_mateSearchLargeStep;
	delete[] cumul_Prob_Male_Mate;
	delete[] Reproduction_Activation;
}

void Simul_Proc::Mtg_Unit_Discrete_Assortative_Female_Choice(int p_Deme, int p_Niche, int p_Spc)
{
	//male_List is a part of agt
	//num_Male is the actual number of males in the male_List
	//female_List is a part of agt
	//num_Female is the actual number of females in the female_List
	//p_Spc: pointer for the target species	

	//resets memory
	Set_Rpd_Memory_Cnt_Discrete();

	//For deciding the activation of reproduction
	int new_Born_Stage = 0;//assumes that the only stage for the new borns is 0
	bool *Reproduction_Activation = new bool[num_Sex];

	for (int sex = 0; sex < num_Sex; sex++)
	{
		//the number of immature life stages having chance of early reproduction
		//num_Immat_Stg_SexMaturation[spc][sex];
		if (new_Born_Stage >= (num_Stg[p_Spc][sex] - num_Immat_Stg_SexMaturation[p_Spc][sex] - 1))
		{
			Reproduction_Activation[sex] = true;
		}
		else
		{
			Reproduction_Activation[sex] = false;
		}
	}//end sex loop

	bool sampling_Status = false;//sampling status

	if (counter_Sampling == 0)
	{
		sampling_Status = true;
	}//end if

	//sets mates by bodylength
	Set_Female_Mates_By_BodyLength_Male_By_One(p_Deme, p_Niche, p_Spc);

	int i, j, matingClass_Female, chosen_Male, sex, fertility;
	double real_rand;//uniform random deviate
	vector<unsigned short int> gamete_0(num_Locus[p_Spc]);
	vector<unsigned short int> gamete_1(num_Locus[p_Spc]);
	//all traits
	vector<int> trait(num_Trait[p_Spc]);

	//the proportion of females at birth
	double propFemaleAtBirth = prop_Female_At_Birth[p_Spc];

	//mating & birth
	//the number of bodylength bins
	int end_0 = num_Bin_Bodylength[p_Spc];
	int end_1;

	//the number of bodylength bins
	//int end_Trt_Female = num_Bin_Bodylength[p_Spc];
	double sum_Male_Mates;
	int *tmp_CelNum_M = &celNum_M[p_Spc];
	int * tmp_Cel_Num_New_AgtLst = &celNum_New_AgtLst[p_Spc];
	int end_2 = num_MateSearchLargeStep_Discrete[p_Spc];

	int **tmp_CelNum;
	int **tmp_Size_Agt;

	int tmp_Trait_Number_Assortative_Mating = TRAIT_NUMBER_MATING_ASSORTATIVE;
	int sequence_Mating, tmp_Female_Trait_Value_Int;
	double tmp_Bodylength_Male, tmp_Bodylength_Female, tmp_Mate_Preference;

	//scaling factor for female choosiness
	double tmp_Female_Choosiness_Scale = female_Choosiness_Scale[p_Spc];
	double tmp_Female_Choosiness_Intensity = female_Choosiness_Intensity[p_Spc];

	//the maximum bodylength
	double tmp_Max_BodyLength = bodyLength_Max_Spc[p_Spc];
	//the minumum body length
	double tmp_Min_BodyLength = bodyLength_Min_Spc[p_Spc];

	//mating status
	bool status_Mating;

	//the total number of alleles for assortative mating traits
	double tmp_tot_Num_Allele_Trait_Assortative_Mating = (double)(trt_Loc_Int[p_Spc][TRAIT_NUMBER_MATING_ASSORTATIVE] * num_Position[p_Spc]);

	Agent *mate_Male;
	Agent *mate_Female;

	//assortative mating mode
	bool tmp_Assortative_Mating_Mode = mode_Assortative_Mating;
	double tmp_Mean_Fertility, tmp_SD_Fertility, tmp_BodyMass_Female;

	//bodylength
	double tmp_Bodylength;
	//bodylength bin number
	int tmp_Bin_Bodylength_male, tmp_Bin_Bodylength_Female, tmp_Bin_GillRaker_Male, tmp_Bin_GillRaker_Female;

	//the total number of bins for calculating the bodylength bin number for a mating individual
	int tmp_max_Num_Clas_Trt = max_Num_Clas_Trt;
	//bin_Width for calculating the bodylength bin number for a mating individual
	double tmp_Bodylength_Bin_Width;
	//the minimum bodylength
	double tmp_Ltb_Minimum = 0.0;

	//calculate the bin_Width for calculating the bin number for a mating individual
	Get_Bin_Width_Discrete_BodyLength_Mating(&tmp_Bodylength_Bin_Width, &p_Spc, &tmp_max_Num_Clas_Trt);

	if (tick_t == true)
	{
		vector<vector<vector<Agent*>>> &tmp_Agt_Sink = agtLst_1[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		vector<vector<vector<Agent*>>> &tmp_Agt_Source = agtLst_0[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		tmp_CelNum = celNum_1[p_Deme][p_Niche][p_Spc];//growth cell number;	
		tmp_Size_Agt = size_AgtLst[p_Deme][p_Niche][p_Spc];//available cell size;

		for (matingClass_Female = 0; matingClass_Female < end_0; matingClass_Female++)
		{
			//the number of females in a mating class
			end_1 = celNum_MatingList_Discrete[matingClass_Female][SEX_FEMALE];

			//end_1 > 0
			if (end_1 > 0)
			{//if there is a female in a mating group
				//returns bodymass from a bin number
				Get_BodyMass_From_Bin_Number(&tmp_BodyMass_Female, &matingClass_Female, &p_Spc);

				//calculates mean and sd for fertility
				Get_Mean_SD_Fertility_From_BodyMass(&tmp_Mean_Fertility, &tmp_SD_Fertility, &tmp_BodyMass_Female, &p_Spc);

				if (tmp_SD_Fertility > 0.0)
				{
					//calculates the probability of mating
					sum_Male_Mates = (double)celNum_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][SEX_MALE];

					if (sum_Male_Mates > 0.0)
					{
						//normal fertility function reflecting the effect of the body size
						boost::math::normal tmp_NormDist(tmp_Mean_Fertility, tmp_SD_Fertility);

						//female individual loop
						for (i = 0; i < end_1; i++)
						{
							//set female mate
							mate_Female = matingList_Discrete[matingClass_Female][SEX_FEMALE][i];
							//get female bodylength here
							mate_Female->Get_BodyLength(&tmp_Bodylength_Female);

							if (tmp_Assortative_Mating_Mode == true)
							{//genetics-driven assortative mating mode
								//get female trait value in integer
								mate_Female->Get_Trait_Raw(&tmp_Female_Trait_Value_Int, &tmp_Trait_Number_Assortative_Mating);
								//calculate female choosiness
								tmp_Female_Choosiness_Intensity = ((double)tmp_Female_Trait_Value_Int) / tmp_tot_Num_Allele_Trait_Assortative_Mating;
							}
							//else
							//{//fixed assortative mating mode
							//tmp_Female_Choosiness_Intensity;
							//}

							//initialize mating status
							status_Mating = false;

							for (sequence_Mating = 0; sequence_Mating < NUMBER_MATE_ENCOUNTER; sequence_Mating++)
							{
								//choose male number
								chosen_Male = (int)(sum_Male_Mates * ((*rand_Uni_Real_Mtg)()));

								//get male bodylength
								matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][SEX_MALE][chosen_Male]->Get_BodyLength(&tmp_Bodylength_Male);

								Get_Competibility_Mates_BodyMass_Abs(&tmp_Mate_Preference, &tmp_Bodylength_Male, &tmp_Bodylength_Female, &tmp_Female_Choosiness_Intensity, &tmp_Female_Choosiness_Scale, &tmp_Max_BodyLength, &tmp_Min_BodyLength);

								//decide to mate
								if (((*rand_Uni_Real_Mtg)()) < tmp_Mate_Preference)
								{
									//set male mate
									mate_Male = matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][SEX_MALE][chosen_Male];
									//update mating status
									status_Mating = true;
									//exit loop
									break;
								}//end if
							}//end mating sequence loop

							if (status_Mating == true)
							{//if mating happens
								//record mate information here
								if (sampling_Status == true)
								{
									Data_Buffer_Mating_Pair_Add_Discrete(&p_Deme, &p_Niche, &p_Spc, mate_Male, mate_Female, &tmp_Bin_Bodylength_male, &tmp_Bin_Bodylength_Female, &tmp_Bin_GillRaker_Male, &tmp_Bin_GillRaker_Female, &tmp_Bodylength, &tmp_Ltb_Minimum, &tmp_Bodylength_Bin_Width, &tmp_max_Num_Clas_Trt);
								}//end if(sampling_Status == true)

								real_rand = 0.0;
								//gets the value between 0.0015 & 0.9985
								while ((real_rand < QUANTILE_FOUR_SIGMA_LEFT) || (real_rand > QUANTILE_FOUR_SIGMA_RIGHT))
								{
									real_rand = (*rand_Uni_Real_Mtg)();
								}//end while

								//calculates fertility (quantile of the normal-distribution)
								fertility = boost::math::iround((boost::math::quantile(tmp_NormDist, real_rand)));

								if (fertility > 0)
								{//if there is offspring
									for (j = 0; j < fertility; j++)
									{
										//a gamete from male
										Get_Gamete_No_Linkage(gamete_0, p_Spc, mate_Male);
										//a gamete from female
										Get_Gamete_No_Linkage(gamete_1, p_Spc, mate_Female);

										//sex determination
										Get_Sex_New_Born(&sex, &propFemaleAtBirth);

										//adds a new agent
										Add_Mut_Agent_Unit(&p_Deme, &p_Niche, &p_Spc, gamete_0, gamete_1, trait, &tmp_CelNum[sex][new_Born_Stage], &tmp_Size_Agt[sex][new_Born_Stage], tmp_Agt_Sink[sex][new_Born_Stage], tmp_Agt_Source[sex][new_Born_Stage], tmp_CelNum_M, tmp_Cel_Num_New_AgtLst, &Reproduction_Activation[sex]);
									}//end fertility loop
								}//end if
							}//end if

						}//end female loop
					}//end if males
				}//end if fertility				
			}//if there is a female
		}//end mating class loop
	}
	else
	{//tick_t == false
		vector<vector<vector<Agent*>>> &tmp_Agt_Sink = agtLst_0[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		vector<vector<vector<Agent*>>> &tmp_Agt_Source = agtLst_1[((p_Deme * num_Niche) + p_Niche)][p_Spc];
		tmp_CelNum = celNum_0[p_Deme][p_Niche][p_Spc];//growth cell number;	
		tmp_Size_Agt = size_AgtLst[p_Deme][p_Niche][p_Spc];//available cell size;

		for (matingClass_Female = 0; matingClass_Female < end_0; matingClass_Female++)
		{
			//the number of females in a mating class
			end_1 = celNum_MatingList_Discrete[matingClass_Female][SEX_FEMALE];

			//end_1 > 0
			if (end_1 > 0)
			{//if there is a female in a mating group
				//returns bodymass from a bin number
				Get_BodyMass_From_Bin_Number(&tmp_BodyMass_Female, &matingClass_Female, &p_Spc);

				//calculates mean and sd for fertility
				Get_Mean_SD_Fertility_From_BodyMass(&tmp_Mean_Fertility, &tmp_SD_Fertility, &tmp_BodyMass_Female, &p_Spc);

				if (tmp_SD_Fertility > 0.0)
				{
					//calculates the probability of mating
					sum_Male_Mates = (double)celNum_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][SEX_MALE];

					if (sum_Male_Mates > 0.0)
					{
						//normal fertility function reflecting the effect of the body size
						boost::math::normal tmp_NormDist(tmp_Mean_Fertility, tmp_SD_Fertility);

						//female individual loop
						for (i = 0; i < end_1; i++)
						{
							//set female mate
							mate_Female = matingList_Discrete[matingClass_Female][SEX_FEMALE][i];
							//get female bodylength here
							mate_Female->Get_BodyLength(&tmp_Bodylength_Female);

							if (tmp_Assortative_Mating_Mode == true)
							{//genetics-driven assortative mating mode
								//get female trait value in integer
								mate_Female->Get_Trait_Raw(&tmp_Female_Trait_Value_Int, &tmp_Trait_Number_Assortative_Mating);
								//calculate female choosiness
								tmp_Female_Choosiness_Intensity = ((double)tmp_Female_Trait_Value_Int) / tmp_tot_Num_Allele_Trait_Assortative_Mating;
							}
							//else
							//{//fixed assortative mating mode
							//tmp_Female_Choosiness_Intensity;
							//}

							//initialize mating status
							status_Mating = false;

							for (sequence_Mating = 0; sequence_Mating < NUMBER_MATE_ENCOUNTER; sequence_Mating++)
							{
								//choose male number
								chosen_Male = (int)(sum_Male_Mates * ((*rand_Uni_Real_Mtg)()));

								//get male bodylength
								matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][SEX_MALE][chosen_Male]->Get_BodyLength(&tmp_Bodylength_Male);

								Get_Competibility_Mates_BodyMass_Abs(&tmp_Mate_Preference, &tmp_Bodylength_Male, &tmp_Bodylength_Female, &tmp_Female_Choosiness_Intensity, &tmp_Female_Choosiness_Scale, &tmp_Max_BodyLength, &tmp_Min_BodyLength);

								//decide to mate
								if (((*rand_Uni_Real_Mtg)()) < tmp_Mate_Preference)
								{
									//set male mate
									mate_Male = matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][SEX_MALE][chosen_Male];
									//update mating status
									status_Mating = true;
									//exit loop
									break;
								}//end if
							}//end mating sequence loop

							if (status_Mating == true)
							{//if mating happens
								//record mate information here
								if (sampling_Status == true)
								{
									Data_Buffer_Mating_Pair_Add_Discrete(&p_Deme, &p_Niche, &p_Spc, mate_Male, mate_Female, &tmp_Bin_Bodylength_male, &tmp_Bin_Bodylength_Female, &tmp_Bin_GillRaker_Male, &tmp_Bin_GillRaker_Female, &tmp_Bodylength, &tmp_Ltb_Minimum, &tmp_Bodylength_Bin_Width, &tmp_max_Num_Clas_Trt);
								}//end if(sampling_Status == true)

								real_rand = 0.0;
								//gets the value between 0.0015 & 0.9985
								while ((real_rand < QUANTILE_FOUR_SIGMA_LEFT) || (real_rand > QUANTILE_FOUR_SIGMA_RIGHT))
								{
									real_rand = (*rand_Uni_Real_Mtg)();
								}//end while

								//calculates fertility (quantile of the normal-distribution)
								fertility = boost::math::iround((boost::math::quantile(tmp_NormDist, real_rand)));

								if (fertility > 0)
								{//if there is offspring
									for (j = 0; j < fertility; j++)
									{
										//a gamete from male
										Get_Gamete_No_Linkage(gamete_0, p_Spc, mate_Male);
										//a gamete from female
										Get_Gamete_No_Linkage(gamete_1, p_Spc, mate_Female);

										//sex determination
										Get_Sex_New_Born(&sex, &propFemaleAtBirth);

										//adds a new agent
										Add_Mut_Agent_Unit(&p_Deme, &p_Niche, &p_Spc, gamete_0, gamete_1, trait, &tmp_CelNum[sex][new_Born_Stage], &tmp_Size_Agt[sex][new_Born_Stage], tmp_Agt_Sink[sex][new_Born_Stage], tmp_Agt_Source[sex][new_Born_Stage], tmp_CelNum_M, tmp_Cel_Num_New_AgtLst, &Reproduction_Activation[sex]);
									}//end fertility loop
								}//end if
							}//end if

						}//end female loop
					}//end if males
				}//end if fertility				
			}//if there is a female
		}//end mating class loop
	}//end else

	delete[] Reproduction_Activation;
}

void Simul_Proc::Set_Mates_By_BodyLength(int p_Deme, int p_Niche, int p_Spc)
{
	//male_List is a part of agt
	//num_Male is the actual number of males in the male_List
	//female_List is a part of agt
	//num_Female is the actual number of females in the female_List
	//p_Spc: pointer for the target species
	int deme_Niche = p_Deme * num_Niche + p_Niche;
	int i, matingClass, tmp_Cel, stg, sex, end_Stg, start_Stg, end_i, end_Trt;
	int end_Sex = num_Sex;
	Agent *tmp_ID;

	//the number of classes in bodylength
	int end_Mt = num_Bin_Bodylength[p_Spc];
	double tmp_Unit_Bin_Width = bin_Width_BodyLength[p_Spc];
	double tmp_BodyLength;
	bool status_Sexual_Maturation;

	//initialization
	for (matingClass = 0; matingClass < end_Mt; matingClass++){
		for (i = 0; i < end_Sex; i++){
			celNum_MatingList_Discrete[matingClass][i] = 0;//initializes a temporary container
		}//end i loop
	}//end mating class loop

	if (tick_t == true){
		//for assigning male mates into mating classes
		for (sex = 0; sex < end_Sex; sex++)
		{
			end_Stg = num_Stg[p_Spc][sex];
			start_Stg = num_Stg[p_Spc][sex] - num_Immat_Stg_SexMaturation[p_Spc][sex] - 1;//start sexual maturation
			end_Trt = num_Bin_Bodylength[p_Spc];

			for (stg = start_Stg; stg < end_Stg; stg++)
			{
				end_i = celNum_0[p_Deme][p_Niche][p_Spc][sex][stg];

				if (end_i > 0)
				{//if there is an individual
					vector<Agent*> &tmp_Mate = agtLst_0[deme_Niche][p_Spc][sex][stg];

					for (i = 0; i < end_i; i++){
						//checks the sexual maturation status
						tmp_Mate[i]->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

						if (status_Sexual_Maturation == true)
						{
							tmp_ID = tmp_Mate[i];//gets ID

							//gets bodylength
							tmp_ID->Get_BodyLength(&tmp_BodyLength);

							//converts bodylength to bin number
							Convert_BodyLength_To_Bin_Number(&matingClass, &tmp_BodyLength, &tmp_Unit_Bin_Width, &end_Mt);

							tmp_Cel = celNum_MatingList_Discrete[matingClass][sex];

							//increases the counter
							celNum_MatingList_Discrete[matingClass][sex]++;

							if (tmp_Cel < size_MatingList_Discrete[matingClass][sex])
							{
								//if there are enough cells in the matingList
								matingList_Discrete[matingClass][sex][tmp_Cel] = tmp_ID;
							}
							else
							{
								//if there are not enough cells in the matingList
								matingList_Discrete[matingClass][sex].push_back(tmp_ID);
								size_MatingList_Discrete[matingClass][sex]++;
							}//end else
						}//end if
					}//end i loop
				}//end if
			}//end life stage loop
		}//end sex loop
	}
	else
	{
		//for assigning male mates into mating classes
		for (sex = 0; sex < end_Sex; sex++)
		{
			end_Stg = num_Stg[p_Spc][sex];
			//start sexual maturation
			start_Stg = num_Stg[p_Spc][sex] - num_Immat_Stg_SexMaturation[p_Spc][sex] - 1;
			end_Trt = num_Bin_Bodylength[p_Spc];

			for (stg = start_Stg; stg < end_Stg; stg++)
			{
				end_i = celNum_1[p_Deme][p_Niche][p_Spc][sex][stg];

				if (end_i > 0)
				{//if there is an individual
					vector<Agent*> &tmp_Mate = agtLst_1[deme_Niche][p_Spc][sex][stg];

					for (i = 0; i < end_i; i++){
						//checks the sexual maturation status
						tmp_Mate[i]->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

						if (status_Sexual_Maturation == true)
						{
							tmp_ID = tmp_Mate[i];//gets ID

							//gets bodylength
							tmp_ID->Get_BodyLength(&tmp_BodyLength);

							//converts bodylength to bin number
							Convert_BodyLength_To_Bin_Number(&matingClass, &tmp_BodyLength, &tmp_Unit_Bin_Width, &end_Mt);

							tmp_Cel = celNum_MatingList_Discrete[matingClass][sex];

							//increases the counter
							celNum_MatingList_Discrete[matingClass][sex]++;

							if (tmp_Cel < size_MatingList_Discrete[matingClass][sex])
							{
								//if there are enough cells in the matingList
								matingList_Discrete[matingClass][sex][tmp_Cel] = tmp_ID;
							}
							else
							{
								//if there are not enough cells in the matingList
								matingList_Discrete[matingClass][sex].push_back(tmp_ID);
								size_MatingList_Discrete[matingClass][sex]++;
							}//end else
						}//end if
					}//end i loop
				}//end if
			}//end life stage loop
		}//end sex loop
	}//end else

	//verification
	//initialization
	//for (matingClass = 0; matingClass < end_Mt; matingClass++){
	//for (i = 0; i < end_Sex; i++){
	//if (celNum_MatingList_Discrete[matingClass][i] > 0)
	//{
	//std::cout << "Mating class " << matingClass << " Sex " << i << " Number of agents " << celNum_MatingList_Discrete[matingClass][i] << std::endl;
	//}//end if
	//}//end i loop
	//}//end mating class loop

	//Check_In("Set_Mates_By_BodyLength()");
}

void Simul_Proc::Set_Female_Mates_By_BodyLength_Male_By_One(int p_Deme, int p_Niche, int p_Spc)
{
	//male_List is a part of agt
	//num_Male is the actual number of males in the male_List
	//female_List is a part of agt
	//num_Female is the actual number of females in the female_List
	//p_Spc: pointer for the target species
	int deme_Niche = p_Deme * num_Niche + p_Niche;
	int i, matingClass, tmp_Cel, stg, sex, end_Stg, start_Stg, end_i, end_Trt;
	int end_Sex = num_Sex;
	Agent *tmp_ID;

	//the number of classes in bodylength
	int end_Mt = num_Bin_Bodylength[p_Spc];
	double tmp_Unit_Bin_Width = bin_Width_BodyLength[p_Spc];
	double tmp_BodyLength;
	bool status_Sexual_Maturation;

	//initialization
	for (matingClass = 0; matingClass < end_Mt; matingClass++){
		for (i = 0; i < end_Sex; i++){
			celNum_MatingList_Discrete[matingClass][i] = 0;//initializes a temporary container
		}//end i loop
	}//end mating class loop

	if (tick_t == true)
	{
		//for assigning male mates into mating classes
		for (sex = 0; sex < end_Sex; sex++)
		{
			end_Stg = num_Stg[p_Spc][sex];
			start_Stg = num_Stg[p_Spc][sex] - num_Immat_Stg_SexMaturation[p_Spc][sex] - 1;//start sexual maturation
			end_Trt = num_Bin_Bodylength[p_Spc];

			if (sex == SEX_MALE)
			{//for male mates
				for (stg = start_Stg; stg < end_Stg; stg++)
				{
					end_i = celNum_0[p_Deme][p_Niche][p_Spc][sex][stg];

					if (end_i > 0)
					{//if there is an individual
						vector<Agent*> &tmp_Mate = agtLst_0[deme_Niche][p_Spc][sex][stg];

						for (i = 0; i < end_i; i++){
							//checks the sexual maturation status
							tmp_Mate[i]->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

							if (status_Sexual_Maturation == true)
							{
								tmp_ID = tmp_Mate[i];//gets ID

								//gets bodylength
								//tmp_ID->Get_BodyLength(&tmp_BodyLength);

								//converts bodylength to bin number
								//Convert_BodyLength_To_Bin_Number(&matingClass, &tmp_BodyLength, &tmp_Unit_Bin_Width, &end_Mt);

								tmp_Cel = celNum_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex];

								//increases the counter
								celNum_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex]++;

								if (tmp_Cel < size_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex])
								{
									//if there are enough cells in the matingList
									matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex][tmp_Cel] = tmp_ID;
								}
								else
								{
									//if there are not enough cells in the matingList
									matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex].push_back(tmp_ID);
									size_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex]++;
								}//end else
							}//end if
						}//end i loop
					}//end if
				}//end life stage loop
			}
			else
			{//for female mates
				for (stg = start_Stg; stg < end_Stg; stg++)
				{
					end_i = celNum_0[p_Deme][p_Niche][p_Spc][sex][stg];

					if (end_i > 0)
					{//if there is an individual
						vector<Agent*> &tmp_Mate = agtLst_0[deme_Niche][p_Spc][sex][stg];

						for (i = 0; i < end_i; i++){
							//checks the sexual maturation status
							tmp_Mate[i]->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

							if (status_Sexual_Maturation == true)
							{
								tmp_ID = tmp_Mate[i];//gets ID

								//gets bodylength
								tmp_ID->Get_BodyLength(&tmp_BodyLength);

								//converts bodylength to bin number
								Convert_BodyLength_To_Bin_Number(&matingClass, &tmp_BodyLength, &tmp_Unit_Bin_Width, &end_Mt);

								tmp_Cel = celNum_MatingList_Discrete[matingClass][sex];

								//increases the counter
								celNum_MatingList_Discrete[matingClass][sex]++;

								if (tmp_Cel < size_MatingList_Discrete[matingClass][sex])
								{
									//if there are enough cells in the matingList
									matingList_Discrete[matingClass][sex][tmp_Cel] = tmp_ID;
								}
								else
								{
									//if there are not enough cells in the matingList
									matingList_Discrete[matingClass][sex].push_back(tmp_ID);
									size_MatingList_Discrete[matingClass][sex]++;
								}//end else
							}//end if
						}//end i loop
					}//end if
				}//end life stage loop
			}//end else
		}//end sex loop
	}
	else
	{
		//for assigning male mates into mating classes
		for (sex = 0; sex < end_Sex; sex++)
		{
			end_Stg = num_Stg[p_Spc][sex];
			//start sexual maturation
			start_Stg = num_Stg[p_Spc][sex] - num_Immat_Stg_SexMaturation[p_Spc][sex] - 1;
			end_Trt = num_Bin_Bodylength[p_Spc];

			if (sex == SEX_MALE)
			{//for male mates
				for (stg = start_Stg; stg < end_Stg; stg++)
				{
					end_i = celNum_1[p_Deme][p_Niche][p_Spc][sex][stg];

					if (end_i > 0)
					{//if there is an individual
						vector<Agent*> &tmp_Mate = agtLst_1[deme_Niche][p_Spc][sex][stg];

						for (i = 0; i < end_i; i++){
							//checks the sexual maturation status
							tmp_Mate[i]->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

							if (status_Sexual_Maturation == true)
							{
								tmp_ID = tmp_Mate[i];//gets ID

								//gets bodylength
								//tmp_ID->Get_BodyLength(&tmp_BodyLength);

								//converts bodylength to bin number
								//Convert_BodyLength_To_Bin_Number(&matingClass, &tmp_BodyLength, &tmp_Unit_Bin_Width, &end_Mt);

								tmp_Cel = celNum_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex];

								//increases the counter
								celNum_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex]++;

								if (tmp_Cel < size_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex])
								{
									//if there are enough cells in the matingList
									matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex][tmp_Cel] = tmp_ID;
								}
								else
								{
									//if there are not enough cells in the matingList
									matingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex].push_back(tmp_ID);
									size_MatingList_Discrete[BIN_NUMBER_MALE_DEFAULT][sex]++;
								}//end else
							}//end if
						}//end i loop
					}//end if
				}//end life stage loop
			}
			else
			{//for female mates
				for (stg = start_Stg; stg < end_Stg; stg++)
				{
					end_i = celNum_1[p_Deme][p_Niche][p_Spc][sex][stg];

					if (end_i > 0)
					{//if there is an individual
						vector<Agent*> &tmp_Mate = agtLst_1[deme_Niche][p_Spc][sex][stg];

						for (i = 0; i < end_i; i++){
							//checks the sexual maturation status
							tmp_Mate[i]->Get_Status_Sexual_Maturation(&status_Sexual_Maturation);

							if (status_Sexual_Maturation == true)
							{
								tmp_ID = tmp_Mate[i];//gets ID

								//gets bodylength
								tmp_ID->Get_BodyLength(&tmp_BodyLength);

								//converts bodylength to bin number
								Convert_BodyLength_To_Bin_Number(&matingClass, &tmp_BodyLength, &tmp_Unit_Bin_Width, &end_Mt);

								tmp_Cel = celNum_MatingList_Discrete[matingClass][sex];

								//increases the counter
								celNum_MatingList_Discrete[matingClass][sex]++;

								if (tmp_Cel < size_MatingList_Discrete[matingClass][sex])
								{
									//if there are enough cells in the matingList
									matingList_Discrete[matingClass][sex][tmp_Cel] = tmp_ID;
								}
								else
								{
									//if there are not enough cells in the matingList
									matingList_Discrete[matingClass][sex].push_back(tmp_ID);
									size_MatingList_Discrete[matingClass][sex]++;
								}//end else
							}//end if
						}//end i loop
					}//end if
				}//end life stage loop
			}//end else
		}//end sex loop
	}//end else

	//verification
	//initialization
	//for (matingClass = 0; matingClass < end_Mt; matingClass++){
	//for (i = 0; i < end_Sex; i++){
	//if (celNum_MatingList_Discrete[matingClass][i] > 0)
	//{
	//std::cout << "Mating class " << matingClass << " Sex " << i << " Number of agents " << celNum_MatingList_Discrete[matingClass][i] << std::endl;
	//}//end if
	//}//end i loop
	//}//end mating class loop

	//Check_In("Set_Mates_By_BodyLength()");
}

void Simul_Proc::Set_Cumul_Prob_Choice_Trait_Freq_BodyLength(double *p_Cumul_Prob_choice, double *p_Sum_Prod_Freq_Preference, int p_Spc, int *p_Bin_BodyLength_Female, int *p_LargeStepSearch)
{
	//p_Bin_BodyLength_Female: the bin number for the female bodylength
	int tot_Num_Bin_BodyLength = num_Bin_Bodylength[(p_Spc)];//the number of classes for a chosen trait
	//int end_Trt = num_Clas_Trt_Spc[p_Spc][0];	

	//the total number of males in the niche
	int sum_Males_Int = 0;
	int cls;

	//calculates the total number of male
	for (cls = 0; cls < tot_Num_Bin_BodyLength; cls++){
		sum_Males_Int += celNum_MatingList_Discrete[cls][0];
	}//end trait loop

	//the total number of males in the niche in double
	double sum_Males = (double)sum_Males_Int;
	//sum_Prod_Freq_Preference
	//the sum of the product of a mate frequency and a mate preference
	double sum_Prod_Freq_Preference = 0.0;

	if (sum_Males > 0.0)
	{//if there is a male
		double tmp_Tot_Mating_Class = (double)tot_Num_Bin_BodyLength;
		//the bodymass of females
		double bodyMass_Female;
		//calculates female bodymass
		Get_BodyMass_From_Bin_Number(&bodyMass_Female, p_Bin_BodyLength_Female, &p_Spc);

		double tmp_Female_Choosiness_Phenotype = female_Choosiness_Intensity[p_Spc];
		double tmp_Female_Choosiness_Scale = female_Choosiness_Scale[p_Spc];

		//the difference between the maximum adult bodymass and the minimum adult body mass
		double tmp_Max_Length = bodyLength_Max_Spc[p_Spc];
		double tmp_Min_Length = bodyLength_Max_Spc[p_Spc];
		//cout << "Max diff: " << tmp_Diff_Max_Min_Male_Mate_Weight<<endl;
		//cout << "Female choosiness intensity: " << tmp_Female_Choosiness_Phenotype<<endl;
		//char tt;
		//cin >> tt;

		double frequencyFactor;//Mate frequency factor in mate choice
		double matePreference;//mate preference factor in mate choice
		//temporary male bodymass
		double tmp_BodyMass_Male;

		if (mode_Gene_Trait[p_Spc][1] == 0)
		{
			if (tmp_Female_Choosiness_Phenotype > 0.0)
			{//assortative mating power function
				for (cls = 0; cls < tot_Num_Bin_BodyLength; cls++){
					if (celNum_MatingList_Discrete[cls][0] > 0)
					{//if there is a male
						//calculates a frequency
						frequencyFactor = (((double)celNum_MatingList_Discrete[cls][0]) / sum_Males);
						//calculates male bodymass
						Get_BodyMass_From_Bin_Number(&tmp_BodyMass_Male, &cls, &p_Spc);
						//calculates mate competibility
						Get_Competibility_Mates_BodyMass_Abs(&matePreference, &tmp_BodyMass_Male, &bodyMass_Female, &tmp_Female_Choosiness_Phenotype, &tmp_Female_Choosiness_Scale, &tmp_Max_Length, &tmp_Min_Length);

						p_Cumul_Prob_choice[cls] = frequencyFactor * matePreference;
						sum_Prod_Freq_Preference += p_Cumul_Prob_choice[cls];//calculates the denominator
					}
					else
					{//there is no male
						p_Cumul_Prob_choice[cls] = 0.0;
					}//end else

					//for verification
					//if ((*p_Mating_Trait_Female) == trait_Male)
					//{
					//cout << "Mate type: " << cls << endl;
					//cout << "frequency factor: " << frequencyFactor << endl;
					//cout << "Choosiness scale: " << tmp_Female_Choosiness_Scale << endl;
					//cout << "Female bodymass: " << mating_Trait_Female << " Male Bodymass: " << bodyMass[p_Spc][0][stg_Male][trait_Male] << " Diff: " << (mating_Trait_Female - bodyMass[p_Spc][0][stg_Male][trait_Male]) << endl;
					//cout << "Max Diff: " << tmp_Diff_Max_Min_Male_Mate_Weight << endl;
					//cout << "Male life stage: " << stg_Male << " Male trait: " << trait_Male << endl;
					//cout << "mate preference: " << matePreference << endl;
					//char tt;
					//cin >> tt;
					//}//end if
				}//end trait loop
			}
			else
			{//random mating
				for (cls = 0; cls < tot_Num_Bin_BodyLength; cls++){
					if (celNum_MatingList_Discrete[cls][0] > 0)
					{//if there is a male
						//calculates a frequency
						p_Cumul_Prob_choice[cls] = (((double)celNum_MatingList_Discrete[cls][0]) / sum_Males);
						sum_Prod_Freq_Preference += p_Cumul_Prob_choice[cls];//calculates the denominator
					}
					else
					{//there is no male
						p_Cumul_Prob_choice[cls] = 0.0;
					}//end else
				}//end trait loop
			}//end else
		}
		else if (mode_Gene_Trait[p_Spc][1] == 2)
		{//assortative mating Normal function
			//Thibert-Plante & Gavrilets (2013)
		}
		else
		{
			for (cls = 0; cls < tot_Num_Bin_BodyLength; cls++){
				p_Cumul_Prob_choice[cls] = 0.0;
			}//end trait loop
		}//end else
	}//end if

	//normalization
	(*p_Sum_Prod_Freq_Preference) = sum_Prod_Freq_Preference;//sets the sum of the product of a mate frequency and a mate preference 

	int i, end1;
	//normalizes the probability and calculates the cumulative probability
	if (sum_Prod_Freq_Preference > 0.0)
	{
		//cout<<endl << "Line 5952: There is a male." << endl<<endl;
		double tmp_Cumul_Prob = 0.0;

		for (cls = 0; cls < tot_Num_Bin_BodyLength; cls++)
		{
			tmp_Cumul_Prob += (p_Cumul_Prob_choice[cls] / sum_Prod_Freq_Preference);
			p_Cumul_Prob_choice[cls] = tmp_Cumul_Prob;
		}//end trait loop

		//sets search start point for a large-step-mate search		
		end1 = num_MateSearchLargeStep_Discrete[p_Spc];
		int startPoint = 0;
		double currentCumulProb;

		for (i = 0; i < end1; i++)
		{
			currentCumulProb = mateSearchLargeStep_Discrete[p_Spc][i];//current search target

			for (cls = startPoint; cls < tot_Num_Bin_BodyLength; cls++)
			{
				if (p_Cumul_Prob_choice[cls] >= currentCumulProb)
				{
					p_LargeStepSearch[i] = cls;
					startPoint = cls;
					break;
				}//end if
			}//end mating class loop
		}//end large step search loop
	}
	else
	{
		end1 = num_MateSearchLargeStep_Discrete[p_Spc];

		for (i = 0; i < end1; i++)
		{
			p_LargeStepSearch[i] = tot_Num_Bin_BodyLength;
		}//end large step search loop
	}//end else

	//verification
	//for (cls = 0; cls < tot_Num_Bin_BodyLength; cls++){
	//std::cout << "class:" << cls << " probability mating " << p_Cumul_Prob_choice[cls] << std::endl;
	//}//end trait loop

	//std::cout << "total number of male mates " << sum_Males << std::endl;
}

void Simul_Proc::Get_Competibility_Mates_BodyMass_Abs(double *p_Mate_Competibility, double *p_BodyLength_A, double *p_BodyLength_B, double *p_Choosiness_Phenotype, double *p_Choosiness_Scale, double *p_Max_Bodylength, double *_Min_Bodylength)
{
	//calculates the difference
	(*p_Mate_Competibility) = abs(((*p_BodyLength_A) - (*p_BodyLength_B)));

	if ((*p_Mate_Competibility) > (*p_Max_Bodylength))
	{
		(*p_Mate_Competibility) = (*p_Max_Bodylength);
	}//end if

	//calculates mate competibility
	(*p_Mate_Competibility) = 1.0 - ((*p_Mate_Competibility) / ((*p_Max_Bodylength) - (*_Min_Bodylength)));

	if ((*p_Mate_Competibility) > 0.0)
	{
		(*p_Mate_Competibility) = (pow((*p_Mate_Competibility), ((*p_Choosiness_Scale) * (*p_Choosiness_Phenotype))));
	}
	else
	{
		(*p_Mate_Competibility) = 0.0;
	}//end else

	if ((*p_Mate_Competibility) > 1.0)
	{
		(*p_Mate_Competibility) = 1.0;
	}//end if
}

void Simul_Proc::Get_Mean_SD_Fertility_From_BodyMass(double *p_Mean_Fertility, double *p_SD_Fertility, double *p_BodyMass, int *p_Species)
{
	double tmp_Max_Prop_Fertility;
	//the maximum proportion of fertility	
	Get_Mean_Fertility_Predicted_By_Bodymass(&tmp_Max_Prop_Fertility, &bodyMass_Max[(*p_Species)]);

	//std::cout << "Line 10166 maxprop" << tmp_Max_Prop_Fertility << std::endl;

	double tmp_Prop_Fertility;
	//the maximum proportion of fertility	
	//calculates the total number of offspring
	Get_Mean_Fertility_Predicted_By_Bodymass(&tmp_Prop_Fertility, p_BodyMass);

	//std::cout << "Line 10166 tmp prop" << tmp_Prop_Fertility << std::endl;

	//std::cout << "max fertility prop" << tmp_Max_Prop_Fertility << std::endl;
	//std::cout << "current fertility prop" << tmp_Prop_Fertility << std::endl;

	tmp_Prop_Fertility = tmp_Prop_Fertility / tmp_Max_Prop_Fertility;

	//std::cout << "Line 10166 result prop" << tmp_Prop_Fertility << std::endl;

	//std::cout << "fertility prop" << tmp_Prop_Fertility << std::endl;

	//factor for calculating a sex ratio at birth
	double sexPropFactor;
	//gets a factor for coverting the number of female offsprings to the number of male and female offsprings
	Get_Offsprinf_Convergence_Factor_From_FemaleOffspring(&sexPropFactor, &proportion_Female_At_Birth[(*p_Species)]);

	//std::cout << "Line 10166 sex prop" << sexPropFactor << std::endl;
	//std::cout << "Line 10166 prop female" << proportion_Female_At_Birth[(*p_Species)] << std::endl;

	//mean fertility
	(*p_Mean_Fertility) = maximum_Fertility[(*p_Species)] * sexPropFactor * tmp_Prop_Fertility;
	//standar deviation for fertility
	(*p_SD_Fertility) = (*p_Mean_Fertility) * maximum_Fertility_CV[(*p_Species)];




	//std::cout << "Line 10166 max fertility" << maximum_Fertility[(*p_Species)] << std::endl;
	//std::cout << "Line 10166 max cv" << maximum_Fertility_CV[(*p_Species)] << std::endl;
	//std::cout << "Line 10166 mean fert" << (*p_Mean_Fertility) << std::endl;
	//std::cout << "Line 10166 sd fertility" << (*p_SD_Fertility) << std::endl;
}

void Simul_Proc::Get_BodyMass_From_BodyLength(double *p_BodyMass, double *p_BodyLength, int *p_Species)
{//returns bodylength from a bin number
	//the maximum bodylength
	//double maximum_BodyLength = ((double)num_Bin_Bodylength[(*p_Species)]) * bin_Width_BodyLength[(*p_Species)];
	//double proportion = ((double)(*p_Bin_Number)) / ((double)(num_Bin_Bodylength[(*p_Species)] - 1));
	//(*p_BodyLength) = proportion * maximum_BodyLength;

	//calculates bodymass
	(*p_BodyMass) = (scalingFactor_BodyLength_BodyMass_Conversion[(*p_Species)] * (pow((*p_BodyLength), exponent_BodyLength_BodyMass_Conversion[(*p_Species)])));
}

void Simul_Proc::Get_Bound_Fertility(int *p_Upper_Bound, int *p_Lower_Bound, double *p_Mean_Fertility, double *p_SD_Fertility)
{
	(*p_Upper_Bound) = boost::math::iround(((*p_Mean_Fertility) + 3.0 * (*p_SD_Fertility)));
	(*p_Lower_Bound) = boost::math::iround(((*p_Mean_Fertility) - 3.0 * (*p_SD_Fertility)));

	if ((*p_Lower_Bound) < 0)
	{
		(*p_Lower_Bound) = 0;
	}//end if
}

void Simul_Proc::Check_Fertility(int *p_Fertility, int *p_Upper_Bound, int *p_Lower_Bound)
{
	if ((*p_Fertility) > (*p_Upper_Bound))
	{
		(*p_Fertility) = (*p_Upper_Bound);
	}//end if

	if ((*p_Fertility) < (*p_Lower_Bound))
	{
		(*p_Fertility) = (*p_Lower_Bound);
	}//end if
}

void Simul_Proc::Get_Sex_New_Born(int *p_Sex, double *p_Prop_Female_At_Birth)
{
	//sex determination
	if (((*rand_Uni_Real_Mtg)()) < (*p_Prop_Female_At_Birth))
	{
		(*p_Sex) = 1;//female
	}
	else
	{
		(*p_Sex) = 0;//male
	}//end else
}

void Simul_Proc::Data_Buffer_Mating_Pair_Add_Discrete(int *p_Deme, int *p_Niche, int *p_Spc, Agent *p_ID_Male, Agent *p_ID_Female, int *p_Bin_Num_Bodylength_Male, int *p_Bin_Num_Bodylength_Female, int *p_Bin_Num_GillRaker_Male, int *p_Bin_Num_GillRaker_Female, double *p_tmp_Bodylength, double *p_Minimum_Bodylength, double *p_Bin_width_BodyLength, int *p_Tot_Num_Bins)
{
	//get male bodylength
	p_ID_Male->Get_BodyLength(p_tmp_Bodylength);
	//calculate male bodylength bin number
	Get_Bin_Number_Bodylength_Mating_Indiv(p_Bin_Num_Bodylength_Male, p_tmp_Bodylength, p_Minimum_Bodylength, p_Bin_width_BodyLength, p_Tot_Num_Bins);

	//get male bodylength
	p_ID_Female->Get_BodyLength(p_tmp_Bodylength);
	//calculate male bodylength bin number
	Get_Bin_Number_Bodylength_Mating_Indiv(p_Bin_Num_Bodylength_Female, p_tmp_Bodylength, p_Minimum_Bodylength, p_Bin_width_BodyLength, p_Tot_Num_Bins);

	//update a data container
	data_Buffer_Bodysize_Mtg[(*p_Deme)][(*p_Niche)][(*p_Spc)][(*p_Bin_Num_Bodylength_Male)][(*p_Bin_Num_Bodylength_Female)]++;

	//male traits
	int *trait_Male = p_ID_Male->Get_Trait();
	//male bodysize genotype
	(*p_Bin_Num_Bodylength_Male) = trait_Male[0];
	//male gillraker genotype
	(*p_Bin_Num_GillRaker_Male) = trait_Male[1];

	//female trait
	int *trait_Female = p_ID_Female->Get_Trait();
	//male bodysize genotype
	(*p_Bin_Num_Bodylength_Female) = trait_Female[0];
	//male gillraker genotype
	(*p_Bin_Num_GillRaker_Female) = trait_Female[1];

	//adds trait types of a mating pair
	tmp_Data_Buffer_Mtg_Trait[(*p_Deme)][(*p_Niche)][(*p_Spc)][(*p_Bin_Num_Bodylength_Male)][(*p_Bin_Num_GillRaker_Male)][(*p_Bin_Num_Bodylength_Female)][(*p_Bin_Num_GillRaker_Female)]++;

	//adds mate bodysize genotype data
	data_Buffer_Bodysize_Genotype_Mtg[(*p_Deme)][(*p_Niche)][(*p_Spc)][(*p_Bin_Num_Bodylength_Male)][(*p_Bin_Num_Bodylength_Female)]++;

	//adds mate gillraker genotype data
	data_Buffer_Gillraker_Genotype_Mtg[(*p_Deme)][(*p_Niche)][(*p_Spc)][(*p_Bin_Num_GillRaker_Male)][(*p_Bin_Num_GillRaker_Female)]++;

	//index_Trait_Bodymass[(*p_Spc)][0][(*p_LifeStage_Male)][(*p_Trait_0_Male)]
	//index_Trait_Bodymass[(*p_Spc)][1][(*p_LifeStage_Female)][(*p_Trait_0_Female)]
	//tmp_Data_Buffer_Mtg_Bodymass[(*p_Deme)][(*p_Niche)][(*p_Spc)][(index_Trait_Bodymass[(*p_Spc)][0][(*p_LifeStage_Male)][trait_Male[0]])][(index_Trait_Bodymass[(*p_Spc)][1][(*p_LifeStage_Female)][trait_Female[0]])] += 1.0;

	//index_Trait_GillRaker[(*p_Spc)][0][(*p_LifeStage_Male)][(*p_Trait_1_Male)]
	//index_Trait_GillRaker[(*p_Spc)][1][(*p_LifeStage_Female)][(*p_Trait_1_Female)]
	//tmp_Data_Buffer_Mtg_GillRaker[(*p_Deme)][(*p_Niche)][(*p_Spc)][(index_Trait_GillRaker[(*p_Spc)][0][(*p_LifeStage_Male)][trait_Male[1]])][(index_Trait_GillRaker[(*p_Spc)][1][(*p_LifeStage_Female)][trait_Female[1]])] += 1.0;
}

void Simul_Proc::Set_Rpd_Memory_Cnt_Discrete(void)
{
	int i, j;

	for (i = 0; i < max_Num_Bin_Bodylength; i++){
		for (j = 0; j < num_Sex; j++){
			if (celNum_MatingList_Discrete[i][j] > maxCelNum_MatingList_Discrete[i][j])
			{
				maxCelNum_MatingList_Discrete[i][j] = celNum_MatingList_Discrete[i][j];
			}//end if

			celNum_MatingList_Discrete[i][j] = 0;
		}//end sex loop
	}//end mating class loop

	//std::cout << "bin length " << max_Num_Bin_Bodylength << std::endl;
	//Check_In("Set_Rpd_Memory_Cnt_Discrete(void)");
}

void Simul_Proc::Rpd_Mtg_Cls_Discrete(void)
{
	int deme, niche, spc;

	for (spc = 0; spc < num_Spc; spc++){
		for (deme = 0; deme < num_Deme; deme++){
			for (niche = 0; niche < num_Niche; niche++){
				//Mtg_Unit_Discrete(deme, niche, spc);
				Mtg_Unit_Discrete_Assortative_Female_Choice(deme, niche, spc);
			}//end niche loop
		}//end deme loop
	}//end species loop
}

void Simul_Proc::Check_In(string tmp_Str)
{
	std::cout << "Check-in" << std::endl;
	std::cout << tmp_Str << std::endl;
	std::cout << "Type a character" << std::endl;
	char tt;
	std::cin >> tt;
}

void Simul_Proc::Convert_BodyLength_To_Bin_Number(int *p_Bin_Number, double *p_BodyLength, double *p_Unit_Bin_Width, int *p_Maximun_Bin_Number)
{//converts bodylength to bin number
	(*p_Bin_Number) = (int)(((*p_BodyLength)) / (*p_Unit_Bin_Width));

	if ((*p_Bin_Number) >= (*p_Maximun_Bin_Number))
	{
		(*p_Bin_Number) = (*p_Maximun_Bin_Number) - 1;
	}//end if
}

void Simul_Proc::Convert_Mu_SD_NormDist_To_M_V_LogNormDist(double *p_M, double *p_V, double *p_Mu, double *p_SD)
{
	//converts mu and sd for normal distribution to m and v for log-normal distribution
	//mu: mean for normal distribution
	//sd: standard deviation for normal distribution
	//m: m for log-normal distribution
	//v: v for log-normal distribution

	if ((*p_Mu) > 0.0)
	{
		//v
		(*p_V) = log((1.0 + (((*p_SD) * (*p_SD)) / ((*p_Mu)))));

		//m
		(*p_M) = log((*p_Mu)) - (0.5 * ((*p_V) * (*p_V)));
	}
	else
	{
		(*p_V) = 0.0;
		(*p_M) = 0.0;
	}//end else
}

void Simul_Proc::Get_Random_Deviate_Log_Normal_Dist(double *p_Random_Deviate_LogNormalDist, double *p_Random_Deviate_UniformDist, double *p_M, double *p_V)
{
	//p_Random_Deviate_LogNormalDist: randome deviate from log normal distribution
	//p_Random_Deviate_UniformDist: random deviate from uniform distribution
	//m: location for log normal distribution
	//v: scale for log normal distribution
	if ((*p_M) > 0.0)
	{
		(*p_Random_Deviate_LogNormalDist) = boost::math::quantile(boost::math::lognormal((*p_M), (*p_V)), (*p_Random_Deviate_UniformDist));
	}
	else
	{
		(*p_Random_Deviate_LogNormalDist) = 0.0;
	}//end else	
}

void Simul_Proc::Get_Random_Growth_BodyLength(double *p_Random_Deviate_LogNormalDist, double *p_Random_Deviate_UniformDist, double *p_M, double *p_V, double *p_Mu, double *p_SD, double *p_Upper_Limit)
{//returns random bodylength growth
	//p_Upper_Limit: the upper limit for cdf 
	//(i.e. p_Upper_Limit = 0.9973)

	//p_Random_Deviate_LogNormalDist: randome deviate from log normal distribution
	//bodylength growth for a time step
	//p_Random_Deviate_UniformDist: random deviate from uniform distribution
	//m: location for log normal distribution
	//v: scale for log normal distribution

	//converts mu and sd for normal distribution to m and v for log-normal distribution
	Convert_Mu_SD_NormDist_To_M_V_LogNormDist(p_M, p_V, p_Mu, p_SD);

	(*p_Random_Deviate_UniformDist) = 1.0;

	while ((*p_Random_Deviate_UniformDist) >= (*p_Upper_Limit))
	{
		//draw a random deviate from uniform distribution
		(*p_Random_Deviate_UniformDist) = (*rand_Uni_Real)();
	}//end while

	//gets a random deviate from log-normal distribution
	(*p_Random_Deviate_LogNormalDist) = boost::math::quantile(boost::math::lognormal((*p_M), (*p_V)), (*p_Random_Deviate_UniformDist));
}

void Simul_Proc::Get_VBK(double *p_VBK, double *p_K, double *p_Age, double *p_Denominator, double *p_TimeStep)
{
	//returns a bodylength growth factor regarding to age
	(*p_VBK) = (exp((((*p_K) * (*p_Age)) / (*p_Denominator)))) - (exp((((*p_K) * ((*p_Age) + (*p_TimeStep))) / (*p_Denominator))));
}

void Simul_Proc::Get_SxIniB(Agent *p_ID, double *p_SxIniB, double *p_LGX, double *p_Edd, double *p_Ce, double *p_Ci, double *p_Allele_Dose_BodyLength, int *p_Stg, int *p_Age_Rprd_Ini, int *p_Age_Rprd_Last, double *p_Inverse_Num_Repd_Imm_Stg_Double)
{
	//p_Age_Rprd_Last: the first life stage that all genotype are sexually matured
	//p_Age_Rprd_Ini: the first life stage having a chance of sexual maturation
	//p_Inverse_Num_Repd_Imm_Stg_Double: 1.0 / the number of reproductive stages in immature life stages in double
	//p_IGN: the proportion of the minimum asymptotic bodylength to the maximum asymptotic bodylength
	//p_Tot_Num_Allele_BodyLength: the total number of alleles controling bodylength for a species
	//p_BodyLength: temporary variable for storing the number of allele controlling body size

	//returns a bodylength growth factor regarding to genetics and environment
	if ((*p_Stg) < (*p_Age_Rprd_Ini))
	{

	}
	else if ((*p_Stg) == (*p_Age_Rprd_Last))
	{//activates sexual maturation
		p_ID->Activate_Status_Sexual_Maturation();
	}
	else
	{
		//get the number of allele determining body size
		//calculate the chance of reproduction initiation
		(*p_SxIniB) = (1.0 - (*p_LGX)) + (*p_LGX) * (((1.0 - (*p_Ce) - (*p_Ci)) * (1.0 - (*p_Allele_Dose_BodyLength))) + ((*p_Ce) * (*p_Edd)) + ((*p_Ci) * (1.0 - (*p_Allele_Dose_BodyLength)) * (*p_Edd)));

		if ((*p_SxIniB) > 0.0)
		{
			if (((*rand_Uni_Real)()) < (pow((*p_SxIniB), (*p_Inverse_Num_Repd_Imm_Stg_Double))))
			{
				p_ID->Activate_Status_Sexual_Maturation();
			}//end if
		}//end if
	}//end else
}

void Simul_Proc::Get_VBC_Foraging(double *p_VBc, double *p_Ltg, double *p_Edd, double *p_Ltrn, double *p_Lgn, double *p_Ce, double *p_Ci)
{
	//return the factor controlling asymptotic bodylength based on von Bertallanffy function
	(*p_VBc) = (1.0 - (*p_Lgn)) + ((*p_Lgn) *   ((*p_Ltrn) + (1.0 - (*p_Ltrn)) * (((1.0 - (*p_Ce) - (*p_Ci)) * (*p_Ltg)) + ((*p_Ce) * (*p_Edd)) + ((*p_Ci) * (*p_Ltg) * (*p_Edd)))));
}

void Simul_Proc::Get_VBC(double *p_VBc, double *p_Ltg, double *p_Edd, double *p_Ltrn, double *p_Lgn, double *p_Ce, double *p_Ci)
{
	//return the factor controlling asymptotic bodylength based on von Bertallanffy function
	(*p_VBc) = ((*p_Lgn) * ((*p_Ltrn) + (1.0 - (*p_Ltrn)) * (((1.0 - (*p_Ce) - (*p_Ci)) * (*p_Ltg)) + ((*p_Ce) * (*p_Edd)) + ((*p_Ci) * (*p_Ltg) * (*p_Edd)))));
}

void Simul_Proc::Get_Baseline_Growth_BodyLength(double *p_Baseline_Growth_BodyLength, double *p_Maximum_Asymp_BodyLength, double *p_Lgn, double *p_VBK)
{
	(*p_Baseline_Growth_BodyLength) = (*p_Maximum_Asymp_BodyLength) * (1.0 - (*p_Lgn)) * (*p_VBK);
}

void Simul_Proc::Get_Cumulative_VBk(double *p_VBk_Cumul, double *p_K, double *p_Age, double *p_TotNumStg)
{
	//return the cumulative degree of growth over age based on von Bertallanffy function
	(*p_VBk_Cumul) = 1.0 - (exp(((*p_K) * (*p_Age) / (*p_TotNumStg))));
}

void Simul_Proc::Get_BodyLength_Von_Bertallanffy(double *p_BodyLength, double *p_BodyLength_Asymptotic_Max, double *p_VBC, double *p_VBK_Cumulative)
{
	//return bodylength based on von Bertallanffy function
	(*p_BodyLength) = (*p_BodyLength_Asymptotic_Max) * (*p_VBC) * (*p_VBK_Cumulative);
}

void Simul_Proc::Get_BodyLength_Von_Bertallanffy_Foraging(double *p_BodyLength, double *p_BodyLength_Previous, double *p_BodyLength_Asymptotic_Max, double *p_VBC, double *p_VBK)
{
	//return bodylength based on von Bertallanffy function
	(*p_BodyLength) = (*p_BodyLength_Previous) + ((*p_BodyLength_Asymptotic_Max) * (*p_VBC) * (*p_VBK));
}

void Simul_Proc::Get_CE_Trait(double *p_CE, vector<int> &p_Trait, double *p_Max_Num_Allele_Trait_Plasticity, double *p_CE_Fixed)
{
	if (mode_Plasticity == true)
	{
		//p_Max_Num_Allele_Trait_Plasticity: the maximum number of allele for plasticity trait
		(*p_CE) = ((double)(p_Trait[TRAIT_NUMBER_PLASTICITY_BODYLENGTH])) / (p_Max_Num_Allele_Trait_Plasticity[TRAIT_NUMBER_PLASTICITY_BODYLENGTH]);
	}
	else
	{//no plasticity mode
		(*p_CE) = (*p_CE_Fixed);
	}//end else
}

void Simul_Proc::Get_Bin_Width_Discrete_BodyLength_Mating(double *p_Bodylength_Bin_Width, int *p_Spc, int *p_Num_Bins)
{
	//double maximum bodylength = (num_Bin_Bodylength[(*p_Species)] * bin_Width_BodyLength[(*p_Species)]);

	//minimum bodylength
	double tmp_Minimum_Bodylength = 0.0;

	//calculate bin width
	(*p_Bodylength_Bin_Width) = (((num_Bin_Bodylength[(*p_Spc)] * bin_Width_BodyLength[(*p_Spc)])) - tmp_Minimum_Bodylength) / ((double)(*p_Num_Bins));
}

void Simul_Proc::Get_Bin_Number_Bodylength_Mating_Indiv(int *p_BinNumber, double *p_Bodylength, double *p_Minimum_Bodylength, double *p_Bin_width_BodyLength, int *p_Tot_Num_Bins)
{
	//calculate the bodylength bin number for a mating individual
	(*p_BinNumber) = ((int)(((*p_Bodylength) - (*p_Minimum_Bodylength)) / (*p_Bin_width_BodyLength)));

	//limit maximum bin number
	if ((*p_BinNumber) > ((*p_Tot_Num_Bins) - 1))
	{
		(*p_BinNumber) = (*p_Tot_Num_Bins) - 1;
	}//end if
}

void Simul_Proc::Data_Output_Buffer_Write_Bodylength_Mate_Pair(int *p_Current_Tstep)
{
	//writes mate pair bodylength data
	int i, j, spc, l, m, numRow, numCol, counter_Column, end_0;
	end_0 = max_Num_Clas_Trt;
	////////////////////////////
	//Add data to trait output//
	numRow = 1;//the total number of cells in the trait-output-data buffer
	numCol = 1 + (num_Deme * num_Niche * num_Spc * max_Num_Clas_Trt * max_Num_Clas_Trt);

	//////////////////////////
	//print out summary data//
	//////////////////////////

	//time step
	data_Buffer_Bodysize_Mtg_2D[0][0] = (*p_Current_Tstep);

	//initialize data buffer
	int tmp_NumCells = 1 + (num_Deme * num_Niche * num_Spc * end_0);

	for (l = 1; l < tmp_NumCells; l++){
		data_Buffer_Bodysize_Mtg_2D[0][l] = 0;
	}//end lower-bound trait loop

	int offset = 0;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				offset = 1 + (i * (num_Niche * num_Spc * end_0)) + (j * (num_Spc * end_0)) + (spc * end_0);
								
				//calculate summary data
				for (l = 0; l < end_0; l++){
					for (m = 0; m < end_0; m++){
						//map data on the data buffer for output
						data_Buffer_Bodysize_Mtg_2D[0][(offset + l)] += data_Buffer_Bodysize_Mtg[i][j][spc][l][m];
						data_Buffer_Bodysize_Mtg_2D[0][(offset + m)] += data_Buffer_Bodysize_Mtg[i][j][spc][l][m];
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Bodysize_Mtg_Adt, data_Buffer_Bodysize_Mtg_2D, &numRow, &tmp_NumCells);


	////////////
	//Bodysize//
	////////////

	//initialize column counter
	counter_Column = 0;

	//time step
	data_Buffer_Bodysize_Mtg_2D[0][counter_Column] = (*p_Current_Tstep);

	//increase the column counter
	counter_Column++;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (l = 0; l < end_0; l++){
					for (m = 0; m < end_0; m++){
						//map data on the data buffer for output
						data_Buffer_Bodysize_Mtg_2D[0][counter_Column] = data_Buffer_Bodysize_Mtg[i][j][spc][l][m];

						//initialize data container
						data_Buffer_Bodysize_Mtg[i][j][spc][l][m] = 0;

						//increase the column counter
						counter_Column++;
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Bodysize_Mtg, data_Buffer_Bodysize_Mtg_2D, &numRow, &numCol);

	/////////////////////
	//Bodysize genotype//
	/////////////////////

	//////////////////////////
	//print out summary data//
	//////////////////////////

	//time step
	data_Buffer_Bodysize_Mtg_2D[0][0] = (*p_Current_Tstep);

	//initialize data buffer
	tmp_NumCells = 1 + (num_Deme * num_Niche * num_Spc * end_0);

	for (l = 1; l < tmp_NumCells; l++){
		data_Buffer_Bodysize_Mtg_2D[0][l] = 0;
	}//end lower-bound trait loop

	offset = 0;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				offset = 1 + (i * (num_Niche * num_Spc * end_0)) + (j * (num_Spc * end_0)) + (spc * end_0);

				//calculate summary data
				for (l = 0; l < end_0; l++){
					for (m = 0; m < end_0; m++){
						//map data on the data buffer for output
						data_Buffer_Bodysize_Mtg_2D[0][(offset + l)] += data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l][m];
						data_Buffer_Bodysize_Mtg_2D[0][(offset + m)] += data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l][m];
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Bodysize_Genotype_Mtg_Adt, data_Buffer_Bodysize_Mtg_2D, &numRow, &tmp_NumCells);
	
	/////////////////////
	//Bodysize genotype//
	/////////////////////

	//initialize column counter
	counter_Column = 0;

	//time step
	data_Buffer_Bodysize_Mtg_2D[0][counter_Column] = (*p_Current_Tstep);

	//increase the column counter
	counter_Column++;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (l = 0; l < end_0; l++){
					for (m = 0; m < end_0; m++){
						//map data on the data buffer for output
						data_Buffer_Bodysize_Mtg_2D[0][counter_Column] = data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l][m];

						//initialize data container
						data_Buffer_Bodysize_Genotype_Mtg[i][j][spc][l][m] = 0;

						//increase the column counter
						counter_Column++;
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_Bodysize_Genotype_Mtg, data_Buffer_Bodysize_Mtg_2D, &numRow, &numCol);

	/////////////////////
	//GillRaker genotype//
	/////////////////////

	//print out summary data
	//time step
	data_Buffer_Bodysize_Mtg_2D[0][0] = (*p_Current_Tstep);

	//initialize data buffer
	tmp_NumCells = 1 + (num_Deme * num_Niche * num_Spc * end_0);

	for (l = 1; l < tmp_NumCells; l++){
		data_Buffer_Bodysize_Mtg_2D[0][l] = 0;
	}//end lower-bound trait loop

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				offset = 1 + (i * (num_Niche * num_Spc * end_0)) + (j * (num_Spc * end_0)) + (spc * end_0);

				//calculate summary data
				for (l = 0; l < end_0; l++){
					for (m = 0; m < end_0; m++){
						//map data on the data buffer for output
						data_Buffer_Bodysize_Mtg_2D[0][(offset + l)] += data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l][m];
						data_Buffer_Bodysize_Mtg_2D[0][(offset + m)] += data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l][m];
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_GillRaker_Genotype_Mtg_Adt, data_Buffer_Bodysize_Mtg_2D, &numRow, &tmp_NumCells);

	/////////////////////
	//GillRaker genotype//
	/////////////////////

	//initialize column counter
	counter_Column = 0;

	//time step
	data_Buffer_Bodysize_Mtg_2D[0][counter_Column] = (*p_Current_Tstep);

	//increase the column counter
	counter_Column++;

	for (i = 0; i < num_Deme; i++){
		for (j = 0; j < num_Niche; j++){
			for (spc = 0; spc < num_Spc; spc++){
				for (l = 0; l < end_0; l++){
					for (m = 0; m < end_0; m++){
						//map data on the data buffer for output
						data_Buffer_Bodysize_Mtg_2D[0][counter_Column] = data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l][m];

						//initialize data container
						data_Buffer_Gillraker_Genotype_Mtg[i][j][spc][l][m] = 0;

						//increase the column counter
						counter_Column++;
					}//end lower-bound trait loop
				}//end lower-bound trait loop
			}//end species loop
		}//end niche loop
	}//end deme loop

	csvRW.CSV_Write_Add_Integer_2D(fileName_GillRaker_Genotype_Mtg, data_Buffer_Bodysize_Mtg_2D, &numRow, &numCol);
}